@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2004-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2004 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -28,10 +28,9 @@
  */
 
 #include "vmware.h"
+#include "vm_version.h"
 #include "util.h"
 #include "str.h"
-#include "unicode.h"
-#include "vixCommands.h"
 
 #include "vixOpenSource.h"
 
@@ -51,14 +50,6 @@
 // To be safe, we always use 8 bytes.
 #define  PROPERTY_SIZE_POINTER   8
 
-static VixError
-VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,
-                               const char *buffer,
-                               size_t bufferSize,
-                               Bool clobber,
-                               VixPropertyListBadEncodingAction action);
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -116,15 +107,8 @@ VixPropertyList_RemoveAllWithoutHandles(VixPropertyListImpl *propList)   // IN
       propList->properties = property->next;
 
       if (VIX_PROPERTYTYPE_STRING == property->type) {
-         if (property->isSensitive) {
-            Util_ZeroString(property->value.strValue);
-         }
          free(property->value.strValue);
       } else if (VIX_PROPERTYTYPE_BLOB == property->type) {
-         if (property->isSensitive) {
-            Util_Zero(property->value.blobValue.blobContents,
-                      property->value.blobValue.blobSize);
-         }
          free(property->value.blobValue.blobContents);
       }
       
@@ -133,37 +117,6 @@ VixPropertyList_RemoveAllWithoutHandles(VixPropertyListImpl *propList)   // IN
 } // VixPropertyList_RemoveAllWithoutHandles
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_MarkAllSensitive --
- *
- *       Mark all properties in a list sensitive.
- *
- * Results:
- *       As above
- *
- * Side effects:
- *       None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-VixPropertyList_MarkAllSensitive(VixPropertyListImpl *propList)  // IN/OUT:
-{
-   if (NULL != propList) {
-      VixPropertyValue *property = propList->properties;
-
-      while (NULL != property) {
-         property->isSensitive = TRUE;
-
-         property = property->next;
-      }
-   }
-} // VixPropertyList_MarkAllSensitive
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -186,10 +139,10 @@ VixPropertyList_MarkAllSensitive(VixPropertyListImpl *propList)  // IN/OUT:
  */
 
 VixError
-VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
-                          Bool dirtyOnly,                 // IN:
-                          size_t *resultSize,             // OUT:
-                          char **resultBuffer)            // OUT:
+VixPropertyList_Serialize(VixPropertyListImpl    *propList,       // IN
+                          Bool dirtyOnly,                         // IN
+                          size_t *resultSize,                     // OUT
+                          char **resultBuffer)                    // OUT
 {
    VixError err = VIX_OK;
    VixPropertyValue *property = NULL;
@@ -202,8 +155,6 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
    size_t bufferSize = 0;
    size_t pos = 0;
  
-   ASSERT_ON_COMPILE(PROPERTY_LENGTH_SIZE == sizeof valueLength);
-
    if ((NULL == propList) ||
        (NULL == resultSize) ||
        (NULL == resultBuffer)) {
@@ -233,7 +184,7 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
 
       bufferSize += headerSize;
       
-      switch (property->type) {
+      switch(property->type) {
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_INTEGER:
             bufferSize += PROPERTY_SIZE_INT32;
@@ -242,26 +193,7 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_STRING:
             if (property->value.strValue) {
-               valueLength = strlen(property->value.strValue) + 1;
-               /*
-                * The deserialization code rejects all non-UTF-8 strings.
-                * There should not be any non-UTF-8 strings passing
-                * through our code since we should have either converted
-                * non-UTF-8 strings from system APIs to UTF-8, or validated
-                * that any client-provided strings were UTF-8. But this
-                * if we've missed something, this should hopefully catch the
-                * offending code close to the act.
-                */
-               if (!Unicode_IsBufferValid(property->value.strValue,
-                                          valueLength,
-                                          STRING_ENCODING_UTF8)) {
-                  Log("%s: attempted to send a non-UTF-8 string for "
-                      "property %d.\n",
-                      __FUNCTION__, property->propertyID);
-                  ASSERT(0);
-                  err = VIX_E_INVALID_UTF8_STRING;
-               }
-               bufferSize += valueLength;
+               bufferSize += (strlen(property->value.strValue) + 1);
             } else {
                err = VIX_E_INVALID_ARG;
                goto abort;
@@ -285,14 +217,8 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
 
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_POINTER:
-            /*
-             * We should not serialize any pointer.
-             * Catch such programming errors.
-             */
-            err = VIX_E_INVALID_ARG;
-            Log("%s:%d, pointer properties cannot be serialized.\n",
-                __FUNCTION__, __LINE__);
-            goto abort;
+            bufferSize += PROPERTY_SIZE_POINTER;
+            break;
 
          ////////////////////////////////////////////////////////
          default:
@@ -303,11 +229,7 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
       property = property->next;
    }
 
-   *resultBuffer = (char*) VixMsg_MallocClientData(bufferSize);
-   if (NULL == *resultBuffer) {
-      err = VIX_E_OUT_OF_MEMORY;
-      goto abort;
-   }
+   *resultBuffer = (char*) Util_SafeCalloc(1, bufferSize);
    serializeBuffer = *resultBuffer;
 
    pos = 0;
@@ -333,7 +255,7 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
       memcpy(&(serializeBuffer[pos]), &(property->type), propertyTypeSize);
       pos += propertyTypeSize;
 
-      switch (property->type) {
+      switch(property->type) {
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_INTEGER:
              valueLength = PROPERTY_SIZE_INT32;
@@ -383,7 +305,16 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
 
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_POINTER:
-            NOT_IMPLEMENTED();
+            if (property->value.ptrValue) {
+               valueLength = PROPERTY_SIZE_POINTER;
+               memcpy(&(serializeBuffer[pos]), &valueLength, propertyValueLengthSize);
+               pos += propertyValueLengthSize;
+               memcpy(&(serializeBuffer[pos]), &(property->value.ptrValue), sizeof(property->value.ptrValue));
+            } else {
+               err = VIX_E_INVALID_ARG;
+               goto abort;
+            }
+            break;
 
          ////////////////////////////////////////////////////////
          default:
@@ -401,12 +332,8 @@ VixPropertyList_Serialize(VixPropertyListImpl *propList,  // IN:
 abort:
    if (VIX_OK != err) {
       free(serializeBuffer);
-      if (NULL != resultBuffer) {
-         *resultBuffer = NULL;
-      }
-      if (NULL != resultSize) {
-         *resultSize = 0;
-      }
+      *resultBuffer = NULL;
+      *resultSize = 0;
    }
 
    return err;
@@ -418,72 +345,11 @@ abort:
  *
  * VixPropertyList_Deserialize --
  *
- *       Deserialize a property list from a buffer. Repeated properties
- *       are clobbered.
- *
- * Results:
- *      VixError.
- *
- * Side effects:
- *       None.
- *
- *-----------------------------------------------------------------------------
- */
-
-VixError
-VixPropertyList_Deserialize(VixPropertyListImpl *propList,            // IN
-                            const char *buffer,                       // IN
-                            size_t bufferSize,                        // IN
-                            VixPropertyListBadEncodingAction action)  // IN
-{
-   return VixPropertyListDeserializeImpl(propList,
-                                         buffer,
-                                         bufferSize,
-                                         TRUE, // clobber
-                                         action);
-} // VixPropertyList_Deserialize
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_DeserializeNoClobber --
- *
- *       Deserialize a property list from a buffer. Repeated properties
- *       are preserved.
- *
- * Results:
- *      VixError.
- *
- * Side effects:
- *       None.
- *
- *-----------------------------------------------------------------------------
- */
-
-VixError
-VixPropertyList_DeserializeNoClobber(VixPropertyListImpl *propList,     // IN
-                                     const char *buffer,                // IN
-                                     size_t bufferSize,                 // IN
-                                     VixPropertyListBadEncodingAction action) // IN
-{
-   return VixPropertyListDeserializeImpl(propList,
-                                         buffer,
-                                         bufferSize,
-                                         FALSE, // clobber
-                                         action);
-} // VixPropertyList_DeserializeNoClobber
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyListDeserializeImpl --
- *
  *       Deserialize a property list from a buffer. 
  *
  *       This function should be modified to deal with the case of 
  *       properties of type VIX_PROPERTYTYPE_HANDLE.
+ *       
  *
  * Results:
  *      VixError.
@@ -495,11 +361,9 @@ VixPropertyList_DeserializeNoClobber(VixPropertyListImpl *propList,     // IN
  */
 
 VixError
-VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
-                               const char *buffer,                       // IN
-                               size_t bufferSize,                        // IN
-                               Bool clobber,                             // IN
-                               VixPropertyListBadEncodingAction action)  // IN
+VixPropertyList_Deserialize(VixPropertyListImpl *propList,     // IN
+                            const char *buffer,                // IN
+                            size_t bufferSize)                 // IN
 {
    VixError err = VIX_OK;
    VixPropertyValue *property = NULL;
@@ -508,6 +372,7 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
    int *intPtr;
    Bool *boolPtr;
    int64 *int64Ptr;
+   void **ptrPtr;
    unsigned char* blobPtr;
    int *propertyIDPtr;
    int *lengthPtr;
@@ -516,8 +381,6 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
    size_t propertyValueLengthSize;
    size_t headerSize;
    VixPropertyType *propertyTypePtr;
-   Bool allocateFailed;
-   Bool needToEscape;
 
    if ((NULL == propList)
        || (NULL == buffer)) {
@@ -540,7 +403,7 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
       pos += propertyTypeSize;
       lengthPtr = (int*) &(buffer[pos]);
       pos += propertyValueLengthSize;
-
+   
       /*
        * Do not allow lengths of 0 or fewer bytes. Those do not make sense,
        * unless you can pass a NULL blob, which Serialize() does not allow.
@@ -550,32 +413,25 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
          err = VIX_E_INVALID_SERIALIZED_DATA;
          goto abort;
       }
-
+      
       /*
        * Create the property if missing
        */
-      if (clobber) {
-         err = VixPropertyList_FindProperty(propList,
-                                            *propertyIDPtr,
-                                            *propertyTypePtr,
-                                            0, // index
-                                            TRUE, //createIfMissing
-                                            &property);
-      } else {
-         err = VixPropertyListAppendProperty(propList,
-                                             *propertyIDPtr,
-                                             *propertyTypePtr,
-                                             &property);
-      }
-
+      err = VixPropertyList_FindProperty(propList,
+                                         *propertyIDPtr,
+                                         *propertyTypePtr,
+                                         0, // index
+                                         TRUE, //createIfMissing
+                                         &property);
+      
       if (VIX_OK != err) {
          goto abort;
       }
-
+      
       /*
        * Initialize the property to the received value
        */
-      switch (*propertyTypePtr) {
+      switch(*propertyTypePtr) {
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_INTEGER:
             if (PROPERTY_SIZE_INT32 != *lengthPtr) {
@@ -597,44 +453,8 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
                err = VIX_E_INVALID_SERIALIZED_DATA;
                goto abort;
             }
-
-            needToEscape = FALSE;
-
-            /*
-             * Make sure the string is valid UTF-8 before copying it. We
-             * expect all strings stored in the process to be UTF-8.
-             */
-            if (!Unicode_IsBufferValid(strPtr, *lengthPtr,
-                                       STRING_ENCODING_UTF8)) {
-               Log("%s: non-UTF-8 string received for property %d.\n",
-                   __FUNCTION__, *propertyIDPtr);
-               switch (action) {
-               case VIX_PROPERTY_LIST_BAD_ENCODING_ERROR:
-                  err = VIX_E_INVALID_UTF8_STRING;
-                  goto abort;
-               case VIX_PROPERTY_LIST_BAD_ENCODING_ESCAPE:
-                  needToEscape = TRUE;
-               }
-
-            }
             free(property->value.strValue);
-
-            if (needToEscape) {
-               property->value.strValue =
-                  Unicode_EscapeBuffer(strPtr, *lengthPtr,
-                                       STRING_ENCODING_UTF8);
-               if (NULL == property->value.strValue) {
-                  err = VIX_E_OUT_OF_MEMORY;
-                  goto abort;
-               }
-            } else {
-               property->value.strValue =
-                  VixMsg_StrdupClientData(strPtr, &allocateFailed);
-               if (allocateFailed) {
-                  err = VIX_E_OUT_OF_MEMORY;
-                  goto abort;
-               }
-            }
+            property->value.strValue = Util_SafeStrdup(strPtr);
             break;
 
          ////////////////////////////////////////////////////////
@@ -646,7 +466,7 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
             boolPtr = (Bool*) &(buffer[pos]);
             property->value.boolValue = *boolPtr;
             break;
-
+         
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_INT64:
             if (PROPERTY_SIZE_INT64 != *lengthPtr) {
@@ -666,9 +486,7 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
              * process. Admittedly we've already bounds checked it, but this is
              * pretty easy to handle.
              */
-            free(property->value.blobValue.blobContents);
-            property->value.blobValue.blobContents =
-               VixMsg_MallocClientData(*lengthPtr);
+            property->value.blobValue.blobContents = malloc(*lengthPtr);
             if (NULL == property->value.blobValue.blobContents) {
                err = VIX_E_OUT_OF_MEMORY;
                goto abort;
@@ -678,25 +496,26 @@ VixPropertyListDeserializeImpl(VixPropertyListImpl *propList,            // IN
 
          ////////////////////////////////////////////////////////
          case VIX_PROPERTYTYPE_POINTER:
-            /*
-             * Deserialize an pointer property should not be allowed.
-             * An evil peer could send us such data.
-             */
-            err = VIX_E_INVALID_SERIALIZED_DATA;
-            Log("%s:%d, pointer properties cannot be serialized.\n",
-                __FUNCTION__, __LINE__);
-            goto abort;
+            // The size may be different on different machines.
+            // To be safe, we always use 8 bytes.
+            if (PROPERTY_SIZE_POINTER != *lengthPtr) {
+               err = VIX_E_INVALID_SERIALIZED_DATA;
+               goto abort;
+            }
+            ptrPtr = (void**) &(buffer[pos]);
+            property->value.ptrValue = *ptrPtr;
+            break;
 
          ////////////////////////////////////////////////////////
          default:
             err = VIX_E_UNRECOGNIZED_PROPERTY;
-            goto abort;
+            goto abort;     
       }
 
       pos += *lengthPtr;
    }
 
-abort:  
+abort:   
    if ((VIX_OK != err) && (NULL != propList)) {
       VixPropertyList_RemoveAllWithoutHandles(propList);
    }
@@ -818,7 +637,6 @@ VixPropertyListAppendProperty(VixPropertyListImpl *propList,   // IN
    property->type = type;
    property->propertyID = propertyID;
    property->isDirty = TRUE;
-   property->isSensitive = FALSE;
 
    /*
     * We only have to initialize the values that we release, so
@@ -873,7 +691,7 @@ abort:
  *
  * Results:
  *       VixError. VIX_OK if the property was found.
- *                 VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
+ *                     VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
  *
  * Side effects:
  *       None
@@ -918,12 +736,20 @@ abort:
 /*
  *-----------------------------------------------------------------------------
  *
- * VixPropertyListSetStringImpl --
+ * VixPropertyList_SetString --
  *
- *       Saves a copy of a string property value. Sets sensitivity.
+ *       Saves a copy of a string property value. The value is identified
+ *       by the integer property ID.
+ *
+ *       Value names are unique within a single proeprty list.
+ *       If a previous value with the same propertyID value already
+ *       existed in this property list, then it is replaced with the new
+ *       value. Otherwise, a new value is added.
+ *
+ *       This fails if the value is present but has a different type.
  *
  * Results:
- *       As above
+ *       VixError
  *
  * Side effects:
  *       None
@@ -931,15 +757,33 @@ abort:
  *-----------------------------------------------------------------------------
  */
 
-static void
-VixPropertyListSetStringImpl(VixPropertyValue *property,  // IN:
-                             const char *value,           // IN:
-                             Bool isSensitive)            // IN:
+VixError
+VixPropertyList_SetString(VixPropertyListImpl *propList,    // IN
+                          int propertyID,                   // IN
+                          const char *value)                // IN
 {
+   VixError err = VIX_OK;
+   VixPropertyValue *property = NULL;
+
+   if (NULL == propList) {
+      err = VIX_E_INVALID_ARG;
+      goto abort;
+   }
+   
+   /*
+    * Find or create an entry for this property.
+    */
+   err = VixPropertyList_FindProperty(propList,
+                                      propertyID, 
+                                      VIX_PROPERTYTYPE_STRING, 
+                                      0,
+                                      TRUE, 
+                                      &property);
+   if (VIX_OK != err) {
+      goto abort;
+   }
+
    if (NULL != property->value.strValue) {
-      if (property->isSensitive) {
-         Util_ZeroString(property->value.strValue);
-      }
       free(property->value.strValue);
       property->value.strValue = NULL;
    }
@@ -947,120 +791,9 @@ VixPropertyListSetStringImpl(VixPropertyValue *property,  // IN:
       property->value.strValue = Util_SafeStrdup(value);
    }
    property->isDirty = TRUE;
-   property->isSensitive = isSensitive;
-} // VixPropertyListSetStringImpl
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_SetString --
- *
- *       Saves a copy of a string property value. The value is identified
- *       by the integer property ID.
- *
- *       Value names are unique within a single property list.
- *       If a previous value with the same propertyID value already
- *       existed in this property list, then it is replaced with the new
- *       value. Otherwise, a new value is added.
- *
- *       This fails if the value is present but has a different type.
- *
- * Results:
- *       VixError
- *
- * Side effects:
- *       None
- *
- *-----------------------------------------------------------------------------
- */
-
-VixError
-VixPropertyList_SetString(VixPropertyListImpl *propList,  // IN:
-                          int propertyID,                 // IN:
-                          const char *value)              // IN:
-{
-   VixError err = VIX_OK;
-   VixPropertyValue *property = NULL;
-
-   if (NULL == propList) {
-      err = VIX_E_INVALID_ARG;
-      goto abort;
-   }
-
-   /*
-    * Find or create an entry for this property.
-    */
-   err = VixPropertyList_FindProperty(propList,
-                                      propertyID,
-                                      VIX_PROPERTYTYPE_STRING,
-                                      0,
-                                      TRUE,
-                                      &property);
-   if (VIX_OK == err) {
-      VixPropertyListSetStringImpl(property, value, property->isSensitive);
-   }
 
 abort:
-
-   return err;
-} // VixPropertyList_SetString
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_SetStringSensitive --
- *
- *       Saves a copy of a string property value. The value is identified
- *       by the integer property ID. Mark sensitive.
- *
- *       Value names are unique within a single property list.
- *       If a previous value with the same propertyID value already
- *       existed in this property list, then it is replaced with the new
- *       value. Otherwise, a new value is added.
- *
- *       This fails if the value is present but has a different type.
- *
- * Results:
- *       VixError
- *
- * Side effects:
- *       None
- *
- *-----------------------------------------------------------------------------
- */
-
-VixError
-VixPropertyList_SetStringSensitive(VixPropertyListImpl *propList,  // IN:
-                                   int propertyID,                 // IN:
-                                   const char *value)              // IN:
-{
-   VixError err = VIX_OK;
-   VixPropertyValue *property = NULL;
-
-   if (NULL == propList) {
-      err = VIX_E_INVALID_ARG;
-      goto abort;
-   }
-
-   /*
-    * Find or create an entry for this property.
-    */
-   err = VixPropertyList_FindProperty(propList,
-                                      propertyID,
-                                      VIX_PROPERTYTYPE_STRING,
-                                      0,
-                                      TRUE,
-                                      &property);
-
-   if (VIX_OK == err) {
-      VixPropertyListSetStringImpl(property, value, TRUE);
-   }
-
-abort:
-
-   return err;
+   return(err);
 } // VixPropertyList_SetString
 
 
@@ -1078,7 +811,7 @@ abort:
  *
  * Results:
  *       VixError. VIX_OK if the property was found.
- *                 VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
+ *                     VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
  *
  * Side effects:
  *       None
@@ -1125,7 +858,7 @@ abort:
  *       Saves a copy of a integer property value. The value is identified
  *       by the integer property ID.
  *
- *       Value names are unique within a single property list.
+ *       Value names are unique within a single proeprty list.
  *       If a previous value with the same propertyID value already
  *       existed in this property list, then it is replaced with the new
  *       value. Otherwise, a new value is added.
@@ -1171,7 +904,7 @@ VixPropertyList_SetInteger(VixPropertyListImpl *propList,      // IN
    property->isDirty = TRUE;
 
 abort:
-   return err;
+   return(err);
 } // VixPropertyList_SetInteger
 
 
@@ -1189,7 +922,7 @@ abort:
  *
  * Results:
  *       VixError. VIX_OK if the property was found.
- *                 VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
+ *                     VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
  *
  * Side effects:
  *       None
@@ -1240,7 +973,7 @@ abort:
  *       Saves a copy of a Bool property value. The value is identified
  *       by the integer property ID.
  *
- *       Value names are unique within a single property list.
+ *       Value names are unique within a single proeprty list.
  *       If a previous value with the same propertyID value already
  *       existed in this property list, then it is replaced with the new
  *       value. Otherwise, a new value is added.
@@ -1286,7 +1019,7 @@ VixPropertyList_SetBool(VixPropertyListImpl *propList,      // IN
    property->isDirty = TRUE;
 
 abort:
-   return err;
+   return(err);
 }
 
 
@@ -1304,7 +1037,7 @@ abort:
  *
  * Results:
  *       VixError. VIX_OK if the property was found.
- *                 VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
+ *                     VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
  *
  * Side effects:
  *       None
@@ -1351,7 +1084,7 @@ abort:
  *       Saves a copy of a int64 property value. The value is identified
  *       by the integer property ID.
  *
- *       Value names are unique within a single property list.
+ *       Value names are unique within a single proeprty list.
  *       If a previous value with the same propertyID value already
  *       existed in this property list, then it is replaced with the new
  *       value. Otherwise, a new value is added.
@@ -1397,7 +1130,7 @@ VixPropertyList_SetInt64(VixPropertyListImpl *propList,     // IN
    property->isDirty = TRUE;
 
 abort:
-   return err;
+   return(err);
 } // VixPropertyList_SetInt64
 
 
@@ -1415,7 +1148,7 @@ abort:
  *
  * Results:
  *       VixError. VIX_OK if the property was found.
- *                 VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
+ *                     VIX_E_UNRECOGNIZED_PROPERTY if the property was not found.
  *
  * Side effects:
  *       None
@@ -1468,12 +1201,20 @@ abort:
 /*
  *-----------------------------------------------------------------------------
  *
- * VixPropertyListSetBlobImpl --
+ * VixPropertyList_SetBlob --
  *
- *       Saves a copy of a blob property value. Set sensitivity.
+ *       Saves a copy of a blob property value. The value is identified
+ *       by the integer property ID.
+ *
+ *       Value names are unique within a single proeprty list.
+ *       If a previous value with the same propertyID value already
+ *       existed in this property list, then it is replaced with the new
+ *       value. Otherwise, a new value is added.
+ *
+ *       This fails if the value is present but has a different type.
  *
  * Results:
- *       As above.
+ *       VixError.
  *
  * Side effects:
  *       None
@@ -1481,18 +1222,34 @@ abort:
  *-----------------------------------------------------------------------------
  */
 
-static void
-VixPropertyListSetBlobImpl(VixPropertyValue *property,  // IN:
-                           int blobSize,                // IN:
-                           const unsigned char *value,  // IN:
-                           Bool isSensitive)            // IN:
+VixError
+VixPropertyList_SetBlob(VixPropertyListImpl *propList,      // IN
+                        int propertyID,                     // IN
+                        int blobSize,                       // IN
+                        const unsigned char *value)         // IN
 {
-   if (NULL != property->value.blobValue.blobContents) {
-      if (property->isSensitive) {
-         Util_Zero(property->value.blobValue.blobContents,
-                   property->value.blobValue.blobSize);
-      }
+   VixError err = VIX_OK;
+   VixPropertyValue *property = NULL;
 
+   if (NULL == propList) {
+      err = VIX_E_INVALID_ARG;
+      goto abort;
+   }
+   
+   /*
+    * Find or create an entry for this property.
+    */
+   err = VixPropertyList_FindProperty(propList,
+                                      propertyID, 
+                                      VIX_PROPERTYTYPE_BLOB, 
+                                      0,
+                                      TRUE, 
+                                      &property);
+   if (VIX_OK != err) {
+      goto abort;
+   }
+
+   if (NULL != property->value.blobValue.blobContents) {
       free(property->value.blobValue.blobContents);
       property->value.blobValue.blobContents = NULL;
    }
@@ -1504,122 +1261,9 @@ VixPropertyListSetBlobImpl(VixPropertyValue *property,  // IN:
    }
 
    property->isDirty = TRUE;
-   property->isSensitive = isSensitive;
-} // VixPropertyListSetBlobImpl
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_SetBlob --
- *
- *       Saves a copy of a blob property value. The value is identified
- *       by the integer property ID.
- *
- *       Value names are unique within a single property list.
- *       If a previous value with the same propertyID value already
- *       existed in this property list, then it is replaced with the new
- *       value. Otherwise, a new value is added.
- *
- *       This fails if the value is present but has a different type.
- *
- * Results:
- *       VixError.
- *
- * Side effects:
- *       None
- *
- *-----------------------------------------------------------------------------
- */
-
-VixError
-VixPropertyList_SetBlob(VixPropertyListImpl *propList,  // IN:
-                        int propertyID,                 // IN:
-                        int blobSize,                   // IN:
-                        const unsigned char *value)     // IN:
-{
-   VixError err = VIX_OK;
-   VixPropertyValue *property = NULL;
-
-   if (NULL == propList) {
-      err = VIX_E_INVALID_ARG;
-      goto abort;
-   }
-
-   /*
-    * Find or create an entry for this property.
-    */
-   err = VixPropertyList_FindProperty(propList,
-                                      propertyID,
-                                      VIX_PROPERTYTYPE_BLOB,
-                                      0,
-                                      TRUE,
-                                      &property);
-
-   if (VIX_OK == err) {
-      VixPropertyListSetBlobImpl(property, blobSize, value,
-                                 property->isSensitive);
-   }
 
 abort:
-   return err;
-} // VixPropertyList_SetBlob
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_SetBlobSensitive --
- *
- *       Saves a copy of a blob property value. The value is identified
- *       by the integer property ID. Set sentivity.
- *
- *       Value names are unique within a single property list.
- *       If a previous value with the same propertyID value already
- *       existed in this property list, then it is replaced with the new
- *       value. Otherwise, a new value is added.
- *
- *       This fails if the value is present but has a different type.
- *
- * Results:
- *       VixError.
- *
- * Side effects:
- *       None
- *
- *-----------------------------------------------------------------------------
- */
-
-VixError
-VixPropertyList_SetBlobSensitive(VixPropertyListImpl *propList,  // IN:
-                                 int propertyID,                 // IN:
-                                 int blobSize,                   // IN:
-                                 const unsigned char *value)     // IN:
-{
-   VixError err = VIX_OK;
-   VixPropertyValue *property = NULL;
-
-   if (NULL == propList) {
-      err = VIX_E_INVALID_ARG;
-      goto abort;
-   }
-
-   /*
-    * Find or create an entry for this property.
-    */
-   err = VixPropertyList_FindProperty(propList,
-                                      propertyID,
-                                      VIX_PROPERTYTYPE_BLOB,
-                                      0,
-                                      TRUE,
-                                      &property);
-
-   if (VIX_OK == err) {
-      VixPropertyListSetBlobImpl(property, blobSize, value, TRUE);
-   }
-
-abort:
-   return err;
+   return(err);
 } // VixPropertyList_SetBlob
 
 
@@ -1690,7 +1334,7 @@ abort:
  *       This is a SHALLOW copy. It only copies the pointer, not what the
  *       pointer references.
  *
- *       Value names are unique within a single property list.
+ *       Value names are unique within a single proeprty list.
  *       If a previous value with the same propertyID value already
  *       existed in this property list, then it is replaced with the new
  *       value. Otherwise, a new value is added.
@@ -1736,7 +1380,7 @@ VixPropertyList_SetPtr(VixPropertyListImpl *propList,     // IN
    property->isDirty = TRUE;
 
 abort:
-   return err;
+   return(err);
 } // VixPropertyList_SetPtr
 
 
@@ -1773,64 +1417,7 @@ VixPropertyList_PropertyExists(VixPropertyListImpl *propList,     // IN
       foundIt = TRUE;
    }
 
-   return foundIt;
+   return(foundIt);
 } // VixPropertyList_PropertyExists
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_NumItems --
- *
- *       Returns a count of the properties in the list.
- *
- * Results:
- *       int - Number of properties in property list.
- *
- * Side effects:
- *       None.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-VixPropertyList_NumItems(VixPropertyListImpl *propList)     // IN
-{
-   VixPropertyValue *prop;
-   int count = 0;
-
-   if (propList == NULL) {
-      return 0;
-   }
-
-   for (prop = propList->properties; prop != NULL; prop = prop->next) {
-      ++count;
-   }
-
-   return count;
-} // VixPropertyList_NumItems
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VixPropertyList_Empty --
- *
- *       Returns whether the property list has no properties.
- *
- * Results:
- *       Bool - True iff property list has no properties.
- *
- * Side effects:
- *       None.
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-VixPropertyList_Empty(VixPropertyListImpl *propList)     // IN
-{
-   return (propList == NULL || propList->properties == NULL);
-} // VixPropertyList_Empty
-
-
