@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,20 +16,6 @@
  *
  *********************************************************/
 
-/*********************************************************
- * The contents of this file are subject to the terms of the Common
- * Development and Distribution License (the "License") version 1.0
- * and no later version.  You may not use this file except in
- * compliance with the License.
- *
- * You can obtain a copy of the License at
- *         http://www.opensource.org/licenses/cddl1.php
- *
- * See the License for the specific language governing permissions
- * and limitations under the License.
- *
- *********************************************************/
-
 /*
  * cpNameLinux.c --
  *
@@ -38,14 +24,53 @@
  *
  */
 
-#if defined sun && !defined SOL9
+#if defined(sun) && defined(SOL10)
 #include <memory.h>
 #endif
 
 #include "cpName.h"
 #include "cpNameInt.h"
 #include "vm_assert.h"
-#include "hgfsEscape.h"
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * CPName_GetComponent --
+ *
+ *    Get the next component of the CP name.
+ *
+ *    Returns the length of the component starting with the begin
+ *    pointer, and a pointer to the next component in the buffer, if
+ *    any. The "next" pointer is set to "end" if there is no next
+ *    component.
+ *
+ * Results:
+ *    length (not including NUL termination) >= 0 of next
+ *    component on success.
+ *    error < 0 on failure (invalid component).
+ *
+ * Side effects:
+ *    None
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+CPName_GetComponent(char const *begin,  // IN: Beginning of buffer
+                    char const *end,    // IN: End of buffer
+                    char const **next)  // OUT: Start of next component
+{
+   ASSERT(begin);
+   ASSERT(end);
+   ASSERT(next);
+
+   /*
+    * '/' is not a legal character on Linux, since it is a path
+    * separator.
+    */
+   return CPName_GetComponentGeneric(begin, end, "/", next);
+}
 
 
 /*
@@ -79,7 +104,7 @@ CPName_ConvertFrom(char const **bufIn, // IN/OUT: Input to convert
    ASSERT(outSize);
    ASSERT(bufOut);
 
-   return CPNameEscapeAndConvertFrom(bufIn, inSize, outSize, bufOut, '/');
+   return CPNameConvertFrom(bufIn, inSize, outSize, bufOut, '/');
 }
 
 
@@ -127,7 +152,7 @@ CPName_ConvertFromRoot(char const **bufIn, // IN/OUT: Input to convert
     */
    len = CPName_GetComponent(*bufIn, *bufIn + *inSize, &next);
    if (len < 0) {
-      Log("%s: get first component failed\n", __FUNCTION__);
+      Log("CPName_ConvertFromRoot: get first component failed\n");
       return HGFS_NAME_STATUS_FAILURE;
    }
 
@@ -163,10 +188,6 @@ CPName_ConvertFromRoot(char const **bufIn, // IN/OUT: Input to convert
  *
  *    Makes a cross-platform name representation from the Linux path input
  *    string and writes it into the output buffer.
- *    If the name being converter may be a result a of name escaping the
- *    function performs unescaping. HGFS convention is to always exchange
- *    unescaped names between guest and host and to perform necessary
- *    name escaping on both ends.
  *
  * Results:
  *    On success, returns the number of bytes used in the
@@ -184,7 +205,5 @@ CPName_ConvertTo(char const *nameIn, // IN:  Buf to convert
                  size_t bufOutSize,  // IN:  Size of the output buffer
                  char *bufOut)       // OUT: Output buffer
 {
-   int result;
-   result = CPName_LinuxConvertTo(nameIn, bufOutSize, bufOut);
-   return result;
+   return CPName_LinuxConvertTo(nameIn, bufOutSize, bufOut);
 }
