@@ -16,20 +16,6 @@
  *
  *********************************************************/
 
-/*********************************************************
- * The contents of this file are subject to the terms of the Common
- * Development and Distribution License (the "License") version 1.0
- * and no later version.  You may not use this file except in
- * compliance with the License.
- *
- * You can obtain a copy of the License at
- *         http://www.opensource.org/licenses/cddl1.php
- *
- * See the License for the specific language governing permissions
- * and limitations under the License.
- *
- *********************************************************/
-
 /*
  * hgfsUtil.c --
  *
@@ -109,7 +95,7 @@ int
 HgfsConvertFromNtTimeNsec(struct timespec *unixTime, // OUT: Time in UNIX format
 			  uint64 ntTime) // IN: Time in Windows NT format
 {
-#if !defined(VM_X86_64) && !defined(__arm__)
+#ifndef VM_X86_64
    uint32 sec;
    uint32 nsec;
 
@@ -134,7 +120,7 @@ HgfsConvertFromNtTimeNsec(struct timespec *unixTime, // OUT: Time in UNIX format
       return -1;
    }
 
-#if !defined(VM_X86_64) && !defined(__arm__)
+#ifndef VM_X86_64
    Div643232(ntTime - UNIX_EPOCH, 10000000, &sec, &nsec);
    unixTime->tv_sec = sec;
    unixTime->tv_nsec = nsec * 100;
@@ -231,15 +217,6 @@ HgfsConvertFromInternalStatus(HgfsInternalStatus status) // IN
       return HGFS_STATUS_OPERATION_NOT_SUPPORTED;
    case ERROR_INVALID_PARAMETER:
       return HGFS_STATUS_INVALID_PARAMETER;
-   case ERROR_NOT_SAME_DEVICE:
-      return HGFS_STATUS_NOT_SAME_DEVICE;
-   case ERROR_FILENAME_EXCED_RANGE:
-      return HGFS_STATUS_NAME_TOO_LONG;
-   case ERROR_CONNECTION_INVALID:  // HGFS_ERROR_STALE_SESSION
-      return HGFS_STATUS_STALE_SESSION;
-   case ERROR_MAX_SESSIONS_REACHED:
-      return HGFS_STATUS_TOO_MANY_SESSIONS;
-   case ERROR_INTERNAL_ERROR:
    case HGFS_INTERNAL_STATUS_ERROR:
    default:
       return HGFS_STATUS_GENERIC_ERROR;
@@ -260,7 +237,6 @@ HgfsConvertFromInternalStatus(HgfsInternalStatus status) // IN
       return HGFS_STATUS_INVALID_HANDLE;
    case EPERM:
       return HGFS_STATUS_OPERATION_NOT_PERMITTED;
-   case EISDIR:
    case EEXIST:
       return HGFS_STATUS_FILE_EXISTS;
    case ENOTDIR:
@@ -281,13 +257,6 @@ HgfsConvertFromInternalStatus(HgfsInternalStatus status) // IN
       return HGFS_STATUS_NAME_TOO_LONG;
    case EPARAMETERNOTSUPPORTED:
       return HGFS_STATUS_INVALID_PARAMETER;
-   case EXDEV:
-      return HGFS_STATUS_NOT_SAME_DEVICE;
-   case ENETRESET:  // HGFS_ERROR_STALE_SESSION
-      return HGFS_STATUS_STALE_SESSION;
-   case ECONNREFUSED:
-      return HGFS_STATUS_TOO_MANY_SESSIONS;
-   case EINTERNAL:
    case HGFS_INTERNAL_STATUS_ERROR:
    default:
       return HGFS_STATUS_GENERIC_ERROR;
