@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,20 +16,6 @@
  *
  *********************************************************/
 
-/*********************************************************
- * The contents of this file are subject to the terms of the Common
- * Development and Distribution License (the "License") version 1.0
- * and no later version.  You may not use this file except in
- * compliance with the License.
- *
- * You can obtain a copy of the License at
- *         http://www.opensource.org/licenses/cddl1.php
- *
- * See the License for the specific language governing permissions
- * and limitations under the License.
- *
- *********************************************************/
-
 /*
  * hgfsBd.c --
  *
@@ -262,16 +248,14 @@ HgfsBd_Dispatch(RpcOut *out,            // IN: Channel to send on
    Bool success;
    char const *reply;
    size_t replyLen;
-   char *bdPacket = packetIn - HGFS_SYNC_REQREP_CLIENT_CMD_LEN;
 
    ASSERT(out);
    ASSERT(packetIn);
    ASSERT(packetSize);
    ASSERT(packetOut);
 
-   memcpy(bdPacket, HGFS_SYNC_REQREP_CLIENT_CMD, HGFS_SYNC_REQREP_CLIENT_CMD_LEN);
-
-   success = RpcOut_send(out, bdPacket, *packetSize + HGFS_CLIENT_CMD_LEN,
+   success = RpcOut_send(out, packetIn - HGFS_CLIENT_CMD_LEN, 
+                         *packetSize + HGFS_CLIENT_CMD_LEN, 
                          &reply, &replyLen);
    if (success == FALSE) {
       Debug("HgfsBd_Dispatch: RpcOut_send returned failure\n");
@@ -308,8 +292,8 @@ HgfsBd_Enabled(RpcOut *out,         // IN: RPCI Channel
                char *requestPacket) // IN: Buffer (obtained from HgfsBd_GetBuf)
 {
    char const *replyPacket; // Buffer returned by HgfsBd_Dispatch
-   size_t replyLen;
-   Bool success;
+   size_t packetSize;
+   int error;
 
    /*
     * Send a bogus (empty) request to the VMX. If hgfs is disabled on
@@ -318,14 +302,16 @@ HgfsBd_Enabled(RpcOut *out,         // IN: RPCI Channel
     * (it will be an error packet because our request was malformed,
     * but we just discard it anyway).
     */
-   success = RpcOut_send(out, requestPacket - HGFS_CLIENT_CMD_LEN,
-                         HGFS_CLIENT_CMD_LEN,
-                         &replyPacket, &replyLen);
-   if (success == TRUE) {
-      ASSERT(replyLen <= HGFS_LARGE_PACKET_MAX);
+   packetSize = 0;
+   error = HgfsBd_Dispatch(out,
+                           requestPacket,
+                           &packetSize,
+                           &replyPacket);
+   if (error < 0) {
+      return FALSE;
    }
 
-   return success;
+   return TRUE;
 }
 
 
