@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -48,16 +48,6 @@ typedef struct HgfsServerPolicyState {
 
 static HgfsServerPolicyState myState;
 
-static void *
-HgfsServerPolicyEnumSharesInit(void);
-static Bool
-HgfsServerPolicyEnumSharesGet(void *data,
-                              char const **name,
-                              size_t *len,
-                              Bool *done);
-static Bool
-HgfsServerPolicyEnumSharesExit(void *data);
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -135,18 +125,15 @@ HgfsServerPolicyDestroyShares(DblLnkLst_Links *head) // IN
  */
 
 Bool
-HgfsServerPolicy_Init(HgfsInvalidateObjectsFunc invalidateObjects,  // Unused
-                      HgfsRegisterSharedFolderFunc registerFolder,  // Unused
-                      HgfsServerResEnumCallbacks *enumResources)    // OUT enum callbacks
+HgfsServerPolicy_Init(HgfsInvalidateObjectsFunc *invalidateObjects) // Ignored
 {
    HgfsSharedFolder *rootShare;
 
    /*
-    * Currently these callbacks are not used, so make sure our caller doesn't pass
+    * We do not recognize this callback, so make sure our caller doesn't pass
     * it in.
     */
-   ASSERT(invalidateObjects == NULL);
-   ASSERT(registerFolder == NULL);
+   ASSERT(!invalidateObjects);
 
    DblLnkLst_Init(&myState.shares);
 
@@ -172,18 +159,10 @@ HgfsServerPolicy_Init(HgfsInvalidateObjectsFunc invalidateObjects,  // Unused
    /* These are strictly optimizations to save work later */
    rootShare->pathLen = strlen(rootShare->path);
    rootShare->nameLen = strlen(rootShare->name);
-   rootShare->handle = HGFS_INVALID_FOLDER_HANDLE;
 
    /* Add the root node to the end of the list */
    DblLnkLst_LinkLast(&myState.shares, &rootShare->links);
 
-   /*
-    * Fill the share enumeration callback table.
-    */
-   enumResources->init = HgfsServerPolicyEnumSharesInit;
-   enumResources->get = HgfsServerPolicyEnumSharesGet;
-   enumResources->exit = HgfsServerPolicyEnumSharesExit;
-
    return TRUE;
 }
 
@@ -299,7 +278,7 @@ HgfsServerPolicyGetShare(HgfsServerPolicyState *state, // IN
 }
 
 
-/* State used by HgfsServerPolicyEnumSharesGet and friends */
+/* State used by HgfsServerPolicy_GetShares and friends */
 typedef struct State {
    DblLnkLst_Links *next;
 } GetSharesState;
@@ -308,9 +287,9 @@ typedef struct State {
 /*
  *-----------------------------------------------------------------------------
  *
- * HgfsServerPolicyEnumSharesInit --
+ * HgfsServerPolicy_GetSharesInit --
  *
- *    Setup state for HgfsServerPolicyEnumSharesGet
+ *    Setup state for HgfsServerPolicy_GetShares
  *
  * Results:
  *    Pointer to state on success.
@@ -322,14 +301,14 @@ typedef struct State {
  *-----------------------------------------------------------------------------
  */
 
-static void *
-HgfsServerPolicyEnumSharesInit(void)
+void *
+HgfsServerPolicy_GetSharesInit(void)
 {
    GetSharesState *that;
 
    that = malloc(sizeof *that);
    if (!that) {
-      LOG(4, ("HgfsServerPolicyEnumSharesInit: couldn't allocate state\n"));
+      LOG(4, ("HgfsServerPolicy_GetSharesInit: couldn't allocate state\n"));
       return NULL;
    }
 
@@ -341,14 +320,14 @@ HgfsServerPolicyEnumSharesInit(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * HgfsServerPolicyEnumSharesGet --
+ * HgfsServerPolicy_GetShares --
  *
  *    Enumerate share names one at a time.
  *
  *    When finished, sets "done" to TRUE.
  *
  *    Should be called with the results obtained by calling
- *    HgfsServerPolicyEnumSharesInit.
+ *    HgfsServerPolicy_GetSharesInit.
  *
  * Results:
  *    TRUE on success.
@@ -360,11 +339,11 @@ HgfsServerPolicyEnumSharesInit(void)
  *-----------------------------------------------------------------------------
  */
 
-static Bool
-HgfsServerPolicyEnumSharesGet(void *data,        // IN:  Callback data
-                              char const **name, // OUT: Share name
-                              size_t *len,       // OUT: Name length
-                              Bool *done)        // OUT: Completion status
+Bool
+HgfsServerPolicy_GetShares(void *data,        // IN:  Callback data
+                           char const **name, // OUT: Share name
+                           size_t *len,       // OUT: Name length
+                           Bool *done)        // OUT: Completion status
 {
    GetSharesState *that;
    HgfsSharedFolder *share;
@@ -386,7 +365,7 @@ HgfsServerPolicyEnumSharesGet(void *data,        // IN:  Callback data
    that->next = share->links.next;
    *name = share->name;
    *len = share->nameLen;
-   LOG(4, ("HgfsServerPolicyEnumSharesGet: Share name is \"%s\"\n",
+   LOG(4, ("HgfsServerPolicy_GetShares: Share name is \"%s\"\n",
            *name));
    *done = FALSE;
    return TRUE;
@@ -396,9 +375,9 @@ HgfsServerPolicyEnumSharesGet(void *data,        // IN:  Callback data
 /*
  *-----------------------------------------------------------------------------
  *
- * HgfsServerPolicyEnumSharesExit --
+ * HgfsServerPolicy_GetSharesCleanup --
  *
- *    Cleanup state from HgfsServerPolicyEnumSharesGet
+ *    Cleanup state from HgfsServerPolicy_GetShares
  *
  * Results:
  *    TRUE on success.
@@ -410,8 +389,8 @@ HgfsServerPolicyEnumSharesGet(void *data,        // IN:  Callback data
  *-----------------------------------------------------------------------------
  */
 
-static Bool
-HgfsServerPolicyEnumSharesExit(void *data) // IN: Callback data
+Bool
+HgfsServerPolicy_GetSharesCleanup(void *data) // IN: Callback data
 {
    GetSharesState *that;
 
@@ -502,126 +481,6 @@ HgfsServerPolicy_GetSharePath(char const *nameIn,        // IN: Name to check
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsServerPolicy_ProcessCPName --
- *
- *    Get the local path for a share name by looking at the requested
- *    name, finding the matching share (if any) and returning the share's
- *    local path local path and permissions.
- *
- * Results:
- *    An HgfsNameStatus value indicating the result is returned.
- *
- *    The local path for the shareName is also returned if a match is found.
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-HgfsNameStatus
-HgfsServerPolicy_ProcessCPName(char const *nameIn,            // IN: name in CPName form
-                               size_t nameInLen,              // IN: length of the name
-                               Bool *readAccess,              // OUT: Read permissions
-                               Bool *writeAccess,             // OUT: Write permissions
-                               HgfsSharedFolderHandle *handle,// OUT: folder handle
-                               char const **shareBaseDir)     // OUT: Shared directory
-{
-   HgfsSharedFolder *myShare;
-
-   ASSERT(nameIn);
-   ASSERT(shareBaseDir);
-
-   myShare = HgfsServerPolicyGetShare(&myState, nameIn, nameInLen);
-   if (!myShare) {
-      LOG(4, ("%s: No matching share name\n", __FUNCTION__));
-      return HGFS_NAME_STATUS_DOES_NOT_EXIST;
-   }
-
-   *readAccess = myShare->readAccess;
-   *writeAccess = myShare->writeAccess;
-   *shareBaseDir = myShare->path;
-   *handle = myShare->handle;
-   return HGFS_NAME_STATUS_COMPLETE;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsServerPolicy_GetShareOptions --
- *
- *    Get the HGFS share config options by looking at the requested name,
- *    finding the matching share (if any).
- *
- * Results:
- *    HGFS_NAME_STATUS_COMPLETE on success, and HGFS_NAME_STATUS_DOES_NOT_EXIST
- *    if no matching share.
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-HgfsNameStatus
-HgfsServerPolicy_GetShareOptions(char const *nameIn,             // IN: Share name
-                                 size_t nameInLen,               // IN: Share name length
-                                 HgfsShareOptions *configOptions)// OUT: Config options
-{
-   HgfsSharedFolder *share;
-   char const *inEnd;
-   char *next;
-   int len;
-
-   ASSERT(nameIn);
-   ASSERT(configOptions);
-
-   inEnd = nameIn + nameInLen;
-   len = CPName_GetComponent(nameIn, inEnd, (char const **) &next);
-   if (len < 0) {
-      LOG(4, ("HgfsServerPolicy_GetShareOptions: get first component failed\n"));
-      return HGFS_NAME_STATUS_FAILURE;
-   }
-
-   share = HgfsServerPolicyGetShare(&myState, nameIn, len);
-   if (!share) {
-      LOG(4, ("HgfsServerPolicy_GetShareOptions: No matching share name.\n"));
-      return HGFS_NAME_STATUS_DOES_NOT_EXIST;
-   }
-   *configOptions = share->configOptions;
-   return HGFS_NAME_STATUS_COMPLETE;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsServerPolicy_IsShareOptionSet --
- *
- *    Check if the specified config option is set.
- *
- * Results:
- *    TRUE if set.
- *    FALSE otherwise.
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-HgfsServerPolicy_IsShareOptionSet(HgfsShareOptions configOptions, // IN: config options
-                                  uint32 option)                  // IN: option to check
-{
-   return (configOptions & option) == option;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -676,62 +535,3 @@ HgfsServerPolicy_GetShareMode(char const *nameIn,        // IN: Share name to re
 
    return HGFS_NAME_STATUS_COMPLETE;
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsServerPolicy_CheckMode --
- *
- *    Checks if the requested mode may be granted depending on read/write permissions.
- *
- * Results:
- *    TRUE if the requested mode can be granted, FALSE otherwise.
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-HgfsServerPolicy_CheckMode(HgfsOpenMode mode,          // IN: mode to check
-                           Bool writePermissions,      // IN: callers write permissions
-                           Bool readPermissions)       // IN: callers read permissions
-{
-   /*
-    * See if access is allowed in the requested mode.
-    *
-    * XXX Yeah, this is retarded. We should be using bits instead of
-    * an enum for HgfsOpenMode. Add it to the todo list. [bac]
-    */
-   switch (HGFS_OPEN_MODE_ACCMODE(mode)) {
-   case HGFS_OPEN_MODE_READ_ONLY:
-      if (!readPermissions) {
-         LOG(4, ("HgfsServerPolicyCheckMode: Read access denied\n"));
-         return FALSE;
-      }
-      break;
-
-   case HGFS_OPEN_MODE_WRITE_ONLY:
-      if (!writePermissions) {
-         LOG(4, ("HgfsServerPolicyCheckMode: Write access denied\n"));
-         return FALSE;
-      }
-      break;
-
-   case HGFS_OPEN_MODE_READ_WRITE:
-      if (!readPermissions || !writePermissions) {
-         LOG(4, ("HgfsServerPolicyCheckMode: Read/write access denied\n"));
-         return FALSE;
-      }
-      break;
-
-   default:
-      LOG(0, ("HgfsServerPolicyCheckMode: Invalid mode\n"));
-      ASSERT(FALSE);
-      return FALSE;
-   }
-
-   return TRUE;
-}
