@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2011 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -21,13 +21,14 @@
 
 #define INCLUDE_ALLOW_USERLEVEL
 #define INCLUDE_ALLOW_VMMON
+#define INCLUDE_ALLOW_VMNIXMOD
 #define INCLUDE_ALLOW_VMCORE
 #define INCLUDE_ALLOW_VMKERNEL
-
 #include "includeCheck.h"
 
+#include "x86cpuid.h"
 #include "vm_basic_asm.h"
-#include "x86cpuid_asm.h"
+
 
 
 typedef struct CPUID0 {
@@ -65,31 +66,41 @@ typedef struct CPUIDSummary {
    CPUIDRegs id88, id8a;
 } CPUIDSummary;
 
-
-/*
- *----------------------------------------------------------------------
- *
- * CPUIDSummary_RegsFromCpuid0 --
- *
- *      Fills in the given CPUIDRegs struct with the values from the CPUID0 struct.
- *
- * Results:
- *      Returns the CPUIDRegs pointer passed in.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-static INLINE CPUIDRegs*
-CPUIDSummary_RegsFromCpuid0(CPUID0* id0In,
-                            CPUIDRegs* id0Out)
+/* The following two functions return the number of cores per package
+   and set *numThreadsPerCore to the number of hardware threads per core. */ 
+static INLINE uint32 
+CPUIDSummary_AMDCoresPerPackage(CPUIDSummary *cpuid,
+                                uint32 *numThreadsPerCore)
 {
-   id0Out->eax = id0In->numEntries;
-   id0Out->ebx = *(uint32 *) (id0In->name + 0);
-   id0Out->edx = *(uint32 *) (id0In->name + 4);
-   id0Out->ecx = *(uint32 *) (id0In->name + 8);
-   return id0Out;
+   uint32 version = cpuid->id1.version, numEntries = cpuid->id80.numEntries;
+   *numThreadsPerCore = 1;
+   return CPUID_FAMILY_IS_K8STAR(version) && numEntries >= 0x80000008 ?
+             CPUID_AMDCoresPerPackage(cpuid->id88.ecx) : 1;
+}
+
+static INLINE uint32
+CPUIDSummary_IntelCoresPerPackage(CPUIDSummary *cpuid,
+                                  uint32 *numThreadsPerCore)
+{
+   uint32 numCoresPerPackage;
+   
+   *numThreadsPerCore = numCoresPerPackage = 1;
+   /*
+    * Multi-core processors have the HT feature bit set even if they don't
+    * support HT.  The reported number of HT is the total, not per core.
+    */
+   if (cpuid->id1.edxFeatures & CPUID_FEATURE_COMMON_ID1EDX_HT) {
+      *numThreadsPerCore = CPUID_LCPU_COUNT(cpuid->id1.ebx);
+       if (cpuid->id0.numEntries >= 4) {
+         numCoresPerPackage =
+            CPUID_IntelCoresPerPackage(__GET_EAX_FROM_CPUID4(0));
+#ifdef ASSERT
+         ASSERT(*numThreadsPerCore % numCoresPerPackage == 0);
+#endif
+         *numThreadsPerCore /= numCoresPerPackage;
+      }
+   }
+   return numCoresPerPackage;
 }
 
 #endif
