@@ -54,7 +54,6 @@
 /* The maximum number of arguments we can hold */
 #define DNDMSG_MAX_ARGS 64
 
-/* Linux only defines. Should be in separate dndLinux.h */
 /* Strings used for formatting various types of data */
 #define DND_URI_LIST_PRE     "file://"
 #define DND_URI_LIST_PRE_KDE "file:"
@@ -66,45 +65,15 @@
 #define FCP_GNOME_LIST_PRE   "file://"
 #define FCP_GNOME_LIST_POST  "\n"
 
-/* FCP target used in gnome. */
-#define FCP_TARGET_NAME_GNOME_COPIED_FILES   "x-special/gnome-copied-files"
-#define FCP_TARGET_INFO_GNOME_COPIED_FILES   0
-/* FCP target used in KDE. */
-#define FCP_TARGET_NAME_URI_LIST             "text/uri-list"
-#define FCP_TARGET_INFO_URI_LIST             1
-/* Number of FCP targets. */
-#define NR_FCP_TARGETS                       2
-
-#define VMWARE_TARGET                        "vmware-target"
-
-#define FCP_COPY_DELAY                       1000000  // 1 second
-#define TARGET_NAME_TIMESTAMP                "TIMESTAMP"
-#define TARGET_NAME_STRING                   "STRING"
-#define TARGET_NAME_TEXT_PLAIN               "text/plain"
-#define TARGET_NAME_UTF8_STRING              "UTF8_STRING"
-#define TARGET_NAME_COMPOUND_TEXT            "COMPOUND_TEXT"
-#define TARGET_NAME_APPLICATION_RTF          "application/rtf"
-#define TARGET_NAME_TEXT_RICHTEXT            "text/richtext"
-
-#define DRAG_TARGET_NAME_URI_LIST  "text/uri-list"
-#define DRAG_LEAVE_TIMEOUT         500
-
 /* Guest detection window width and height. */
 #define DRAG_DET_WINDOW_WIDTH 15
 
-/* Clipboard image size limit. */
-#define CLIPBOARD_IMAGE_MAX_WIDTH  4000
-#define CLIPBOARD_IMAGE_MAX_HEIGHT 4000
-
 typedef enum
 {
    CPFORMAT_UNKNOWN = 0,
    CPFORMAT_TEXT,       /* NUL terminated UTF-8. */
    CPFORMAT_FILELIST,
    CPFORMAT_RTF,
-   CPFORMAT_FILELIST_URI,
-   CPFORMAT_FILECONTENTS,
-   CPFORMAT_IMG_PNG,
    CPFORMAT_MAX,
 } DND_CPFORMAT;
 
@@ -170,22 +139,13 @@ typedef struct DnDTransportBuffer {
                                                DND_TRANSPORT_PACKET_HEADER_SIZE)
 #define DND_MAX_TRANSPORT_LATENCY_TIME        3 * 1000000 /* 3 seconds. */
 
-/*
- * Structure to access methods of currently used blocking mechanism.
- */
-typedef struct DnDBlockControl {
-   int fd;
-   const char *blockRoot;
-   Bool (*AddBlock)(int blockFd, const char *blockPath);
-   Bool (*RemoveBlock)(int blockFd, const char *blockedPath);
-} DnDBlockControl;
-
 #ifdef _WIN32
 /*
  * Windows-specific functions
  */
+EXTERN uint32 DnD_GetClipboardFormatFromName(ConstUnicode pFormatName);
 EXTERN Unicode DnD_GetClipboardFormatName(UINT cf);
-EXTERN HGLOBAL DnD_CopyStringToGlobal(ConstUnicode str);
+EXTERN HGLOBAL  DnD_CopyStringToGlobal(ConstUnicode str);
 EXTERN HGLOBAL DnD_CopyDWORDToGlobal(DWORD *pDWORD);
 EXTERN HGLOBAL DnD_CreateHDrop(ConstUnicode path, ConstUnicode fileList);
 EXTERN HGLOBAL DnD_CreateHDropForGuest(ConstUnicode path,
@@ -198,17 +158,6 @@ EXTERN Bool DnD_SetCPClipboardFromLocalText(CPClipboard *clip,
                                             utf16_t *bufIn);
 EXTERN Bool DnD_SetCPClipboardFromLocalRtf(CPClipboard *clip,
                                            char *bufIn);
-EXTERN Bool DnD_SetCPClipboardFromBMPInfo(CPClipboard *clip,
-                                          const LPBITMAPINFOHEADER bmi,
-                                          DND_CPFORMAT fmt);
-EXTERN Bool DnD_SetCPClipboardFromHBITMAP(CPClipboard *clip,
-                                          HBITMAP hBitmap,
-                                          DND_CPFORMAT fmt);
-EXTERN Bool DnD_PNGToLocalFormat(const unsigned char *pngData,
-                                 unsigned int pngDataLen,
-                                 int pngReadFlags,
-                                 DynBuf *bmpData,
-                                 HBITMAP *hBitmap);
 EXTERN Bool DnD_FakeMouseEvent(DWORD flag);
 EXTERN Bool DnD_FakeMouseState(DWORD key, Bool isDown);
 EXTERN Bool DnD_FakeEscapeKey(void);
@@ -235,7 +184,9 @@ EXTERN char *DnD_UriListGetNextFile(char const *uriList,
  */
 ConstUnicode DnD_GetFileRoot(void);
 char *DnD_CreateStagingDirectory(void);
-Bool DnD_DeleteStagingFiles(ConstUnicode stagingDir, Bool onReboot);
+Bool DnD_DeleteStagingFiles(ConstUnicode fileList, Bool onReboot);
+Bool DnD_DataContainsIllegalCharacters(const char *data,
+                                       const size_t dataSize);
 Bool DnD_PrependFileRoot(ConstUnicode fileRoot, char **src, size_t *srcSize);
 int DnD_LegacyConvertToCPName(const char *nameIn,
                               size_t bufOutSize,
@@ -246,19 +197,10 @@ Bool DnD_CPNameListToDynBufArray(char *fileList,
 Unicode DnD_GetLastDirName(const char *str);
 
 /* vmblock support functions. */
-Bool DnD_InitializeBlocking(DnDBlockControl *blkCtrl);
-Bool DnD_UninitializeBlocking(DnDBlockControl *blkCtrl);
-Bool DnD_CompleteBlockInitialization(int fd, DnDBlockControl *blkCtrl);
-
-static INLINE Bool
-DnD_BlockIsReady(DnDBlockControl *blkCtrl)   // IN: blocking control structure
-{
-   if (blkCtrl->fd >= 0) {
-      ASSERT(blkCtrl->AddBlock && blkCtrl->RemoveBlock);
-      return TRUE;
-   }
-   return FALSE;
-}
+int DnD_InitializeBlocking(void);
+Bool DnD_UninitializeBlocking(int blockFd);
+Bool DnD_AddBlock(int blockFd, const char *blockPath);
+Bool DnD_RemoveBlock(int blockFd, const char *blockedPath);
 
 /* Transport layer big buffer support functions. */
 void DnD_TransportBufInit(DnDTransportBuffer *buf,
