@@ -20,20 +20,13 @@
 #ifndef _DND_GUEST_H_
 #define _DND_GUEST_H_
 
-#include "vm_basic_types.h"
-
 #if defined(_WIN32)
 
 typedef struct {
    HWND detWnd;
-   void (*setMode) (Bool);
 } UnityDnD;
 
-/*
- * Callers who desire the gtk implementation must include at least one gtk
- * header before this one.
- */
-#elif defined(GTK_CHECK_VERSION)
+#else
 
 #include <gtk/gtk.h>
 
@@ -43,12 +36,8 @@ typedef struct {
 } UnityDnD;
 
 
-#else
-
-/* Probably compiling a stub. */
-typedef struct UnityDnD UnityDnD;
-   
 #endif
 
+
 #endif // _DND_GUEST_H_
 
