@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (c) 1998-2018, 2021-2022 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -50,32 +50,48 @@
 #include "vm_basic_types.h"
 #include "unicodeTypes.h"
 
+#if !defined(__FreeBSD__)
 #include "iovector.h"        // for struct iovec
-
-#if defined(__cplusplus)
-extern "C" {
 #endif
 
-struct FileLockToken;
+#if defined(VMX86_STATS)
+
+struct StatsUserBlock;
+
+#define FILEIO_STATS_VARS \
+   uint32 readIn, readDirect; \
+   uint32 writeIn, writeDirect; \
+   uint32 readvIn, readvDirect; \
+   uint32 writevIn, writevDirect; \
+   uint32 preadvIn, preadDirect; \
+   uint32 pwritevIn, pwriteDirect; \
+   uint64 bytesRead, bytesWritten; \
+   uint32 numReadCoalesced, numWriteCoalesced; \
+   struct StatsUserBlock *stats;
+#else
+#define FILEIO_STATS_VARS
+#endif
 
 #if defined(_WIN32)
 
 # include <windows.h>
 
 typedef struct FileIODescriptor {
-   HANDLE                win32;
-   uint32                flags;
-   char                 *fileName;
-   struct FileLockToken *lockToken;
+   HANDLE win32;
+   uint32 flags;
+   Unicode fileName;
+   void *lockToken;
+   FILEIO_STATS_VARS
 } FileIODescriptor;
 
 #else
 
 typedef struct FileIODescriptor {
-   int                   posix;
-   int                   flags;
-   char                 *fileName;
-   struct FileLockToken *lockToken;
+   int posix;
+   int flags;
+   Unicode fileName;
+   void *lockToken;
+   FILEIO_STATS_VARS
 } FileIODescriptor;
 
 #endif
@@ -140,7 +156,7 @@ typedef enum {
  * unshare the flags between the two at which point this could be fixed.
  * --Tommy
  */
-#define  FILEIO_OPEN_PRIVILEGED_IOCTL    (1 << 12)
+#define  FILEIO_OPEN_PRIVILEGED_IOCTL  (1 << 12)
 /*
  * Lock the file on open with a exclusive leased lock that can be broken
  * (supported on ESX file systems)
@@ -152,100 +168,16 @@ typedef enum {
  */
 #define FILEIO_OPEN_MULTIWRITER_LOCK     (1 << 14)
 /*
- * Lock the file on open with an SWMR leased lock that can be broken
- * (supported on ESX file systems)
+ * Flag the file as not to be backed up by Time Machine on Mac OS X.
  */
-#define FILEIO_OPEN_SWMR_LOCK            (1 << 15)
-/*
- * Valid only for MacOS. It eventually results into O_EXLOCK flag passed to open
- * system call.
- *
- * O_EXLOCK, O_SHLOCK behavior is tested on Mac OS X Server 10.6, kernel 10.0.0.
- *
- * |                      | Block devices      | Regular files
- * |----------------------|--------------------|----------------
- * | Locking behavior     | mandatory          | advisory
- * |                      |                    |
- * | If O_NONBLOCK absent | open doesn't block | open blocks
- * |                      | on conflicts       | on conflicts
- */
-#define FILEIO_OPEN_EXCLUSIVE_LOCK_MACOS (1 << 16)
-/*
- * Open file in APPEND-only mode.  All writes go to the current end of file,
- * not to the current file pointer location.
- */
-#define FILEIO_OPEN_APPEND               (1 << 17)
-/*
- * Valid only on POSIXen. Don't follow a symbolic link.
- */
-#define FILEIO_OPEN_ACCESS_NOFOLLOW      (1 << 18)
-/*
- * Valid only on Windows. Set FILE_SHARE_DELETE.
- */
-#define FILEIO_OPEN_SHARE_DELETE         (1 << 19)
-/*
- * Strengths of file lock.
- * Advisory:
- *   Must use FileIO plus lock flags to get locking.
- *   Never uses kernel/fs-level lock, so naked open() bypasses locking.
- * Mandatory:
- *   Requires kernel/fs-level, so naked open() respects lock.
- *   Kernel/fs-level locks are available on ESX but not hosted.
- * Best:
- *   Adaptively picks between mandatory and advisory.
- * Almost all cases should use the "best" lock.
- */
-#define FILEIO_OPEN_LOCK_BEST            FILEIO_OPEN_LOCKED /* historical */
-#define FILEIO_OPEN_LOCK_ADVISORY        (1 << 20)
-#define FILEIO_OPEN_LOCK_MANDATORY       (1 << 21)
+#define FILEIO_OPEN_NO_TIME_MACHINE      (1 << 15)
 
-/*
- * Flag passed to open() to enable use of swmr-reader locks on VMFS.  This
- * definition must match USEROBJ_OPEN_SWMR_LOCK in user_vsiTypes.h.
- */
-#define O_SWMR_LOCK                      (1 << 21)  // 0x00200000
-
-/*
- * OPTIMISTIC is an alternative to EXCLUSIVE and MANDATORY. It applies
- * only on ESX, and gives VMkernel permission to use a type of lock
- * called "optimistic" to speed up opens.  A general guideline is to use it
- * only for read-only opens of small files (< 1KB).
- */
-#define FILEIO_OPEN_OPTIMISTIC_LOCK      (1 << 22)  // 0x00400000
-
-/*
- * Flag passed to open() to enable use of oplocks on VMFS.  This definition
- * must match USEROBJ_OPEN_OPTIMISTIC_LOCK in user_vsiTypes.h.
- */
-#define O_OPTIMISTIC_LOCK                (1 << 22)  // 0x00400000
-
-/*
- * POSIX specific close the file descriptor when the program uses a variant
- * of the exec system call capability. This is useful in fork/exec scenarios.
- */
-#define FILEIO_OPEN_CLOSE_ON_EXEC        (1 << 23)  // 0x00800000
-
-/*
- * Flag passed to open() to not attempt to get the LUN attributes as part of
- * the open operation. Applicable only to opening of SCSI devices. This
- * definition must match the definition of USEROBJ_OPEN_NOATTR in
- * user_vsiTypes.h and FS_OPEN_NOATTR in fs_public.h
- */
-#define O_NOATTR                         (1 << 26)  // 0x04000000
-
-/*
- * Flag passed to open() to get multiwriter VMFS lock.  This definition must
- * match USEROBJ_OPEN_MULTIWRITER_LOCK in user_vsiTypes.h.
- */
-
-#define O_MULTIWRITER_LOCK               (1 << 27)  // 0x08000000
-
-/*
- * Flag passed to open() to get exclusive VMFS lock.  This definition must
- * match USEROBJ_OPEN_EXCLUSIVE_LOCK in user_vsiTypes.h.
- */
-
-#define O_EXCLUSIVE_LOCK                 (1 << 28)  // 0x10000000
+// Flag passed to open() to get exclusive VMFS lock.  This definition must
+// match USEROBJ_OPEN_EXCLUSIVE_LOCK in user_vsiTypes.h.
+#define O_EXCLUSIVE_LOCK 0x10000000
+// Flag passed to open() to get multiwriter VMFS lock.  This definition must
+// match USEROBJ_OPEN_MULTIWRITER_LOCK in user_vsiTypes.h.
+#define O_MULTIWRITER_LOCK 0x08000000
 
 /* File Access check args */
 #define FILEIO_ACCESS_READ       (1 << 0)
@@ -317,42 +249,21 @@ typedef enum {
 
 } FileIOResult;
 
-#if defined(__APPLE__)
-typedef int (FileIOPrivilegedOpener)(const char *path,
-                                     int flags);
-#endif
-
-
 const char *FileIO_MsgError(FileIOResult status);
-
 void FileIO_Invalidate(FileIODescriptor *file);
-
 Bool FileIO_IsValid(const FileIODescriptor *fd);
 
 FileIOResult FileIO_Create(FileIODescriptor *file,
-                           const char *pathName,
+                           ConstUnicode pathName,
                            int access,
                            FileIOOpenAction action,
                            int mode);
 
-FileIOResult FileIO_CreateRetry(FileIODescriptor *file,
-                               const char *pathName,
-                               int access,
-                               FileIOOpenAction action,
-                               int mode,
-                               uint32 maxWaitTimeMsec);
-
 FileIOResult FileIO_Open(FileIODescriptor *file,
-                         const char *pathName,
+                         ConstUnicode pathName,
                          int access,
                          FileIOOpenAction action);
 
-FileIOResult FileIO_OpenRetry(FileIODescriptor *file,
-                              const char *pathName,
-                              int access,
-                              FileIOOpenAction action,
-                              uint32 maxWaitTimeMsec);
-
 uint64 FileIO_Seek(const FileIODescriptor *file,
                    int64 distance,
                    FileIOSeekOrigin origin);
@@ -367,111 +278,82 @@ FileIOResult FileIO_Write(FileIODescriptor *file,
                           size_t requested,
                           size_t *actual);
 
-char *FileIO_AtomicTempPath(const char *path);
-
-FileIOResult FileIO_AtomicTempFile(FileIODescriptor *fileFD,
-                                   FileIODescriptor *tempFD);
-
-Bool FileIO_AtomicUpdate(FileIODescriptor *newFD,
-                         FileIODescriptor *currFD);
-
-int FileIO_AtomicUpdateEx(FileIODescriptor *newFD,
-                          FileIODescriptor *currFD,
-                          Bool renameOnNFS);
-
 #if !defined(VMX86_TOOLS) || !defined(__FreeBSD__)
 
 FileIOResult FileIO_Readv(FileIODescriptor *fd,
-                          struct iovec const *v,
+                          struct iovec  *v,
                           int count,
                           size_t totalSize,
                           size_t *bytesRead);
 
 FileIOResult FileIO_Writev(FileIODescriptor *fd,
-                           struct iovec const *v,
+                           struct iovec *v,
                            int count,
                            size_t totalSize,
                            size_t *bytesWritten);
+
+FileIOResult FileIO_Preadv(FileIODescriptor *fd,   // IN: File descriptor
+                           struct iovec *entries,  // IN: Vector to read into
+                           int numEntries,         // IN: Number of vector entries
+                           uint64 offset,          // IN: Offset to start reading
+                           size_t totalSize);      // IN: totalSize (bytes) in entries
+
+FileIOResult FileIO_Pwritev(FileIODescriptor *fd,  // IN: File descriptor
+                            struct iovec *entries, // IN: Vector to write from
+                            int numEntries,        // IN: Number of vector entries
+                            uint64 offset,         // IN: Offset to start writing
+                            size_t totalSize);     // IN: Total size (bytes) in entries
+
+FileIOResult FileIO_Pread(FileIODescriptor *fd,    // IN: File descriptor
+                          void *buf,               // IN: Buffer to read into
+                          size_t len,              // IN: Length of the buffer
+                          uint64 offset);          // IN: Offset to start reading
+
+FileIOResult FileIO_Pwrite(FileIODescriptor *fd,   // IN: File descriptor
+                           void const *buf,        // IN: Buffer to write from
+                           size_t len,             // IN: Length of the buffer
+                           uint64 offset);         // IN: Offset to start writing
 #endif
 
-FileIOResult FileIO_Preadv(
-               FileIODescriptor *fd,        // IN: File descriptor
-               struct iovec const *entries, // IN: Vector to read into
-               int numEntries,              // IN: Number of vector entries
-               uint64 offset,               // IN: Offset to start reading
-               size_t totalSize,            // IN: totalSize (bytes) in entries
-               size_t *actual);             // OUT: number of bytes read
-
-FileIOResult FileIO_Pwritev(
-              FileIODescriptor *fd,        // IN: File descriptor
-              struct iovec const *entries, // IN: Vector to write from
-              int numEntries,              // IN: Number of vector entries
-              uint64 offset,               // IN: Offset to start writing
-              size_t totalSize,            // IN: Total size (bytes) in entries
-              size_t *actual);             // OUT: number of bytes written
-
-FileIOResult FileIO_Pread(
-                        FileIODescriptor *fd,    // IN: File descriptor
-                        void *buf,               // IN: Buffer to read into
-                        size_t len,              // IN: Length of the buffer
-                        uint64 offset);          // IN: Offset to start reading
-
-FileIOResult FileIO_Pwrite(
-                         FileIODescriptor *fd,   // IN: File descriptor
-                         void const *buf,        // IN: Buffer to write from
-                         size_t len,             // IN: Length of the buffer
-                         uint64 offset);         // IN: Offset to start writing
-
-FileIOResult FileIO_Access(const char *pathName,
+FileIOResult FileIO_Access(ConstUnicode pathName,
                            int accessMode);
 
 Bool    FileIO_Truncate(FileIODescriptor *file,
                         uint64 newSize);
 
-FileIOResult  FileIO_Sync(const FileIODescriptor *file);
-
-FileIOResult FileIO_GetAllocSize(const FileIODescriptor *fd,
-                                 uint64 *logicalBytes,
-                                 uint64 *allocedBytes);
+int     FileIO_Sync(const FileIODescriptor *file);
 
 int64   FileIO_GetSize(const FileIODescriptor *fd);
 
-Bool    FileIO_SetAllocSize(const FileIODescriptor *fd,
-                            uint64 size);
+int64   FileIO_GetSizeByPath(ConstUnicode pathName);
 
-FileIOResult FileIO_GetAllocSizeByPath(const char *pathName,
-                                       uint64 *logicalBytes,
-                                       uint64 *allocedBytes);
-
-int64   FileIO_GetSizeByPath(const char *pathName);
-
-FileIOResult    FileIO_Close(FileIODescriptor *file);
-
-FileIOResult    FileIO_CloseAndUnlink(FileIODescriptor *file);
+int     FileIO_Close(FileIODescriptor *file);
 
 uint32  FileIO_GetFlags(FileIODescriptor *file);
 
-#if defined(_WIN32)
 Bool    FileIO_GetVolumeSectorSize(const char *name,
                                    uint32 *sectorSize);
-#endif
 
 Bool    FileIO_SupportsFileSize(const FileIODescriptor *file,
                                 uint64 testSize);
 
-int64   FileIO_GetModTime(const FileIODescriptor *fd);
-
-FileIOResult FileIO_Lock(FileIODescriptor *file,
-                         int access);
+FileIOResult FileIO_Lock(FileIODescriptor *file,  // IN/OUT
+                         int access);             // IN
 
 FileIOResult FileIO_Unlock(FileIODescriptor *file);
 
 /* Only users not using FileIO_Open should use these two */
 void FileIO_Init(FileIODescriptor *fd,
-                 const char *pathName);
+                 ConstUnicode pathName);
 
 void FileIO_Cleanup(FileIODescriptor *fd);
 
+void FileIO_StatsInit(FileIODescriptor *fd);
+
+void FileIO_StatsLog(FileIODescriptor *fd);
+
+void FileIO_StatsExit(const FileIODescriptor *fd);
+
 const char *FileIO_ErrorEnglish(FileIOResult status);
 
 void FileIO_OptionalSafeInitialize(void);
@@ -484,28 +366,15 @@ FileIODescriptor FileIO_CreateFDWin32(HANDLE win32,
 FileIODescriptor FileIO_CreateFDPosix(int posix,
                                       int flags);
 
-int FileIO_PrivilegedPosixOpen(const char *pathName,
+int FileIO_PrivilegedPosixOpen(ConstUnicode pathName,
                                int flags);
 #endif
 
-FILE *FileIO_DescriptorToStream(FileIODescriptor *fd,
-                                Bool textMode);
-
-const char *FileIO_Filename(FileIODescriptor *fd);
-
-#ifdef VMX86_SERVER
-FileIOResult FileIO_CreateDeviceFileNoPrompt(FileIODescriptor *fd,
-                                             const char *pathName,
-                                             int openMode,
-                                             FileIOOpenAction action,
-                                             int perms,
-                                             const char *device);
-#endif
 
 /*
  *-------------------------------------------------------------------------
- *
- * FileIO_IsSuccess --
+ * 
+ * FileIO_Success --
  *
  *      Returns TRUE if the error code is success.
  *
@@ -518,21 +387,26 @@ FileIOResult FileIO_CreateDeviceFileNoPrompt(FileIODescriptor *fd,
  *-------------------------------------------------------------------------
  */
 
+#if !defined(sun) || __GNUC__ >= 3
 static INLINE Bool
 FileIO_IsSuccess(FileIOResult res)      // IN
 {
    return res == FILEIO_SUCCESS;
 }
-
-Bool FileIO_SupportsPrealloc(const char *pathName,
-                             Bool fsCheck);
-
-#if defined(__APPLE__)
-void FileIO_SetPrivilegedOpener(FileIOPrivilegedOpener *opener);
+#else
+/*
+ * XXX: Crosscompiler used for Solaris tools builds (gcc 2.95.3) complains
+ * whenever the above definition is unused, even though that shouldn't be
+ * a problem for static functions that are also inline.  So for Solaris, we
+ * have a separate definition that is neither static nor inline.
+ */
+Bool FileIO_IsSuccess(FileIOResult res);
 #endif
 
-#if defined(__cplusplus)
-}  // extern "C"
+#if defined(__APPLE__)
+EXTERN Bool FileIO_ResetExcludedFromTimeMachine(char const *pathName);
+EXTERN Bool FileIO_SetExcludedFromTimeMachine(char const *pathName,
+                                              Bool isExcluded);
 #endif
 
 #endif // _FILEIO_H_
