@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2003-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2003 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,23 +16,10 @@
  *
  *********************************************************/
 
-/**
- * @file guestInfo.h
+/*
+ * guestInfo.h --
  *
- * Common declarations that aid in sending guest information to the host.
- */
-
-/**
- * @defgroup vmtools_guestInfoAPI GuestInfo API Reference
- * @{
- *
- * @brief APIs implementing the GuestInfo feature.
- *
- * Definitions below are used for communication across the backdoor between
- * the VMware Tools Service (running in the guest) and the VMX (running in
- * the host).
- *
- * @sa @ref vmtools_guestInfo for a high level overview.
+ *    Common declarations that aid in sending guest information to the host.
  */
 
 #ifndef _GUEST_INFO_H_
@@ -44,35 +31,29 @@
 #include "vm_basic_types.h"
 
 #include "dbllnklst.h"
-#ifndef N_PLAT_NLM
+#include "guestStats.h"
 #include "guestrpc/nicinfo.h"
-#endif // #ifndef N_PLAT_NLM
 
 #define GUEST_INFO_COMMAND "SetGuestInfo"
 #define MAX_VALUE_LEN 100
-
 #define MAX_NICS     16
 #define MAX_IPS      8     // Max number of IP addresses for a single NIC
-#define INFO_IPADDRESS_V2_MAX_IPS 64
 #define MAC_ADDR_SIZE 19
 #define IP_ADDR_SIZE 16
 #define PARTITION_NAME_SIZE MAX_VALUE_LEN
-
-/* Value to be used when "primary" IP address is indeterminable. */
-#define GUESTINFO_IP_UNKNOWN "unknown"
+#define GUESTINFO_TIME_INTERVAL_MSEC 3000  /* time interval in msec */
 
 typedef enum {
    INFO_ERROR,       /* Zero is unused so that errors in atoi can be caught. */
    INFO_DNS_NAME,
    INFO_IPADDRESS,
    INFO_DISK_FREE_SPACE,
-   INFO_BUILD_NUMBER,
+   INFO_TOOLS_VERSION,
    INFO_OS_NAME_FULL,
    INFO_OS_NAME,
    INFO_UPTIME,
    INFO_MEMORY,
    INFO_IPADDRESS_V2,
-   INFO_IPADDRESS_V3,
    INFO_MAX
 } GuestInfoType;
 
@@ -107,8 +88,19 @@ typedef struct _DiskInfo {
    PPartitionEntry partitionList;
 } GuestDiskInfo, *PGuestDiskInfo;
 
-/**
- * @}
+
+
+/*
+ * Global functions
  */
 
+extern Bool GuestInfo_GetFqdn(int outBufLen, char fqdn[]);
+extern Bool GuestInfo_GetNicInfo(GuestNicList *nicInfo);
+extern Bool GuestInfo_GetDiskInfo(PGuestDiskInfo di);
+extern Bool GuestInfo_GetOSName(unsigned int outBufFullLen,
+                                unsigned int outBufLen, char *osNameFull,
+                                char *osName);
+extern int GuestInfo_GetSystemBitness(void);
+
 #endif // _GUEST_INFO_H_
+
