@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -16,20 +16,6 @@
  *
  *********************************************************/
 
-/*********************************************************
- * The contents of this file are subject to the terms of the Common
- * Development and Distribution License (the "License") version 1.0
- * and no later version.  You may not use this file except in
- * compliance with the License.
- *
- * You can obtain a copy of the License at
- *         http://www.opensource.org/licenses/cddl1.php
- *
- * See the License for the specific language governing permissions
- * and limitations under the License.
- *
- *********************************************************/
-
 
 
 /*
@@ -152,7 +138,6 @@ typedef enum {
  * HgfsLookup to fail unexpectedly (normally HGFS_STATUS_INVALID_NAME is
  * converted to -ENOENT, an expected result in HgfsLookup).
  */
-
 typedef enum {
    HGFS_STATUS_SUCCESS,
    HGFS_STATUS_NO_SUCH_FILE_OR_DIR,
@@ -170,15 +155,6 @@ typedef enum {
    HGFS_STATUS_OPERATION_NOT_SUPPORTED,
    HGFS_STATUS_NAME_TOO_LONG,
    HGFS_STATUS_INVALID_PARAMETER,
-   HGFS_STATUS_NOT_SAME_DEVICE,
-   /*
-    * Following error codes are for V4 and above protocol only.
-    * Server must never retun these codes for legacy clients.
-    */
-   HGFS_STATUS_STALE_SESSION,
-   HGFS_STATUS_TOO_MANY_SESSIONS,
-
-   HGFS_STATUS_TRANSPORT_ERROR,
 } HgfsStatus;
 
 /*
