@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2003 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -15,7 +15,7 @@
  * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
  *********************************************************/
-
+  
 #ifndef _LOGLEVEL_USER_H_
 #define _LOGLEVEL_USER_H_
 
@@ -37,8 +37,6 @@
    LOGLEVEL_VAR(vui), \
    LOGLEVEL_VAR(stats), \
    LOGLEVEL_VAR(cpucount), \
-   LOGLEVEL_VAR(ovhdmem), \
-   LOGLEVEL_VAR(vigor), \
    \
    /* user/io */ \
    LOGLEVEL_VAR(disk), \
@@ -57,38 +55,31 @@
    LOGLEVEL_VAR(serial), \
    LOGLEVEL_VAR(parallel), \
    LOGLEVEL_VAR(chipset), \
-   LOGLEVEL_VAR(smram), \
    LOGLEVEL_VAR(smc), \
    LOGLEVEL_VAR(ich7m), \
    LOGLEVEL_VAR(hpet), \
    LOGLEVEL_VAR(extcfgdevice), \
    LOGLEVEL_VAR(flashram), \
-   LOGLEVEL_VAR(efinv), \
-   LOGLEVEL_VAR(pvnvram), \
    LOGLEVEL_VAR(pci), \
    LOGLEVEL_VAR(pci_vide), \
    LOGLEVEL_VAR(pci_uhci), \
    LOGLEVEL_VAR(uhci), \
    LOGLEVEL_VAR(pci_ehci), \
    LOGLEVEL_VAR(ehci), \
-   LOGLEVEL_VAR(pci_xhci), \
-   LOGLEVEL_VAR(usb_xhci), \
    LOGLEVEL_VAR(usb), \
    LOGLEVEL_VAR(vusbmouse), \
    LOGLEVEL_VAR(vusbkeyboard), \
-   LOGLEVEL_VAR(vusbhid), \
-   LOGLEVEL_VAR(vusbtablet), \
-   LOGLEVEL_VAR(vusbaudio), \
-   LOGLEVEL_VAR(vusbvideo),\
-   LOGLEVEL_VAR(hidQueue), \
+   LOGLEVEL_VAR(pci_1394), \
+   LOGLEVEL_VAR(1394), \
    LOGLEVEL_VAR(pci_vlance), \
    LOGLEVEL_VAR(pci_svga), \
    LOGLEVEL_VAR(pci_e1000), \
    LOGLEVEL_VAR(pci_hyper), \
    LOGLEVEL_VAR(pcibridge), \
    LOGLEVEL_VAR(vide), \
-   LOGLEVEL_VAR(atapiCdrom), \
+   LOGLEVEL_VAR(ideCdrom), \
    LOGLEVEL_VAR(hostonly), \
+   LOGLEVEL_VAR(backdoorAbsMouse), \
    LOGLEVEL_VAR(oprom), \
    LOGLEVEL_VAR(http), \
    LOGLEVEL_VAR(vmci), \
@@ -96,12 +87,6 @@
    LOGLEVEL_VAR(vmxnet3), \
    LOGLEVEL_VAR(pci_vmxnet3), \
    LOGLEVEL_VAR(vcpuhotplug), \
-   LOGLEVEL_VAR(vcpuNUMA), \
-   LOGLEVEL_VAR(heci), \
-   LOGLEVEL_VAR(pciplugin), \
-   LOGLEVEL_VAR(vsock), \
-   LOGLEVEL_VAR(vrdma), \
-   LOGLEVEL_VAR(nvdimm), \
    \
    /* user/disk */ \
    LOGLEVEL_VAR(aioMgr), \
@@ -117,51 +102,24 @@
    LOGLEVEL_VAR(checkpoint), \
    LOGLEVEL_VAR(dumper), \
    LOGLEVEL_VAR(migrate), \
-   LOGLEVEL_VAR(fsresx), \
    \
    /* user/gui */ \
    LOGLEVEL_VAR(gui), \
    LOGLEVEL_VAR(guiWin32), \
    LOGLEVEL_VAR(mks), \
-   LOGLEVEL_VAR(mksInput), \
-   LOGLEVEL_VAR(mksSWB), \
    LOGLEVEL_VAR(mksClient), \
    LOGLEVEL_VAR(mksServer), \
    LOGLEVEL_VAR(mksKeyboard), \
-   LOGLEVEL_VAR(keymap), \
    LOGLEVEL_VAR(mksMouse), \
-   LOGLEVEL_VAR(mksHostCursor), \
-   LOGLEVEL_VAR(mksCursorPosition), \
-   LOGLEVEL_VAR(mksBasicOps), \
-   LOGLEVEL_VAR(mksRenderOps), \
-   LOGLEVEL_VAR(mksGLBasic), \
+   LOGLEVEL_VAR(mksHostOps), \
    LOGLEVEL_VAR(mksGLManager), \
-   LOGLEVEL_VAR(mksGLFBO), \
    LOGLEVEL_VAR(mksGLShader), \
-   LOGLEVEL_VAR(mksGLState), \
-   LOGLEVEL_VAR(mksGLWindow), \
-   LOGLEVEL_VAR(mksGLContextMux), \
-   LOGLEVEL_VAR(mksGLDraw), \
-   LOGLEVEL_VAR(mksGLQuery), \
-   LOGLEVEL_VAR(mksGLTextureView), \
-   LOGLEVEL_VAR(mksWinBSOD), \
-   LOGLEVEL_VAR(mksDX11Renderer), \
-   LOGLEVEL_VAR(mksDX11ResourceView), \
-   LOGLEVEL_VAR(mksDX11ShimOps), \
-   LOGLEVEL_VAR(vdpPlugin), \
    \
    /* user/sound */ \
    LOGLEVEL_VAR(sound), \
-   LOGLEVEL_VAR(hdaudio), \
-   LOGLEVEL_VAR(pci_hdaudio), \
-   LOGLEVEL_VAR(hdaudio_alsa), \
-   \
-   /* user video */ \
-   LOGLEVEL_VAR(AVCapture), \
    \
    /* user/disklib */ \
    LOGLEVEL_VAR(disklib), \
-   LOGLEVEL_VAR(dmg), \
    LOGLEVEL_VAR(sparseChecker), \
    LOGLEVEL_VAR(dataCache), \
    /* more */ \
@@ -177,15 +135,13 @@
    LOGLEVEL_VAR(macbw), \
    LOGLEVEL_VAR(macfi), \
    LOGLEVEL_VAR(vmkcfg), \
-   LOGLEVEL_VAR(policy), \
+   LOGLEVEL_VAR(vmxfer), \
    LOGLEVEL_VAR(poll), \
    LOGLEVEL_VAR(barrier), \
    LOGLEVEL_VAR(mstat), \
    LOGLEVEL_VAR(vmLock), \
    LOGLEVEL_VAR(buslogic), \
    LOGLEVEL_VAR(lsilogic), \
-   LOGLEVEL_VAR(pvscsi), \
-   LOGLEVEL_VAR(ahci), \
    LOGLEVEL_VAR(diskVmnix), \
    LOGLEVEL_VAR(hbaCommon), \
    LOGLEVEL_VAR(backdoor), \
@@ -200,16 +156,11 @@
    LOGLEVEL_VAR(ipc), \
    LOGLEVEL_VAR(smbios), \
    LOGLEVEL_VAR(acpi), \
-   LOGLEVEL_VAR(acpiGPE), \
-   LOGLEVEL_VAR(vmgenc), \
-   LOGLEVEL_VAR(xpmode), \
    LOGLEVEL_VAR(snapshot), \
    LOGLEVEL_VAR(asyncsocket), \
    LOGLEVEL_VAR(mainMem), \
    LOGLEVEL_VAR(mainMemReplayCheck), \
    LOGLEVEL_VAR(memoryHotplug), \
-   LOGLEVEL_VAR(numa), \
-   LOGLEVEL_VAR(numaHost), \
    LOGLEVEL_VAR(remoteDevice), \
    LOGLEVEL_VAR(vncDecode), \
    LOGLEVEL_VAR(vncEncode), \
@@ -223,6 +174,7 @@
    LOGLEVEL_VAR(battery), \
    LOGLEVEL_VAR(fakeDma), \
    LOGLEVEL_VAR(shader), \
+   LOGLEVEL_VAR(numa), \
    LOGLEVEL_VAR(machPoll), \
    LOGLEVEL_VAR(replayVMX), \
    LOGLEVEL_VAR(vmWindowController), \
@@ -230,71 +182,20 @@
    LOGLEVEL_VAR(duiMKS), \
    LOGLEVEL_VAR(worker), \
    LOGLEVEL_VAR(duiDevices), \
-   LOGLEVEL_VAR(duiLocalization), \
-   LOGLEVEL_VAR(duiProxyApps), \
-   LOGLEVEL_VAR(docker), \
-   LOGLEVEL_VAR(vmIPC), \
    LOGLEVEL_VAR(uwt), /* lib/unityWindowTracker */ \
    LOGLEVEL_VAR(cui), \
    LOGLEVEL_VAR(automation), \
    LOGLEVEL_VAR(oemDevice), \
    LOGLEVEL_VAR(cptOps), \
-   LOGLEVEL_VAR(vprobe), \
-   LOGLEVEL_VAR(VProbeClient), \
+   LOGLEVEL_VAR(VProbeExec), \
+   LOGLEVEL_VAR(VProbeVP), \
    LOGLEVEL_VAR(device), \
    LOGLEVEL_VAR(devicePowerOn), \
    LOGLEVEL_VAR(vmxvmdbCallbacks), \
    LOGLEVEL_VAR(guestInstall), \
-   LOGLEVEL_VAR(migrateVM), \
    LOGLEVEL_VAR(vmUpsellController), \
    LOGLEVEL_VAR(objc), /* lib/objc */ \
-   LOGLEVEL_VAR(blit), /* lib/blit */ \
-   LOGLEVEL_VAR(vmnetBridge), \
-   LOGLEVEL_VAR(wifi), /* macWireless and wpa_supplicant */ \
-   LOGLEVEL_VAR(pvfslib), \
-   LOGLEVEL_VAR(brtalk), \
-   LOGLEVEL_VAR(button), \
-   LOGLEVEL_VAR(util), \
-   LOGLEVEL_VAR(vmcf), \
-   LOGLEVEL_VAR(win32util), \
-   LOGLEVEL_VAR(largepage), \
-   LOGLEVEL_VAR(guestAppMonitor), \
-   LOGLEVEL_VAR(syncWaitQ), \
-   LOGLEVEL_VAR(sg), /* lib/sg */ \
-   LOGLEVEL_VAR(ftcpt), \
-   LOGLEVEL_VAR(wrapLib),  \
-   LOGLEVEL_VAR(digestlib), \
-   LOGLEVEL_VAR(inputdevtap), \
-   LOGLEVEL_VAR(objlib), \
-   LOGLEVEL_VAR(vsanobj), \
-   LOGLEVEL_VAR(vvolbe), \
-   LOGLEVEL_VAR(svgadevtap), \
-   LOGLEVEL_VAR(masReceipt), /* lib/masReceipt */ \
-   LOGLEVEL_VAR(serviceImpl), /* lib/serviceImpl */ \
-   LOGLEVEL_VAR(serviceUser), /* lib/serviceUser */ \
-   LOGLEVEL_VAR(ssl), \
-   LOGLEVEL_VAR(namespaceDb), \
-   LOGLEVEL_VAR(namespaceMgr), \
-   LOGLEVEL_VAR(grainTrack), \
-   LOGLEVEL_VAR(shim3D), \
-   LOGLEVEL_VAR(crc32), \
-   LOGLEVEL_VAR(vmkmgmtlib), \
-   LOGLEVEL_VAR(vflash), \
-   LOGLEVEL_VAR(vva),  /* apps/rde/vva */ \
-   LOGLEVEL_VAR(ftConfig), /*lib/ftConfig */ \
-   LOGLEVEL_VAR(vmname),  /* lib/vmname */ \
-   LOGLEVEL_VAR(gpumgmt), \
-   LOGLEVEL_VAR(unityMsg),  /* mks/remote/vdpUnityVmdb */ \
-   LOGLEVEL_VAR(sharedFolderMgr),  /* mks/remote/vdpFolderSharedMgrVmdb */ \
-   LOGLEVEL_VAR(crtbora),  /* apps/crtbora */ \
-   LOGLEVEL_VAR(mirror), \
-   LOGLEVEL_VAR(filtlib), \
-   LOGLEVEL_VAR(epd), \
-   LOGLEVEL_VAR(ddecomd), \
-   LOGLEVEL_VAR(hostctl), \
-   LOGLEVEL_VAR(pmemobj), \
-   /* end of list */
 
 LOGLEVEL_EXTENSION_DECLARE(LOGLEVEL_USER);
 
-#endif /* _LOGLEVEL_USER_H_ */
+#endif /* _LOGLEVEL_USER_H_ */ 
