@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,27 +30,28 @@
 #define INCLUDE_ALLOW_VMCORE
 #include "includeCheck.h"
 #include "vm_basic_types.h"
-#include "vm_assert.h"
 
 #if defined(__APPLE__)
 
 #include <sys/types.h>
 #include <unistd.h>
 
+void Id_SetSuperUser(Bool yes);
 int Id_SetGid(gid_t egid);
 int Id_SetREUid(uid_t ruid, uid_t euid);
 int Id_SetRESUid(uid_t ruid, uid_t euid, uid_t suid);
 
+#define IsSuperUser() (0 == geteuid())
+#define SuperUser(yes) Id_SetSuperUser(yes)
 #define Id_GetEUid() geteuid()
 
 void *Id_AuthGetLocal();
 void *Id_AuthGetExternal(size_t *size);
 Bool Id_AuthSet(void const *buf, size_t size);
 Bool Id_AuthCheck(char const *right,
-                  char const *localizedDescription,
-                  Bool showDialogIfNeeded);
+                  char const *localizedDescription);
 
-#elif (defined(__linux__) || defined(sun) || defined(__FreeBSD__))
+#elif (defined(__linux__) || defined(sun) || defined(__FreeBSD__)) && !defined(N_PLAT_NLM)
 
 #include <sys/types.h>
 #include <unistd.h>
@@ -112,19 +113,17 @@ Id_SetEGid(gid_t egid)
 {
    return Id_SetRESGid((gid_t)-1, egid, (gid_t)-1);
 }
-#endif /* linux */
 
-#if defined(_WIN32)
-static INLINE int Id_BeginSuperUser(void) { return -1; }
-static INLINE void Id_EndSuperUser(int id) { }
-static INLINE Bool Id_IsSuperUser(void) { return TRUE; }
-static INLINE Bool Id_IsSetUGid(void) { return FALSE; }
-#else
-static INLINE Bool Id_IsSuperUser(void) { return 0 == geteuid(); }
-uid_t Id_BeginSuperUser(void);
-void Id_EndSuperUser(uid_t uid);
-Bool Id_IsSetUGid(void);
-#endif
+#define IsSuperUser()	(0 == geteuid())
+#define SuperUser(yes)	((void) ((yes) ? Id_SetEUid(0) : Id_SetEUid(getuid())))
+
+#else /* linux */
+
+// XXX does Windows need this?
+#define IsSuperUser()	TRUE
+#define SuperUser(yes)	
+
+#endif /* linux */
 
 #endif /* USER_SU_H */
 
