@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -27,6 +27,13 @@
 
 #include "vm_basic_types.h"
 
+/*
+ * SyncDriverHandle is a different thing depending on the platform we're
+ * running: on Windows it's a thread handle, used to monitor the status
+ * of the thread used to asynchronously freeze devices. On POSIX systems,
+ * it's a file descriptor, used to send commands to the driver via ioctls.
+ */
+
 #ifdef _WIN32 /* { */
 
 # include <windows.h>
@@ -36,9 +43,8 @@ typedef HANDLE SyncDriverHandle;
 #else /* }{ POSIX */
 
 # define INFINITE -1
-# define SYNCDRIVER_INVALID_HANDLE NULL
-
-typedef struct SyncHandle * SyncDriverHandle;
+# define SYNCDRIVER_INVALID_HANDLE -1
+typedef int SyncDriverHandle;
 
 #endif /* } */
 
@@ -49,9 +55,9 @@ typedef enum {
 } SyncDriverStatus;
 
 Bool SyncDriver_Init(void);
-Bool SyncDriver_Freeze(const char *drives, Bool enableNullDriver,
-                       SyncDriverHandle *handle);
+Bool SyncDriver_Freeze(const char *drives, SyncDriverHandle *handle);
 Bool SyncDriver_Thaw(const SyncDriverHandle handle);
+Bool SyncDriver_DrivesAreFrozen(void);
 SyncDriverStatus SyncDriver_QueryStatus(const SyncDriverHandle handle,
                                         int32 timeout);
 void SyncDriver_CloseHandle(SyncDriverHandle *handle);
