@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -28,6 +28,7 @@
 #define INCLUDE_ALLOW_USERLEVEL
 #define INCLUDE_ALLOW_VMCORE
 #define INCLUDE_ALLOW_VMKERNEL
+#define INCLUDE_ALLOW_VMNIXMOD
 #include "includeCheck.h"
 
 #ifdef __cplusplus
@@ -40,22 +41,36 @@ extern "C" {
 #include "unicodeTypes.h"
 
 
-#define UNICODE_SUBSTITUTION_CHAR "\xEF\xBF\xBD"
+/*
+ * Static Unicode string literal macros.
+ *
+ * Given an ASCII const char * literal ("abcd"), returns a static
+ * ConstUnicode string.
+ *
+ * These strings are never freed, so this is not to be used for
+ * general runtime Unicode string creation. Instead, use it to
+ * replace:
+ *
+ *   const char *extension = ".vmdk";
+ *
+ * with:
+ *
+ *   ConstUnicode extension = U(".vmdk");
+ */
+
+#define U(x) Unicode_GetStatic(x, FALSE)
 
 /*
- * Unescapes \\uABCD in string literals to Unicode code point
+ * Same as U(), but also unescapes \\uABCD to Unicode code point
  * U+ABCD, and \\U001FABCD to Unicode code point U+1FABCD.
  *
- * The resulting string is never freed, so this is not to be used for
- * general runtime Unicode string creation.
- *
  * Use to replace:
  *
  *   const char *utf8Copyright = "Copyright \302\251 COMPANY_NAME";
  *
  * with:
  *
- *   const char *copyright = U_UNESCAPE("Copyright \\u00A9 COMPANY_NAME");
+ *   ConstUnicode copyright = U_UNESCAPE("Copyright \\u00A9 COMPANY_NAME");
  */
 
 #define U_UNESCAPE(x) Unicode_GetStatic(x, TRUE)
@@ -79,7 +94,7 @@ extern "C" {
  * allocated.
  */
 
-char *Unicode_AllocWithLength(const void *buffer,
+Unicode Unicode_AllocWithLength(const void *buffer,
                                 ssize_t lengthInBytes,
                                 StringEncoding encoding);
 
@@ -100,7 +115,7 @@ char *Unicode_AllocWithLength(const void *buffer,
  * Results:
  *      An allocated Unicode string containing the decoded characters
  *      in buffer, or NULL if input is NULL.  Caller must pass to
- *      free to free.
+ *      Unicode_Free to free.
  *
  * Side effects:
  *      None
@@ -108,7 +123,7 @@ char *Unicode_AllocWithLength(const void *buffer,
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
+static INLINE Unicode
 Unicode_Alloc(const void *buffer,      // IN
               StringEncoding encoding) // IN
 {
@@ -128,7 +143,7 @@ Unicode_Alloc(const void *buffer,      // IN
  * Results:
  *      An allocated Unicode string containing the characters in
  *      utf8String, or NULL if utf8String is NULL.  Caller must pass to
- *      free to free.
+ *      Unicode_Free to free.
  *
  * Side effects:
  *      None
@@ -136,7 +151,7 @@ Unicode_Alloc(const void *buffer,      // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
+static INLINE Unicode
 Unicode_AllocWithUTF8(const char *utf8String) // IN
 {
    return Unicode_AllocWithLength(utf8String, -1, STRING_ENCODING_UTF8);
@@ -156,7 +171,7 @@ Unicode_AllocWithUTF8(const char *utf8String) // IN
  * Results:
  *      An allocated Unicode string containing the characters in
  *      utf16String, or NULL if utf16String is NULL.  Caller must pass to
- *      free to free.
+ *      Unicode_Free to free.
  *
  * Side effects:
  *      None
@@ -164,20 +179,22 @@ Unicode_AllocWithUTF8(const char *utf8String) // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
+static INLINE Unicode
 Unicode_AllocWithUTF16(const utf16_t *utf16String) // IN
 {
    return Unicode_AllocWithLength(utf16String, -1, STRING_ENCODING_UTF16);
 }
 
 
-char *Unicode_Duplicate(const char *str);
+Unicode Unicode_Duplicate(ConstUnicode str);
+void Unicode_Free(Unicode str);
 
-char **Unicode_AllocList(char **srcList, ssize_t length,
+
+Unicode *Unicode_AllocList(char **srcList, ssize_t length,
                            StringEncoding encoding);
 
-char **Unicode_GetAllocList(char *const srcList[], ssize_t length,
-                            StringEncoding encoding);
+Unicode *Unicode_GetAllocList(Unicode const srcList[], ssize_t length, 
+                              StringEncoding encoding);
 
 /*
  *-----------------------------------------------------------------------------
@@ -190,7 +207,7 @@ char **Unicode_GetAllocList(char *const srcList[], ssize_t length,
  *
  * Results:
  *      An allocated list (vector) of Unicode strings.
- *      The result must be freed with Util_FreeStringList.
+ *      The result must be freed with Unicode_FreeList.
  *
  * Side effects:
  *      None
@@ -198,7 +215,7 @@ char **Unicode_GetAllocList(char *const srcList[], ssize_t length,
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char **
+static INLINE Unicode *
 Unicode_AllocListWithUTF16(utf16_t **utf16list, // IN:
                            ssize_t length)      // IN:
 {
@@ -206,6 +223,27 @@ Unicode_AllocListWithUTF16(utf16_t **utf16list, // IN:
 			    STRING_ENCODING_UTF16);
 }
 
+void Unicode_FreeList(Unicode *list, ssize_t length);
+
+
+/*
+ * Accessor for the NUL-terminated UTF-8 representation of this
+ * Unicode string.
+ *
+ * Memory is managed inside this string, so callers do not need to
+ * free.
+ */
+const char *Unicode_GetUTF8(ConstUnicode str);
+
+/*
+ * Convenience wrapper with a short name for this often-used function.
+ */
+static INLINE const char *
+UTF8(ConstUnicode str)
+{
+   return Unicode_GetUTF8(str);
+}
+
 /*
  * Compute the number of bytes in a string.
  */
@@ -232,7 +270,7 @@ Bool Unicode_IsBufferValid(const void *buffer,
  * Tests if the buffer's unicode contents can be converted to the
  * specified encoding.
  */
-Bool Unicode_CanGetBytesWithEncoding(const char *ustr,
+Bool Unicode_CanGetBytesWithEncoding(ConstUnicode ustr,
                                      StringEncoding encoding);
 
 /*
@@ -247,7 +285,7 @@ char *Unicode_EscapeBuffer(const void *buffer,
  * Returns the length in number of native code units (UTF-8 bytes or
  * UTF-16 words) of the string.
  */
-UnicodeIndex Unicode_LengthInCodeUnits(const char *str);
+UnicodeIndex Unicode_LengthInCodeUnits(ConstUnicode str);
 
 /*
  *-----------------------------------------------------------------------------
@@ -266,10 +304,14 @@ UnicodeIndex Unicode_LengthInCodeUnits(const char *str);
  */
 
 static INLINE Bool
-Unicode_IsEmpty(const char *str)  // IN:
+Unicode_IsEmpty(ConstUnicode str)  // IN:
 {
+#ifdef SUPPORT_UNICODE_OPAQUE
+   NOT_IMPLEMENTED();
+#else
    ASSERT(str != NULL);
    return str[0] == '\0';
+#endif
 }
 
 
@@ -278,7 +320,7 @@ Unicode_IsEmpty(const char *str)  // IN:
  * to encode the Unicode string in the specified encoding, including
  * NUL termination.
  */
-size_t Unicode_BytesRequired(const char *str,
+size_t Unicode_BytesRequired(ConstUnicode str,
                              StringEncoding encoding);
 
 /*
@@ -290,18 +332,14 @@ size_t Unicode_BytesRequired(const char *str,
  * the NUL termination, upon return.
  */
 Bool Unicode_CopyBytes(void *destBuffer,
-                       const char *srcBuffer,
+                       ConstUnicode srcBuffer,
                        size_t maxLengthInBytes,
                        size_t *retLength,
                        StringEncoding encoding);
 
-void *Unicode_GetAllocBytes(const char *str,
+void *Unicode_GetAllocBytes(ConstUnicode str,
                             StringEncoding encoding);
 
-void *Unicode_GetAllocBytesWithLength(const char *str,
-                                      StringEncoding encoding,
-                                      ssize_t lengthInBytes);
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -327,7 +365,7 @@ void *Unicode_GetAllocBytesWithLength(const char *str,
  */
 
 static INLINE utf16_t *
-Unicode_GetAllocUTF16(const char *str)  // IN:
+Unicode_GetAllocUTF16(ConstUnicode str)  // IN:
 {
    /* Cast for compatibility with C++ compilers. */
    return (utf16_t *) Unicode_GetAllocBytes(str, STRING_ENCODING_UTF16);
@@ -337,11 +375,21 @@ Unicode_GetAllocUTF16(const char *str)  // IN:
 /*
  * Helper function for Unicode string literal macros.
  */
-const char *Unicode_GetStatic(const char *asciiBytes,
-                              Bool unescape);
+ConstUnicode Unicode_GetStatic(const char *asciiBytes,
+                               Bool unescape);
 
 /*
  * Helper macros for Win32 Unicode string transition.
+ *
+ * Eventually, when the Unicode type becomes opaque
+ * (SUPPORT_UNICODE_OPAQUE), we can store UTF-16 directly inside
+ * it, which is a huge optimization for Win32 programmers.
+ *
+ * At that point, UNICODE_GET_UTF16/UNICODE_RELEASE_UTF16 can become
+ * constant-time operations.
+ *
+ * Until then, Win32 programmers need to alloc and free the UTF-16
+ * representation of the Unicode string.
  */
 #if defined(_WIN32)
    #define UNICODE_GET_UTF16(s)     Unicode_GetAllocUTF16(s)
