@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -41,6 +41,7 @@
 #define INCLUDE_ALLOW_USERLEVEL
 #define INCLUDE_ALLOW_VMCORE
 #define INCLUDE_ALLOW_VMKERNEL
+#define INCLUDE_ALLOW_VMNIXMOD
 #include "includeCheck.h"
 
 #include <string.h>
@@ -60,44 +61,43 @@ extern "C" {
  * of string".
  */
 
-int Unicode_CompareRange(const char *str1,
+int Unicode_CompareRange(ConstUnicode str1,
                          UnicodeIndex str1Start,
                          UnicodeIndex str1Length,
-                         const char *str2,
+                         ConstUnicode str2,
                          UnicodeIndex str2Start,
                          UnicodeIndex str2Length,
                          Bool ignoreCase);
 
-UnicodeIndex Unicode_FindSubstrInRange(const char *str,
+UnicodeIndex Unicode_FindSubstrInRange(ConstUnicode str,
                                        UnicodeIndex strStart,
                                        UnicodeIndex strLength,
-                                       const char *strToFind,
+                                       ConstUnicode strToFind,
                                        UnicodeIndex strToFindStart,
                                        UnicodeIndex strToFindLength);
 
-UnicodeIndex Unicode_FindLastSubstrInRange(const char *str,
+UnicodeIndex Unicode_FindLastSubstrInRange(ConstUnicode str,
                                            UnicodeIndex strStart,
                                            UnicodeIndex strLength,
-                                           const char *strToFind,
+                                           ConstUnicode strToFind,
                                            UnicodeIndex strToFindStart,
                                            UnicodeIndex strToFindLength);
-char *Unicode_Substr(const char *str,
-                     UnicodeIndex start,
-                     UnicodeIndex length);
+Unicode Unicode_Substr(ConstUnicode str,
+                       UnicodeIndex start,
+                       UnicodeIndex length);
 
-char *Unicode_ReplaceRange(const char *destination,
-                           UnicodeIndex destinationStart,
-                           UnicodeIndex destinationLength,
-                           const char *source,
-                           UnicodeIndex sourceStart,
-                           UnicodeIndex sourceLength);
+Unicode Unicode_ReplaceRange(ConstUnicode destination,
+                             UnicodeIndex destinationStart,
+                             UnicodeIndex destinationLength,
+                             ConstUnicode source,
+                             UnicodeIndex sourceStart,
+                             UnicodeIndex sourceLength);
 
-char *Unicode_Join(const char *first,
-                   ...);
+Unicode Unicode_Join(ConstUnicode first,
+                     ...);
 
-char *Unicode_Format(const char *fmt, ...);
+Unicode Unicode_Format(const char *fmt, ...);
 
-UnicodeIndex Unicode_LengthInCodePoints(const char *str);
 
 /*
  * Simple in-line functions that may be used below.
@@ -109,7 +109,7 @@ UnicodeIndex Unicode_LengthInCodePoints(const char *str);
  *
  * Unicode_IsIndexAtCodePointBoundary --
  *
- *      Check a string index (in bytes) for code point boundary.
+ *      Check a string index for code point boundary.
  *
  *	The index must be valid (>= 0 and <= string length).
  *	The end of the string is considered a valid boundary.
@@ -124,12 +124,16 @@ UnicodeIndex Unicode_LengthInCodePoints(const char *str);
  */
 
 static INLINE Bool
-Unicode_IsIndexAtCodePointBoundary(const char *str,     // IN:
-                                   UnicodeIndex index)  // IN:
+Unicode_IsIndexAtCodePointBoundary(ConstUnicode str,    // IN
+                                   UnicodeIndex index)  // IN
 {
    ASSERT(index >= 0 && index <= Unicode_LengthInCodeUnits(str));
 
+#ifdef SUPPORT_UNICODE_OPAQUE
+   NOT_IMPLEMENTED();
+#else
    return (str[index] & 0xc0) != 0x80;
+#endif
 }
 
 
@@ -147,7 +151,7 @@ Unicode_IsIndexAtCodePointBoundary(const char *str,     // IN:
  *      followed by 'source'.
  *
  * Results:
- *      The newly-allocated string.  Caller must free with free.
+ *      The newly-allocated string.  Caller must free with Unicode_Free.
  *
  * Side effects:
  *      None
@@ -155,9 +159,9 @@ Unicode_IsIndexAtCodePointBoundary(const char *str,     // IN:
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
-Unicode_Append(const char *destination, // IN
-               const char *source)      // IN
+static INLINE Unicode
+Unicode_Append(ConstUnicode destination, // IN
+               ConstUnicode source)      // IN
 {
    return Unicode_ReplaceRange(destination,
                                -1,
@@ -177,7 +181,7 @@ Unicode_Append(const char *destination, // IN
  *      followed by the specified range of 'source'.
  *
  * Results:
- *      The newly-allocated string.  Caller must free with free.
+ *      The newly-allocated string.  Caller must free with Unicode_Free.
  *
  * Side effects:
  *      None
@@ -185,18 +189,18 @@ Unicode_Append(const char *destination, // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
-Unicode_AppendRange(const char *dest,        // IN:
-                    const char *src,         // IN:
-                    UnicodeIndex srcStart,   // IN:
-                    UnicodeIndex srcLength)  // IN:
+static INLINE Unicode
+Unicode_AppendRange(ConstUnicode destination,  // IN
+                    ConstUnicode source,       // IN
+                    UnicodeIndex sourceStart,  // IN
+                    UnicodeIndex sourceLength) // IN
 {
-   return Unicode_ReplaceRange(dest,
-                               Unicode_LengthInCodePoints(dest),
+   return Unicode_ReplaceRange(destination,
+                               Unicode_LengthInCodeUnits(destination),
                                0,
-                               src,
-                               srcStart,
-                               srcLength);
+                               source,
+                               sourceStart,
+                               sourceLength);
 }
 
 
@@ -222,8 +226,8 @@ Unicode_AppendRange(const char *dest,        // IN:
  */
 
 static INLINE int
-Unicode_Compare(const char *str1, // IN
-                const char *str2) // IN
+Unicode_Compare(ConstUnicode str1, // IN
+                ConstUnicode str2) // IN
 {
    return Unicode_CompareRange(str1,
                                0,
@@ -257,8 +261,8 @@ Unicode_Compare(const char *str1, // IN
  */
 
 static INLINE int
-Unicode_CompareIgnoreCase(const char *str1, // IN
-                          const char *str2) // IN
+Unicode_CompareIgnoreCase(ConstUnicode str1, // IN
+                          ConstUnicode str2) // IN
 {
    return Unicode_CompareRange(str1,
                                0,
@@ -289,17 +293,20 @@ Unicode_CompareIgnoreCase(const char *str1, // IN
  */
 
 static INLINE Bool
-UnicodeEndsWith(const char *str,      // IN:
-                const char *suffix,   // IN:
-                Bool ignoreCase)      // IN:
+UnicodeEndsWith(ConstUnicode str,    // IN
+                ConstUnicode suffix, // IN
+                Bool ignoreCase)     // IN
 {
-   UnicodeIndex strLength = Unicode_LengthInCodePoints(str);
-   UnicodeIndex suffixLength = Unicode_LengthInCodePoints(suffix);
+   UnicodeIndex strLength = Unicode_LengthInCodeUnits(str);
+   UnicodeIndex suffixLength = Unicode_LengthInCodeUnits(suffix);
    UnicodeIndex offset = strLength - suffixLength;
 
    if (suffixLength > strLength) {
       return FALSE;
    }
+   if (!Unicode_IsIndexAtCodePointBoundary(str, offset)) {
+      return FALSE;
+   }
 
    return Unicode_CompareRange(str,
                                offset,
@@ -312,16 +319,16 @@ UnicodeEndsWith(const char *str,      // IN:
 
 
 static INLINE Bool
-Unicode_EndsWith(const char *str,    // IN
-                 const char *suffix) // IN
+Unicode_EndsWith(ConstUnicode str,    // IN
+                 ConstUnicode suffix) // IN
 {
    return UnicodeEndsWith(str, suffix, FALSE);
 }
 
 
 static INLINE Bool
-Unicode_EndsWithIgnoreCase(const char *str,    // IN
-                           const char *suffix) // IN
+Unicode_EndsWithIgnoreCase(ConstUnicode str,    // IN
+                           ConstUnicode suffix) // IN
 {
    return UnicodeEndsWith(str, suffix, TRUE);
 }
@@ -347,8 +354,8 @@ Unicode_EndsWithIgnoreCase(const char *str,    // IN
  */
 
 static INLINE UnicodeIndex
-Unicode_Find(const char *str,       // IN
-             const char *strToFind) // IN
+Unicode_Find(ConstUnicode str,       // IN
+             ConstUnicode strToFind) // IN
 {
    return Unicode_FindSubstrInRange(str,
                                     0,
@@ -380,8 +387,8 @@ Unicode_Find(const char *str,       // IN
  */
 
 static INLINE UnicodeIndex
-Unicode_FindFromIndex(const char *str,        // IN
-                      const char *strToFind,  // IN
+Unicode_FindFromIndex(ConstUnicode str,       // IN
+                      ConstUnicode strToFind, // IN
                       UnicodeIndex fromIndex) // IN
 {
    return Unicode_FindSubstrInRange(str,
@@ -414,8 +421,8 @@ Unicode_FindFromIndex(const char *str,        // IN
  */
 
 static INLINE UnicodeIndex
-Unicode_FindInRange(const char *str,        // IN
-                    const char *strToFind,  // IN
+Unicode_FindInRange(ConstUnicode str,       // IN
+                    ConstUnicode strToFind, // IN
                     UnicodeIndex start,     // IN
                     UnicodeIndex length)    // IN
 {
@@ -448,8 +455,8 @@ Unicode_FindInRange(const char *str,        // IN
  */
 
 static INLINE UnicodeIndex
-Unicode_FindLast(const char *str,       // IN
-                 const char *strToFind) // IN
+Unicode_FindLast(ConstUnicode str,       // IN
+                 ConstUnicode strToFind) // IN
 {
    return Unicode_FindLastSubstrInRange(str,
                                         0,
@@ -481,8 +488,8 @@ Unicode_FindLast(const char *str,       // IN
  */
 
 static INLINE UnicodeIndex
-Unicode_FindLastFromIndex(const char *str,        // IN
-                          const char *strToFind,  // IN
+Unicode_FindLastFromIndex(ConstUnicode str,       // IN
+                          ConstUnicode strToFind, // IN
                           UnicodeIndex fromIndex) // IN
 {
    return Unicode_FindLastSubstrInRange(str,
@@ -515,8 +522,8 @@ Unicode_FindLastFromIndex(const char *str,        // IN
  */
 
 static INLINE UnicodeIndex
-Unicode_FindLastInRange(const char *str,        // IN
-                        const char *strToFind,  // IN
+Unicode_FindLastInRange(ConstUnicode str,       // IN
+                        ConstUnicode strToFind, // IN
                         UnicodeIndex start,     // IN
                         UnicodeIndex length)    // IN
 {
@@ -538,7 +545,7 @@ Unicode_FindLastInRange(const char *str,        // IN
  *      string 'source' inserted at the index 'destinationStart'.
  *
  * Results:
- *      The newly-allocated string.  Caller must free with free.
+ *      The newly-allocated string.  Caller must free with Unicode_Free.
  *
  * Side effects:
  *      None
@@ -546,10 +553,10 @@ Unicode_FindLastInRange(const char *str,        // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
-Unicode_Insert(const char *destination,       // IN
+static INLINE Unicode
+Unicode_Insert(ConstUnicode destination,      // IN
                UnicodeIndex destinationStart, // IN
-               const char *source)            // IN
+               ConstUnicode source)           // IN
 {
    return Unicode_ReplaceRange(destination,
                                destinationStart,
@@ -570,7 +577,7 @@ Unicode_Insert(const char *destination,       // IN
  *      'destinationStart'.
  *
  * Results:
- *      The newly-allocated string.  Caller must free with free.
+ *      The newly-allocated string.  Caller must free with Unicode_Free.
  *
  * Side effects:
  *      None
@@ -578,10 +585,10 @@ Unicode_Insert(const char *destination,       // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
-Unicode_InsertRange(const char *destination,
+static INLINE Unicode
+Unicode_InsertRange(ConstUnicode destination,
                     UnicodeIndex destinationStart,
-                    const char *source,
+                    ConstUnicode source,
                     UnicodeIndex sourceStart,
                     UnicodeIndex sourceLength)
 {
@@ -611,8 +618,8 @@ Unicode_InsertRange(const char *destination,
  */
 
 static INLINE Bool
-Unicode_IsEqual(const char *str1, // IN
-                const char *str2) // IN
+Unicode_IsEqual(ConstUnicode str1, // IN
+                ConstUnicode str2) // IN
 {
    return Unicode_CompareRange(str1,
                                0,
@@ -634,7 +641,7 @@ Unicode_IsEqual(const char *str1, // IN
  *      removed.
  *
  * Results:
- *      The newly-allocated string.  Caller must free with free.
+ *      The newly-allocated string.  Caller must free with Unicode_Free.
  *
  * Side effects:
  *      None
@@ -642,12 +649,12 @@ Unicode_IsEqual(const char *str1, // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
-Unicode_RemoveRange(const char *destination,
+static INLINE Unicode
+Unicode_RemoveRange(ConstUnicode destination,
                     UnicodeIndex start,
                     UnicodeIndex length)
 {
-   return Unicode_ReplaceRange(destination, start, length, "", 0, 0);
+   return Unicode_ReplaceRange(destination, start, length, U(""), 0, 0);
 }
 
 
@@ -662,7 +669,7 @@ Unicode_RemoveRange(const char *destination,
  *      with 'source'.
  *
  * Results:
- *      The newly-allocated string.  Caller must free with free.
+ *      The newly-allocated string.  Caller must free with Unicode_Free.
  *
  * Side effects:
  *      None
@@ -670,11 +677,11 @@ Unicode_RemoveRange(const char *destination,
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
-Unicode_Replace(const char *destination,
+static INLINE Unicode
+Unicode_Replace(ConstUnicode destination,
                 UnicodeIndex destinationStart,
                 UnicodeIndex destinationLength,
-                const char *source)
+                ConstUnicode source)
 {
    return Unicode_ReplaceRange(destination,
                                destinationStart,
@@ -704,16 +711,19 @@ Unicode_Replace(const char *destination,
  */
 
 static INLINE Bool
-UnicodeStartsWith(const char *str,     // IN:
-                  const char *prefix,  // IN:
-                  Bool ignoreCase)     // IN:
+UnicodeStartsWith(ConstUnicode str,    // IN
+                  ConstUnicode prefix, // IN
+                  Bool ignoreCase)     // IN
 {
-   UnicodeIndex strLength = Unicode_LengthInCodePoints(str);
-   UnicodeIndex prefixLength = Unicode_LengthInCodePoints(prefix);
+   UnicodeIndex strLength = Unicode_LengthInCodeUnits(str);
+   UnicodeIndex prefixLength = Unicode_LengthInCodeUnits(prefix);
 
    if (prefixLength > strLength) {
       return FALSE;
    }
+   if (!Unicode_IsIndexAtCodePointBoundary(str, prefixLength)) {
+      return FALSE;
+   }
 
    return Unicode_CompareRange(str,
                                0,
@@ -726,16 +736,16 @@ UnicodeStartsWith(const char *str,     // IN:
 
 
 static INLINE Bool
-Unicode_StartsWith(const char *str,    // IN
-                   const char *prefix) // IN
+Unicode_StartsWith(ConstUnicode str,    // IN
+                   ConstUnicode prefix) // IN
 {
    return UnicodeStartsWith(str, prefix, FALSE);
 }
 
 
 static INLINE Bool
-Unicode_StartsWithIgnoreCase(const char *str,    // IN
-                             const char *prefix) // IN
+Unicode_StartsWithIgnoreCase(ConstUnicode str,    // IN
+                             ConstUnicode prefix) // IN
 {
    return UnicodeStartsWith(str, prefix, TRUE);
 }
@@ -751,7 +761,7 @@ Unicode_StartsWithIgnoreCase(const char *str,    // IN
  *
  * Results:
  *      The newly-allocated truncated copy of 'str'.  Caller must free
- *      with free.
+ *      with Unicode_Free.
  *
  * Side effects:
  *      None
@@ -759,8 +769,8 @@ Unicode_StartsWithIgnoreCase(const char *str,    // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE char *
-Unicode_Truncate(const char *str,     // IN
+static INLINE Unicode
+Unicode_Truncate(ConstUnicode str,    // IN
                  UnicodeIndex length) // IN
 {
    return Unicode_Substr(str, 0, length);
