@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -32,6 +32,7 @@
 #define INCLUDE_ALLOW_USERLEVEL
 #define INCLUDE_ALLOW_VMCORE
 #define INCLUDE_ALLOW_VMKERNEL
+#define INCLUDE_ALLOW_VMNIXMOD
 #include "includeCheck.h"
 
 #include "unicodeTypes.h"
@@ -44,14 +45,14 @@ extern "C" {
  * Standardizes the case of the string by doing a locale-agnostic
  * uppercase operation, then a locale-agnostic lowercase operation.
  */
-char *Unicode_FoldCase(const char *str);
+Unicode Unicode_FoldCase(ConstUnicode str);
 
 /*
  * Trims whitespace from either side of the string.
  */
-char *Unicode_Trim(const char *str);
-char *Unicode_TrimLeft(const char *str);
-char *Unicode_TrimRight(const char *str);
+Unicode Unicode_Trim(ConstUnicode str);
+Unicode Unicode_TrimLeft(ConstUnicode str);
+Unicode Unicode_TrimRight(ConstUnicode str);
 
 #ifdef __cplusplus
 }
