@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2004-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2004-2008 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -60,15 +60,8 @@ enum {
    VIX_HANDLETYPE_METADATA_CONTAINER   = 11
 };
 
-/*
- * The "//{{ Begin VIX_ERROR  }}" and "//{{ End VIX_ERROR }}" lines are
- * to bracket the error code definitions that will be copied over
- * to vixDiskLib.h during build time. If you modify these two lines, please
- * make sure you also change bora/lib/distribute/vixDiskLib.h and
- * bora/support/scripts/replaceVixErrors.py
- */
 
-// {{ Begin VIX_ERROR }}
+
 /*
  * An error is a 64-bit value. If there is no error, then the value is
  * set to VIX_OK. If there is an error, then the least significant bits
@@ -111,16 +104,7 @@ enum {
    VIX_E_BUFFER_TOOSMALL                        = 24,
    VIX_E_OBJECT_NOT_FOUND                       = 25,
    VIX_E_HOST_NOT_CONNECTED                     = 26,
-   VIX_E_INVALID_UTF8_STRING                    = 27,
-   VIX_E_OPERATION_ALREADY_IN_PROGRESS          = 31,
-   VIX_E_UNFINISHED_JOB                         = 29,
-   VIX_E_NEED_KEY                               = 30,
-   VIX_E_LICENSE                                = 32,
-   VIX_E_VM_HOST_DISCONNECTED                   = 34,
-   VIX_E_AUTHENTICATION_FAIL                    = 35,
-   VIX_E_HOST_CONNECTION_LOST                   = 36,
-   VIX_E_DUPLICATE_NAME                         = 41,
-   VIX_E_ARGUMENT_TOO_BIG                       = 44,
+
 
    /* Handle Errors */
    VIX_E_INVALID_HANDLE                         = 1000,
@@ -158,37 +142,35 @@ enum {
    VIX_E_CONSOLE_GUEST_OPERATIONS_PROHIBITED    = 3026,
    VIX_E_MUST_BE_CONSOLE_USER                   = 3027,
    VIX_E_VMX_MSG_DIALOG_AND_NO_UI               = 3028,
-   /* VIX_E_NOT_ALLOWED_DURING_VM_RECORDING        = 3029, Removed in version 1.11 */
-   /* VIX_E_NOT_ALLOWED_DURING_VM_REPLAY           = 3030, Removed in version 1.11 */
+   VIX_E_NOT_ALLOWED_DURING_VM_RECORDING        = 3029,
+   VIX_E_NOT_ALLOWED_DURING_VM_REPLAY           = 3030,
    VIX_E_OPERATION_NOT_ALLOWED_FOR_LOGIN_TYPE   = 3031,
    VIX_E_LOGIN_TYPE_NOT_SUPPORTED               = 3032,
    VIX_E_EMPTY_PASSWORD_NOT_ALLOWED_IN_GUEST    = 3033,
    VIX_E_INTERACTIVE_SESSION_NOT_PRESENT        = 3034,
-   VIX_E_INTERACTIVE_SESSION_USER_MISMATCH      = 3035,
-   /* VIX_E_UNABLE_TO_REPLAY_VM                    = 3039, Removed in version 1.11 */
-   VIX_E_CANNOT_POWER_ON_VM                     = 3041,
-   VIX_E_NO_DISPLAY_SERVER                      = 3043,
-   /* VIX_E_VM_NOT_RECORDING                       = 3044, Removed in version 1.11 */
-   /* VIX_E_VM_NOT_REPLAYING                       = 3045, Removed in version 1.11 */
-   VIX_E_TOO_MANY_LOGONS                        = 3046,
-   VIX_E_INVALID_AUTHENTICATION_SESSION         = 3047,
+   VIX_E_INTERACTIVE_SESSION_USER_MISMATCH      = 3035,          
+   VIX_E_UNABLE_TO_REPLAY_VM                    = 3039,
 
-   /* VM Errors */
+
+   /* VM Errors */ 
    VIX_E_VM_NOT_FOUND                           = 4000,
    VIX_E_NOT_SUPPORTED_FOR_VM_VERSION           = 4001,
    VIX_E_CANNOT_READ_VM_CONFIG                  = 4002,
    VIX_E_TEMPLATE_VM                            = 4003,
    VIX_E_VM_ALREADY_LOADED                      = 4004,
    VIX_E_VM_ALREADY_UP_TO_DATE                  = 4006,
-   VIX_E_VM_UNSUPPORTED_GUEST                   = 4011,
 
+   /* Wrapper Errors */
+   VIX_E_WRAPPER_WORKSTATION_NOT_INSTALLED      = 5001,
+   VIX_E_WRAPPER_VERSION_NOT_FOUND              = 5002,
+   VIX_E_WRAPPER_SERVICEPROVIDER_NOT_FOUND      = 5003,
+ 
    /* Property Errors */
    VIX_E_UNRECOGNIZED_PROPERTY                  = 6000,
    VIX_E_INVALID_PROPERTY_VALUE                 = 6001,
    VIX_E_READ_ONLY_PROPERTY                     = 6002,
    VIX_E_MISSING_REQUIRED_PROPERTY              = 6003,
    VIX_E_INVALID_SERIALIZED_DATA                = 6004,
-   VIX_E_PROPERTY_TYPE_MISMATCH                 = 6005,
 
    /* Completion Errors */
    VIX_E_BAD_VM_INDEX                           = 8000,
@@ -217,11 +199,6 @@ enum {
    VIX_E_SNAPSHOT_INDEPENDENTDISK               = 13016,
    VIX_E_SNAPSHOT_NONUNIQUE_NAME                = 13017,
    VIX_E_SNAPSHOT_MEMORY_ON_INDEPENDENT_DISK    = 13018,
-   VIX_E_SNAPSHOT_MAXSNAPSHOTS                  = 13019,
-   VIX_E_SNAPSHOT_MIN_FREE_SPACE                = 13020,
-   VIX_E_SNAPSHOT_HIERARCHY_TOODEEP             = 13021,
-   // DEPRECRATED VIX_E_SNAPSHOT_RRSUSPEND                     = 13022,
-   VIX_E_SNAPSHOT_NOT_REVERTABLE                = 13024,
 
    /* Host Errors */
    VIX_E_HOST_DISK_INVALID_VALUE                = 14003,
@@ -269,11 +246,6 @@ enum {
    VIX_E_DISK_CANTREPAIR                        = 16062,
    VIX_E_DISK_INVALIDDISK                       = 16063,
    VIX_E_DISK_NOLICENSE                         = 16064,
-   VIX_E_DISK_NODEVICE                          = 16065,
-   VIX_E_DISK_UNSUPPORTEDDEVICE                 = 16066,
-   VIX_E_DISK_CAPACITY_MISMATCH                 = 16067,
-   VIX_E_DISK_PARENT_NOTALLOWED                 = 16068,
-   VIX_E_DISK_ATTACH_ROOTLINK                   = 16069,
 
    /* Crypto Library Errors */
    VIX_E_CRYPTO_UNKNOWN_ALGORITHM               = 17000,
@@ -281,7 +253,7 @@ enum {
    VIX_E_CRYPTO_INVALID_OPERATION               = 17002,
    VIX_E_CRYPTO_RANDOM_DEVICE                   = 17003,
    VIX_E_CRYPTO_NEED_PASSWORD                   = 17004,
-   VIX_E_CRYPTO_BAD_PASSWORD                    = 17005,
+   VIX_E_CRYPTO_BAS_PASSWORD                    = 17005,
    VIX_E_CRYPTO_NOT_IN_DICTIONARY               = 17006,
    VIX_E_CRYPTO_NO_CRYPTO                       = 17007,
    VIX_E_CRYPTO_ERROR                           = 17008,
@@ -307,105 +279,8 @@ enum {
    VIX_E_NOT_A_DIRECTORY                        = 20002,
    VIX_E_NO_SUCH_PROCESS                        = 20003,
    VIX_E_FILE_NAME_TOO_LONG                     = 20004,
-   VIX_E_OPERATION_DISABLED                     = 20005,
-
-   /* Tools install errors */
-   VIX_E_TOOLS_INSTALL_NO_IMAGE                 = 21000,
-   VIX_E_TOOLS_INSTALL_IMAGE_INACCESIBLE        = 21001,
-   VIX_E_TOOLS_INSTALL_NO_DEVICE                = 21002,
-   VIX_E_TOOLS_INSTALL_DEVICE_NOT_CONNECTED     = 21003,
-   VIX_E_TOOLS_INSTALL_CANCELLED                = 21004,
-   VIX_E_TOOLS_INSTALL_INIT_FAILED              = 21005,
-   VIX_E_TOOLS_INSTALL_AUTO_NOT_SUPPORTED       = 21006,
-   VIX_E_TOOLS_INSTALL_GUEST_NOT_READY          = 21007,
-   /* VIX_E_TOOLS_INSTALL_SIG_CHECK_FAILED         = 21008, Removed in version 1.15.0 */
-   VIX_E_TOOLS_INSTALL_ERROR                    = 21009,
-   VIX_E_TOOLS_INSTALL_ALREADY_UP_TO_DATE       = 21010,
-   VIX_E_TOOLS_INSTALL_IN_PROGRESS              = 21011,
-   VIX_E_TOOLS_INSTALL_IMAGE_COPY_FAILED        = 21012,
-
-   /* Wrapper Errors */
-   VIX_E_WRAPPER_WORKSTATION_NOT_INSTALLED      = 22001,
-   VIX_E_WRAPPER_VERSION_NOT_FOUND              = 22002,
-   VIX_E_WRAPPER_SERVICEPROVIDER_NOT_FOUND      = 22003,
-   VIX_E_WRAPPER_PLAYER_NOT_INSTALLED           = 22004,
-   VIX_E_WRAPPER_RUNTIME_NOT_INSTALLED          = 22005,
-   VIX_E_WRAPPER_MULTIPLE_SERVICEPROVIDERS      = 22006,
-
-   /* FuseMnt errors*/
-   VIX_E_MNTAPI_MOUNTPT_NOT_FOUND               = 24000,
-   VIX_E_MNTAPI_MOUNTPT_IN_USE                  = 24001,
-   VIX_E_MNTAPI_DISK_NOT_FOUND                  = 24002,
-   VIX_E_MNTAPI_DISK_NOT_MOUNTED                = 24003,
-   VIX_E_MNTAPI_DISK_IS_MOUNTED                 = 24004,
-   VIX_E_MNTAPI_DISK_NOT_SAFE                   = 24005,
-   VIX_E_MNTAPI_DISK_CANT_OPEN                  = 24006,
-   VIX_E_MNTAPI_CANT_READ_PARTS                 = 24007,
-   VIX_E_MNTAPI_UMOUNT_APP_NOT_FOUND            = 24008,
-   VIX_E_MNTAPI_UMOUNT                          = 24009,
-   VIX_E_MNTAPI_NO_MOUNTABLE_PARTITONS          = 24010,
-   VIX_E_MNTAPI_PARTITION_RANGE                 = 24011,
-   VIX_E_MNTAPI_PERM                            = 24012,
-   VIX_E_MNTAPI_DICT                            = 24013,
-   VIX_E_MNTAPI_DICT_LOCKED                     = 24014,
-   VIX_E_MNTAPI_OPEN_HANDLES                    = 24015,
-   VIX_E_MNTAPI_CANT_MAKE_VAR_DIR               = 24016,
-   VIX_E_MNTAPI_NO_ROOT                         = 24017,
-   VIX_E_MNTAPI_LOOP_FAILED                     = 24018,
-   VIX_E_MNTAPI_DAEMON                          = 24019,
-   VIX_E_MNTAPI_INTERNAL                        = 24020,
-   VIX_E_MNTAPI_SYSTEM                          = 24021,
-   VIX_E_MNTAPI_NO_CONNECTION_DETAILS           = 24022,
-   /* FuseMnt errors: Do not exceed 24299 */
-
-   /* VixMntapi errors*/
-   VIX_E_MNTAPI_INCOMPATIBLE_VERSION            = 24300,
-   VIX_E_MNTAPI_OS_ERROR                        = 24301,
-   VIX_E_MNTAPI_DRIVE_LETTER_IN_USE             = 24302,
-   VIX_E_MNTAPI_DRIVE_LETTER_ALREADY_ASSIGNED   = 24303,
-   VIX_E_MNTAPI_VOLUME_NOT_MOUNTED              = 24304,
-   VIX_E_MNTAPI_VOLUME_ALREADY_MOUNTED          = 24305,
-   VIX_E_MNTAPI_FORMAT_FAILURE                  = 24306,
-   VIX_E_MNTAPI_NO_DRIVER                       = 24307,
-   VIX_E_MNTAPI_ALREADY_OPENED                  = 24308,
-   VIX_E_MNTAPI_ITEM_NOT_FOUND                  = 24309,
-   VIX_E_MNTAPI_UNSUPPROTED_BOOT_LOADER         = 24310,
-   VIX_E_MNTAPI_UNSUPPROTED_OS                  = 24311,
-   VIX_E_MNTAPI_CODECONVERSION                  = 24312,
-   VIX_E_MNTAPI_REGWRITE_ERROR                  = 24313,
-   VIX_E_MNTAPI_UNSUPPORTED_FT_VOLUME           = 24314,
-   VIX_E_MNTAPI_PARTITION_NOT_FOUND             = 24315,
-   VIX_E_MNTAPI_PUTFILE_ERROR                   = 24316,
-   VIX_E_MNTAPI_GETFILE_ERROR                   = 24317,
-   VIX_E_MNTAPI_REG_NOT_OPENED                  = 24318,
-   VIX_E_MNTAPI_REGDELKEY_ERROR                 = 24319,
-   VIX_E_MNTAPI_CREATE_PARTITIONTABLE_ERROR     = 24320,
-   VIX_E_MNTAPI_OPEN_FAILURE                    = 24321,
-   VIX_E_MNTAPI_VOLUME_NOT_WRITABLE             = 24322,
-
-   /* Success on operation that completes asynchronously */
-   VIX_ASYNC                                    = 25000,
-
-   /* Async errors */
-   VIX_E_ASYNC_MIXEDMODE_UNSUPPORTED            = 26000,
-
-   /* Network Errors */
-   VIX_E_NET_HTTP_UNSUPPORTED_PROTOCOL     = 30001,
-   VIX_E_NET_HTTP_URL_MALFORMAT            = 30003,
-   VIX_E_NET_HTTP_COULDNT_RESOLVE_PROXY    = 30005,
-   VIX_E_NET_HTTP_COULDNT_RESOLVE_HOST     = 30006,
-   VIX_E_NET_HTTP_COULDNT_CONNECT          = 30007,
-   VIX_E_NET_HTTP_HTTP_RETURNED_ERROR      = 30022,
-   VIX_E_NET_HTTP_OPERATION_TIMEDOUT       = 30028,
-   VIX_E_NET_HTTP_SSL_CONNECT_ERROR        = 30035,
-   VIX_E_NET_HTTP_TOO_MANY_REDIRECTS       = 30047,
-   VIX_E_NET_HTTP_TRANSFER                 = 30200,
-   VIX_E_NET_HTTP_SSL_SECURITY             = 30201,
-   VIX_E_NET_HTTP_GENERIC                  = 30202,
 };
 
-// {{ End VIX_ERROR }}
-
 const char *Vix_GetErrorText(VixError err, const char *locale);
 
 
@@ -447,7 +322,6 @@ enum {
    /* VIX_HANDLETYPE_HOST properties */
    VIX_PROPERTY_HOST_HOSTTYPE                         = 50,
    VIX_PROPERTY_HOST_API_VERSION                      = 51,
-   VIX_PROPERTY_HOST_SOFTWARE_VERSION                 = 52,
 
    /* VIX_HANDLETYPE_VM properties */
    VIX_PROPERTY_VM_NUM_VCPUS                          = 101,
@@ -455,16 +329,13 @@ enum {
    VIX_PROPERTY_VM_VMTEAM_PATHNAME                    = 105, 
    VIX_PROPERTY_VM_MEMORY_SIZE                        = 106,
    VIX_PROPERTY_VM_READ_ONLY                          = 107,
-   VIX_PROPERTY_VM_NAME                               = 108,
-   VIX_PROPERTY_VM_GUESTOS                            = 109,
    VIX_PROPERTY_VM_IN_VMTEAM                          = 128,
    VIX_PROPERTY_VM_POWER_STATE                        = 129,
    VIX_PROPERTY_VM_TOOLS_STATE                        = 152,
    VIX_PROPERTY_VM_IS_RUNNING                         = 196,
    VIX_PROPERTY_VM_SUPPORTED_FEATURES                 = 197,
-   /* VIX_PROPERTY_VM_IS_RECORDING                       = 236, Removed in version 1.11 */
-   /* VIX_PROPERTY_VM_IS_REPLAYING                       = 237, Removed in version 1.11 */
-   VIX_PROPERTY_VM_SSL_ERROR                          = 293,
+   VIX_PROPERTY_VM_IS_RECORDING                       = 236,
+   VIX_PROPERTY_VM_IS_REPLAYING                       = 237,
 
    /* Result properties; these are returned by various procedures */
    VIX_PROPERTY_JOB_RESULT_ERROR_CODE                 = 3000,
@@ -492,7 +363,6 @@ enum {
    VIX_PROPERTY_JOB_RESULT_SCREEN_IMAGE_DATA          = 3059,
    VIX_PROPERTY_JOB_RESULT_FILE_SIZE                  = 3061,
    VIX_PROPERTY_JOB_RESULT_FILE_MOD_TIME              = 3062,
-   VIX_PROPERTY_JOB_RESULT_EXTRA_ERROR_INFO           = 3084,
 
    /* Event properties; these are sent in the moreEventInfo for some events. */
    VIX_PROPERTY_FOUND_ITEM_LOCATION                   = 4010,
@@ -501,12 +371,8 @@ enum {
    VIX_PROPERTY_SNAPSHOT_DISPLAYNAME                  = 4200,   
    VIX_PROPERTY_SNAPSHOT_DESCRIPTION                  = 4201,
    VIX_PROPERTY_SNAPSHOT_POWERSTATE                   = 4205,
-   /* VIX_PROPERTY_SNAPSHOT_IS_REPLAYABLE                = 4207, Removed in version 1.11 */
+   VIX_PROPERTY_SNAPSHOT_IS_REPLAYABLE                = 4207,
 
-   VIX_PROPERTY_GUEST_SHAREDFOLDERS_SHARES_PATH       = 4525,
-
-   /* Virtual machine encryption properties */
-   VIX_PROPERTY_VM_ENCRYPTION_PASSWORD                = 7001,
 };
 
 /*
@@ -548,7 +414,9 @@ typedef void VixEventProc(VixHandle handle,
 
 void Vix_ReleaseHandle(VixHandle handle);
 
+#ifndef VIX_HIDE_FROM_JAVA
 void Vix_AddRefHandle(VixHandle handle);
+#endif
 
 VixHandleType Vix_GetHandleType(VixHandle handle);
 
@@ -571,21 +439,15 @@ void Vix_FreeBuffer(void *p);
 
 typedef int VixHostOptions;
 enum {
-   /*
-    * The following option was removed in version 1.11.
-      VIX_HOSTOPTION_USE_EVENT_PUMP        = 0x0008,
-    */
-   VIX_HOSTOPTION_VERIFY_SSL_CERT       = 0x4000,
+   VIX_HOSTOPTION_USE_EVENT_PUMP        = 0x0008,
 };
 
 typedef int VixServiceProvider;
 enum {
-   VIX_SERVICEPROVIDER_DEFAULT                   = 1,
-   VIX_SERVICEPROVIDER_VMWARE_SERVER             = 2,
-   VIX_SERVICEPROVIDER_VMWARE_WORKSTATION        = 3,
-   VIX_SERVICEPROVIDER_VMWARE_PLAYER             = 4,
-   VIX_SERVICEPROVIDER_VMWARE_VI_SERVER          = 10,
-   VIX_SERVICEPROVIDER_VMWARE_WORKSTATION_SHARED = 11,
+   VIX_SERVICEPROVIDER_DEFAULT               = 1,
+   VIX_SERVICEPROVIDER_VMWARE_SERVER         = 2,
+   VIX_SERVICEPROVIDER_VMWARE_WORKSTATION    = 3,
+   VIX_SERVICEPROVIDER_VMWARE_VI_SERVER      = 10,
 };
 
 /*
@@ -630,8 +492,8 @@ VixHandle VixHost_UnregisterVM(VixHandle hostHandle,
 
 typedef int VixFindItemType;
 enum {
-   VIX_FIND_RUNNING_VMS         = 1,
-   VIX_FIND_REGISTERED_VMS      = 4,
+    VIX_FIND_RUNNING_VMS         = 1,
+    VIX_FIND_REGISTERED_VMS      = 4,
 };
 
 VixHandle VixHost_FindItems(VixHandle hostHandle,
@@ -641,44 +503,18 @@ VixHandle VixHost_FindItems(VixHandle hostHandle,
                             VixEventProc *callbackProc,
                             void *clientData);
 
-
 /*
- * VixHost_OpenVM() supercedes VixVM_Open() since it allows for
- * the passing of option flags and extra data in the form of a
- * property list.
- * It is recommended to use VixHost_OpenVM() instead of VixVM_Open().
+ * Event pump
  */
 
-typedef int VixVMOpenOptions;
+typedef int VixPumpEventsOptions;
 enum {
-   VIX_VMOPEN_NORMAL  = 0x0,
+   VIX_PUMPEVENTOPTION_NONE = 0,
 };
 
-VixHandle VixHost_OpenVM(VixHandle hostHandle,
-                         const char *vmxFilePathName,
-                         VixVMOpenOptions options,
-                         VixHandle propertyListHandle,
-                         VixEventProc *callbackProc,
-                         void *clientData);
+void Vix_PumpEvents(VixHandle hostHandle, VixPumpEventsOptions options);
 
 
-/*
- * Following functions were removed in version 1.11.
- *
-   typedef int VixPumpEventsOptions;
-   enum {
-    VIX_PUMPEVENTOPTION_NONE = 0,
-   };
-   void Vix_PumpEvents(VixHandle hostHandle, VixPumpEventsOptions options);
-
-   VixHandle VixVM_OpenUrlInGuest(VixHandle vmHandle,
-                                  const char *url,
-                                  int windowState,
-                                  VixHandle propertyListHandle,
-                                  VixEventProc *callbackProc,
-                                  void *clientData);
- */
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -716,8 +552,7 @@ enum {
    VIX_VMPOWEROP_NORMAL                      = 0,
    VIX_VMPOWEROP_FROM_GUEST                  = 0x0004,
    VIX_VMPOWEROP_SUPPRESS_SNAPSHOT_POWERON   = 0x0080,
-   VIX_VMPOWEROP_LAUNCH_GUI                  = 0x0200,
-   VIX_VMPOWEROP_START_VM_PAUSED             = 0x1000,
+   VIX_VMPOWEROP_LAUNCH_GUI                  = 0x0200
 };
 
 /*
@@ -768,9 +603,7 @@ VixHandle VixVM_Delete(VixHandle vmHandle,
                        void *clientData);
 
 /*
- * This is the state of an individual VM.  These values are bitwise flags.
- * The actual value returned for may be a bitwise OR of one more of these
- * flags, along with other reserved values not documented here.
+ * This is the state of an individual VM.
  */
 
 typedef int VixPowerState;
@@ -807,6 +640,14 @@ enum {
       VIX_VM_SUPPORT_HARDWARE_UPGRADE     = 0x0008,
 };
 
+
+/*
+ * These are special names for an anonymous user and the system administrator.
+ * The password is ignored if you specify these.
+ */
+
+#define VIX_ANONYMOUS_USER_NAME        "__VMware_Vix_Guest_User_Anonymous__"
+
 /*
  * VIX_ADMINISTRATOR_USER_NAME and VIX_CONSOLE_USER_NAME are no longer
  * supported. If your code includes references to these constants please
@@ -817,6 +658,38 @@ enum {
 //#define VIX_ADMINISTRATOR_USER_NAME    "__VMware_Vix_Guest_User_Admin__"
 //#define VIX_CONSOLE_USER_NAME          "__VMware_Vix_Guest_Console_User__"
 
+
+/*
+ * Record/replay operations
+ */
+VixHandle VixVM_BeginRecording(VixHandle vmHandle,
+                               const char *displayName,
+                               const char *description,
+                               int options,
+                               VixHandle propertyList,
+                               VixEventProc *callbackProc,
+                               void *clientData);
+
+VixHandle VixVM_EndRecording(VixHandle vmHandle,
+                             int options,
+                             VixHandle propertyList,
+                             VixEventProc *callbackProc,
+                             void *clientData);
+
+VixHandle VixVM_BeginReplay(VixHandle vmHandle,
+                            VixHandle snapshotHandle,
+                            int options,
+                            VixHandle propertyList,
+                            VixEventProc *callbackProc,
+                            void *clientData);
+
+VixHandle VixVM_EndReplay(VixHandle vmHandle,
+                          int options,
+                          VixHandle propertyList,
+                          VixEventProc *callbackProc,
+                          void *clientData);
+
+
 /*
  * Guest operations
  */
@@ -882,6 +755,14 @@ VixHandle VixVM_RunScriptInGuest(VixHandle vmHandle,
                                  VixEventProc *callbackProc,
                                  void *clientData);
 
+VixHandle VixVM_OpenUrlInGuest(VixHandle vmHandle,
+                               const char *url,
+                               int windowState,
+                               VixHandle propertyListHandle,
+                               VixEventProc *callbackProc,
+                               void *clientData);
+
+
 /* 
  * Guest File functions 
  */
@@ -926,10 +807,12 @@ VixHandle VixVM_CreateTempFileInGuest(VixHandle vmHandle,
                                       VixEventProc *callbackProc,
                                       void *clientData);
 
+#ifndef VIX_HIDE_FROM_JAVA
 VixHandle VixVM_GetFileInfoInGuest(VixHandle vmHandle,
                                    const char *pathName,
                                    VixEventProc *callbackProc,
                                    void *clientData);
+#endif   // VIX_HIDE_FROM_JAVA
 
 
 /* 
@@ -1132,12 +1015,6 @@ VixHandle VixVM_UpgradeVirtualHardware(VixHandle vmHandle,
                                        VixEventProc *callbackProc,
                                        void *clientData);
 
-enum {
-   VIX_INSTALLTOOLS_MOUNT_TOOLS_INSTALLER = 0x00,
-   VIX_INSTALLTOOLS_AUTO_UPGRADE          = 0x01,
-   VIX_INSTALLTOOLS_RETURN_IMMEDIATELY    = 0x02
-};
-
 VixHandle VixVM_InstallTools(VixHandle vmHandle,
                              int options,
                              const char *commandLineArgs,
