@@ -16,48 +16,6 @@
  *
  *********************************************************/
 
-/*********************************************************
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of VMware Inc. nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission of VMware Inc.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *********************************************************/
-
-/*********************************************************
- * The contents of this file are subject to the terms of the Common
- * Development and Distribution License (the "License") version 1.0
- * and no later version.  You may not use this file except in
- * compliance with the License.
- *
- * You can obtain a copy of the License at
- *         http://www.opensource.org/licenses/cddl1.php
- *
- * See the License for the specific language governing permissions
- * and limitations under the License.
- *
- *********************************************************/
-
 /*
  * vm_assert.h --
  *
@@ -71,9 +29,10 @@
 #define _VM_ASSERT_H_
 
 #define INCLUDE_ALLOW_USERLEVEL
-
+#define INCLUDE_ALLOW_VMMEXT
 #define INCLUDE_ALLOW_MODULE
 #define INCLUDE_ALLOW_VMMON
+#define INCLUDE_ALLOW_VMNIXMOD
 #define INCLUDE_ALLOW_VMKERNEL
 #define INCLUDE_ALLOW_VMKDRIVERS
 #define INCLUDE_ALLOW_VMK_MODULE
@@ -86,131 +45,48 @@
 #include "vm_basic_types.h"
 #include "vm_basic_defs.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 /*
- * Some bits of vmcore are used in VMKernel code and cannot have
- * the VMKERNEL define due to other header dependencies.
- */
-#if defined(VMKERNEL) && !defined(VMKPANIC)
-#define VMKPANIC 1
-#endif
-
-/*
- * Internal macros, functions, and strings
- *
- * The monitor wants to save space at call sites, so it has specialized
- * functions for each situation.  User level wants to save on implementation
- * so it uses generic functions.
+ * XXX old file code
  */
 
-#if !defined VMM || defined MONITOR_APP // {
-
-#if defined VMKPANIC
-
-// vmkernel Panic() function does not want a trailing newline.
-#define _ASSERT_PANIC(name) \
-           Panic(_##name##Fmt, __FILE__, __LINE__)
-#define _ASSERT_PANIC_BUG(bug, name) \
-           Panic(_##name##Fmt " bugNr=%d", __FILE__, __LINE__, bug)
-#define _ASSERT_PANIC_NORETURN(name) \
-           Panic_NoReturn(_##name##Fmt, __FILE__, __LINE__)
-
-#else /* !VMKPANIC */
-#define _ASSERT_PANIC(name) \
-           Panic(_##name##Fmt "\n", __FILE__, __LINE__)
-#define _ASSERT_PANIC_BUG(bug, name) \
-           Panic(_##name##Fmt " bugNr=%d\n", __FILE__, __LINE__, bug)
-#endif /* VMKPANIC */
-
-#ifdef VMX86_DEVEL
-#   define _ASSERT_WARNING(name) \
-           Warning(_##name##Fmt "\n", __FILE__, __LINE__)
-#else
-#   define _ASSERT_WARNING(name) \
-           Log(_##name##Fmt "\n", __FILE__, __LINE__)
+#ifdef FILECODEINT
+#error "Don't define FILECODEINT.  It is obsolete."
+#endif
+#ifdef FILECODE
+#error "Don't define FILECODE.  It is obsolete."
 #endif
-
-#endif // }
-
-
-// these don't have newline so a bug can be tacked on
-#define _AssertPanicFmt            "PANIC %s:%d"
-#define _AssertAssertFmt           "ASSERT %s:%d"
-#define _AssertNotImplementedFmt   "NOT_IMPLEMENTED %s:%d"
-#define _AssertNotReachedFmt       "NOT_REACHED %s:%d"
-#define _AssertMemAllocFmt         "MEM_ALLOC %s:%d"
-#define _AssertNotTestedFmt        "NOT_TESTED %s:%d"
-
 
 
 /*
  * Panic and log functions
  */
 
-void Log(const char *fmt, ...) PRINTF_DECL(1, 2);
-void Warning(const char *fmt, ...) PRINTF_DECL(1, 2);
-#if defined VMKPANIC
-void Panic_SaveRegs(void);
+EXTERN void Log(const char *fmt, ...) PRINTF_DECL(1, 2);
+EXTERN void Warning(const char *fmt, ...) PRINTF_DECL(1, 2);
+EXTERN NORETURN void Panic(const char *fmt, ...) PRINTF_DECL(1, 2);
 
-#ifdef VMX86_DEBUG
-void Panic_NoSave(const char *fmt, ...) PRINTF_DECL(1, 2);
-#else
-NORETURN void Panic_NoSave(const char *fmt, ...) PRINTF_DECL(1, 2);
-#endif
-
-NORETURN void Panic_NoSaveNoReturn(const char *fmt, ...)
-   PRINTF_DECL(1, 2);
-
-#define Panic(fmt...) do { \
-   Panic_SaveRegs(); \
-   Panic_NoSave(fmt); \
-} while(0)
-
-#define Panic_NoReturn(fmt...) do { \
-   Panic_SaveRegs(); \
-   Panic_NoSaveNoReturn(fmt); \
-} while(0)
-
-
-#else
-NORETURN void Panic(const char *fmt, ...) PRINTF_DECL(1, 2);
-#endif
-
-void LogThrottled(uint32 *count, const char *fmt, ...)
+EXTERN void LogThrottled(uint32 *count, const char *fmt, ...)
             PRINTF_DECL(2, 3);
-void WarningThrottled(uint32 *count, const char *fmt, ...)
+EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
             PRINTF_DECL(2, 3);
 
+/* DB family:  messages which are parsed by logfile database system */
+#define WarningDB Warning
+#define LogDB Log
+#define WarningThrottledDB WarningThrottled
+#define LogThrottledDB LogThrottled
+
 
 /*
  * Stress testing: redefine ASSERT_IFNOT() to taste
  */
 
 #ifndef ASSERT_IFNOT
-   /*
-    * PR 271512: When compiling with gcc, catch assignments inside an ASSERT.
-    *
-    * 'UNLIKELY' is defined with __builtin_expect, which does not warn when
-    * passed an assignment (gcc bug 36050). To get around this, we put 'cond'
-    * in an 'if' statement and make sure it never gets executed by putting
-    * that inside of 'if (0)'. We use gcc's statement expression syntax to
-    * make ASSERT an expression because some code uses it that way.
-    *
-    * Since statement expression syntax is a gcc extension and since it's
-    * not clear if this is a problem with other compilers, the ASSERT
-    * definition was not changed for them. Using a bare 'cond' with the
-    * ternary operator may provide a solution.
-    */
-
-   #ifdef __GNUC__
-      #define ASSERT_IFNOT(cond, panic)                                              \
-                 ({if (UNLIKELY(!(cond))) { panic; if (0) { if (cond) { ; } } } (void)0;})
+   #ifdef __cplusplus
+      #define ASSERT_IFNOT(cond, panic) (UNLIKELY(!(cond)) ? (panic) : (void)0)
    #else
-      #define ASSERT_IFNOT(cond, panic)                               \
-                 (UNLIKELY(!(cond)) ? (panic) : (void)0)
+      #define ASSERT_IFNOT(cond, panic) (UNLIKELY(!(cond)) ? (panic) : 0)
    #endif
 #endif
 
@@ -234,32 +110,28 @@ void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define PANIC()        _ASSERT_PANIC(AssertPanic)
 #define PANIC_BUG(bug) _ASSERT_PANIC_BUG(bug, AssertPanic)
 
-#ifdef VMKPANIC
-#define ASSERT_OR_IN_PANIC(cond) ASSERT((cond) || Panic_IsSystemInPanic())
-#endif
-
 #define ASSERT_NOT_IMPLEMENTED(cond) \
            ASSERT_IFNOT(cond, NOT_IMPLEMENTED())
 #define ASSERT_NOT_IMPLEMENTED_BUG(bug, cond) \
            ASSERT_IFNOT(cond, NOT_IMPLEMENTED_BUG(bug))
 
-#if defined VMKPANIC && defined VMX86_DEBUG
-#define NOT_IMPLEMENTED()        _ASSERT_PANIC_NORETURN(AssertNotImplemented)
-#else
 #define NOT_IMPLEMENTED()        _ASSERT_PANIC(AssertNotImplemented)
-#endif
 #define NOT_IMPLEMENTED_BUG(bug) _ASSERT_PANIC_BUG(bug, AssertNotImplemented)
 
-#if defined VMKPANIC && defined VMX86_DEBUG
-#define NOT_REACHED()            _ASSERT_PANIC_NORETURN(AssertNotReached)
-#else
 #define NOT_REACHED()            _ASSERT_PANIC(AssertNotReached)
-#endif
 #define NOT_REACHED_BUG(bug)     _ASSERT_PANIC_BUG(bug, AssertNotReached)
 
 #define ASSERT_MEM_ALLOC(cond) \
            ASSERT_IFNOT(cond, _ASSERT_PANIC(AssertMemAlloc))
 
+#ifdef VMX86_DEVEL
+   #define ASSERT_LENGTH(real, expected) \
+              ASSERT_IFNOT((real) == (expected), \
+                 Panic(AssertLengthFmt, __FILE__, __LINE__, real, expected))
+#else
+   #define ASSERT_LENGTH(real, expected) ASSERT((real) == (expected))
+#endif
+
 #ifdef VMX86_DEVEL
    #define ASSERT_DEVEL(cond) ASSERT(cond)
 #else
@@ -269,9 +141,31 @@ void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define ASSERT_NO_INTERRUPTS()  ASSERT(!INTERRUPTS_ENABLED())
 #define ASSERT_HAS_INTERRUPTS() ASSERT(INTERRUPTS_ENABLED())
 
-#define NOT_TESTED() _ASSERT_WARNING(AssertNotTested)
-#define ASSERT_NOT_TESTED(cond) (UNLIKELY(!(cond)) ? NOT_TESTED() : (void)0)
-#define NOT_TESTED_ONCE() DO_ONCE(NOT_TESTED())
+#define ASSERT_LOG_UNEXPECTED(bug, cond) \
+           (UNLIKELY(!(cond)) ? LOG_UNEXPECTED(bug) : 0)
+#ifdef VMX86_DEVEL
+   #define LOG_UNEXPECTED(bug) \
+              Warning(AssertUnexpectedFmt, __FILE__, __LINE__, bug)
+#else
+   #define LOG_UNEXPECTED(bug) \
+              Log(AssertUnexpectedFmt, __FILE__, __LINE__, bug)
+#endif
+
+#define ASSERT_NOT_TESTED(cond) (UNLIKELY(!(cond)) ? NOT_TESTED() : 0)
+#ifdef VMX86_DEVEL
+   #define NOT_TESTED() Warning(AssertNotTestedFmt, __FILE__, __LINE__)
+#else
+   #define NOT_TESTED() Log(AssertNotTestedFmt, __FILE__, __LINE__)
+#endif
+
+#define NOT_TESTED_ONCE()                                               \
+   do {                                                                 \
+      static Bool alreadyPrinted = FALSE;                               \
+      if (UNLIKELY(!alreadyPrinted)) {                                  \
+	 alreadyPrinted = TRUE;                                         \
+	 NOT_TESTED();                                                  \
+      }                                                                 \
+   } while (0)
 
 #define NOT_TESTED_1024()                                               \
    do {                                                                 \
@@ -280,16 +174,14 @@ void WarningThrottled(uint32 *count, const char *fmt, ...)
       count = (count + 1) & 1023;                                       \
    } while (0)
 
-#define LOG_ONCE(_s) DO_ONCE(Log _s)
-
-#ifdef VMX86_DEVEL
-   #define DEPRECATED(_fix) DO_ONCE(                                        \
-                               Warning("%s:%d: %s is DEPRECATED; %s\n",     \
-                                          __FILE__, __LINE__, __FUNCTION__, \
-                                          _fix))
-#else
-   #define DEPRECATED(_fix) do {} while (0)
-#endif
+#define LOG_ONCE(_s)                                                    \
+   do {                                                                 \
+      static Bool logged = FALSE;                                       \
+      if (!logged) {                                                    \
+	 Log _s;                                                        \
+         logged = TRUE;                                                 \
+      }                                                                 \
+   } while (0)
 
 
 /*
@@ -349,26 +241,12 @@ void WarningThrottled(uint32 *count, const char *fmt, ...)
 
 
 /*
- * Compile-time assertions.
- *
- * ASSERT_ON_COMPILE does not use the common
- * switch (0) { case 0: case (e): ; } trick because some compilers (e.g. MSVC)
- * generate code for it.
- *
- * The implementation uses both enum and typedef because the typedef alone is
- * insufficient; gcc allows arrays to be declared with non-constant expressions
- * (even in typedefs, where it makes no sense).
- *
- * NOTE: if GCC ever changes so that it ignores unused types altogether, this
- * assert might not fire!  We explicitly mark it as unused because GCC 4.8+
- * uses -Wunused-local-typedefs as part of -Wall, which means the typedef will
- * generate a warning.
+ * Compile-time assertions
  */
 
 #define ASSERT_ON_COMPILE(e) \
    do { \
-      enum { AssertOnCompileMisused = ((e) ? 1 : -1) }; \
-      UNUSED_TYPE(typedef char AssertOnCompileFailed[AssertOnCompileMisused]); \
+      typedef char AssertOnCompileType[(e) ? 1 : -1]; \
    } while (0)
 
 
@@ -393,8 +271,38 @@ void WarningThrottled(uint32 *count, const char *fmt, ...)
       assertions \
    }
 
-#ifdef __cplusplus
-} /* extern "C" */
-#endif
+
+/*
+ * Internal macros, functions, and strings
+ *
+ * The monitor wants to save space at call sites, so it has specialized
+ * functions for each situation.  User level wants to save on implementation
+ * so it uses generic functions.
+ */
+
+#if !defined VMM || defined MONITOR_APP // {
+
+#define _ASSERT_PANIC(name) \
+           Panic(_##name##Fmt "\n", __FILE__, __LINE__)
+#define _ASSERT_PANIC_BUG(bug, name) \
+           Panic(_##name##Fmt " bugNr=%d\n", __FILE__, __LINE__, bug)
+
+#define AssertLengthFmt     _AssertLengthFmt
+#define AssertUnexpectedFmt _AssertUnexpectedFmt
+#define AssertNotTestedFmt  _AssertNotTestedFmt
+
+#endif // }
+
+// these don't have newline so a bug can be tacked on
+#define _AssertPanicFmt            "PANIC %s:%d"
+#define _AssertAssertFmt           "ASSERT %s:%d"
+#define _AssertNotImplementedFmt   "NOT_IMPLEMENTED %s:%d"
+#define _AssertNotReachedFmt       "NOT_REACHED %s:%d"
+#define _AssertMemAllocFmt         "MEM_ALLOC %s:%d"
+
+// these are complete formats with newline
+#define _AssertLengthFmt           "LENGTH %s:%d r=%#x e=%#x\n"
+#define _AssertUnexpectedFmt       "UNEXPECTED %s:%d bugNr=%d\n"
+#define _AssertNotTestedFmt        "NOT_TESTED %s:%d\n"
 
 #endif /* ifndef _VM_ASSERT_H_ */
