@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2004-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2004 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,17 +26,16 @@
 #ifndef _WIPER_H_
 # define _WIPER_H_
 
+
 #if defined(_WIN32) && defined(_MSC_VER)
 #include <windows.h>
 #endif
 
-#include "vm_basic_types.h"
-#include "dbllnklst.h"
 
-#define TOOLS_WIPE_CANCEL "Wipe cancelled by user.\n"
+#include "vm_basic_types.h"
+
 
 typedef enum {
-   PARTITION_UNSUPPORTED = 0,
    PARTITION_EXT2,
    PARTITION_EXT3,
    PARTITION_REISERFS,
@@ -44,11 +43,6 @@ typedef enum {
    PARTITION_FAT,
    PARTITION_UFS,
    PARTITION_PCFS,
-   PARTITION_EXT4,
-   PARTITION_HFS,
-   PARTITION_ZFS,
-   PARTITION_XFS,
-   PARTITION_BTRFS,
 } WiperPartition_Type;
 
 /* Max size of a path */
@@ -58,31 +52,28 @@ typedef enum {
 typedef struct WiperPartition {
    unsigned char mountPoint[NATIVE_MAX_PATH];
 
-   /* Type of the partition */
-   WiperPartition_Type type;
-   
    /* 
-    * Clients should specifically set this flag to TRUE to enable free space
-    * reclamation using unmaps.
+    * Empty if type is set, otherwise describes why the partition can not be
+    * wiped
     */
-   Bool attemptUnmaps;
+   char *comment;
 
-   /*
-    * NULL if type is not PARTITION_UNSUPPORTED, otherwise describes
-    * why the partition can not be wiped.
-    */
-   const char *comment;
+   /* Type of the partition we know how to wipe */
+   WiperPartition_Type type;
 
 #if defined(_WIN32)
    /* Private flags used by the Win32 implementation */
    DWORD flags;
 #endif
 
-   DblLnkLst_Links link;
+#if defined(N_PLAT_NLM)
+   unsigned long volumeNumber;
+#endif
 } WiperPartition;
 
 typedef struct WiperPartition_List {
-   DblLnkLst_Links link;
+   WiperPartition *partitions;
+   unsigned int size;
 } WiperPartition_List;
 
 typedef struct WiperInitData {
@@ -91,18 +82,16 @@ typedef struct WiperInitData {
 #endif
 } WiperInitData;
 
-Bool Wiper_Init(WiperInitData *clientData);
-Bool WiperPartition_Open(WiperPartition_List *pl);
+Bool Wiper_Init(void *clientData);
+WiperPartition_List *WiperPartition_Open(void);
 void WiperPartition_Close(WiperPartition_List *pl);
 
-WiperPartition *WiperSinglePartition_Allocate(void);
-WiperPartition *WiperSinglePartition_Open(const char *mntpt);
-void WiperSinglePartition_Close(WiperPartition *);
-Bool Wiper_IsWipeSupported(const WiperPartition *);
+WiperPartition *SingleWiperPartition_Open(const char *mntpt);
+void SingleWiperPartition_Close(WiperPartition *pl);
 
-unsigned char *WiperSinglePartition_GetSpace(const WiperPartition *p,
-                              uint64 *free,
-                              uint64 *total);
+unsigned char *WiperSinglePartition_GetSpace(const WiperPartition *p,  
+                              uint64 *free,       
+                              uint64 *total);      
 
 /* External definition of the wiper state */
 struct Wiper_State;
