@@ -54,61 +54,6 @@ static const char Base64[] =
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 static const char Pad64 = '=';
 
-// Special markers
-enum {
-   ILLEGAL = -1, EOM = -2, WS = -3
-};
-
-/*
- * Reverse byte map used for decoding. Except for specials (negative values), contains the index
- * into Base64[] where given value is found, ie: base64Reverse[Base64[n]] = n, for 0 <= n < 64
- *
- * This static initialization replaces, and should have identical result to, this runtime init:
- *
- *  for (i = 0; i < 256; ++i) {
- *     base64Reverse[i] = isspace(i) ? WS : ILLEGAL;
- *  }
- *  base64Reverse['\0']  = EOM;
- *  base64Reverse['=']   = EOM;
- *  for (i = 0; Base64[i]; ++i) {
- *     base64Reverse[(unsigned)Base64[i]] = (char) i;
- *  }
- */
-
-static const signed char base64Reverse[256] = {
-   EOM,     ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 00-07 */
-   ILLEGAL, WS,      WS,      WS,      WS,      WS,      ILLEGAL, ILLEGAL,   /* 08-0F */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 10-17 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 18-1F */
-   WS,      ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 20-27 */
-   ILLEGAL, ILLEGAL, ILLEGAL, 62,      ILLEGAL, ILLEGAL, ILLEGAL, 63,        /* 28-2F */
-   52,      53,      54,      55,      56,      57,      58,      59,        /* 30-37 */
-   60,      61,      ILLEGAL, ILLEGAL, ILLEGAL, EOM,     ILLEGAL, ILLEGAL,   /* 38-3F */
-   ILLEGAL, 0,       1,       2,       3,       4,       5,       6,         /* 40-47 */
-   7,       8,       9,       10,      11,      12,      13,      14,        /* 48-4F */
-   15,      16,      17,      18,      19,      20,      21,      22,        /* 50-57 */
-   23,      24,      25,      ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 58-5F */
-   ILLEGAL, 26,      27,      28,      29,      30,      31,      32,        /* 60-67 */
-   33,      34,      35,      36,      37,      38,      39,      40,        /* 68-6F */
-   41,      42,      43,      44,      45,      46,      47,      48,        /* 70-77 */
-   49,      50,      51,      ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 78-7F */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 80-87 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 88-8F */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 90-97 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* 98-9F */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* A0-A7 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* A8-AF */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* B0-B7 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* B8-BF */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* C0-C7 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* C8-CF */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* D0-D7 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* D8-DF */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* E0-E7 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* E8-EF */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL,   /* F0-F7 */
-   ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL, ILLEGAL }; /* F8-FF */
-
 /* (From RFC1521 and draft-ietf-dnssec-secext-03.txt)
    The following encoding technique is taken from RFC 1521 by Borenstein
    and Freed.  It is reproduced here in a slightly edited form for
@@ -131,16 +76,16 @@ static const signed char base64Reverse[256] = {
    Table 1: The Base64 Alphabet
 
    Value Encoding  Value Encoding  Value Encoding  Value Encoding
-   0 A             17 R            34 i            51 z
-   1 B             18 S            35 j            52 0
-   2 C             19 T            36 k            53 1
-   3 D             20 U            37 l            54 2
-   4 E             21 V            38 m            55 3
-   5 F             22 W            39 n            56 4
-   6 G             23 X            40 o            57 5
-   7 H             24 Y            41 p            58 6
-   8 I             25 Z            42 q            59 7
-   9 J             26 a            43 r            60 8
+   0 A            17 R            34 i            51 z
+   1 B            18 S            35 j            52 0
+   2 C            19 T            36 k            53 1
+   3 D            20 U            37 l            54 2
+   4 E            21 V            38 m            55 3
+   5 F            22 W            39 n            56 4
+   6 G            23 X            40 o            57 5
+   7 H            24 Y            41 p            58 6
+   8 I            25 Z            42 q            59 7
+   9 J            26 a            43 r            60 8
    10 K            27 b            44 s            61 9
    11 L            28 c            45 t            62 +
    12 M            29 d            46 u            63 /
@@ -191,24 +136,21 @@ static const signed char base64Reverse[256] = {
  */
 
 Bool
-Base64_Encode(uint8 const *src,  // IN:
-              size_t srcSize,    // IN:
-              char *dst,         // OUT:
-              size_t dstMax,     // IN: max result length, including NUL byte
-              size_t *dstSize)   // OUT/OPT: result length
+Base64_Encode(uint8 const *src,        // IN
+              size_t       srcSize,    // IN
+              char        *dst,        // OUT
+              size_t       dstMax,     // IN: max result length, including NUL byte
+              size_t      *dstSize)    // OUT: result length, may be NULL
 {
    char *dst0 = dst;
-   Bool retval = TRUE;
-
    ASSERT(src || srcSize == 0);
    ASSERT(dst);
 
-   /* Security: Carefully written to avoid fixed arithmetic attacks. */
-   if (   srcSize + 2 < srcSize
-       || dstMax < 1
-       || (srcSize + 2) / 3 > (dstMax - 1) / 4) {
-      retval = FALSE;
-      goto exit;
+   if (4 * ((srcSize+2) / 3) >= dstMax) {
+      if (dstSize) {
+         *dstSize = 0;
+      }
+      return FALSE;
    }
 
    while (LIKELY(srcSize > 2)) {
@@ -234,16 +176,12 @@ Base64_Encode(uint8 const *src,  // IN:
    }
 
    dst[0] = '\0';	/* Returned value doesn't count \0. */
-
-exit:
    if (dstSize) {
       *dstSize = dst - dst0;
    }
-
-   return retval;
+   return TRUE;
 }
 
-
 #ifdef __I_WANT_TO_TEST_THIS__
 main()
 {
@@ -269,7 +207,7 @@ main()
             bufMax = sizeof buf;
          }
 
-         printf("\nBuffer size %ld:\n", bufMax);
+         printf("\nBuffer size %d:\n", bufMax);
 
          test = tests;
          for (; test->in; ++test) {
@@ -277,31 +215,24 @@ main()
 
             r = Base64_Decode(test->in, buf, bufMax, &bufSize);
 
-            if ((bufMax > strlen(test->out)) && (bufSize < strlen(test->out))) {
-               printf("Decoding of %s failed. Decoded size %ld < expected %ld\n",
-                      test->in, bufSize, strlen(test->out));
-            }
-            if (memcmp(test->out, buf, bufSize) != 0) {
-               printf("Decoding of %s failed.  Got %s (%ld), not %s\n",
+            if (bufSize != strlen(test->out) ||
+                strncmp(test->out, buf, bufSize) != 0) {
+               printf("Decoding of %s failed.  Got %s (%d), not %s\n",
                       test->in, buf, bufSize, test->out);
             } else {
-               printf("Good: %s -> %s (%ld)\n", test->in, buf, bufSize);
+               printf("Good: %s -> %s (%d)\n", test->in, buf, bufSize);
             }
 
             r = Base64_Encode(test->out, strlen(test->out),
                               buf, bufMax, &bufSize);
             buf[bufMax] = 0;
 
-            if (bufMax <= strlen(test->in) && r == 0) {
-               printf("Good: %s. Failed for bufMax %ld (required %ld)\n", test->out, bufMax, strlen(test->in));
+            if (!r || bufSize != strlen(test->in) ||
+                strncmp(test->in, buf, bufSize) != 0) {
+               printf("Encoding of %s failed.  Got %s (%d), not %s\n",
+                      test->out, buf, bufSize, test->in);
             } else {
-                if (!r || bufSize != strlen(test->in) ||
-                    strncmp(test->in, buf, bufSize) != 0) {
-                   printf("Encoding of %s failed.  r = %d. Got %s (%ld), not %s\n",
-                          test->out, r, buf, bufSize, test->in);
-                } else {
-                   printf("Good: %s -> %s (%ld)\n", test->out, buf, bufSize);
-                }
+               printf("Good: %s -> %s (%d)\n", test->out, buf, bufSize);
             }
          }
       }
@@ -322,7 +253,6 @@ main()
 }
 #endif
 
-
 /*
  *----------------------------------------------------------------------------
  *
@@ -343,71 +273,54 @@ main()
  */
 
 Bool
-Base64_Decode(char const *in,      // IN:
-              uint8 *out,          // OUT:
-              size_t outSize,      // IN:
-              size_t *dataLength)  // OUT:
+Base64_Decode(char const *in,         // IN
+              uint8      *out,        // OUT
+              size_t      outSize,    // IN
+              size_t     *dataLength) // OUT
 {
-   return Base64_ChunkDecode(in, -1, out, outSize, dataLength);
-}
+   static char base64Reverse[256];
+   static Bool base64ReverseInit = 0;
 
-
-/*
- *----------------------------------------------------------------------------
- *
- * Base64_ChunkDecode --
- *
- *       Skips all whitespace anywhere. Converts characters, four at
- *       a time, starting at (or after) src from base - 64 numbers into three
- *       8 bit bytes in the target area. Conversion stops after inSize (which
- *       must be a multiple of 4) characters, or an EOM marker. Returns the
- *       number of data bytes stored at the target in the provided out parameter.
- *
- * Results:
- *      TRUE on success, FALSE on failure.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------------
- */
-
-Bool
-Base64_ChunkDecode(char const *in,      // IN:
-                   size_t inSize,       // IN:
-                   uint8 *out,          // OUT:
-                   size_t outSize,      // IN:
-                   size_t *dataLength)  // OUT:
-{
    uint32 b = 0;
    int n = 0;
    uintptr_t i = 0;
-   size_t inputIndex = 0;
+
+   // Special markers
+   enum {
+      ILLEGAL = -1, EOM = -2, WS = -3
+   };
 
    ASSERT(in);
    ASSERT(out || outSize == 0);
    ASSERT(dataLength);
-   ASSERT((inSize == -1) || (inSize % 4) == 0);
+
    *dataLength = 0;
 
+   if (!base64ReverseInit) {
+      // Default is illegal chars (or whitespace)
+      for (i = 0; i < 256; ++i) {
+         base64Reverse[i] = isspace(i) ? WS : ILLEGAL;
+      }
+      base64Reverse['\0']  = EOM;
+      base64Reverse['=']   = EOM;
+
+      for (i = 0; Base64[i]; ++i) {
+         base64Reverse[(unsigned)Base64[i]] = (char) i;
+      }
+      base64ReverseInit = 1;
+   }
+
    i = 0;
-   for (;inputIndex < inSize;) {
-      int p = base64Reverse[(unsigned char)in[inputIndex]];
+   for (;;) {
+      int p = base64Reverse[*(unsigned char *)in++];
 
       if (UNLIKELY(p < 0)) {
          switch (p) {
-         case WS:
-            inputIndex++;
-            break;
-         case EOM:
-            *dataLength = i;
-            return TRUE;
-         case ILLEGAL:
-         default:
-            return FALSE;
+         case ILLEGAL: return FALSE;
+         case WS: continue;
+         case EOM: *dataLength = i; return TRUE;
          }
       } else {
-         inputIndex++;
          if (UNLIKELY(i >= outSize)) {
             return FALSE;
          }
@@ -419,8 +332,6 @@ Base64_ChunkDecode(char const *in,      // IN:
          }
       }
    }
-   *dataLength = i;
-   return TRUE;
 }
 
 
@@ -441,11 +352,10 @@ Base64_ChunkDecode(char const *in,      // IN:
  */
 
 Bool
-Base64_ValidEncoding(char const *src,   // IN:
-                     size_t srcLength)  // IN:
+Base64_ValidEncoding(char const *src, // IN
+                     size_t srcLength) // IN
 {
    size_t i;
-
    ASSERT(src);
    for (i = 0; i < srcLength; i++) {
       uint8 c = src[i]; /* MSVC CRT will die on negative arguments to is* */
@@ -455,7 +365,6 @@ Base64_ValidEncoding(char const *src,   // IN:
          return FALSE;
       }
    }
-
    return TRUE;
 }
 
@@ -477,9 +386,7 @@ Base64_ValidEncoding(char const *src,   // IN:
  */
 
 size_t
-Base64_EncodedLength(uint8 const *src,  // IN:
-                     size_t srcLength)  // IN:
-{
+Base64_EncodedLength(uint8 const *src, size_t srcLength) {
    return ((srcLength + 2) / 3 * 4) + 1;
 }
 
@@ -504,23 +411,17 @@ Base64_EncodedLength(uint8 const *src,  // IN:
  */
 
 size_t
-Base64_DecodedLength(char const *src,   // IN:
-                     size_t srcLength)  // IN:
-{
+Base64_DecodedLength(char const *src, size_t srcLength) {
    size_t length;
 
    ASSERT(src);
 
    length = srcLength / 4 * 3;
-   // PR 303173 - do the following check to avoid a negative value returned
-   // from this function. Note: length can only be in a multiple of 3
-   if (length > 2) {
-      if (src[srcLength - 1] == '=') {
-         length--;
-      }
-      if (src[srcLength - 2] == '=') {
-         length--;
-      }
+   if (src[srcLength-1] == '=') {
+      length--;
+   }
+   if (src[srcLength-2] == '=') {
+      length--;
    }
    return length;
 }
@@ -557,7 +458,7 @@ Base64_EasyEncode(const uint8 *src,  // IN: data to encode
 
    size = Base64_EncodedLength(src, srcLength);
 
-   *target = malloc(size);
+   *target = (char *) malloc(size);
 
    if (!*target) {
       goto exit;
@@ -611,7 +512,7 @@ Base64_EasyDecode(const char *src,   // IN: data to decode
 
    theDataSize = Base64_DecodedLength(src, strlen(src));
 
-   theData = malloc(theDataSize);
+   theData = (uint8 *) malloc(theDataSize);
 
    if (!theData) {
       goto exit;
@@ -635,48 +536,3 @@ exit:
 
    return succeeded;
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Base64_DecodeFixed --
- *
- *      Base64-decode 'src' into a preallocated, fixed sized buffer.
- *
- * Results:
- *      TRUE  outBuf is populated with the decoded string
- *      FALSE Decoding or memory allocation failed.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-Base64_DecodeFixed(const char *src,    // IN: data to decode
-                   char *outBuf,       // OUT: decoded data
-                   size_t outBufSize)  // IN: data size
-{
-   Bool success;
-   uint8 *theData;
-   size_t theDataSize;
-
-   ASSERT(src);
-   ASSERT(outBuf);
-
-   success = Base64_EasyDecode(src, &theData, &theDataSize);
-
-   if (success) {
-      success = (theDataSize <= outBufSize);
-
-      if (success) {
-         memcpy(outBuf, theData, theDataSize);
-      }
-
-      free(theData);
-   }
-
-   return success;
-}
