@@ -1,5 +1,5 @@
 /* **********************************************************
- * Copyright (C) 1998-2015 VMware, Inc.  All rights reserved.
+ * Copyright 1998 VMware, Inc.  All rights reserved.
  * **********************************************************/
 
 /*
@@ -62,33 +62,18 @@
 #   include <errno.h>
 #   include <su.h>
 #   include <sys/stat.h>
-#   include "hostinfo.h"
-#   include "hostType.h"
-#endif
-
-#if defined(__APPLE__)
-#include <mach-o/dyld.h>
 #endif
 
 #include <stdio.h>
 #include "vmware.h"
 #include "vm_product.h"
-#include "vm_atomic.h"
-#if !defined(NO_ICU)
-#  include "unicode/ucnv.h"
-#  include "unicode/udata.h"
-#  include "unicode/putil.h"
-#endif
+#include "unicode/ucnv.h"
+#include "unicode/putil.h"
 #include "file.h"
 #include "util.h"
 #include "codeset.h"
 #include "codesetOld.h"
 #include "str.h"
-#include "win32util.h"
-#if defined __APPLE__
-#   define LOCATION_WEAK
-#   include "location.h"
-#endif
 
 /*
  * Macros
@@ -96,14 +81,10 @@
 
 #define CODESET_CAN_FALLBACK_ON_NON_ICU TRUE
 
-#if defined __APPLE__
-#   define POSIX_ICU_DIR DEFAULT_LIBDIRECTORY
-#elif !defined _WIN32
-#   if defined(VMX86_TOOLS)
-#      define POSIX_ICU_DIR "/etc/vmware-tools"
-#   else
-#      define POSIX_ICU_DIR "/etc/vmware"
-#   endif
+#if defined(__APPLE__)
+#define POSIX_ICU_DIR DEFAULT_LIBDIRECTORY "/icu"
+#elif !defined(WIN32)
+#define POSIX_ICU_DIR "/etc/vmware/icu"
 #endif
 
 /*
@@ -111,36 +92,34 @@
  * but I don't have time to deal with bora-vmsoft.  -- edward
  */
 
-#define ICU_DATA_FILE "icudt44l.dat"
+#define ICU_DATA_FILE "icudt38l.dat"
 #ifdef _WIN32
-#define ICU_DATA_ITEM "icudt44l"
-#define ICU_DATA_FILE_W XCONC(L, ICU_DATA_FILE)
+#define ICU_DATA_FILE_DIR "%TCROOT%/noarch/icu-data-3.8-1"
+#else
+#define ICU_DATA_FILE_DIR "/build/toolchain/noarch/icu-data-3.8-1"
 #endif
 
-#ifdef VMX86_DEVEL
 #ifdef _WIN32
-#define ICU_DATA_FILE_DIR "%TCROOT%/noarch/icu-data-4.4-1"
+#define ICU_DATA_FILE_W XCONC(L, ICU_DATA_FILE)
 #define ICU_DATA_FILE_DIR_W XCONC(L, ICU_DATA_FILE_DIR)
 #define ICU_DATA_FILE_PATH ICU_DATA_FILE_DIR_W DIRSEPS_W ICU_DATA_FILE_W
 #else
-#define ICU_DATA_FILE_DIR "/build/toolchain/noarch/icu-data-4.4-1"
 #define ICU_DATA_FILE_PATH ICU_DATA_FILE_DIR DIRSEPS ICU_DATA_FILE
 #endif
-#endif
+
 
 /*
  * Variables
  */
 
 static Bool dontUseIcu = TRUE;
+DEBUG_ONLY(static Bool initedIcu = FALSE;)
 
 
 /*
  * Functions
  */
 
-#if !defined NO_ICU
-
 #ifdef _WIN32
 
 /*
@@ -161,7 +140,7 @@ static Bool dontUseIcu = TRUE;
  *----------------------------------------------------------------------------
  */
 
-static utf16_t *
+utf16_t *
 CodeSetGetModulePath(HANDLE hModule) // IN
 {
    utf16_t *pathW = NULL;
@@ -170,7 +149,7 @@ CodeSetGetModulePath(HANDLE hModule) // IN
    while (TRUE) {
       DWORD res;
 
-      pathW = realloc(pathW, size * (sizeof *pathW));
+      pathW = realloc(pathW, size * sizeof(wchar_t));
       if (!pathW) {
          return NULL;
       }
@@ -181,13 +160,7 @@ CodeSetGetModulePath(HANDLE hModule) // IN
          /* fatal error */
          goto exit;
       } else if (res == size) {
-         /*
-          * The buffer is too small; double its size. The maximum path length
-          * on Windows is ~64KB so doubling will not push a DWORD into
-          * integer overflow before success or error due to another reason
-          * will occur.
-          */
-
+         /* buffer too small */
          size *= 2;
       } else {
          /* success */
@@ -199,143 +172,8 @@ CodeSetGetModulePath(HANDLE hModule) // IN
    return pathW;
 }
 
-
-#elif vmx86_devel && !defined(TEST_CUSTOM_ICU_DATA_FILE) // _WIN32
-
-/*
- *-----------------------------------------------------------------------------
- *
- * CodeSetGetModulePath --
- *
- *	Retrieve the full path to the executable. Not supported under
- *	VMvisor. Based on HostInfo_GetModulePath, simplified for safe
- *	use before ICU is initialized.
- *
- * Results:
- *      On success: The allocated, NUL-terminated file path.
- *         Note: This path can be a symbolic or hard link; it's just one
- *         possible path to access the executable.
- *         
- *      On failure: NULL.
- *
- * Side effects:
- *	None
- *
- *-----------------------------------------------------------------------------
- */
-
-static char *
-CodeSetGetModulePath(uint32 priv)
-{
-   char path[PATH_MAX];
-   Bool ret = FALSE;
-#if defined(__APPLE__)
-   uint32_t size;
-#else
-   ssize_t size;
-   uid_t uid = -1;
-#endif
-
-   if ((priv != HGMP_PRIVILEGE) && (priv != HGMP_NO_PRIVILEGE)) {
-      return NULL;
-   }
-
-#if defined(__APPLE__)
-   size = sizeof path;
-   if (_NSGetExecutablePath(path, &size)) {
-      goto exit;
-   }
-
-#else
-#if defined(VMX86_SERVER)
-   if (HostType_OSIsVMK()) {
-      goto exit;
-   }
-#endif
-
-   if (priv == HGMP_PRIVILEGE) {
-      uid = Id_BeginSuperUser();
-   }
-
-   size = readlink("/proc/self/exe", path, sizeof path - 1);
-   if (-1 == size) {
-      if (priv == HGMP_PRIVILEGE) {
-         Id_EndSuperUser(uid);
-      }
-      goto exit;
-   }
-
-   path[size] = '\0';
-
-   if (priv == HGMP_PRIVILEGE) {
-      Id_EndSuperUser(uid);
-   }
-#endif
-
-   ret = TRUE;
-
-  exit:
-   if (ret) {
-      return strdup(path);
-   } else {
-      return NULL;
-   }
-}
-
 #endif // _WIN32
 
-#endif // !NO_ICU
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * CodeSet_GetAltPathName --
- *
- *	The Unicode path is probably not compatible in the current encoding.
- *	Try to convert the path into a short name so it may work with
- *      local encoding.
- *
- *      XXX -- this function is a temporary fix. It should be removed once 
- *             we fix the 3rd party library pathname issue.
- *
- * Results:
- *      NULL, or a char string (free with free).
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-char *
-CodeSet_GetAltPathName(const utf16_t *pathW) // IN
-{
-   char *path = NULL;
-#if defined(_WIN32)
-   DWORD res;
-   utf16_t shortPathW[_MAX_PATH];
-
-   ASSERT(pathW);
-
-   res = GetShortPathNameW(pathW, shortPathW, ARRAYSIZE(shortPathW));
-
-   if ((res == 0) || (res >= ARRAYSIZE(shortPathW))) {
-      goto exit;
-   }
-
-   if (!CodeSetOld_Utf16leToCurrent((const char *)shortPathW,
-                                    wcslen(shortPathW) * sizeof *shortPathW,
-                                    &path, NULL)) {
-      goto exit;
-   }
-
-  exit:
-#endif // _WIN32
-
-   return path;
-}
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -375,8 +213,7 @@ CodeSet_DontUseIcu(void)
  *    directory. If already inited, returns the current state (init
  *    failed/succeeded).
  *
- *    For Windows, CodeSet_Init is thread-safe (with critical section).
- *    For Linux/Apple, call while single-threaded.
+ *    Call while single-threaded.
  *
  *    *********** WARNING ***********
  *    Do not call CodeSet_Init directly, it is called already by
@@ -395,22 +232,13 @@ CodeSet_DontUseIcu(void)
  */
 
 Bool
-CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current code page.
-                                     //     Default is used if NULL.
+CodeSet_Init(void)
 {
-#ifdef NO_ICU
-   /* Nothing required if not using ICU. */
-   return TRUE;
-#else // NO_ICU
    DynBuf dbpath;
 #ifdef _WIN32
    DWORD attribs;
    utf16_t *modPath = NULL;
    utf16_t *lastSlash;
-   utf16_t *wpath;
-   HANDLE hFile = INVALID_HANDLE_VALUE;
-   HANDLE hMapping = NULL;
-   void *memMappedData = NULL;
 #else
    struct stat finfo;
 #endif
@@ -419,6 +247,9 @@ CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current co
 
    DynBuf_Init(&dbpath);
 
+   DEBUG_ONLY(ASSERT(!initedIcu);)
+   DEBUG_ONLY(initedIcu = TRUE;)
+
 #ifdef USE_ICU
    /*
     * We're using system ICU, which finds its own data. So nothing to
@@ -440,10 +271,11 @@ CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current co
 
 #ifdef _WIN32 // {
 
-#if vmx86_devel && !defined(TEST_CUSTOM_ICU_DATA_FILE)
+#if vmx86_devel
    /*
     * Devel builds use toolchain directory first.
     */
+
    {
       WCHAR icuFilePath[MAX_PATH] = { 0 };
       DWORD n = ExpandEnvironmentStringsW(ICU_DATA_FILE_PATH,
@@ -463,169 +295,74 @@ CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current co
    }
 #endif
 
-   if (icuDataDir) {
-      /*
-       * Data file must be in the specified directory.
-       */
-      size_t length = strlen(icuDataDir);
+   /*
+    * Data file must be in current module directory.
+    */
 
-      if (!DynBuf_Append(&dbpath, icuDataDir, length)) {
-         goto exit;
-      }
-      if (length && icuDataDir[length - 1] != DIRSEPC) {
-         if (!DynBuf_Append(&dbpath, DIRSEPS, strlen(DIRSEPS))) {
-            goto exit;
-         }
-      }
-      if (!DynBuf_Append(&dbpath, ICU_DATA_FILE, strlen(ICU_DATA_FILE)) ||
-          !DynBuf_Append(&dbpath, "\0", 1)) {
-         goto exit;
-      }
+   modPath = CodeSetGetModulePath(NULL);
+   if (!modPath) {
+      goto exit;
+   }
 
-      /*
-       * Check for file existence.
-       */
-      attribs = GetFileAttributesA(DynBuf_Get(&dbpath));
+   lastSlash = wcsrchr(modPath, DIRSEPC_W);
+   if (!lastSlash) {
+      goto exit;
+   }
 
-      if ((INVALID_FILE_ATTRIBUTES == attribs) ||
-          (attribs & FILE_ATTRIBUTE_DIRECTORY)) {
-         goto exit;
-      }
+   *lastSlash = L'\0';
 
-      path = (char *) DynBuf_Detach(&dbpath);
-   } else {
-      /*
-       * Data file must be in the directory of the current module
-       * (i.e. the module that contains CodeSet_Init()).
-       */
-      HMODULE hModule = W32Util_GetModuleByAddress((void *) CodeSet_Init);
-      if (!hModule) {
-         goto exit;
-      }
+   if (!DynBuf_Append(&dbpath, modPath,
+                      wcslen(modPath) * sizeof(utf16_t)) ||
+       !DynBuf_Append(&dbpath, DIRSEPS_W,
+                      wcslen(DIRSEPS_W) * sizeof(utf16_t)) ||
+       !DynBuf_Append(&dbpath, ICU_DATA_FILE_W,
+                      wcslen(ICU_DATA_FILE_W) * sizeof(utf16_t)) ||
+       !DynBuf_Append(&dbpath, L"\0", 2)) {
+      goto exit;
+   }
 
-      modPath = CodeSetGetModulePath(hModule);
-      if (!modPath) {
-         goto exit;
-      }
+   /*
+    * Check for file existence.
+    */
 
-      lastSlash = wcsrchr(modPath, DIRSEPC_W);
-      if (!lastSlash) {
-         goto exit;
-      }
+   attribs = GetFileAttributesW((LPCWSTR) DynBuf_Get(&dbpath));
 
-      *lastSlash = L'\0';
+   if ((INVALID_FILE_ATTRIBUTES == attribs) ||
+       (attribs & FILE_ATTRIBUTE_DIRECTORY)) {
+      goto exit;
+   }
 
-      if (!DynBuf_Append(&dbpath, modPath,
-                         wcslen(modPath) * sizeof(utf16_t)) ||
-          !DynBuf_Append(&dbpath, DIRSEPS_W,
-                         wcslen(DIRSEPS_W) * sizeof(utf16_t)) ||
-          !DynBuf_Append(&dbpath, ICU_DATA_FILE_W,
-                         wcslen(ICU_DATA_FILE_W) * sizeof(utf16_t)) ||
-          !DynBuf_Append(&dbpath, L"\0", 2)) {
-         goto exit;
-      }
+   /*
+    * Convert path to local encoding using system APIs (old codeset).
+    */
 
-      /*
-       * Since u_setDataDirectory can't handle UTF-16, we would have to
-       * now convert this path to local encoding. But that fails when
-       * the module is in a path containing characters not in the
-       * local encoding (see 282524). So we'll memory-map the file
-       * instead and call udata_setCommonData() below.
-       */
-      wpath = (utf16_t *) DynBuf_Get(&dbpath);
-      hFile = CreateFileW(wpath, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0,
-                          NULL);
-      if (INVALID_HANDLE_VALUE == hFile) {
-         goto exit;
-      }
-      hMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
-      if (NULL == hMapping) {
-         goto exit;
-      }
-      memMappedData = MapViewOfFile(hMapping, FILE_MAP_READ, 0, 0, 0);
-      if (NULL == memMappedData) {
-         goto exit;
-      }
+   if (!CodeSetOld_Utf16leToCurrent(DynBuf_Get(&dbpath),
+                                    DynBuf_GetSize(&dbpath),
+                                    &path, NULL)) {
+      goto exit;
    }
 
 #else // } _WIN32 {
 
-#if vmx86_devel && !defined(TEST_CUSTOM_ICU_DATA_FILE)
-   {
-      char *modPath;
-      char *lastSlash;
+#if vmx86_devel
+   /*
+    * Devel builds use toolchain directory first.
+    */
 
-      /*
-       * Devel builds use toolchain directory first.
-       */
-      if (stat(ICU_DATA_FILE_PATH, &finfo) >= 0 && !S_ISDIR(finfo.st_mode)) {
-         if ((path = strdup(ICU_DATA_FILE_PATH)) == NULL) {
-            goto exit;
-         }
-         goto found;
-      }
-
-      /*
-       * Then we try module directory, if we can get it.
-       */
-      modPath = CodeSetGetModulePath(HGMP_PRIVILEGE);
-      if (modPath) {
-         lastSlash = strrchr(modPath, DIRSEPC);
-         if (lastSlash) {
-            *lastSlash = '\0';
-
-            if (DynBuf_Append(&dbpath, modPath, strlen(modPath)) &&
-                DynBuf_Append(&dbpath, DIRSEPS, strlen(DIRSEPS)) &&
-                DynBuf_Append(&dbpath, ICU_DATA_FILE,
-                              strlen(ICU_DATA_FILE)) &&
-                DynBuf_Append(&dbpath, "\0", 1)) {
-
-               if ((stat((const char *) DynBuf_Get(&dbpath), &finfo) >= 0) &&
-                   !S_ISDIR(finfo.st_mode)) {
-                  free(modPath);
-                  path = DynBuf_Detach(&dbpath);
-                  goto found;
-               } else {
-                  DynBuf_SetSize(&dbpath, 0);
-               }
-            }
-         }
-
-         free(modPath);
+   if (stat(ICU_DATA_FILE_PATH, &finfo) >= 0 && !S_ISDIR(finfo.st_mode)) {
+      if ((path = strdup(ICU_DATA_FILE_PATH)) == NULL) {
+	 goto exit;
       }
+      goto found;
    }
-#endif // vmx86_devel
+#endif
 
-   if (icuDataDir) {
-      /* Use the caller-specified ICU data dir. */
-      if (!DynBuf_Append(&dbpath, icuDataDir, strlen(icuDataDir))) {
-         goto exit;
-      }
-   } else {
-      /* Use a default ICU data dir. */
-#   if defined __APPLE__
-      Location_Get_Type *Location_Get = Location_Get_Addr();
+   /*
+    * Data file must be in POSIX_ICU_DIR.
+    */
 
-      if (Location_Get) {
-         char *icuDir = Location_Get("icuDir");
-         Bool success =    icuDir
-                        && DynBuf_Append(&dbpath, icuDir, strlen(icuDir));
-
-         free(icuDir);
-         if (!success) {
-            goto exit;
-         }
-      } else
-#   endif
-
-      {
-         if (!DynBuf_Append(&dbpath, POSIX_ICU_DIR, strlen(POSIX_ICU_DIR)) ||
-             !DynBuf_Append(&dbpath, "/icu", strlen("/icu"))) {
-            goto exit;
-         }
-      }
-   }
-   if (!DynBuf_Append(&dbpath, DIRSEPS, strlen(DIRSEPS)) ||
+   if (!DynBuf_Append(&dbpath, POSIX_ICU_DIR, strlen(POSIX_ICU_DIR)) ||
+       !DynBuf_Append(&dbpath, DIRSEPS, strlen(DIRSEPS)) ||
        !DynBuf_Append(&dbpath, ICU_DATA_FILE, strlen(ICU_DATA_FILE)) ||
        !DynBuf_Append(&dbpath, "\0", 1)) {
       goto exit;
@@ -634,6 +371,7 @@ CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current co
    /*
     * Check for file existence. (DO NOT CHANGE TO 'stat' WRAPPER).
     */
+
    path = (char *) DynBuf_Detach(&dbpath);
    if (stat(path, &finfo) < 0 || S_ISDIR(finfo.st_mode)) {
       goto exit;
@@ -641,37 +379,14 @@ CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current co
 
 #endif // } _WIN32
 
-#if vmx86_devel && !defined(TEST_CUSTOM_ICU_DATA_FILE)
+#if vmx86_devel
 found:
 #endif
 
-#ifdef _WIN32
-   if (memMappedData) {
-      /*
-       * Tell ICU to use this mapped data.
-       */
-      UErrorCode uerr = U_ZERO_ERROR;
-      ASSERT(memMappedData);
-
-      udata_setCommonData(memMappedData, &uerr);
-      if (uerr != U_ZERO_ERROR) {
-         UnmapViewOfFile(memMappedData);
-         goto exit;
-      }
-      udata_setAppData(ICU_DATA_ITEM, memMappedData, &uerr);
-      if (uerr != U_ZERO_ERROR) {
-         UnmapViewOfFile(memMappedData);
-         goto exit;
-      }
-   } else {
-#endif
-      /*
-       * Tell ICU to use this directory.
-       */
-      u_setDataDirectory(path);
-#ifdef _WIN32
-   }
-#endif
+   /*
+    * Tell ICU to use this directory.
+    */
+   u_setDataDirectory(path);
 
    dontUseIcu = FALSE;
    ret = TRUE;
@@ -685,27 +400,17 @@ found:
       if (CODESET_CAN_FALLBACK_ON_NON_ICU) {
          ret = TRUE;
          dontUseIcu = TRUE;
-
-#ifdef _WIN32
-         OutputDebugStringW(L"CodeSet_Init: no ICU\n");
-#endif
       }
    }
 
 #ifdef _WIN32
    free(modPath);
-   if (hMapping) {
-      CloseHandle(hMapping);
-   }
-   if (hFile != INVALID_HANDLE_VALUE) {
-      CloseHandle(hFile);
-   }
 #endif
+
    free(path);
    DynBuf_Destroy(&dbpath);
 
    return ret;
-#endif
 }
 
 
@@ -732,22 +437,17 @@ static Bool
 CodeSetDuplicateUtf8Str(const char *bufIn,  // IN: Input string
                         size_t sizeIn,      // IN: Input string length
                         char **bufOut,      // OUT: "Converted" string
-                        size_t *sizeOut)    // OUT/OPT: Length of string
+                        size_t *sizeOut)    // OUT: Length of string
 {
    char *myBufOut;
-   size_t newSize = sizeIn + sizeof *myBufOut;
 
-   if (newSize < sizeIn) {   // Prevent integer overflow
-      return FALSE;
-   }
-
-   myBufOut = malloc(newSize);
+   myBufOut = malloc(sizeIn + 1);
    if (myBufOut == NULL) {
       return FALSE;
    }
 
    memcpy(myBufOut, bufIn, sizeIn);
-   myBufOut[sizeIn] = '\0';
+   memset(myBufOut + sizeIn, 0, 1);
 
    *bufOut = myBufOut;
    if (sizeOut) {
@@ -782,7 +482,7 @@ static Bool
 CodeSetDynBufFinalize(Bool ok,          // IN: Earlier steps succeeded
                       DynBuf *db,       // IN: Buffer with converted string
                       char **bufOut,    // OUT: Converted string
-                      size_t *sizeOut)  // OUT/OPT: Length of string in bytes
+                      size_t *sizeOut)  // OUT: Length of string in bytes
 {
    /*
     * NUL can be as long as 4 bytes if UTF-32, make no assumptions.
@@ -858,6 +558,30 @@ CodeSet_Utf8Normalize(const char *bufIn,     // IN
 #endif /* defined(__APPLE__) */
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * CodeSet_GetCurrentCodeSet --
+ *
+ *    Return native code set name. Always calls down to
+ *    CodeSetOld_GetCurrentCodeSet. See there for more details.
+ *
+ * Results:
+ *    See CodeSetOld_GetCurrentCodeSet.
+ *
+ * Side effects:
+ *    See CodeSetOld_GetCurrentCodeSet.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+const char *
+CodeSet_GetCurrentCodeSet(void)
+{
+   return CodeSetOld_GetCurrentCodeSet();
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -884,10 +608,6 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
                            unsigned int flags,  // IN
                            DynBuf *db)          // IN/OUT
 {
-#if defined(NO_ICU)
-   return CodeSetOld_GenericToGenericDb(codeIn, bufIn, sizeIn, codeOut,
-                                        flags, db);
-#else
    Bool result = FALSE;
    UErrorCode uerr;
    const char *bufInCur;
@@ -899,7 +619,6 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
    char *bufOut;
    char *bufOutCur;
    char *bufOutEnd;
-   size_t newSize;
    size_t bufOutSize;
    size_t bufOutOffset;
    UConverter *cvin = NULL;
@@ -991,11 +710,7 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
 
    bufInCur = bufIn;
    bufInEnd = bufIn + sizeIn;
-   newSize = sizeIn + 4;
-   if (newSize < sizeIn) {  // Prevent integer overflow
-      goto exit;
-   }
-   bufOutSize = newSize;
+   bufOutSize = sizeIn + 4;
    bufOutOffset = 0;
    bufPivSource = bufPiv;
    bufPivTarget = bufPiv;
@@ -1018,8 +733,11 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
 
       if (!U_FAILURE(uerr)) {
          /*
-          * "This was a triumph. I'm making a note here: HUGE SUCCESS. It's
-          * hard to overstate my satisfaction."
+          * "This was a triumph.
+          *  I'm making a note here:
+          *  HUGE SUCCESS.
+          *  It's hard to overstate
+          *  my satisfaction."
           */
 
          break;
@@ -1036,19 +754,7 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
        * based on bufOutOffset.
        */
 
-      newSize = 2 * bufOutSize;
-
-      /*
-       * Prevent integer overflow. We can use this form of checking
-       * specifically because a multiple by 2 is used. This type of checking
-       * does not work in the general case.
-       */
-
-      if (newSize < bufOutSize) {
-         goto exit;
-      }
-
-      bufOutSize = newSize;
+      bufOutSize *= 2;
       bufOutOffset = bufOutCur - bufOut;
    }
 
@@ -1070,7 +776,6 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
    }
 
    return result;
-#endif
 }
 
 
@@ -1098,7 +803,7 @@ CodeSet_GenericToGeneric(const char *codeIn,  // IN
                          const char *codeOut, // IN
                          unsigned int flags,  // IN
                          char **bufOut,       // OUT
-                         size_t *sizeOut)     // OUT/OPT
+                         size_t *sizeOut)     // OUT
 {
    DynBuf db;
    Bool ok;
@@ -1176,7 +881,7 @@ Bool
 CodeSet_Utf8ToCurrent(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT/OPT
+                      size_t *sizeOut)    // OUT
 {
 #if !defined(CURRENT_IS_UTF8)
    DynBuf db;
@@ -1225,7 +930,7 @@ Bool
 CodeSet_CurrentToUtf8(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT/OPT
+                      size_t *sizeOut)    // OUT
 {
 #if !defined(CURRENT_IS_UTF8)
    DynBuf db;
@@ -1311,7 +1016,7 @@ Bool
 CodeSet_Utf16leToUtf8(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT/OPT
+                      size_t *sizeOut)    // OUT
 {
    DynBuf db;
    Bool ok;
@@ -1355,7 +1060,7 @@ Bool
 CodeSet_Utf8ToUtf16le(const char *bufIn,  // IN
                       size_t sizeIn,      // IN
                       char **bufOut,      // OUT
-                      size_t *sizeOut)    // OUT/OPT
+                      size_t *sizeOut)    // OUT
 {
    DynBuf db;
    Bool ok;
@@ -1399,7 +1104,7 @@ Bool
 CodeSet_Utf8FormDToUtf8FormC(const char *bufIn,     // IN
                              size_t sizeIn,         // IN
                              char **bufOut,         // OUT
-                             size_t *sizeOut)       // OUT/OPT
+                             size_t *sizeOut)       // OUT
 {
    /*
     * Fallback if necessary.
@@ -1409,13 +1114,11 @@ CodeSet_Utf8FormDToUtf8FormC(const char *bufIn,     // IN
    }
 
 #if defined(__APPLE__)
-   {
-      DynBuf db;
-      Bool ok;
-      DynBuf_Init(&db);
-      ok = CodeSet_Utf8Normalize(bufIn, sizeIn, TRUE, &db);
-      return CodeSetDynBufFinalize(ok, &db, bufOut, sizeOut);
-   }
+   DynBuf db;
+   Bool ok;
+   DynBuf_Init(&db);
+   ok = CodeSet_Utf8Normalize(bufIn, sizeIn, TRUE, &db);
+   return CodeSetDynBufFinalize(ok, &db, bufOut, sizeOut);
 #else
    NOT_IMPLEMENTED();
 #endif
@@ -1448,7 +1151,7 @@ Bool
 CodeSet_Utf8FormCToUtf8FormD(const char *bufIn,     // IN
                              size_t sizeIn,         // IN
                              char **bufOut,         // OUT
-                             size_t *sizeOut)       // OUT/OPT
+                             size_t *sizeOut)       // OUT
 {
    /*
     * Fallback if necessary.
@@ -1458,13 +1161,11 @@ CodeSet_Utf8FormCToUtf8FormD(const char *bufIn,     // IN
    }
 
 #if defined(__APPLE__)
-   {
-      DynBuf db;
-      Bool ok;
-      DynBuf_Init(&db);
-      ok = CodeSet_Utf8Normalize(bufIn, sizeIn, FALSE, &db);
-      return CodeSetDynBufFinalize(ok, &db, bufOut, sizeOut);
-   }
+   DynBuf db;
+   Bool ok;
+   DynBuf_Init(&db);
+   ok = CodeSet_Utf8Normalize(bufIn, sizeIn, FALSE, &db);
+   return CodeSetDynBufFinalize(ok, &db, bufOut, sizeOut);
 #else
    NOT_IMPLEMENTED();
 #endif
@@ -1495,7 +1196,7 @@ Bool
 CodeSet_CurrentToUtf16le(const char *bufIn, // IN
                          size_t sizeIn,     // IN
                          char **bufOut,     // OUT
-                         size_t *sizeOut)   // OUT/OPT
+                         size_t *sizeOut)   // OUT
 {
    DynBuf db;
    Bool ok;
@@ -1538,7 +1239,7 @@ Bool
 CodeSet_Utf16leToCurrent(const char *bufIn,  // IN
                          size_t sizeIn,      // IN
                          char **bufOut,      // OUT
-                         size_t *sizeOut)    // OUT/OPT
+                         size_t *sizeOut)    // OUT
 {
    DynBuf db;
    Bool ok;
@@ -1581,7 +1282,7 @@ Bool
 CodeSet_Utf16beToCurrent(const char *bufIn,  // IN
                          size_t sizeIn,      // IN
                          char **bufOut,      // OUT
-                         size_t *sizeOut)    // OUT/OPT
+                         size_t *sizeOut)    // OUT
 {
    DynBuf db;
    Bool ok;
@@ -1620,9 +1321,6 @@ CodeSet_Utf16beToCurrent(const char *bufIn,  // IN
 Bool
 CodeSet_IsEncodingSupported(const char *name) // IN
 {
-#if defined(NO_ICU)
-   return CodeSetOld_IsEncodingSupported(name);
-#else
    UConverter *cv;
    UErrorCode uerr;
 
@@ -1640,12 +1338,10 @@ CodeSet_IsEncodingSupported(const char *name) // IN
    cv = ucnv_open(name, &uerr);
    if (cv) {
       ucnv_close(cv);
-
       return TRUE;
    }
 
    return FALSE;
-#endif
 }
 
 
@@ -1668,17 +1364,14 @@ CodeSet_IsEncodingSupported(const char *name) // IN
 
 Bool
 CodeSet_Validate(const char *buf,   // IN: the string
-                 size_t size,	    // IN: length of string
+                 size_t size,	   // IN: length of string
                  const char *code)  // IN: encoding
 {
-#if defined(NO_ICU)
-   return CodeSetOld_Validate(buf, size, code);
-#else
    UConverter *cv;
    UErrorCode uerr;
 
    // ucnv_toUChars takes 32-bit int size
-   VERIFY(size <= (size_t) MAX_INT32);
+   ASSERT_NOT_IMPLEMENTED(size <= (size_t) MAX_INT32);
 
    if (size == 0) {
       return TRUE;
@@ -1701,13 +1394,11 @@ CodeSet_Validate(const char *buf,   // IN: the string
 
    uerr = U_ZERO_ERROR;
    cv = ucnv_open(code, &uerr);
-   VERIFY(uerr == U_ZERO_ERROR);
+   ASSERT_NOT_IMPLEMENTED(uerr == U_ZERO_ERROR);
    ucnv_setToUCallBack(cv, UCNV_TO_U_CALLBACK_STOP, NULL, NULL, NULL, &uerr);
-   VERIFY(uerr == U_ZERO_ERROR);
+   ASSERT_NOT_IMPLEMENTED(uerr == U_ZERO_ERROR);
    ucnv_toUChars(cv, NULL, 0, buf, size, &uerr);
    ucnv_close(cv);
 
    return uerr == U_BUFFER_OVERFLOW_ERROR;
-#endif
 }
-
