@@ -48,7 +48,7 @@
  */
 
 void
-DynBuf_Init(DynBuf *b)  // OUT:
+DynBuf_Init(DynBuf *b) // IN
 {
    ASSERT(b);
 
@@ -75,12 +75,37 @@ DynBuf_Init(DynBuf *b)  // OUT:
  */
 
 void
-DynBuf_Destroy(DynBuf *b)  // IN/OUT:
+DynBuf_Destroy(DynBuf *b) // IN
 {
    ASSERT(b);
 
    free(b->data);
-   DynBuf_Init(b);
+   b->data = NULL;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DynBuf_Get --
+ *
+ *      Retrieve a pointer to the data contained in a dynamic buffer --hpreg
+ *
+ * Results:
+ *      The pointer to the data
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void *
+DynBuf_Get(DynBuf const *b) // IN
+{
+   ASSERT(b);
+
+   return b->data;
 }
 
 
@@ -139,7 +164,6 @@ DynBuf_Attach(DynBuf *b,    // IN
               void *data)   // IN
 {
    ASSERT(b);
-   ASSERT((size == 0) == (data == NULL));
 
    free(b->data);
    b->data = data;
@@ -173,7 +197,6 @@ DynBuf_Detach(DynBuf *b) // IN
 
    data = b->data;
    b->data = NULL;
-   b->allocated = 0;
 
    return data;
 }
@@ -197,21 +220,21 @@ DynBuf_Detach(DynBuf *b) // IN
  */
 
 static Bool
-DynBufRealloc(DynBuf *b,            // IN:
-              size_t newAllocated)  // IN:
+DynBufRealloc(DynBuf *b,            // IN
+              size_t new_allocated) // IN
 {
    void *new_data;
 
    ASSERT(b);
 
-   new_data = realloc(b->data, newAllocated);
-   if (new_data == NULL && newAllocated) {
+   new_data = realloc(b->data, new_allocated);
+   if (new_data == NULL) {
       /* Not enough memory */
       return FALSE;
    }
 
    b->data = new_data;
-   b->allocated = newAllocated;
+   b->allocated = new_allocated;
 
    return TRUE;
 }
@@ -223,7 +246,7 @@ DynBufRealloc(DynBuf *b,            // IN:
  * DynBuf_Enlarge --
  *
  *      Enlarge a dynamic buffer. The resulting dynamic buffer is guaranteed to
- *      be larger than the one you passed, and at least 'minSize' bytes
+ *      be larger than the one you passed, and at least 'size' bytes
  *      large --hpreg
  *
  * Results:
@@ -237,19 +260,19 @@ DynBufRealloc(DynBuf *b,            // IN:
  */
 
 Bool
-DynBuf_Enlarge(DynBuf *b,       // IN:
-               size_t minSize)  // IN:
+DynBuf_Enlarge(DynBuf *b,       // IN
+               size_t min_size) // IN
 {
-   size_t newAllocated;
+   size_t new_allocated;
 
    ASSERT(b);
 
-   newAllocated = b->allocated
+   new_allocated = b->allocated
                       ?
 #if defined(DYNBUF_DEBUG)
                         b->allocated + 1
 #else
-                        /*
+                        /* 
                          * Double the previously allocated size if it is less
                          * than 256KB; otherwise grow it linearly by 256KB
                          */
@@ -268,21 +291,11 @@ DynBuf_Enlarge(DynBuf *b,       // IN:
 #endif
                       ;
 
-   if (minSize > newAllocated) {
-      newAllocated = minSize;
+   if (min_size > new_allocated) {
+      new_allocated = min_size;
    }
 
-   /*
-    * Prevent integer overflow. We can use this form of checking specifically
-    * because a multiple by 2 is used (in the worst case). This type of
-    * checking does not work in the general case.
-    */
-
-   if (newAllocated < b->allocated) {
-      return FALSE;
-   }
-
-   return DynBufRealloc(b, newAllocated);
+   return DynBufRealloc(b, new_allocated);
 }
 
 
@@ -310,7 +323,7 @@ DynBuf_Append(DynBuf *b,        // IN
               size_t size)      // IN
 {
    size_t new_size;
-
+   
    ASSERT(b);
 
    if (size <= 0) {
@@ -320,11 +333,6 @@ DynBuf_Append(DynBuf *b,        // IN
    ASSERT(data);
 
    new_size = b->size + size;
-
-   if (new_size < b->size) {  // Prevent integer overflow
-      return FALSE;
-   }
-
    if (new_size > b->allocated) {
       /* Not enough room */
       if (DynBuf_Enlarge(b, new_size) == FALSE) {
@@ -339,38 +347,6 @@ DynBuf_Append(DynBuf *b,        // IN
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * DynBuf_SafeInternalAppend --
- *
- *      Append data at the end of a dynamic buffer. Memory allocation failure
- *      are handled the same way as Util_SafeMalloc, that is to say, with a
- *      Panic.
- *
- * Results:
- *      None
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-DynBuf_SafeInternalAppend(DynBuf *b,            // IN
-                          void const *data,     // IN
-                          size_t size,          // IN
-                          char const *file,     // IN
-                          unsigned int lineno)  // IN
-{
-   if (!DynBuf_Append(b, data, size)) {
-      Panic("Unrecoverable memory allocation failure at %s:%u\n",
-            file, lineno);
-   }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -397,6 +373,83 @@ DynBuf_Trim(DynBuf *b) // IN
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DynBuf_GetSize --
+ *
+ *      Returns the current size of the dynamic buffer --hpreg
+ *
+ * Results:
+ *      The current size of the dynamic buffer
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+size_t
+DynBuf_GetSize(DynBuf const *b) // IN
+{
+   ASSERT(b);
+
+   return b->size;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DynBuf_SetSize --
+ *
+ *      Set the current size of a dynamic buffer --hpreg
+ *
+ * Results:
+ *      None
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+DynBuf_SetSize(DynBuf *b,   // IN
+               size_t size) // IN
+{
+   ASSERT(b);
+   ASSERT(size <= b->allocated);
+
+   b->size = size;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DynBuf_GetAllocatedSize --
+ *
+ *      Returns the current allocated size of the dynamic buffer --hpreg
+ *
+ * Results:
+ *      The current allocated size of the dynamic buffer
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+size_t
+DynBuf_GetAllocatedSize(DynBuf const *b) // IN
+{
+   ASSERT(b);
+
+   return b->allocated;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -424,16 +477,15 @@ DynBuf_Copy(DynBuf *src,   // IN
    ASSERT(dest);
    ASSERT(!dest->data);
 
-   dest->data = malloc(src->allocated);
+   dest->size      = src->size;
+   dest->allocated = src->allocated;
+   dest->data      = malloc(src->allocated);
 
-   if (dest->data == NULL) {
+   if (!dest->data) {
       return FALSE;
    }
 
-   dest->size = src->size;
-   dest->allocated = src->allocated;
-
    memcpy(dest->data, src->data, src->size);
-
+   
    return TRUE;
 }
