@@ -35,7 +35,6 @@
  * decimal number. If the input is not an hexadecimal digit character, the
  * output is -1 --hpreg
  */
-
 static int const Hex2Dec[] = {
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
@@ -60,7 +59,6 @@ static int const Hex2Dec[] = {
  * Table to use to quickly convert a decimal number into an ASCII hexadecimal
  * digit character --hpreg
  */
-
 static char const Dec2Hex[] = {
    '0', '1', '2', '3', '4', '5', '6', '7',
    '8', '9', 'A', 'B', 'C', 'D', 'E', 'F',
@@ -91,7 +89,7 @@ Escape_DoString(const char *escStr,    // IN
                 int const *bytesToEsc, // IN
                 void const *bufIn,     // IN
                 size_t sizeIn,         // IN
-                size_t *sizeOut)       // OUT/OPT
+                size_t *sizeOut)       // OUT
 {
    char const *buf;
    DynBuf b;
@@ -123,23 +121,23 @@ Escape_DoString(const char *escStr,    // IN
          /* We must escape that byte --hpreg */
 
          escSeq[0] = Dec2Hex[ubyte >> 4];
-         escSeq[1] = Dec2Hex[ubyte & 0xF];
-         if (DynBuf_Append(&b, &buf[startUnescaped],
-                           index - startUnescaped) == FALSE ||
-             DynBuf_Append(&b, escStr, escStrLen) == FALSE ||
-             DynBuf_Append(&b, escSeq, sizeof escSeq) == FALSE) {
+    escSeq[1] = Dec2Hex[ubyte & 0xF];
+         if (   DynBuf_Append(&b, &buf[startUnescaped],
+                   index - startUnescaped) == FALSE
+             || DynBuf_Append(&b, escStr, escStrLen) == FALSE
+             || DynBuf_Append(&b, escSeq, sizeof escSeq) == FALSE) {
             goto nem;
          }
          startUnescaped = index + 1;
       }
    }
 
-   if (/* Last unescaped chunk (if any) --hpreg */
-       DynBuf_Append(&b, &buf[startUnescaped],
-                     index - startUnescaped) == FALSE ||
-       /* NUL terminator --hpreg */
-       DynBuf_Append(&b, "", 1) == FALSE ||
-       DynBuf_Trim(&b) == FALSE) {
+   if (   /* Last unescaped chunk (if any) --hpreg */
+          DynBuf_Append(&b, &buf[startUnescaped],
+             index - startUnescaped) == FALSE
+          /* NUL terminator --hpreg */
+       || DynBuf_Append(&b, "", 1) == FALSE
+       || DynBuf_Trim(&b) == FALSE) {
       goto nem;
    }
 
@@ -180,7 +178,7 @@ Escape_Do(char escByte,          // IN
           int const *bytesToEsc, // IN
           void const *bufIn,     // IN
           size_t sizeIn,         // IN
-          size_t *sizeOut)       // OUT/OPT
+          size_t *sizeOut)       // OUT
 {
    const char escStr[] = { escByte, '\0' };
 
@@ -211,7 +209,7 @@ void *
 Escape_Undo(char escByte,      // IN
             void const *bufIn, // IN
             size_t sizeIn,     // IN
-            size_t *sizeOut)   // OUT/OPT
+            size_t *sizeOut)   // OUT
 {
    char const *buf;
    DynBuf b;
@@ -251,9 +249,9 @@ Escape_Undo(char escByte,      // IN
             char escaped;
 
             escaped = h << 4 | l;
-            if (DynBuf_Append(&b, &buf[startUnescaped],
-                              index - 2 - startUnescaped) == FALSE ||
-                DynBuf_Append(&b, &escaped, 1) == FALSE) {
+            if (   DynBuf_Append(&b, &buf[startUnescaped],
+                                        index - 2 - startUnescaped) == FALSE
+                || DynBuf_Append(&b, &escaped, 1) == FALSE) {
                goto nem;
             }
             startUnescaped = index + 1;
@@ -267,12 +265,12 @@ Escape_Undo(char escByte,      // IN
       }
    }
 
-   if (/* Last unescaped chunk (if any) --hpreg */
-       DynBuf_Append(&b, &buf[startUnescaped],
-                     index - startUnescaped) == FALSE ||
-       /* NUL terminator --hpreg */
-       DynBuf_Append(&b, "", 1) == FALSE ||
-       DynBuf_Trim(&b) == FALSE) {
+   if (   /* Last unescaped chunk (if any) --hpreg */
+          DynBuf_Append(&b, &buf[startUnescaped],
+                               index - startUnescaped) == FALSE
+          /* NUL terminator --hpreg */
+       || DynBuf_Append(&b, "", 1) == FALSE
+       || DynBuf_Trim(&b) == FALSE) {
       goto nem;
    }
 
@@ -311,7 +309,7 @@ nem:
 void *
 Escape_AnsiToUnix(void const *bufIn, // IN
                   size_t sizeIn,     // IN
-                  size_t *sizeOut)   // OUT/OPT
+                  size_t *sizeOut)   // OUT
 {
    char const *buf;
    DynBuf b;
@@ -330,7 +328,6 @@ Escape_AnsiToUnix(void const *bufIn, // IN
     * Identify all chunks in buf (\r\n being the chunk separator), and copy
     * them into b --hpreg
     */
-
    for (index = 0; index < sizeIn; index++) {
       char byte;
 
@@ -340,7 +337,7 @@ Escape_AnsiToUnix(void const *bufIn, // IN
          state = 0;
          if (byte == '\n') {
             if (DynBuf_Append(&b, &buf[startUnescaped],
-                              index - 1 - startUnescaped) == FALSE) {
+                                        index - 1 - startUnescaped) == FALSE) {
                goto nem;
             }
             startUnescaped = index;
@@ -360,12 +357,12 @@ Escape_AnsiToUnix(void const *bufIn, // IN
       }
    }
 
-   if (/* Last unescaped chunk (if any) --hpreg */
-       DynBuf_Append(&b, &buf[startUnescaped],
-                     index - startUnescaped) == FALSE ||
-       /* NUL terminator --hpreg */
-       DynBuf_Append(&b, "", 1) == FALSE ||
-       DynBuf_Trim(&b) == FALSE) {
+   if (   /* Last unescaped chunk (if any) --hpreg */
+          DynBuf_Append(&b, &buf[startUnescaped], index - startUnescaped)
+             == FALSE
+          /* NUL terminator --hpreg */
+       || DynBuf_Append(&b, "", 1) == FALSE
+       || DynBuf_Trim(&b) == FALSE) {
       goto nem;
    }
 
@@ -427,8 +424,8 @@ main(int argc,
  *
  * Escape_Sh --
  *
- *    Escape a buffer so that it can be passed verbatim as part of an argument
- *    on a shell command line.
+ *    Escape a buffer so that it is passed as one argument on a sh command
+ *    line --hpreg
  *
  * Results:
  *    The escaped, allocated, NUL terminated buffer on success. If not NULL,
@@ -445,7 +442,7 @@ main(int argc,
 void *
 Escape_Sh(void const *bufIn, // IN
           size_t sizeIn,     // IN
-          size_t *sizeOut)   // OUT/OPT
+          size_t *sizeOut)   // OUT
 {
    static const char be[] = { '\'', };
    static const char escSeq[] = { '\'', '"', '\'', '"', };
@@ -468,22 +465,23 @@ Escape_Sh(void const *bufIn, // IN
       if (buf[index] == '\'') {
          /* We must escape that byte --hpreg */
 
-         if (DynBuf_Append(&b, &buf[startUnescaped],
-                           index - startUnescaped) == FALSE ||
-             DynBuf_Append(&b, escSeq, sizeof(escSeq)) == FALSE) {
+         if (   DynBuf_Append(&b, &buf[startUnescaped],
+                   index - startUnescaped) == FALSE
+             || DynBuf_Append(&b, escSeq,
+                   sizeof(escSeq)) == FALSE) {
             goto nem;
          }
          startUnescaped = index;
       }
    }
 
-   if (/* Last unescaped chunk (if any) --hpreg */
-       DynBuf_Append(&b, &buf[startUnescaped],
-                     index - startUnescaped) == FALSE ||
-       DynBuf_Append(&b, be, sizeof(be)) == FALSE ||
-       /* NUL terminator --hpreg */
-       DynBuf_Append(&b, "", 1) == FALSE ||
-       DynBuf_Trim(&b) == FALSE) {
+   if (   /* Last unescaped chunk (if any) --hpreg */
+          DynBuf_Append(&b, &buf[startUnescaped],
+             index - startUnescaped) == FALSE
+       || DynBuf_Append(&b, be, sizeof(be)) == FALSE
+          /* NUL terminator --hpreg */
+       || DynBuf_Append(&b, "", 1) == FALSE
+       || DynBuf_Trim(&b) == FALSE) {
       goto nem;
    }
 
@@ -500,198 +498,6 @@ nem:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * Escape_BRE --
- *
- *    Escape a buffer so that it can be passed verbatim as part of a Basic
- *    (a.k.a. obsolete) Regular Expression.
- *
- * Results:
- *    The escaped, allocated, NUL terminated buffer on success. If not NULL,
- *     '*sizeOut' contains the size of the buffer (excluding the NUL
- *     terminator)
- *    NULL on failure (not enough memory)
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-void *
-Escape_BRE(void const *bufIn, // IN
-           size_t sizeIn,     // IN
-           size_t *sizeOut)   // OUT/OPT
-{
-   static const char escByte = '\\';
-   /* Escape ] [ ^ . * $ and 'escByte'. */
-   static const int bytesToEsc[] = {
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-   };
-   char const *buf;
-   DynBuf b;
-   size_t startUnescaped;
-   size_t index;
-
-   /* Unsigned does matter. */
-   ASSERT(bytesToEsc[(unsigned char)escByte]);
-
-   buf = (char const *)bufIn;
-   ASSERT(buf);
-
-   DynBuf_Init(&b);
-   startUnescaped = 0;
-
-   for (index = 0; index < sizeIn; index++) {
-      /* Unsigned does matter. */
-      unsigned char ubyte;
-
-      ubyte = buf[index];
-      if (bytesToEsc[ubyte]) {
-         /* We must escape that byte. */
-
-         if (DynBuf_Append(&b, &buf[startUnescaped],
-                           index - startUnescaped) == FALSE ||
-             DynBuf_Append(&b, &escByte, sizeof escByte) == FALSE) {
-            goto nem;
-         }
-         startUnescaped = index;
-      }
-   }
-
-   if (/* Last unescaped chunk (if any). */
-       DynBuf_Append(&b, &buf[startUnescaped],
-                     index - startUnescaped) == FALSE ||
-       /* NUL terminator. */
-       DynBuf_Append(&b, "", 1) == FALSE ||
-       DynBuf_Trim(&b) == FALSE) {
-      goto nem;
-   }
-
-   if (sizeOut) {
-      *sizeOut = DynBuf_GetSize(&b) - 1;
-   }
-
-   return DynBuf_Get(&b);
-
-nem:
-   DynBuf_Destroy(&b);
-
-   return NULL;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Escape_StrChr --
- *
- *      Find the first occurence of c in bufIn that is not preceded by
- *      escByte.
- *
- *      XXX Doesn't handle recursive escaping, so <escByte><escByte><c>
- *          will skip that occurence of <c>
- *
- * Results:
- *      A pointer to the character, if found, NULL if not found.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-const char *
-Escape_Strchr(char escByte,      // IN
-              const char *bufIn, // IN
-              char c)            // IN
-{
-   size_t i;
-   Bool escaped = FALSE;
-
-   ASSERT(escByte != c);
-   ASSERT(bufIn);
-
-   for (i = 0; bufIn[i] != '\0'; i++) {
-      if (escaped) {
-         escaped = FALSE;
-      } else {
-         if (bufIn[i] == c) {
-            return &bufIn[i];
-         }
-
-         if (bufIn[i] == escByte) {
-            escaped = TRUE;
-         }
-      }
-   }
-
-   return NULL;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Escape_Unescape --
- *
- *      Removes all occurences of an escape character from a string, unless
- *      the occurence is escaped.
- *
- * Results:
- *      An allocated string.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-char *
-Escape_Unescape(char escByte,       // IN
-                const char *bufIn)  // IN
-{
-   DynBuf result;
-   Bool escaped = FALSE;
-   char nullbyte = '\0';
-   int i;
-
-   ASSERT(bufIn);
-
-   DynBuf_Init(&result);
-
-   for (i = 0; bufIn[i]; i++) {
-      if (bufIn[i] != escByte || escaped) {
-         DynBuf_Append(&result, &(bufIn[i]), sizeof(char));
-         escaped = FALSE;
-      } else {
-         escaped = TRUE;
-      }
-   }
-
-   DynBuf_Append(&result, &nullbyte, sizeof('\0'));
-
-   return DynBuf_Get(&result);
-}
-
-
 /*
  *----------------------------------------------------------------------------
  *
@@ -782,91 +588,3 @@ main(int argc,
    return 0;
 }
 #endif
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Escape_Comma --
- *
- *       Use backslash character as an escape character to handle comma
- *       character escaping.
- *
- * Results:
- *       Returns a newly allocated string with comma and backslash characters
- *       escaped.
- *
- * Side effects:
- *       None.  It is a caller responsibility to deallocate the result.
- *
- *-----------------------------------------------------------------------------
- */
-
-char *
-Escape_Comma(const char *string) // IN
-{
-   DynBuf b;
-
-   if (NULL == string) {
-      return NULL;
-   }
-
-   DynBuf_Init(&b);
-
-   for (; *string; ++string) {
-      char c = *string;
-
-      if (c == ',' || c == '\\') {
-         if (!DynBuf_Append(&b, "\\", 1)) {
-            goto out_of_memory;
-         }
-      }
-      if (!DynBuf_Append(&b, string, 1)) {
-         goto out_of_memory;
-      }
-   }
-
-   DynBuf_Append(&b, string, 1);
-
-   return DynBuf_Get(&b);
-
-out_of_memory:
-   DynBuf_Destroy(&b);
-   return NULL;
-}
-
-
-#if 0
-/* Unit test suite for Escape_Comma() */
-int
-main(int argc,
-     char **argv)
-{
-   static struct {
-      const char *in;
-      const char *out;
-   } tests[] = {
-      { "123# ", "123# ", },
-      { "123,", "123\\,", },
-      { "'123\\", "'123\\\\", },
-   };
-   unsigned int i;
-
-   for (i = 0; i < sizeof(tests) / sizeof(tests[0]); i++) {
-      char *out;
-
-      out = Escape_Comma(tests[i].in);
-      if (strcmp(out, tests[i].out) != 0) {
-         printf("test %u failed: \"%s\" : \"%s\"\n", i, tests[i].in, out);
-         exit(1);
-      }
-      free(out);
-   }
-
-   printf("all tests passed\n");
-
-   return 0;
-}
-
-#endif
-
