@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -27,45 +27,47 @@
 #include <sys/syscall.h>
 #include <string.h>
 #include <unistd.h>
-#ifdef __linux__
-#if defined(__GLIBC__) && \
-           (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 16))
-#include <sys/auxv.h>
-#endif
-#endif
 #ifdef __APPLE__
 #include <sys/socket.h>
-#include <TargetConditionals.h>
-#if !defined TARGET_OS_IPHONE
-#define TARGET_OS_IPHONE 0
-#endif
-#if !TARGET_OS_IPHONE
 #include <Security/Authorization.h>
 #include <Security/AuthorizationTags.h>
 #endif
-#endif
-#if defined __ANDROID__
-#include <syscall-android.h>
-#endif
 
 #include "vmware.h"
 #include "su.h"
 #include "vm_atomic.h"
 
 #if defined(__linux__)
+#ifndef GLIBC_VERSION_21
+/*
+ * SYS_ constants for glibc 2.0, some of which may already be defined on some of those
+ * older systems.
+ */
+#ifndef SYS_setresuid
+#define SYS_setresuid          164
+#endif
+#ifndef SYS_setresgid
+#define SYS_setresgid          170
+#endif
+#define SYS_setreuid32         203
+#define SYS_setregid32         204
+#define SYS_setresuid32        208
+#define SYS_setresgid32        210
+#define SYS_setuid32           213
+#define SYS_setgid32           214
+#endif // ifndef GLIBC_VERSION_21
+
 /*
  * 64bit linux has no 16 bit versions and
  * the 32bit versions have the un-suffixed names.
  * And obviously, we're not using glibc 2.0
  * for our 64bit builds!
  */
-#ifdef VM_64BIT
+#ifdef VM_X86_64
 #define SYS_setreuid32         (abort(), 0)
 #define SYS_setregid32         (abort(), 0)
 #define SYS_setresuid32        (abort(), 0)
-#define SYS_getresuid32        (abort(), 0)
 #define SYS_setresgid32        (abort(), 0)
-#define SYS_getresgid32        (abort(), 0)
 #define SYS_setuid32           (abort(), 0)
 #define SYS_setgid32           (abort(), 0)
 #endif
@@ -81,14 +83,14 @@
  * with the '32' suffix, so we get the behaviour we want by forcing
  * the code to use the unsuffixed syscalls.
  */
-#ifdef VM_64BIT
+#ifdef VM_X86_64
 static int uid32 = 0;
 #else
 static int uid32 = 1;
 #endif
 #endif // __linux__
 
-#if defined(__APPLE__) && !TARGET_OS_IPHONE
+#if defined(__APPLE__)
 #include <sys/kauth.h>
 
 static AuthorizationRef IdAuthCreate(void);
@@ -97,7 +99,7 @@ static AuthorizationRef IdAuthCreateWithFork(void);
 #endif
 
 
-#if !defined(__APPLE__)
+#if !defined(__APPLE__) && !defined(sun) && !defined(__FreeBSD__)
 /*
  *----------------------------------------------------------------------------
  *
@@ -119,9 +121,6 @@ static AuthorizationRef IdAuthCreateWithFork(void);
 int
 Id_SetUid(uid_t euid)		// IN: new euid
 {
-#if defined(__FreeBSD__) || defined(sun)
-   return setuid(euid);
-#elif defined(linux) || defined __ANDROID__
    if (uid32) {
       int r = syscall(SYS_setuid32, euid);
       if (r != -1 || errno != ENOSYS) {
@@ -130,9 +129,6 @@ Id_SetUid(uid_t euid)		// IN: new euid
       uid32 = 0;
    }
    return syscall(SYS_setuid, euid);
-#else
-#   error "Id_SetUid is not implemented for this platform"
-#endif
 }
 #endif
 
@@ -160,20 +156,21 @@ Id_SetGid(gid_t egid)		// IN: new egid
 {
 #if defined(__APPLE__)
    Warning("XXXMACOS: implement %s\n", __func__);
-
    return -1;
-#elif defined(sun) || defined(__FreeBSD__)
-   return setgid(egid);
+#elif defined(sun)
+   Warning("XXXSolaris: implement %s\n", __FUNCTION__);
+   return -1;
+#elif defined(__FreeBSD__)
+   Warning("XXXFreeBSD: implement %s\n", __FUNCTION__);
+   return -1;
 #else
    if (uid32) {
       int r = syscall(SYS_setgid32, egid);
-
       if (r != -1 || errno != ENOSYS) {
          return r;
       }
       uid32 = 0;
    }
-
    return syscall(SYS_setgid, egid);
 #endif
 }
@@ -201,63 +198,28 @@ Id_SetRESUid(uid_t uid,		// IN: new uid
 	     uid_t euid,	// IN: new effective uid
 	     uid_t suid)	// IN: new saved uid
 {
-#if (defined(__FreeBSD__) && __FreeBSD_version >= 500043)
-   return setresuid(uid, euid, suid);
-#elif defined(linux)
+#if defined(__APPLE__)
+   Warning("XXXMACOS: implement %s\n", __func__);
+   return -1;
+#elif defined(sun)
+   Warning("XXXSolaris: implement %s\n", __FUNCTION__);
+   return -1;
+#elif defined(__FreeBSD__)
+   Warning("XXXFreeBSD: implement %s\n", __FUNCTION__);
+   return -1;
+#else
    if (uid32) {
       int r = syscall(SYS_setresuid32, uid, euid, suid);
-
       if (r != -1 || errno != ENOSYS) {
          return r;
       }
       uid32 = 0;
    }
-
    return syscall(SYS_setresuid, uid, euid, suid);
-#else
-   Warning("XXX: implement %s\n", __func__);
-
-   return -1;
 #endif
 }
 
 
-#if defined(__linux__)
-/*
- *----------------------------------------------------------------------------
- *
- * IdGetRESUid --
- *
- *      Gets uid, effective uid and saved uid.
- *
- * Results:
- *      0 on success, -1 on failure.
- *
- * Side effects:
- *      errno may be modified on success
- *
- *----------------------------------------------------------------------------
- */
-
-static int
-IdGetRESUid(uid_t *uid,   // OUT:
-            uid_t *euid,  // OUT:
-            uid_t *suid)  // OUT:
-{
-   if (uid32) {
-      int r = syscall(SYS_getresuid32, uid, euid, suid);
-
-      if (r != -1 || errno != ENOSYS) {
-         return r;
-      }
-      uid32 = 0;
-   }
-
-   return syscall(SYS_getresuid, uid, euid, suid);
-}
-#endif
-
-
 #if !defined(__APPLE__)
 /*
  *----------------------------------------------------------------------------
@@ -281,64 +243,26 @@ Id_SetRESGid(gid_t gid,		// IN: new gid
 	     gid_t egid,	// IN: new effective gid
 	     gid_t sgid)	// IN: new saved gid
 {
-#if (defined(__FreeBSD__) && __FreeBSD_version >= 500043)
-   return setresgid(gid, egid, sgid);
-#elif defined(linux)
+#ifdef sun
+   Warning("XXXSolaris: implement %s\n", __FUNCTION__);
+   return -1;
+#elif defined(__FreeBSD__)
+   Warning("XXXFreeBSD: implement %s\n", __FUNCTION__);
+   return -1;
+#else
    if (uid32) {
       int r = syscall(SYS_setresgid32, gid, egid, sgid);
-
       if (r != -1 || errno != ENOSYS) {
          return r;
       }
       uid32 = 0;
    }
-
    return syscall(SYS_setresgid, gid, egid, sgid);
-#else
-   Warning("XXX: implement %s\n", __func__);
-
-   return -1;
 #endif
 }
 #endif
 
 
-#if defined(__linux__)
-/*
- *----------------------------------------------------------------------------
- *
- * IdGetRESGid --
- *
- *      Gets gid, effective gid and saved gid.
- *
- * Results:
- *      0 on success, -1 on failure.
- *
- * Side effects:
- *      errno may be modified on success
- *
- *----------------------------------------------------------------------------
- */
-
-static int
-IdGetRESGid(gid_t *gid,   // OUT:
-            gid_t *egid,  // OUT:
-            gid_t *sgid)  // OUT:
-{
-   if (uid32) {
-      int r = syscall(SYS_getresgid32, gid, egid, sgid);
-
-      if (r != -1 || errno != ENOSYS) {
-         return r;
-      }
-      uid32 = 0;
-   }
-
-   return syscall(SYS_getresgid, gid, egid, sgid);
-}
-#endif
-
-
 /*
  *----------------------------------------------------------------------------
  *
@@ -365,24 +289,26 @@ Id_SetREUid(uid_t uid,		// IN: new uid
 #if defined(__APPLE__)
    Warning("XXXMACOS: implement %s\n", __func__);
    return -1;
-#elif defined(sun) || defined(__FreeBSD__)
-   return setreuid(uid, euid);
+#elif defined(sun)
+   Warning("XXXSolaris: implement %s\n", __FUNCTION__);
+   return -1;
+#elif defined(__FreeBSD__)
+   Warning("XXXFreeBSD: implement %s\n", __FUNCTION__);
+   return -1;
 #else
    if (uid32) {
       int r = syscall(SYS_setreuid32, uid, euid);
-
       if (r != -1 || errno != ENOSYS) {
          return r;
       }
       uid32 = 0;
    }
-
    return syscall(SYS_setreuid, uid, euid);
 #endif
 }
 
 
-#if !defined(__APPLE__)
+#if !defined(__APPLE__) && !defined(sun) && !defined(__FreeBSD__)
 /*
  *----------------------------------------------------------------------------
  *
@@ -406,28 +332,58 @@ int
 Id_SetREGid(gid_t gid,		// IN: new gid
 	    gid_t egid)		// IN: new effective gid
 {
-#if defined(sun) || defined(__FreeBSD__)
-   return setregid(gid, egid);
-#else
    if (uid32) {
       int r = syscall(SYS_setregid32, gid, egid);
-
       if (r != -1 || errno != ENOSYS) {
          return r;
       }
       uid32 = 0;
    }
-
    return syscall(SYS_setregid, gid, egid);
-#endif
 }
 #endif
 
 
-#if defined(__APPLE__) && !TARGET_OS_IPHONE
+#if defined(__APPLE__)
 /*
  *----------------------------------------------------------------------------
  *
+ * Id_SetSuperUser --
+ *
+ *      If the calling process does not have euid root, do nothing.
+ *      If the calling process has euid root, make the calling thread acquire
+ *      or release euid root.
+ *
+ * Results:
+ *      None
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+Id_SetSuperUser(Bool yes) // IN: TRUE to acquire super user, FALSE to release
+{
+   if (!IsSuperUser() == !yes) {
+      // settid(2) fails on spurious transitions.
+      return;
+   }
+
+   if (yes) {
+      syscall(SYS_settid, KAUTH_UID_NONE, KAUTH_GID_NONE /* Ignored. */);
+   } else {
+      if (syscall(SYS_settid, getuid(), getgid()) == -1) {
+         Log("Failed to release super user privileges.\n");
+      }
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
  * IdAuthCreate --
  *
  *      Create an Authorization session.
@@ -444,7 +400,7 @@ Id_SetREGid(gid_t gid,		// IN: new gid
  * Side effects:
  *      See IdAuthCreateWithFork.
  *
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 static AuthorizationRef
@@ -465,7 +421,6 @@ IdAuthCreate(void)
     * euid or gid != egid?  Some callers may expect us to check
     * against euid, others may expect us to check against uid.
     */
-
    uid_t thread_uid;
    gid_t thread_gid;
    int ret;
@@ -477,28 +432,26 @@ IdAuthCreate(void)
        * We have per-thread UIDs in use, so Apple's authorization
        * APIs don't work.  Fork so we can use them.
        */
-
       return IdAuthCreateWithFork();
    } else {
       if (errno != ESRCH) {
-         Warning("%s: gettid failed, error %d.\n", __func__, errno);
-
+         Warning("%s: gettid failed, error %d.\n", __FUNCTION__, errno);
          return NULL;
       } else {
           // Per-thread identities are not in use in this thread.
          AuthorizationRef auth;
          OSStatus ret;
 
-         ret = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment,
-                                   kAuthorizationFlagDefaults, &auth);
+         ret = AuthorizationCreate(NULL,
+                                   kAuthorizationEmptyEnvironment,
+                                   kAuthorizationFlagDefaults,
+                                   &auth);
 
          if (ret == errAuthorizationSuccess) {
             return auth;
          } else {
-            ASSERT_ON_COMPILE(sizeof ret == sizeof (int32));
-            Warning("%s: AuthorizationCreate failed, error %d.\n",
-                    __func__, (int32)ret);
-
+            Warning("%s: AuthorizationCreate failed, error %ld.\n",
+                    __FUNCTION__, ret);
             return NULL;
          }
       }
@@ -507,7 +460,7 @@ IdAuthCreate(void)
 
 
 /*
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
  * IdAuthCreateWithFork --
  *
@@ -525,7 +478,7 @@ IdAuthCreate(void)
  * Side effects:
  *      The current process is forked.
  *
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 static AuthorizationRef
@@ -599,7 +552,7 @@ IdAuthCreateWithFork(void)
          result = waitpid(child, &status, 0);
       } while (result == -1 && errno == EINTR);
 
-      VERIFY(result == child);
+      ASSERT_NOT_IMPLEMENTED(result == child);
    } else {
       // Child: use fds[1]
 
@@ -623,10 +576,10 @@ IdAuthCreateWithFork(void)
           * Wait until the child can destroy its process ref to the auth
           * session.
           */
-
          for (;;) {
-            ssize_t actual = read(fds[1], &buf, sizeof buf);
+            ssize_t actual;
 
+            actual = read(fds[1], &buf, sizeof buf);
             ASSERT(actual <= sizeof buf);
             if (actual) {
                break;
@@ -639,7 +592,6 @@ IdAuthCreateWithFork(void)
        * o Destroys the child process ref to the Authorization session.
        * o Closes fds[0] and fds[1]
        */
-
       exit(0);
    }
 
@@ -655,7 +607,7 @@ static Atomic_Ptr procAuth = { 0 };
 
 
 /*
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
  * IdAuthGet --
  *
@@ -669,7 +621,7 @@ static Atomic_Ptr procAuth = { 0 };
  *      If the process' Authorization session does not exist yet, it is
  *      created.
  *
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 static AuthorizationRef
@@ -678,7 +630,9 @@ IdAuthGet(void)
    if (UNLIKELY(Atomic_ReadPtr(&procAuth) == NULL)) {
       AuthorizationRef newProcAuth = IdAuthCreate();
 
-      if (Atomic_ReadIfEqualWritePtr(&procAuth, NULL, newProcAuth)) {
+      if (Atomic_ReadIfEqualWritePtr(&procAuth,
+                                     NULL,
+                                     newProcAuth)) {
          // Someone else snuck in before we did.  Free the new authorization.
          AuthorizationFree(newProcAuth, kAuthorizationFlagDefaults);
       }
@@ -691,7 +645,7 @@ IdAuthGet(void)
 
 
 /*
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
  * Id_AuthGetLocal --
  *
@@ -705,7 +659,7 @@ IdAuthGet(void)
  *      If the process' Authorization session does not exist yet, it is
  *      created.
  *
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 void *
@@ -716,7 +670,7 @@ Id_AuthGetLocal(void)
 
 
 /*
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
  * Id_AuthGetExternal --
  *
@@ -729,7 +683,7 @@ Id_AuthGetLocal(void)
  * Side effects:
  *      None
  *
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 void *
@@ -746,25 +700,22 @@ Id_AuthGetExternal(size_t *size) // OUT
    ext = malloc(sizeof *ext);
    if (!ext) {
       Warning("Unable to allocate an AuthorizationExternalForm.\n");
-
       return NULL;
    }
 
    if (AuthorizationMakeExternalForm(auth, ext) != errAuthorizationSuccess) {
       Warning("AuthorizationMakeExternalForm() failed.\n");
       free(ext);
-
       return NULL;
    }
 
    *size = sizeof *ext;
-
    return ext;
 }
 
 
 /*
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
  * Id_AuthSet --
  *
@@ -778,7 +729,7 @@ Id_AuthGetExternal(size_t *size) // OUT
  * Side effects:
  *      None
  *
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 Bool
@@ -788,29 +739,28 @@ Id_AuthSet(void const *buf, // IN
    AuthorizationRef newProcAuth;
 
    AuthorizationExternalForm const *ext =
-                                      (AuthorizationExternalForm const *)buf;
+      (AuthorizationExternalForm const *)buf;
 
    if (!buf || size != sizeof *ext) {
       Warning("%s: Invalid argument.\n", __func__);
-
       return FALSE;
    }
 
    ASSERT(!Atomic_ReadPtr(&procAuth));
-   if (AuthorizationCreateFromExternalForm(ext,
-                                   &newProcAuth) != errAuthorizationSuccess) {
+   if (AuthorizationCreateFromExternalForm(ext, &newProcAuth)
+       != errAuthorizationSuccess) {
       Warning("AuthorizationCreateFromExternalForm failed.\n");
-
       return FALSE;
    }
 
-   if (Atomic_ReadIfEqualWritePtr(&procAuth, NULL, newProcAuth)) {
+   if (Atomic_ReadIfEqualWritePtr(&procAuth,
+                                  NULL,
+                                  newProcAuth)) {
       /*
        * This is meant to be called very early on in the life of the
        * process.  If someone else has snuck in an authorization,
        * we're toast.
        */
-
       NOT_IMPLEMENTED();
    }
 
@@ -819,7 +769,7 @@ Id_AuthSet(void const *buf, // IN
 
 
 /*
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
  * Id_AuthCheck --
  *
@@ -830,22 +780,19 @@ Id_AuthSet(void const *buf, // IN
  * Results:
  *      TRUE if the right was granted, FALSE if the user cancelled,
  *      entered the wrong password three times in a row, or if an
- *      error was encountered, or if the Authorization session is 
- *      invalid or has not been granted the 'right'.
+ *      error was encountered.
  *
  * Side effects:
- *      If showDialogIfNeeded is set and the specified Authorization session
- *      does not have the required privilege, displays a dialog to the user.
- *      The dialog grabs keyboard focus if Id_AuthSet() was previously called
- *      with a cross-process ref to a GUI process.
+ *      Displays a dialog to the user.  The dialog grabs keyboard
+ *      focus if Id_AuthSet() was previously called with a
+ *      cross-process ref to a GUI process.
  *
- *----------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 Bool
 Id_AuthCheck(char const *right,                // IN
-             char const *localizedDescription, // IN: UTF-8
-             Bool showDialogIfNeeded)          // IN
+             char const *localizedDescription) // IN: UTF-8
 {
    AuthorizationRef auth;
    AuthorizationItem rightsItems[1] = { { 0 } };
@@ -853,7 +800,7 @@ Id_AuthCheck(char const *right,                // IN
    AuthorizationItem environmentItems[1] = { { 0 } };
    AuthorizationEnvironment environmentWithDescription = { 0 };
    const AuthorizationEnvironment *environment =
-                                              kAuthorizationEmptyEnvironment;
+      kAuthorizationEmptyEnvironment;
 
    auth = IdAuthGet();
    if (!auth) {
@@ -872,7 +819,6 @@ Id_AuthCheck(char const *right,                // IN
     * If the localized description is present, the API uses that
     * description and appends a space followed by the above string.
     */
-
    if (localizedDescription) {
       environmentItems[0].name = kAuthorizationEnvironmentPrompt;
       environmentItems[0].valueLength = strlen(localizedDescription);
@@ -887,198 +833,12 @@ Id_AuthCheck(char const *right,                // IN
     * the same AuthorizationRef?  Apple's documentation doesn't
     * actually say whether it is or is not.
     */
-
-   return AuthorizationCopyRights(auth, &rights, environment,
-             (showDialogIfNeeded ? kAuthorizationFlagInteractionAllowed |
-                                   kAuthorizationFlagDefaults :
-                                   kAuthorizationFlagDefaults) |
+   return AuthorizationCopyRights(auth, &rights,
+             environment,
+             kAuthorizationFlagDefaults |
+             kAuthorizationFlagInteractionAllowed |
              kAuthorizationFlagExtendRights,
              NULL) == errAuthorizationSuccess;
 }
-#endif
 
-#if !defined(_WIN32)
-/*
- *----------------------------------------------------------------------------
- *
- * Id_BeginSuperUser --
- *
- *      Transition the calling thread from whatever its current effective
- *      user is to effectively root.
- *
- * Results:
- *      Returns the uid of the effective user from before the transition to
- *      effectively root. A "-1" is used to indicate that the thread was
- *      already root when this routine was called.
- *
- * Side effects:
- *      None
- *
- *----------------------------------------------------------------------------
- */
-
-uid_t
-Id_BeginSuperUser(void)
-{
-   uid_t uid = Id_GetEUid();
-
-   VERIFY(uid != (uid_t) -1);
-
-   if (uid == 0) {
-      uid = (uid_t) -1; // already root; nothing to do
-   } else {
-#if defined(__APPLE__)
-#if TARGET_OS_IPHONE
-      Warning("XXXIOS: implement %s\n", __func__);
-#else
-      syscall(SYS_settid, KAUTH_UID_NONE, KAUTH_GID_NONE /* Ignored. */);
-#endif
-#else
-      Id_SetRESUid((uid_t) -1, (uid_t) 0, (uid_t) -1); // effectively root
-#endif
-   }
-
-   return uid;
-}
-
-
-/*
- *----------------------------------------------------------------------------
- *
- * Id_EndSuperUser --
- *
- *      Transition the calling thread from effective root user to
- *      another user.
- *
- * Results:
- *      None
- *
- * Side effects:
- *      When transitioning from being effectively root to the specified
- *      user (uid) the effective gid of the calling thread may be lost.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-Id_EndSuperUser(uid_t uid)  // IN:
-{
-   if ((uid != (uid_t) -1) && (uid != Id_GetEUid())) {
-      ASSERT(uid != 0);  // Don't allow cheating like this
-
-#if defined(__APPLE__)
-      if (syscall(SYS_settid, uid, getgid()) == -1) {
-         Log("Failed to release super user privileges.\n");
-      }
-#else
-      Id_SetRESUid((uid_t) -1, uid, (uid_t) -1); // revert to uid
-#endif
-   }
-}
-
-
-#if defined(__linux__)
-/*
- *----------------------------------------------------------------------------
- *
- * IdIsSetUGid --
- *
- *      Check if the binary is setuid or setgid.
- *
- * Results:
- *      TRUE if the process should be considered setuid or setgid.
- *
- * Side effects:
- *      None
- *
- *----------------------------------------------------------------------------
- */
-
-static Bool
-IdIsSetUGid(void)
-{
-#if defined(__ANDROID__)
-   /* Android does not have a secure_getenv, so be conservative. */
-   return TRUE;
-#else
-   /*
-    * We use __secure_getenv, which returns NULL if the binary is
-    * setuid or setgid. Alternatives include,
-    *
-    *   a) getauxval(AT_SECURE); not available until glibc 2.16.
-    *   b) __libc_enable_secure; may not be exported.
-    *
-    * Use (a) when we are based on glibc 2.16, or newer.
-    */
-
-#if defined(__GLIBC__) && \
-           (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 16))
-   return getauxval(AT_SECURE) != 0;
-#else
-   static const char envName[] = "VMW_SETUGID_TEST";
-
-   if (setenv(envName, "1", TRUE) == -1) {
-      return TRUE; /* Conservative */
-   }
-   return __secure_getenv(envName) == NULL;
-#endif
-#endif
-}
-#endif
-
-
-/*
- *----------------------------------------------------------------------------
- *
- * Id_IsSetUGid --
- *
- *      Check if the environment should be treated with suspicion in a
- *      security sensitive context.
- *
- *      Most commonly, this will return TRUE when a binary is setuid or
- *      setgid, but will also be TRUE when the uid does not match the
- *      effective uid (for example if the current binary was exec'd from
- *      a setuid or setgid binary without proper uid scrubbing).
- *
- *      Detecting dangerous environments is best-effort and there are
- *      some known holes (see PR 1085298), so the best practice is to
- *      design systems such that this checking is not required.
- *
- * Results:
- *      TRUE if the process should be treated with suspicion in a
- *      security sensitive context. Otherwise FALSE.
- *
- * Side effects:
- *      None
- *
- *----------------------------------------------------------------------------
- */
-
-Bool
-Id_IsSetUGid(void)
-{
-#if defined(__APPLE__) || defined(__FreeBSD__) || defined(sun)
-   uid_t ruid = getuid();
-   uid_t euid = geteuid();
-   gid_t rgid = getgid();
-   gid_t egid = getegid();
-
-   return issetugid() == 1 || ruid != euid || rgid != egid;
-#elif defined(__linux__)
-   uid_t ruid, euid, suid;
-   gid_t rgid, egid, sgid;
-
-   /* We want to make sure that this call always succeeds. */
-   if (IdGetRESUid(&ruid, &euid, &suid) != 0 ||
-       IdGetRESGid(&rgid, &egid, &sgid) != 0) {
-      return TRUE; /* Conservative */
-   }
-
-   return IdIsSetUGid() ||
-          ruid != euid || ruid != suid ||
-          rgid != egid || rgid != sgid;
-#else
-   return TRUE; /* Conservative */
-#endif
-}
 #endif
