@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -109,8 +109,7 @@ IOV_Zero(VMIOVec *iov)   // IN
    while (numBytesLeft > 0) {
       size_t c = MIN(numBytesLeft, iov->entries[i].iov_len);
       void *buf;
-
-      VERIFY(i < iov->numEntries);
+      ASSERT_NOT_IMPLEMENTED(i < iov->numEntries);
       buf = iov->entries[i].iov_base;
       ASSERT(buf && buf != LAZY_ALLOC_MAGIC);
       memset(buf, 0, c);
@@ -177,7 +176,7 @@ IOV_DuplicateStatic(VMIOVec *iovIn,                     // IN
    ASSERT(iovIn);
    ASSERT(iovOut);
 
-   Util_Memcpy(iovOut, iovIn, sizeof *iovOut);
+   memcpy(iovOut, iovIn, sizeof *iovOut);
    if (iovIn->numEntries <= numStaticEntries) {
       iovOut->allocEntries = NULL;
       iovOut->entries = staticEntries;
@@ -186,7 +185,7 @@ IOV_DuplicateStatic(VMIOVec *iovIn,                     // IN
                                              sizeof(struct iovec));
       iovOut->entries = iovOut->allocEntries;
    }
-   Util_Memcpy(iovOut->entries, iovIn->entries, 
+   memcpy(iovOut->entries, iovIn->entries, 
           iovIn->numEntries * sizeof(struct iovec));
 }
 
@@ -260,7 +259,6 @@ IOV_IsZero(VMIOVec* iov)      // IN: the iov to scan
          return FALSE;
       }
    }
-
    return TRUE;
 }
 
@@ -312,17 +310,15 @@ IOVSplitList(VMIOVec *regionV,      // IN/OUT: VMIOVec for this region
       if (regionV->numBytes > regionV->numSectors * sectorSize) {
          int spillover;
          
-         spillover = (int) (regionV->numBytes -
-                                            regionV->numSectors * sectorSize);
+         spillover = (int) (regionV->numBytes - regionV->numSectors * sectorSize);
          ASSERT(spillover < curEntry->iov_len);
          ASSERT(spillover > 0);
 
          /*
-          * Truncate the last overlapping entry and store the excess. After
-          * we finish this region, we'll smash this last entry with its
-          * remainder and just move on to the next region.
+          *  Truncate the last overlapping entry and store the excess. After we
+          *  finish this region, we'll smash this last entry with its remainder
+          *  and just move on to the next region.
           */
-
          regionV->numBytes -= spillover;
          curEntry->iov_len -= spillover;
          overlap->iov_len = spillover;
@@ -330,11 +326,10 @@ IOVSplitList(VMIOVec *regionV,      // IN/OUT: VMIOVec for this region
          break;
       } else if (regionV->numBytes == regionV->numSectors * sectorSize) {
          /*
-          * Clean finish. The last entry for this region will be handled
-          * with no overlap, so increment past it for the start of the next
-          * region's entries.
+          *  Clean finish.  The last entry for this region will be handled with
+          *  no overlap, so increment past it for the start of the next
+          *  region's entries.
           */
-
          overlap->iov_len = 0;
          curEntry++;
          break;
@@ -386,23 +381,21 @@ IOV_Split(VMIOVec *origV,         // IN/OUT: VMIOVec for whole xfer
    /*
     * The resulting iov cannot have more entries than the incoming one.
     */
-
    v = Util_SafeMalloc(sizeof *v + origV->numEntries * sizeof(struct iovec));
    iov = &v->iov;
-   Util_Memcpy(iov, origV, sizeof *iov);
+   memcpy(iov, origV, sizeof *iov);
    iov->allocEntries = NULL;
    iov->numSectors = numSectors;
 
    /*
     * Handle lazy allocation of backing store.
     */
-
    if (origV->entries->iov_base == LAZY_ALLOC_MAGIC && 
        origV->entries->iov_len == 0) {
 
       ASSERT(origV->numEntries == 1);
       iov->entries = v->e;
-      Util_Memcpy(iov->entries, origV->entries, sizeof(struct iovec));
+      memcpy(iov->entries, origV->entries, sizeof(struct iovec));
 
       iov->numBytes = iov->numSectors * sectorSize;
 
@@ -417,7 +410,7 @@ IOV_Split(VMIOVec *origV,         // IN/OUT: VMIOVec for whole xfer
    if (origV->numSectors == numSectors) {
       cpySize = origV->numEntries * sizeof *origV->entries;
       iov->entries = v->e;
-      Util_Memcpy(iov->entries, origV->entries, cpySize);
+      memcpy(iov->entries, origV->entries, cpySize);
 
       origV->startSector += numSectors;
       origV->numSectors = 0;
@@ -437,7 +430,7 @@ IOV_Split(VMIOVec *origV,         // IN/OUT: VMIOVec for whole xfer
       cpySize = iov->numEntries * sizeof *iov->entries;
       tmpPtr = iov->entries;
       iov->entries = v->e;
-      Util_Memcpy(iov->entries, tmpPtr, cpySize);
+      memcpy(iov->entries, tmpPtr, cpySize);
 
       origV->numEntries -= iov->numEntries;
       if (overlap.iov_len != 0) { 
@@ -460,7 +453,7 @@ IOV_Split(VMIOVec *origV,         // IN/OUT: VMIOVec for whole xfer
  *
  *      This function takes an iov and a buffer as input and writes the content
  *      of the buffers pointed to by the iov into buf.
- *
+ * 
  * Result:
  *      None.
  *
@@ -471,10 +464,10 @@ IOV_Split(VMIOVec *origV,         // IN/OUT: VMIOVec for whole xfer
  */
 
 void
-IOV_WriteIovToBuf(struct iovec const *entries, // IN
-                  int numEntries,              // IN
-                  uint8 *bufOut,               // OUT
-                  size_t bufSize)              // IN
+IOV_WriteIovToBuf(struct iovec* entries,   // IN
+                  int numEntries,          // IN
+                  uint8* bufOut,           // OUT
+                  size_t bufSize)          // IN
 {
    size_t count = 0;
    int i;
@@ -490,14 +483,14 @@ IOV_WriteIovToBuf(struct iovec const *entries, // IN
 
       numBytes = MIN(bufSize - count, entries[i].iov_len);
 
-      Util_Memcpy(&bufOut[count], entries[i].iov_base, numBytes);
+      memcpy(&bufOut[count], entries[i].iov_base, numBytes);
       count += numBytes;
 
       if (count >= bufSize) {
          return;
       }
 
-      VERIFY(count <= bufSize);
+      ASSERT_NOT_IMPLEMENTED(count <= bufSize);
    }
 }
 
@@ -525,12 +518,11 @@ IOV_Duplicate(VMIOVec* iovIn)     // IN
    struct VMIOVecAndEntries* v;
 
    v = Util_SafeMalloc(sizeof *v + iovIn->numEntries * sizeof(struct iovec));
-   Util_Memcpy(&v->iov, iovIn, sizeof *iovIn);
+   memcpy(&v->iov, iovIn, sizeof *iovIn);
    v->iov.allocEntries = NULL;
    v->iov.entries = v->e;
-   Util_Memcpy(v->iov.entries, iovIn->entries,
+   memcpy(v->iov.entries, iovIn->entries,
           iovIn->numEntries * sizeof(struct iovec));
-
    return &v->iov;
 }
 
@@ -570,7 +562,7 @@ IOV_Free(VMIOVec* iov)     // IN
  *
  *      This function copies the content of bufIn into the buffer pointed to by
  *      entries. It basically does the opposite of IOV_WriteIovToBuf.
- *
+ * 
  * Result:
  *      None.
  *
@@ -581,16 +573,16 @@ IOV_Free(VMIOVec* iov)     // IN
  */
 
 void
-IOV_WriteBufToIov(const uint8 *bufIn,          // IN
-                  size_t bufSize,              // IN
-                  struct iovec const *entries, // OUT
-                  int numEntries)              // IN
+IOV_WriteBufToIov(uint8* bufIn,                 // IN
+                  size_t bufSize,               // IN
+                  struct iovec* entries,        // OUT
+                  int numEntries)               // IN
 {
    size_t count = 0;
    int i;
 
    ASSERT(entries);
-   VERIFY_BUG(29009, bufIn);
+   ASSERT_BUG(29009, bufIn);
 
    for (i = 0; i < numEntries; i++) {
       size_t numBytes;
@@ -600,12 +592,12 @@ IOV_WriteBufToIov(const uint8 *bufIn,          // IN
 
       numBytes = MIN(bufSize - count, entries[i].iov_len);
 
-      Util_Memcpy(entries[i].iov_base, &bufIn[count], numBytes);
+      memcpy(entries[i].iov_base, &bufIn[count], numBytes);
       count += numBytes;
       if (count >= bufSize) {
          return;
       }
-      VERIFY(count <= bufSize);
+      ASSERT_NOT_IMPLEMENTED(count <= bufSize);
    }
 }
 
@@ -652,7 +644,6 @@ IOVFindFirstEntryOffset(struct iovec* entries,        // IN
       Log(LGPFX"%s:%d i %d (of %d), offsets: entry %"FMTSZ"u, iov %"FMTSZ"u "
           "invalid iov offset\n",
           __FILE__, __LINE__, i, numEntries, entryOffset, iovOffset);
-
       return numEntries;
    }
 
@@ -661,7 +652,6 @@ IOVFindFirstEntryOffset(struct iovec* entries,        // IN
    ASSERT(entryOffset < entryLen);
 
    *entryOffsetp = entryOffset;
-
    return i - 1;
 }
 
@@ -697,23 +687,20 @@ IOV_WriteIovToBufPlus(struct iovec* entries,    // IN
    size_t count = bufSize;
    int i;
 
-   VERIFY_BUG(29009, bufOut);
+   ASSERT_BUG(29009, bufOut);
 
    i = IOVFindFirstEntryOffset(entries, numEntries, iovOffset, &entryOffset);
 
    for (; count && (i < numEntries); i++) {
-      char *base = (char *)(entries[i].iov_base) + entryOffset;
-      size_t iov_len = entries[i].iov_len;
-
       ASSERT(entries[i].iov_base || entries[i].iov_len == 0);
       ASSERT(entries[i].iov_base != LAZY_ALLOC_MAGIC);
 
-      if (iov_len <= 0) {
+      if (entries[i].iov_len <= 0) {
          continue;
       }
-      entryLen = MIN(count, iov_len - entryOffset);
+      entryLen = MIN(count, entries[i].iov_len - entryOffset);
 
-      Util_Memcpy(bufOut, base, entryLen);
+      memcpy(bufOut, (char *)(entries[i].iov_base) + entryOffset, entryLen);
 
       count -= entryLen;
       bufOut += entryLen;
@@ -756,23 +743,20 @@ IOV_WriteBufToIovPlus(uint8* bufIn,             // IN
    size_t count = bufSize;
    int i;
 
-   VERIFY_BUG(29009, bufIn);
+   ASSERT_BUG(29009, bufIn);
 
    i = IOVFindFirstEntryOffset(entries, numEntries, iovOffset, &entryOffset);
 
    for (; count && (i < numEntries); i++) {
-      char *base = (char *)(entries[i].iov_base) + entryOffset;
-      size_t iov_len = entries[i].iov_len;
-
-      VERIFY_BUG(33859, entries[i].iov_base || entries[i].iov_len == 0);
+      ASSERT_BUG(33859, entries[i].iov_base || entries[i].iov_len == 0);
       ASSERT(entries[i].iov_base != LAZY_ALLOC_MAGIC);
 
-      if (iov_len <= 0) {
+      if (entries[i].iov_len <= 0) {
          continue;
       }
-      entryLen = MIN(count, iov_len - entryOffset);
-      
-      Util_Memcpy(base, bufIn, entryLen);
+      entryLen = MIN(count, entries[i].iov_len - entryOffset);
+
+      memcpy((char *)(entries[i].iov_base) + entryOffset, bufIn, entryLen);
 
       count -= entryLen;
       bufIn += entryLen;
@@ -817,10 +801,7 @@ IOV_WriteIovToIov(VMIOVec *srcIov,        // IN
    size_t entryLen = 0, srcEntryOffset, copyLen, retval;
    uint64 srcStartByte, dstStartByte, startByte, endByte;
    int64 count, srcIovOffset, dstIovOffset;
-   struct iovec *srcEntries = srcIov->entries;
-   uint32 srcNumEntries = srcIov->numEntries;
-   struct iovec *dstEntries = dstIov->entries;
-   uint32 dstNumEntries = dstIov->numEntries;
+   struct iovec *srcEntries;
    int i;
 
    ASSERT(srcIov);
@@ -843,7 +824,6 @@ IOV_WriteIovToIov(VMIOVec *srcIov,        // IN
       Log(LGPFX"%s:%d iov [%"FMT64"u:%"FMT64"u] and [%"FMT64"u:%"FMT64"u] - "
           "no overlap!\n", __FILE__, __LINE__, srcIov->startSector,
           srcIov->numSectors, dstIov->startSector, dstIov->numSectors);
-
       return 0;
    }
 
@@ -863,24 +843,24 @@ IOV_WriteIovToIov(VMIOVec *srcIov,        // IN
    retval = (size_t)count;
 
    /* first find the src entry where to start */
-   i = IOVFindFirstEntryOffset(srcEntries, srcNumEntries,
+   i = IOVFindFirstEntryOffset(srcEntries, srcIov->numEntries,
                                (size_t) srcIovOffset, &srcEntryOffset);
 
-   for (; count && (i < srcNumEntries); i++) {
-      size_t iov_len = srcEntries[i].iov_len;
-
+   for (; count && (i < srcIov->numEntries); i++) {
       ASSERT(srcEntries[i].iov_base || srcEntries[i].iov_len == 0);
       ASSERT(srcEntries[i].iov_base != LAZY_ALLOC_MAGIC);
 
-      if (iov_len <= 0) {
+      if (srcEntries[i].iov_len <= 0) {
          continue;
       }
-      entryLen = MIN(count, iov_len - srcEntryOffset);
+      entryLen = MIN(count, srcEntries[i].iov_len - srcEntryOffset);
 
       copyLen = IOV_WriteBufToIovPlus(
-                           (uint8 *)(srcEntries[i].iov_base) + srcEntryOffset,
-                                      entryLen, dstEntries,
-                                      dstNumEntries, dstIovOffset);
+                     (uint8 *)(srcEntries[i].iov_base) + srcEntryOffset,
+                     entryLen,
+                     dstIov->entries,
+                     dstIov->numEntries,
+                     dstIovOffset);
 
       if (copyLen == 0) {  /* finished */
          break;
