@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -31,17 +31,17 @@
 #  include <Windows.h>
 #else // Posix
 #  include <unistd.h>
-#  include <signal.h>
 #endif // Win32 vs Posix
 
 #include "vmware.h"
+#include "productState.h"
+#include "vm_version.h"
 #include "log.h"
 #include "panic.h"
 #include "msg.h"
 #include "str.h"
 #include "config.h"
 #include "util.h"
-#include "userlock.h"
 #if defined(_WIN32) || !defined(VMX86_TOOLS)
 #include "coreDump.h"
 #endif
@@ -218,31 +218,6 @@ Panic_GetCoreDumpFlags(void)
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * Panic_SetCoreDumpFlags --
- *
- *      Although the core dump flags are read at init time, we may want to
- *      update this value later. This is especially true because the default
- *      value is read before the VM config is established.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      Updates global struct panicState.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-Panic_SetCoreDumpFlags(int flags)   // IN
-{
-   panicState.coreDumpFlags = flags;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -290,38 +265,13 @@ Panic_LoopOnPanic(void)
 void
 Panic_BreakOnPanic(void)
 {
-#if defined(_WIN32)
+#ifdef _WIN32
    if (Panic_GetBreakOnPanic()) {
       Warning("Panic: breaking into debugger\n");
       DebugBreak();
    }
-#else // Posix
-   switch (panicState.breakOnPanic) {
-   case PanicBreakLevel_Never:
-      break;
-   case PanicBreakLevel_IfDebuggerAttached:
-      {
-         void (*handler)(int);
-         handler = signal(SIGTRAP, SIG_IGN);
-
-         /*
-          * INT3 is not always ignored, so explicitely use kill() here.
-          */
-         kill(getpid(), SIGTRAP);
-
-         signal(SIGTRAP, handler);
-      }
-      break;
-   default:
-   case PanicBreakLevel_Always:
-      Warning("Panic: breaking into debugger\n");
-#  if defined(__GNUC__) && (defined(__x86_64__) || defined(__i386__))
-      __asm__ __volatile__ ("int3");
-#  else
-      kill(getpid(), SIGTRAP);
-#  endif
-      break;
-   }
+#else
+   /* XXX: Linux experts: is there any equivalent of attaching a JIT debugger? */
 #endif
 }
 
@@ -392,11 +342,7 @@ Panic_GetBreakOnPanic(void)
          }
       }
 #else
-      /*
-       * This case is handled by Panic_BreakOnPanic for Posix as there is no
-       * portable way to know if we're being debugged other than actually
-       * trapping into the debugger.
-       */
+      /* XXX: Linux experts?  How do you know if you're being ptraced? */
 #endif
       break;
    default:
@@ -495,8 +441,6 @@ Panic_Panic(const char *format,
    char buf[1024];
    static int count = 0;
 
-   MXUser_SetInPanic();
-
    Str_Vsnprintf(buf, sizeof buf, format, args);
 
    /*
@@ -509,16 +453,7 @@ Panic_Panic(const char *format,
 
    fputs(buf, stderr);
 #ifdef _WIN32
-   /*
-    * This would nominally be Win32U_OutputDebugString.  However,
-    * OutputDebugString is unusual in that the W version converts
-    * to local encoding and calls the A version.
-    *
-    * Since any such conversion is risky (read: can Panic) and
-    * we haven't yet hit the loop detection, we will conservatively
-    * dump UTF-8 via the A version.
-    */
-   OutputDebugStringA(buf);
+   Win32U_OutputDebugString(buf);
 #endif
 
    /*
@@ -537,7 +472,7 @@ Panic_Panic(const char *format,
    case 0:
       break;
    case 1:
-      Log("PANIC: %s", buf);
+      Log(buf);
       Log("Panic loop\n");
    default:
       fprintf(stderr, "Panic loop\n");
@@ -545,21 +480,14 @@ Panic_Panic(const char *format,
       NOT_REACHED();
    }
 
-#ifdef _WIN32
    /*
-    * Output again, in a way that we hope localizes correctly.  Since
-    * we are converting, this can Panic, so it must run after loop
-    * detection.
-    */
-   Win32U_OutputDebugString(buf);
-#endif
-
-   /*
-    * Log panic information.
+    * Log panic information, and make sure we don't remove
+    * the log file on exit.
     */
 
-   Log("PANIC: %s", buf);
+   Log(buf);
    Util_Backtrace(0);
+   Log_SetAlwaysKeep(TRUE);
 
    /*
     * Do the debugging steps early before we have a chance
@@ -583,10 +511,9 @@ Panic_Panic(const char *format,
 
    Panic_PostPanicMsg(buf);
 
-   /*
+   /* 
     * Bye
     */
-   Log("Exiting\n");
 
    exit(-1);
    NOT_REACHED();
