@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -20,18 +20,6 @@
  * str.c --
  *
  *    User level string wrappers
- *
- * WARNING:
- *    Do not call any variadic functions - those that use "..." repeatedly
- *    with the same va_list or memory corruption and/or crashes will occur.
- *    The suggested way deal with repeated calls is to use a va_copy:
- *
- *    va_list tmpArgs;
- *
- *    va_copy(tmpArgs, ap);
- *    // Call the variadic function
- *    va_end(tmpArgs);
- *
  */
 
 #ifdef _WIN32
@@ -57,139 +45,25 @@
 extern int vasprintf(char **ptr, const char *f, va_list arg);
 /*
  * Declare vswprintf on platforms where it's not known to exist.  We know
- * it's available on glibc >= 2.2, FreeBSD >= 5.0, and all versions of
+ * it's available on glibc >= 2.2, FreeBSD >= 5.0, and all versions of 
  * Solaris.
  * (Re: Solaris, vswprintf has been present since Solaris 8, and we only
  * support Solaris 9 and above, since that was the first release available
  * for x86, so we just assume it's already there.)
  *
  * XXX Str_Vsnwprintf and friends are still protected by _WIN32 and
- * glibc >= 2.2.  I.e., even though they should be able to work on
+ * GLIBC_VERSION_22.  I.e., even though they should be able to work on
  * FreeBSD 5.0+ and Solaris 8+, they aren't made available there.
  */
-#   if !(defined(__linux__) ||                                          \
+#   if !(defined(GLIBC_VERSION_22) ||                                   \
          (defined(__FreeBSD__) && (__FreeBSD_version >= 500000)) ||     \
          defined(sun))
 extern int vswprintf(wchar_t *wcs, size_t maxlen, const wchar_t *format, va_list args);
 #   endif
 #endif // _WIN32
 
-
-/*
- *----------------------------------------------------------------------
- *
- * Str_Vsnprintf --
- *
- *      Compatibility wrapper b/w different libc versions
- *
- * Results:
- *
- *      int - number of bytes stored in 'str' (not including NUL
- *      terminate character), -1 on overflow (insufficient space for
- *      NUL terminate is considered overflow)
- *
- *      NB: on overflow the buffer WILL be NUL terminated at the last
- *      UTF-8 code point boundary within the buffer's bounds.
- *
- * WARNING: See warning at the top of this file.
- *
- * Side effects:
- *      None
- *
- *----------------------------------------------------------------------
- */
-
-int
-Str_Vsnprintf(char *str,          // OUT
-              size_t size,        // IN
-              const char *format, // IN
-              va_list ap)         // IN
-{
-   int retval;
-
-   ASSERT(str != NULL);
-   ASSERT(format != NULL);
-
-#if defined HAS_BSD_PRINTF && !defined __ANDROID__
-   retval = bsd_vsnprintf(&str, size, format, ap);
-#else
-   retval = vsnprintf(str, size, format, ap);
-#endif
-
-   /*
-    * Linux glibc 2.0.x returns -1 and NUL terminates (which we shouldn't
-    * be linking against), but glibc 2.1.x follows c99 and returns
-    * characters that would have been written.
-    *
-    * In the case of Win32 and !HAS_BSD_PRINTF, we are using
-    * _vsnprintf(), which returns -1 on overflow, returns size
-    * when result fits exactly, and does not NUL terminate in
-    * those cases.
-    */
-
-   if ((retval < 0) || (retval >= size)) {
-      if (size > 0) {
-         /* Find UTF-8 code point boundary and place NUL termination there */
-         int trunc = CodeSet_Utf8FindCodePointBoundary(str, size - 1);
-
-         str[trunc] = '\0';
-      }
-   }
-   if (retval >= size) {
-      return -1;
-   }
-   return retval;
-}
-
-
-#ifdef HAS_BSD_PRINTF
-/*
- *----------------------------------------------------------------------
- *
- * Str_Sprintf_C_Locale --
- *
- *      sprintf wrapper that fails on overflow. Enforces numeric C locale.
- *
- * Results:
- *      Returns the number of bytes stored in 'buf'.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-Str_Sprintf_C_Locale(char *buf,        // OUT:
-                     size_t maxSize,   // IN:
-                     const char *fmt,  // IN:
-                     ...)              // IN:
-{
-   va_list args;
-   int retval;
-
-   ASSERT(buf);
-   ASSERT(fmt);
-
-   va_start(args, fmt);
-   retval = bsd_vsnprintf_c_locale(&buf, maxSize, fmt, args);
-   va_end(args);
-
-   if ((retval < 0) || (retval >= maxSize)) {
-      if (maxSize > 0) {
-         /* Find UTF-8 code point boundary and place NUL termination there */
-         int trunc = CodeSet_Utf8FindCodePointBoundary(buf, maxSize - 1);
-
-         buf[trunc] = '\0';
-      }
-   }
-
-   if (retval >= maxSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
-   }
-
-   return retval;
-}
+#ifdef N_PLAT_NLM
+extern int vsnprintf(char *buf, size_t len, const char *f, va_list arg);
 #endif
 
 
@@ -215,14 +89,15 @@ Str_Sprintf(char *buf,       // OUT
             const char *fmt, // IN
             ...)             // IN
 {
+   uint32 *stack = (uint32*) &buf;
    va_list args;
    int i;
-
-   va_start(args, fmt);
+   
+   va_start(args,fmt);
    i = Str_Vsnprintf(buf, maxSize, fmt, args);
    va_end(args);
    if (i < 0) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
    }
    return i;
 }
@@ -231,25 +106,95 @@ Str_Sprintf(char *buf,       // OUT
 /*
  *----------------------------------------------------------------------
  *
- * Str_Snprintf --
+ * Str_Vsnprintf --
  *
- *      Compatibility wrapper b/w different libc versions
+ *	Compatibility wrapper b/w different libc versions
  *
  * Results:
  *
- *      int - number of bytes stored in 'str' (not including NUL
- *      terminate character), -1 on overflow (insufficient space for
- *      NUL terminate is considered overflow)
+ *	int - number of bytes stored in 'str' (not including null
+ *	terminate character), -1 on overflow (insufficient space for
+ *	null terminate is considered overflow)
  *
- *      NB: on overflow the buffer WILL be NUL terminated
+ *	NB: on overflow the buffer WILL be null terminated at the last
+ *	UTF-8 code point boundary within the buffer's bounds.
  *
  * Side effects:
- *      None
+ *	None
  *
  *----------------------------------------------------------------------
  */
 
 int
+Str_Vsnprintf(char *str,          // OUT
+              size_t size,        // IN
+              const char *format, // IN
+              va_list ap)         // IN
+{
+   int retval;
+
+   ASSERT(str != NULL);
+   ASSERT(format != NULL);
+
+#ifndef HAS_BSD_PRINTF
+   retval = vsnprintf(str, size, format, ap);
+#elif defined __linux__
+   {
+      va_list aq;
+
+      va_copy(aq, ap);
+      retval = bsd_vsnprintf(&str, size, format, aq);
+      va_end(aq);
+   }
+#else
+   retval = bsd_vsnprintf(&str, size, format, ap);
+#endif
+
+   /*
+    * Linux glibc 2.0.x returns -1 and null terminates (which we shouldn't
+    * be linking against), but glibc 2.1.x follows c99 and returns
+    * characters that would have been written.
+    *
+    * In the case of Win32 and !HAS_BSD_PRINTF, we are using
+    * _vsnprintf(), which returns -1 on overflow, returns size
+    * when result fits exactly, and does not null terminate in
+    * those cases.
+    */
+
+   if ((retval < 0 || retval >= size) && size > 0) {
+      /* Find UTF-8 code point boundary and place NUL termination there */
+      int trunc = CodeSet_Utf8FindCodePointBoundary(str, size - 1);
+      str[trunc] = '\0';
+   }
+   if (retval >= size) {
+      return -1;
+   }
+   return retval;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Str_Snprintf --
+ *
+ *	Compatibility wrapper b/w different libc versions
+ *
+ * Results:
+ *
+ *	int - number of bytes stored in 'str' (not including null
+ *	terminate character), -1 on overflow (insufficient space for
+ *	null terminate is considered overflow)
+ *
+ *	NB: on overflow the buffer WILL be null terminated
+ *
+ * Side effects:
+ *	None
+ *
+ *----------------------------------------------------------------------
+ */
+
+int      
 Str_Snprintf(char *str,          // OUT
              size_t size,        // IN
              const char *format, // IN
@@ -264,7 +209,6 @@ Str_Snprintf(char *str,          // OUT
    va_start(args, format);
    retval = Str_Vsnprintf(str, size, format, args);
    va_end(args);
-
    return retval;
 }
 
@@ -290,6 +234,7 @@ Str_Strcpy(char *buf,       // OUT
            const char *src, // IN
            size_t maxSize)  // IN
 {
+   uint32 *stack = (uint32 *)&buf;
    size_t len;
 
    ASSERT(buf != NULL);
@@ -297,56 +242,22 @@ Str_Strcpy(char *buf,       // OUT
 
    len = strlen(src);
    if (len >= maxSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      ASSERT_BUG(5686, FALSE);
    }
    return memcpy(buf, src, len + 1);
 }
 
-
-/*
- *----------------------------------------------------------------------
- *
- * Str_Strlen --
- *
- *      Calculate length of the string.
- *
- * Results:
- *      Length of s not including the terminating '\0' character.
- *      If there is no '\0' for first maxLen bytes, then it
- *      returns maxLen.
- *
- * Side Effects:
- *      None
- *
- *----------------------------------------------------------------------
- */
-
-size_t
-Str_Strlen(const char *s,  // IN:
-           size_t maxLen)  // IN:
-
-{
-   const char *end;
-
-   ASSERT(s != NULL);
-
-   if ((end = memchr(s, '\0', maxLen)) == NULL) {
-      return maxLen;
-   }
-   return end - s;
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
  * Str_Strnstr --
  *
- *      Find a substring within a string of length at most n. 'sub' must be
+ *	Find a substring within a string of length at most n. 'sub' must be
  *      NUL-terminated. 'n' is interpreted as an unsigned int.
  *
  * Results:
- *      A pointer to the beginning of the substring, or NULL if not found.
+ *	A pointer to the beginning of the substring, or NULL if not found.
  *
  * Side Effects:
  *      None
@@ -355,9 +266,10 @@ Str_Strlen(const char *s,  // IN:
  */
 
 char *
-Str_Strnstr(const char *src,  // IN:
-            const char *sub,  // IN:
-            size_t n)         // IN:
+Str_Strnstr(const char *src,  	// IN 
+            const char *sub,	// IN
+            size_t n)         	// IN
+
 {
    size_t subLen;
    const char *end;
@@ -376,9 +288,9 @@ Str_Strnstr(const char *src,  // IN:
       return NULL;
    }
    for (;
-       (src = memchr(src, sub[0], end - src)) != NULL &&
-        memcmp(src, sub, subLen) != 0;
-        src++) {
+	(src = memchr(src, sub[0], end - src)) != NULL &&
+	memcmp(src, sub, subLen) != 0;
+	src++) {
    }
    return (char *) src;
 }
@@ -401,10 +313,11 @@ Str_Strnstr(const char *src,  // IN:
  */
 
 char *
-Str_Strcat(char *buf,       // IN/OUT
+Str_Strcat(char *buf,       // IN-OUT
            const char *src, // IN
            size_t maxSize)  // IN
 {
+   uint32 *stack = (uint32 *)&buf;
    size_t bufLen;
    size_t srcLen;
 
@@ -416,7 +329,7 @@ Str_Strcat(char *buf,       // IN/OUT
 
    /* The first comparison checks for numeric overflow */
    if (bufLen + srcLen < srcLen || bufLen + srcLen >= maxSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
    }
 
    memcpy(buf + bufLen, src, srcLen + 1);
@@ -445,16 +358,20 @@ Str_Strcat(char *buf,       // IN/OUT
  */
 
 char *
-Str_Strncat(char *buf,       // IN/OUT
+Str_Strncat(char *buf,       // IN-OUT
             size_t bufSize,  // IN: Size of buf
             const char *src, // IN: String to append
             size_t n)        // IN: Max chars of src to append
 {
-   size_t bufLen;
+   uint32 *stack; 
+   size_t bufLen; 
 
    ASSERT(buf != NULL);
    ASSERT(src != NULL);
 
+   stack = (uint32 *)&buf;
+   bufLen = strlen(buf);
+
    /*
     * Check bufLen + n first so we can avoid the second call to strlen
     * if possible.
@@ -462,20 +379,18 @@ Str_Strncat(char *buf,       // IN/OUT
     * The reason the test with bufLen and n is >= rather than just >
     * is that strncat always NUL-terminates the resulting string, even
     * if it reaches the length limit n. This means that if it happens that
-    * bufLen + n == bufSize, strncat will write a NUL terminator that
+    * bufLen + n == bufSize, strncat will write a null terminator that
     * is outside of the buffer. Therefore, we make sure this does not
     * happen by adding the == case to the Panic test.
     */
 
-   bufLen = strlen(buf);
-
    if (bufLen + n >= bufSize &&
        bufLen + strlen(src) >= bufSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__,__LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__,__LINE__, stack[-1]);
    }
 
    /*
-    * We don't need to worry about NUL termination, because it's only
+    * We don't need to worry about null termination, because it's only
     * needed on overflow and we Panic above in that case.
     */
 
@@ -500,7 +415,7 @@ Str_Strncat(char *buf,       // IN/OUT
  */
 
 char *
-Str_Asprintf(size_t *length,       // OUT/OPT
+Str_Asprintf(size_t *length,       // OUT
              const char *format,   // IN
              ...)                  // IN
 {
@@ -532,13 +447,13 @@ Str_Asprintf(size_t *length,       // OUT/OPT
  */
 
 char *
-Str_SafeAsprintf(size_t *length,       // OUT/OPT
+Str_SafeAsprintf(size_t *length,       // OUT
                  const char *format,   // IN
                  ...)                  // IN
 {
    va_list arguments;
    char *result;
-
+   
    va_start(arguments, format);
    result = Str_SafeVasprintf(length, format, arguments);
    va_end(arguments);
@@ -550,7 +465,7 @@ Str_SafeAsprintf(size_t *length,       // OUT/OPT
 /*
  *-----------------------------------------------------------------------------
  *
- * StrVasprintfInternal --
+ * StrVasprintf_Internal --
  *
  *    Allocate and format a string, using the GNU libc way to specify the
  *    format (i.e. optionally allow the use of positional parameters)
@@ -563,8 +478,6 @@ Str_SafeAsprintf(size_t *length,       // OUT/OPT
  *    ASSERTs or returns NULL on failure, depending on the value of
  *    'assertOnFailure'.
  *
- * WARNING: See warning at the top of this file.
- *
  * Side effects:
  *    None
  *
@@ -572,16 +485,25 @@ Str_SafeAsprintf(size_t *length,       // OUT/OPT
  */
 
 static char *
-StrVasprintfInternal(size_t *length,       // OUT/OPT:
-                     const char *format,   // IN:
-                     va_list arguments,    // IN:
-                     Bool assertOnFailure) // IN:
+StrVasprintf_Internal(size_t *length,       // OUT
+                      const char *format,   // IN
+                      va_list arguments,    // IN
+                      Bool assertOnFailure) // IN
 {
    char *buf = NULL;
    int ret;
 
-#if defined HAS_BSD_PRINTF && !defined __ANDROID__
-   ret = bsd_vsnprintf(&buf, 0, format, arguments);
+#ifdef HAS_BSD_PRINTF
+   #ifdef __linux__
+      {
+	 va_list aq;
+	 va_copy(aq, arguments);
+	 ret = bsd_vsnprintf(&buf, 0, format, aq);
+	 va_end(aq);
+      }
+   #else
+      ret = bsd_vsnprintf(&buf, 0, format, arguments);
+   #endif
 
 #elif !defined sun && !defined STR_NO_WIN32_LIBS
    ret = vasprintf(&buf, format, arguments);
@@ -600,9 +522,7 @@ StrVasprintfInternal(size_t *length,       // OUT/OPT:
        * XXX Yes, this could overflow and spin forever when you get near 2GB
        *     allocations. I don't care. --rrdharan
        */
-
       char *newBuf;
-      va_list tmpArgs;
 
       bufSize *= 2;
       newBuf = realloc(buf, bufSize);
@@ -613,10 +533,7 @@ StrVasprintfInternal(size_t *length,       // OUT/OPT:
       }
 
       buf = newBuf;
-
-      va_copy(tmpArgs, arguments);
-      ret = Str_Vsnprintf(buf, bufSize, format, tmpArgs);
-      va_end(tmpArgs);
+      ret = Str_Vsnprintf(buf, bufSize, format, arguments);
    } while (ret < 0);
 #endif
 
@@ -630,7 +547,7 @@ StrVasprintfInternal(size_t *length,       // OUT/OPT:
 
   exit:
    if (assertOnFailure) {
-      VERIFY(buf);
+      ASSERT_NOT_IMPLEMENTED(buf);
    }
    return buf;
 }
@@ -641,24 +558,24 @@ StrVasprintfInternal(size_t *length,       // OUT/OPT:
  *
  * Str_Vasprintf --
  *
- *    See StrVasprintfInternal.
+ *    See StrVasprintf_Internal.
  *
  * Results:
+ *    See StrVasprintf_Internal.
  *    Returns NULL on failure.
  *
- * WARNING: See warning at the top of this file.
- *
  * Side effects:
  *    None
+ *
  *-----------------------------------------------------------------------------
  */
 
 char *
-Str_Vasprintf(size_t *length,       // OUT/OPT
+Str_Vasprintf(size_t *length,       // OUT
               const char *format,   // IN
               va_list arguments)    // IN
 {
-   return StrVasprintfInternal(length, format, arguments, FALSE);
+   return StrVasprintf_Internal(length, format, arguments, FALSE);
 }
 
 
@@ -667,12 +584,11 @@ Str_Vasprintf(size_t *length,       // OUT/OPT
  *
  * Str_SafeVasprintf --
  *
- *    See StrVasprintfInternal.
+ *    See StrVasprintf_Internal.
  *
  * Results:
- *    Calls VERIFY on failure.
- *
- * WARNING: See warning at the top of this file.
+ *    See StrVasprintf_Internal.
+ *    Calls ASSERT_NOT_IMPLEMENTED on failure.
  *
  * Side effects:
  *    None
@@ -681,14 +597,14 @@ Str_Vasprintf(size_t *length,       // OUT/OPT
  */
 
 char *
-Str_SafeVasprintf(size_t *length,       // OUT/OPT
+Str_SafeVasprintf(size_t *length,       // OUT
                   const char *format,   // IN
                   va_list arguments)    // IN
 {
-   return StrVasprintfInternal(length, format, arguments, TRUE);
+   return StrVasprintf_Internal(length, format, arguments, TRUE);
 }
 
-#if defined(_WIN32) || defined(__linux__)
+#if defined(_WIN32) || defined(GLIBC_VERSION_22)
 
 /*
  *----------------------------------------------------------------------
@@ -712,14 +628,15 @@ Str_Swprintf(wchar_t *buf,       // OUT
              const wchar_t *fmt, // IN
              ...)                // IN
 {
+   uint32 *stack = (uint32*) &buf;
    va_list args;
    int i;
-
+   
    va_start(args,fmt);
    i = Str_Vsnwprintf(buf, maxSize, fmt, args);
    va_end(args);
    if (i < 0) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
    }
    return i;
 }
@@ -730,20 +647,18 @@ Str_Swprintf(wchar_t *buf,       // OUT
  *
  * Str_Vsnwprintf --
  *
- *      Compatibility wrapper b/w different libc versions
+ *	Compatibility wrapper b/w different libc versions
  *
  * Results:
  *
- *      int - number of wchar_ts stored in 'str' (not including NUL
- *      terminate character), -1 on overflow (insufficient space for
- *      NUL terminate is considered overflow)
+ *	int - number of wchar_ts stored in 'str' (not including null
+ *	terminate character), -1 on overflow (insufficient space for
+ *	null terminate is considered overflow)
  *
- *      NB: on overflow the buffer WILL be NUL terminated
- *
- * WARNING: See warning at the top of this file.
+ *	NB: on overflow the buffer WILL be null terminated
  *
  * Side effects:
- *      None
+ *	None
  *
  *----------------------------------------------------------------------
  */
@@ -761,17 +676,21 @@ Str_Vsnwprintf(wchar_t *str,          // OUT
 #elif defined(_WIN32)
    retval = _vsnwprintf(str, size, format, ap);
 #else
-   retval = vswprintf(str, size, format, ap);
+   va_list aq;
+
+   va_copy(aq, ap);
+   retval = vswprintf(str, size, format, aq);
+   va_end(aq);
 #endif
 
    /*
-    * Linux glibc 2.0.x returns -1 and NUL terminates (which we shouldn't
+    * Linux glibc 2.0.x returns -1 and null terminates (which we shouldn't
     * be linking against), but glibc 2.1.x follows c99 and returns
     * characters that would have been written.
     *
     * In the case of Win32 and !HAS_BSD_PRINTF, we are using
     * _vsnwprintf(), which returns -1 on overflow, returns size
-    * when result fits exactly, and does not NUL terminate in
+    * when result fits exactly, and does not null terminate in
     * those cases.
     */
 
@@ -793,23 +712,23 @@ Str_Vsnwprintf(wchar_t *str,          // OUT
  *
  * Str_Snwprintf --
  *
- *      Compatibility wrapper b/w different libc versions
+ *	Compatibility wrapper b/w different libc versions
  *
  * Results:
  *
- *      int - number of wchar_ts stored in 'str' (not including NUL
- *      terminate character), -1 on overflow (insufficient space for
- *      NUL terminate is considered overflow)
+ *	int - number of wchar_ts stored in 'str' (not including null
+ *	terminate character), -1 on overflow (insufficient space for
+ *	null terminate is considered overflow)
  *
- *      NB: on overflow the buffer WILL be NUL terminated
+ *	NB: on overflow the buffer WILL be null terminated
  *
  * Side effects:
- *      None
+ *	None
  *
  *----------------------------------------------------------------------
  */
 
-int
+int      
 Str_Snwprintf(wchar_t *str,          // OUT
               size_t size,           // IN: Size of str, in wide-characters.
               const wchar_t *format, // IN
@@ -846,11 +765,13 @@ Str_Wcscpy(wchar_t *buf,       // OUT
            const wchar_t *src, // IN
            size_t maxSize)     // IN: Size of buf, in wide-characters.
 {
+   uint32 *stack = (uint32 *)&buf;
    size_t len;
 
    len = wcslen(src);
-   if (len >= maxSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+   if (len >= maxSize) { 
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
+      ASSERT_BUG(5686, FALSE); 
    }
    return memcpy(buf, src, (len + 1)*sizeof(wchar_t));
 }
@@ -873,10 +794,11 @@ Str_Wcscpy(wchar_t *buf,       // OUT
  */
 
 wchar_t *
-Str_Wcscat(wchar_t *buf,       // IN/OUT
+Str_Wcscat(wchar_t *buf,       // IN-OUT
            const wchar_t *src, // IN
            size_t maxSize)     // IN: Size of buf, in wide-characters.
 {
+   uint32 *stack = (uint32 *)&buf;
    size_t bufLen;
    size_t srcLen;
 
@@ -885,7 +807,7 @@ Str_Wcscat(wchar_t *buf,       // IN/OUT
 
    /* The first comparison checks for numeric overflow */
    if (bufLen + srcLen < srcLen || bufLen + srcLen >= maxSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
    }
 
    memcpy(buf + bufLen, src, (srcLen + 1)*sizeof(wchar_t));
@@ -914,11 +836,12 @@ Str_Wcscat(wchar_t *buf,       // IN/OUT
  */
 
 wchar_t *
-Str_Wcsncat(wchar_t *buf,       // IN/OUT
+Str_Wcsncat(wchar_t *buf,       // IN-OUT
             size_t bufSize,     // IN: Size of buf, in wide-characters.
             const wchar_t *src, // IN: String to append
             size_t n)           // IN: Max chars of src to append
 {
+   uint32 *stack = (uint32 *)&buf;
    size_t bufLen = wcslen(buf);
 
    /*
@@ -928,18 +851,18 @@ Str_Wcsncat(wchar_t *buf,       // IN/OUT
     * The reason the test with bufLen and n is >= rather than just >
     * is that wcsncat always NUL-terminates the resulting string, even
     * if it reaches the length limit n. This means that if it happens that
-    * bufLen + n == bufSize, wcsncat will write a NUL terminator that
+    * bufLen + n == bufSize, wcsncat will write a null terminator that
     * is outside of the buffer. Therefore, we make sure this does not
     * happen by adding the == case to the Panic test.
     */
 
    if (bufLen + n >= bufSize &&
        bufLen + wcslen(src) >= bufSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__,__LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__,__LINE__, stack[-1]);
    }
 
    /*
-    * We don't need to worry about NUL termination, because it's only
+    * We don't need to worry about null termination, because it's only
     * needed on overflow and we Panic above in that case.
     */
 
@@ -964,15 +887,16 @@ Str_Wcsncat(wchar_t *buf,       // IN/OUT
  */
 
 unsigned char *
-Str_Mbscpy(char *buf,        // OUT
-           const char *src,  // IN
-           size_t maxSize)   // IN
+Str_Mbscpy(char *buf,                // OUT
+           const char *src,          // IN
+           size_t maxSize)           // IN
 {
+   uint32 *stack = (uint32 *)&buf;
    size_t len;
 
-   len = strlen(src);
+   len = strlen((const char *) src);
    if (len >= maxSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
    }
    return memcpy(buf, src, len + 1);
 }
@@ -998,19 +922,20 @@ Str_Mbscpy(char *buf,        // OUT
  */
 
 unsigned char *
-Str_Mbscat(char *buf,        // IN/OUT
-           const char *src,  // IN
-           size_t maxSize)   // IN
+Str_Mbscat(char *buf,                // IN-OUT
+           const char *src,          // IN
+           size_t maxSize)           // IN
 {
+   uint32 *stack = (uint32 *)&buf;
    size_t bufLen;
    size_t srcLen;
 
-   bufLen = strlen(buf);
-   srcLen = strlen(src);
+   bufLen = strlen((const char *) buf);
+   srcLen = strlen((const char *) src);
 
    /* The first comparison checks for numeric overflow */
    if (bufLen + srcLen < srcLen || bufLen + srcLen >= maxSize) {
-      Panic("%s:%d Buffer too small\n", __FILE__, __LINE__);
+      Panic("%s:%d Buffer too small 0x%x\n", __FILE__, __LINE__, stack[-1]);
    }
 
    memcpy(buf + bufLen, src, srcLen + 1);
@@ -1022,7 +947,7 @@ Str_Mbscat(char *buf,        // IN/OUT
 /*
  *-----------------------------------------------------------------------------
  *
- * StrVaswprintfInternal --
+ * StrVaswprintf_Internal --
  *
  *    Allocate and format a string.
  *
@@ -1033,8 +958,6 @@ Str_Mbscat(char *buf,        // IN/OUT
  *    ASSERTs or returns NULL on failure, depending on the value of
  *    'assertOnFailure'.
  *
- * WARNING: See warning at the top of this file.
- *
  * Side effects:
  *    None
  *
@@ -1042,10 +965,10 @@ Str_Mbscat(char *buf,        // IN/OUT
  */
 
 static wchar_t *
-StrVaswprintfInternal(size_t *length,         // OUT/OPT:
-                      const wchar_t *format,  // IN:
-                      va_list arguments,      // IN
-                      Bool assertOnFailure)   // IN
+StrVaswprintf_Internal(size_t *length,         // OUT
+                       const wchar_t *format,  // IN
+                       va_list arguments,      // IN
+                       Bool assertOnFailure)   // IN
 {
    size_t bufSize;
    wchar_t *buf = NULL;
@@ -1059,12 +982,10 @@ StrVaswprintfInternal(size_t *length,         // OUT/OPT:
        * XXX Yes, this could overflow and spin forever when you get near 2GB
        *     allocations. I don't care. --rrdharan
        */
-
-      va_list tmpArgs;
       wchar_t *newBuf;
 
       bufSize *= 2;
-      newBuf = realloc(buf, bufSize * sizeof(wchar_t));
+      newBuf = realloc(buf, bufSize*sizeof(wchar_t));
       if (!newBuf) {
          free(buf);
          buf = NULL;
@@ -1072,10 +993,8 @@ StrVaswprintfInternal(size_t *length,         // OUT/OPT:
       }
 
       buf = newBuf;
+      retval = Str_Vsnwprintf(buf, bufSize, format, arguments);
 
-      va_copy(tmpArgs, arguments);
-      retval = Str_Vsnwprintf(buf, bufSize, format, tmpArgs);
-      va_end(tmpArgs);
    } while (retval == -1);
 
    if (length) {
@@ -1088,7 +1007,7 @@ StrVaswprintfInternal(size_t *length,         // OUT/OPT:
 
   exit:
    if (assertOnFailure) {
-      VERIFY(buf);
+      ASSERT_NOT_IMPLEMENTED(buf);
    }
    return buf;
 }
@@ -1111,13 +1030,13 @@ StrVaswprintfInternal(size_t *length,         // OUT/OPT:
  */
 
 wchar_t *
-Str_Aswprintf(size_t *length,         // OUT/OPT
+Str_Aswprintf(size_t *length,         // OUT
               const wchar_t *format,  // IN
               ...)                    // IN
 {
    va_list arguments;
    wchar_t *result;
-
+   
    va_start(arguments, format);
    result = Str_Vaswprintf(length, format, arguments);
    va_end(arguments);
@@ -1131,13 +1050,12 @@ Str_Aswprintf(size_t *length,         // OUT/OPT
  *
  * Str_Vaswprintf --
  *
- *    See StrVaswprintfInternal.
+ *    See StrVaswprintf_Internal.
  *
  * Results:
+ *    See StrVaswprintf_Internal.
  *    Returns NULL on failure.
  *
- * WARNING: See warning at the top of this file.
- *
  * Side effects:
  *    None
  *
@@ -1145,11 +1063,11 @@ Str_Aswprintf(size_t *length,         // OUT/OPT
  */
 
 wchar_t *
-Str_Vaswprintf(size_t *length,         // OUT/OPT
+Str_Vaswprintf(size_t *length,         // OUT
                const wchar_t *format,  // IN
                va_list arguments)      // IN
 {
-   return StrVaswprintfInternal(length, format, arguments, FALSE);
+   return StrVaswprintf_Internal(length, format, arguments, FALSE);
 }
 
 
@@ -1170,13 +1088,13 @@ Str_Vaswprintf(size_t *length,         // OUT/OPT
  */
 
 wchar_t *
-Str_SafeAswprintf(size_t *length,         // OUT/OPT
+Str_SafeAswprintf(size_t *length,         // OUT
                   const wchar_t *format,  // IN
                   ...)                    // IN
 {
    va_list arguments;
    wchar_t *result;
-
+   
    va_start(arguments, format);
    result = Str_SafeVaswprintf(length, format, arguments);
    va_end(arguments);
@@ -1190,12 +1108,11 @@ Str_SafeAswprintf(size_t *length,         // OUT/OPT
  *
  * Str_SafeVaswprintf --
  *
- *    See StrVaswprintfInternal.
+ *    See StrVaswprintf_Internal.
  *
  * Results:
- *    Calls VERIFY on failure.
- *
- * WARNING: See warning at the top of this file.
+ *    See StrVaswprintf_Internal.
+ *    Calls ASSERT_NOT_IMPLEMENTED on failure.
  *
  * Side effects:
  *    None
@@ -1204,14 +1121,14 @@ Str_SafeAswprintf(size_t *length,         // OUT/OPT
  */
 
 wchar_t *
-Str_SafeVaswprintf(size_t *length,         // OUT/OPT
+Str_SafeVaswprintf(size_t *length,         // OUT
                    const wchar_t *format,  // IN
                    va_list arguments)      // IN
 {
-   return StrVaswprintfInternal(length, format, arguments, TRUE);
+   return StrVaswprintf_Internal(length, format, arguments, TRUE);
 }
 
-#endif // defined(_WIN32) || defined(__linux__)
+#endif // defined(_WIN32) || defined(GLIBC_VERSION_22)
 
 #ifndef _WIN32
 
@@ -1296,31 +1213,26 @@ static Bool bCompare;
    } while (0);
 
 static void
-PrintAndCheck(char *fmt,  // IN:
-              ...)        // IN:
+PrintAndCheck(char *fmt, ...)
 {
    char buf1[1024], buf2[1024];
    int count;
    va_list args;
-
+   
    va_start(args, fmt);
-   count = Str_Vsnprintf(buf1, sizeof buf1, fmt, args);
-   va_end(args);
+   count = Str_Vsnprintf(buf1, 1024, fmt, args);
 
    if (count < 0) {
       FAIL("PrintAndCheck new code count off");
    }
 
    va_start(args, fmt);
-
 #ifdef _WIN32
-   count = _vsnprintf(buf2, sizeof buf2, fmt, args);
+   count = _vsnprintf(buf2, 1024, fmt, args);
 #else
-   count = vsnprintf(buf2, sizeof buf2, fmt, args);
+   count = vsnprintf(buf2, 1024, fmt, args);
 #endif
 
-   va_end(args);
-
    if (count < 0) {
       FAIL("PrintAndCheck old code count off");
    }
@@ -1334,6 +1246,8 @@ PrintAndCheck(char *fmt,  // IN:
    }
 
    printf(buf1);
+
+   va_end(args);
 }
 
 static void
@@ -1342,25 +1256,21 @@ PrintAndCheckW(wchar_t *fmt, ...)
    wchar_t buf1[1024], buf2[1024];
    int count;
    va_list args;
-
+   
    va_start(args, fmt);
-   count = Str_Vsnwprintf(buf1, sizeof buf1, fmt, args);
-   va_end(args);
+   count = Str_Vsnwprintf(buf1, 1024, fmt, args);
 
    if (count < 0) {
       FAIL("PrintAndCheckW new code count off");
    }
 
    va_start(args, fmt);
-
 #ifdef _WIN32
-   count = _vsnwprintf(buf2, sizeof buf2, fmt, args);
+   count = _vsnwprintf(buf2, 1024, fmt, args);
 #else
-   count = vswprintf(buf2, sizeof buf2, fmt, args);
+   count = vswprintf(buf2, 1024, fmt, args);
 #endif
 
-   va_end(args);
-
    if (count < 0) {
       FAIL("PrintAndCheckW old code count off");
    }
@@ -1376,6 +1286,8 @@ PrintAndCheckW(wchar_t *fmt, ...)
 #ifndef _WIN32
    printf("%S", buf1);
 #endif // _WIN32
+
+   va_end(args);
 }
 
 void
