@@ -141,14 +141,6 @@ UnicodeAllocInternal(const void *buffer,      // IN
  *      Otherwise, buffer must be of the specified length, but does
  *      not need to be NUL-terminated.
  *
- *      This function should not be used for heuristic determination of
- *      encodings.  Since the test looks for bit patterns in the buffer
- *      that are invalid in the specified encoding, negative results
- *      guarantee the buffer is not in the specified encoding, but positive
- *      results are inconclusive.  Source buffers containing pure ASCII
- *      will pass all 8-bit encodings, and all source buffers will pass 
- *      a windows-1252 test since win-1252 maps all 256 8-bit combinations.
- *
  * Results:
  *      TRUE if the buffer is valid, FALSE if it's not.
  *
@@ -319,15 +311,13 @@ Unicode_FreeList(Unicode *list,    // IN: the list to free
  *
  * Unicode_GetAllocList --
  *
- *      Given a list of Unicode strings, converts them to a list of
- *      buffers in the specified encoding.
- *
+ *      Allocates a list (actually a vector) of NUL terminated buffers from a 
+ *      list (vector) of strings of specified encoding.
  *      The input list has a specified length or can be an argv-style
  *      NULL-terminated list (if length is negative).
  *
  * Results:
- *      An allocated list (vector) of NUL terminated buffers in the specified
- *      encoding
+ *      An allocated list (vector) of NUL terminated buffers,
  *      or NULL on conversion failure.
  *      The caller is responsible to free the memory allocated by
  *      this routine.
@@ -338,12 +328,12 @@ Unicode_FreeList(Unicode *list,    // IN: the list to free
  *-----------------------------------------------------------------------------
  */
 
-char **
+Unicode *
 Unicode_GetAllocList(Unicode const srcList[], // IN: list of strings
-		     ssize_t length,          // IN: length (-1 for NULL term.)
-		     StringEncoding encoding) // IN: Encoding of returned list
+		     ssize_t length,          // IN: list 
+		     StringEncoding encoding) // IN:
 {
-   char **dstList = NULL;
+   Unicode *dstList = NULL;
    ssize_t i;
 
    ASSERT(srcList != NULL);
@@ -355,7 +345,7 @@ Unicode_GetAllocList(Unicode const srcList[], // IN: list of strings
       while (srcList[length] != NULL) {
          length++;
       }
-
+      
       /* Include the sentinel element. */
       length++;
    }
@@ -468,7 +458,7 @@ Unicode_BytesRequired(ConstUnicode str,        // IN
 
    switch (encoding) {
    case STRING_ENCODING_UTF8:
-      return strlen((const char *)utf8) + 1;
+      return strlen(utf8) + 1;
    case STRING_ENCODING_US_ASCII:
    case STRING_ENCODING_ISO_8859_1:
    case STRING_ENCODING_WINDOWS_1252:
@@ -702,49 +692,7 @@ Unicode_GetAllocBytes(ConstUnicode str,        // IN:
       return NULL;
    }
 
-   return UnicodeGetAllocBytesInternal(str, encoding, -1, NULL);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Unicode_GetAllocBytesWithLength --
- *
- *      Allocates and returns a buffer into which the contents of the unicode
- *      string of the specified length are extracted using the specified
- *      encoding.
- *
- *      NOTE: The buffer returned is always NUL terminated.  The length of
- *            the NUL can depend on the encoding.  UTF-16 NUL is "\0\0";
- *            UTF-32 NUL is "\0\0\0\0".
- *
- *      NULL is returned for NULL argument.
- *
- * Results:
- *      NULL if argument is NULL.
- *      Otherwise, pointer to the dynamically allocated memory
- *      or NULL on conversion failure.
- *      The caller is responsible to free the memory allocated
- *      by this routine.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void *
-Unicode_GetAllocBytesWithLength(ConstUnicode str,        // IN:
-                                StringEncoding encoding, // IN:
-                                ssize_t lengthInBytes)   // IN:
-{
-   if (str == NULL) {
-      return NULL;
-   }
-   ASSERT(lengthInBytes >= 0);
-
-   return UnicodeGetAllocBytesInternal(str, encoding, lengthInBytes, NULL);
+   return UnicodeGetAllocBytesInternal(str, encoding, NULL);
 }
 
 
@@ -772,43 +720,41 @@ Unicode_GetAllocBytesWithLength(ConstUnicode str,        // IN:
 void *
 UnicodeGetAllocBytesInternal(ConstUnicode ustr,       // IN
                              StringEncoding encoding, // IN
-                             ssize_t lengthInBytes,   // IN
                              size_t *retLength)       // OUT: optional
 {
    const char *utf8Str = ustr;
+   size_t len;
    char *result = NULL;
 
    ASSERT(ustr != NULL);
 
    encoding = Unicode_ResolveEncoding(encoding);
 
-   if (lengthInBytes == -1) {
-      lengthInBytes = Unicode_LengthInBytes(ustr, STRING_ENCODING_UTF8);
-   }
+   len = strlen(utf8Str);
 
    switch (encoding) {
    case STRING_ENCODING_US_ASCII:
-      if (!UnicodeSanityCheck(utf8Str, lengthInBytes, encoding)) {
+      if (!UnicodeSanityCheck(utf8Str, len, encoding)) {
 	 break;
       }
       // fall through
    case STRING_ENCODING_UTF8:
-      result = Util_SafeMalloc(lengthInBytes + 1);
-      memcpy(result, utf8Str, lengthInBytes + 1);
+      result = Util_SafeMalloc(len + 1);
+      memcpy(result, utf8Str, len + 1);
       if (retLength != NULL) {
-	 *retLength = lengthInBytes;
+	 *retLength = len;
       }
       break;
 
    case STRING_ENCODING_UTF16_LE:
-      if (!CodeSet_Utf8ToUtf16le(utf8Str, lengthInBytes, &result, retLength)) {
+      if (!CodeSet_Utf8ToUtf16le(utf8Str, len, &result, retLength)) {
 	 // input should be valid UTF-8, no conversion error possible
 	 ASSERT_MEM_ALLOC(FALSE);
       }
       break;
 
    default:
-      if (!CodeSet_GenericToGeneric("UTF-8", utf8Str, lengthInBytes,
+      if (!CodeSet_GenericToGeneric("UTF-8", utf8Str, len,
 				    Unicode_EncodingEnumToName(encoding),
 				    CSGTG_NORMAL,
 				    &result, retLength)) {
