@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -19,13 +19,17 @@
 /*
  * unicodeStatic.c --
  *
- *      Manages memory for static const char *literal strings
+ *      Manages memory for static ConstUnicode literal strings
  *      created like:
  *
- *         const char *c = U_UNESCAPE("Copyright \\u00A9 VMware, Inc.");
+ *         ConstUnicode foo = U("foo");
  *
- *      Uses two HashTables to hold static const char *strings. Static
- *      const char *strings are keyed off the ASCII bytes passed to the
+ *      and
+ *
+ *         ConstUnicode c = U_UNESCAPE("Copyright \\u00A9 VMware, Inc.");
+ *
+ *      Uses two HashTables to hold static ConstUnicode strings. Static
+ *      ConstUnicode strings are keyed off the ASCII bytes passed to the
  *      static macros.
  *
  *      Unescaped strings are kept separate from escaped strings so
@@ -75,7 +79,7 @@ static Atomic_Ptr UnicodeUnescapedStringTable;
 static void
 UnicodeHashFree(void *v)  // IN:
 {
-   free(v);
+   Unicode_Free((Unicode) v);
 }
 
 
@@ -84,9 +88,9 @@ UnicodeHashFree(void *v)  // IN:
  *
  * Unicode_GetStatic --
  *
- *      Helper function for the U_UNESCAPE() macro.
+ *      Helper function for U() and U_UNESCAPE() macros.
  *
- *      Given a NUL-terminated ASCII string, returns a const char *
+ *      Given a NUL-terminated ASCII string, returns a ConstUnicode
  *      string containing the string's contents.
  *
  *      If unescape is TRUE, then \\uABCD becomes the Unicode code
@@ -94,7 +98,7 @@ UnicodeHashFree(void *v)  // IN:
  *      U+1FABCD in the resulting string.
  *
  * Results:
- *      A const char * string.  Memory is managed inside this module;
+ *      A ConstUnicode string.  Memory is managed inside this module;
  *      caller does not need to free.
  *
  * Side effects:
@@ -106,11 +110,11 @@ UnicodeHashFree(void *v)  // IN:
  *-----------------------------------------------------------------------------
  */
 
-const char *
+ConstUnicode
 Unicode_GetStatic(const char *asciiBytes, // IN
                   Bool unescape)          // IN
 {
-   char *result = NULL;
+   Unicode result = NULL;
    HashTable *stringTable;
 
    if (unescape) {
@@ -134,13 +138,13 @@ Unicode_GetStatic(const char *asciiBytes, // IN
     */
 
    if (!HashTable_Lookup(stringTable, asciiBytes, (void **) &result)) {
-      char *newData = UnicodeAllocStatic(asciiBytes, unescape);
+      Unicode newData = UnicodeAllocStatic(asciiBytes, unescape);
 
       if (newData) {
          result = HashTable_LookupOrInsert(stringTable, asciiBytes, newData);
 
          if (result != newData) {
-            free(newData);
+            Unicode_Free(newData);
          }
       }
    }
