@@ -36,9 +36,9 @@
 # include <sys/ucred.h>
 # include <sys/mount.h>
 # include <fstab.h>
-# if defined(__FreeBSD_version) && __FreeBSD_version >= 500000
+# if BSD_VERSION >= 50
 #  include <libgen.h>
-# endif /* __FreeBSD_version >= 500000 */
+# endif /* BSD_VERSION < 50 */
 #endif
 #include <unistd.h>
 
@@ -46,12 +46,9 @@
 #include "wiper.h"
 #include "util.h"
 #include "str.h"
-#include "strutil.h"
 #include "fileIO.h"
-#include "vmstdio.h"
 #include "mntinfo.h"
 #include "posix.h"
-#include "util.h"
 
 
 /* Number of bytes per disk sector */
@@ -67,9 +64,6 @@
 */
 #define WIPER_SECTOR_STEP 128
 
-/* Number of device numbers to store for device-mapper */
-#define WIPER_MAX_DM_NUMBERS 8
-
 #if defined(sun) || defined(__linux__)
 # define PROCFS "proc"
 #elif defined(__FreeBSD__) || defined(__APPLE__)
@@ -113,145 +107,17 @@ typedef struct WiperDiskString {
 } WiperDiskString;
 #endif
 
-typedef struct PartitionInfo {
-   const char          *name;
-   WiperPartition_Type  type;
-   const char          *comment;
-   Bool                 diskBacked;
-} PartitionInfo;
-
 
 /* Variables */
-
-static const char gRemoteFS[] = "Remote filesystem.";
-
-static const PartitionInfo gKnownPartitions[] = {
-   { "autofs",    PARTITION_UNSUPPORTED,  "autofs filesystem.",   FALSE       },
-   { "devpts",    PARTITION_UNSUPPORTED,  "devpts filesystem.",   FALSE       },
-   { "nfs",       PARTITION_UNSUPPORTED,  gRemoteFS,              FALSE       },
-   { "smbfs",     PARTITION_UNSUPPORTED,  gRemoteFS,              FALSE       },
-   { "swap",      PARTITION_UNSUPPORTED,  "Swap partition.",      FALSE       },
-   { "vmhgfs",    PARTITION_UNSUPPORTED,  gRemoteFS,              FALSE       },
-   { PROCFS,      PARTITION_UNSUPPORTED,  "proc filesystem.",     FALSE       },
-   { "ext2",      PARTITION_EXT2,         NULL,                   TRUE        },
-   { "ext3",      PARTITION_EXT3,         NULL,                   TRUE        },
-   { "ext4",      PARTITION_EXT4,         NULL,                   TRUE        },
-   { "hfs",       PARTITION_HFS,          NULL,                   TRUE        },
-   { "msdos",     PARTITION_FAT,          NULL,                   TRUE        },
-   { "ntfs",      PARTITION_NTFS,         NULL,                   TRUE        },
-   { "pcfs",      PARTITION_PCFS,         NULL,                   TRUE        },
-   { "reiserfs",  PARTITION_REISERFS,     NULL,                   TRUE        },
-   { "ufs",       PARTITION_UFS,          NULL,                   TRUE        },
-   { "vfat",      PARTITION_FAT,          NULL,                   TRUE        },
-   { "zfs",       PARTITION_ZFS,          NULL,                   FALSE       },
-};
-
 static Bool initDone = FALSE;
 
 
 /* Local functions */
-static Bool WiperIsDiskDevice(MNTINFO *mnt, struct stat *s);
+static INLINE Bool WiperIsDiskDevice(MNTINFO *mnt, struct stat *s);
 static void WiperPartitionFilter(WiperPartition *item, MNTINFO *mnt);
 static unsigned char *WiperGetSpace(WiperState *state, uint64 *free, uint64 *total);
 static void WiperClean(WiperState *state);
 
-
-#if defined(__linux__)
-
-#define MAX_DISK_MAJORS       256   /* should be enough for now */
-#define NUM_PRESEEDED_MAJORS  5     /* must match the below */
-
-static unsigned int knownDiskMajor[MAX_DISK_MAJORS] = {
-   /*
-    * Pre-seed some major numbers we were comparing against before
-    * we started scanning /proc/devices.
-    */
-   3,    /* First MFM, RLL and IDE hard disk/CD-ROM interface.
-            Inside a VM, this is simply the First IDE hard
-            disk/CD-ROM interface because we don't support
-            others */
-   8,    /* SCSI disk devices */
-   22,   /* Second IDE hard disk/CD-ROM interface */
-   43,   /* Network block device */
-   259,  /* Disks in 2.6.27 */
-};
-
-static int numDiskMajors;
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * WiperCollectDiskMajors --
- *
- *      Collects major numbers of devices that we considering "disks" and
- *      may try to shrink.
- *
- * Results:
- *      None
- *
- * Side effects:
- *      Populates knownDiskMajor array and numDiskMajors counter for
- *      subsequent use by linux version of WiperIsDiskDevice().
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-WiperCollectDiskMajors(void)
-{
-   const char diskDevNames[] = "|ide0|ide1|sd|md|nbd|device-mapper|blkext|";
-   const char blockSeparator[] = "Block devices:";
-   Bool seenBlockSeparator = FALSE;
-   char *buf;
-   int major;
-   char device[64];
-   FILE *f;
-
-   numDiskMajors = NUM_PRESEEDED_MAJORS;
-
-   f = Posix_Fopen("/proc/devices", "r");
-   if (!f) {
-      return;
-   }
-
-   while (StdIO_ReadNextLine(f, &buf, 0, NULL) == StdIO_Success) {
-
-      if (!seenBlockSeparator) {
-         if (!memcmp(buf, blockSeparator, sizeof(blockSeparator) - 1)) {
-            seenBlockSeparator = TRUE;
-         }
-      } else if (sscanf(buf, "%d %61s\n", &major, device + 1) == 2) {
-
-         device[0] = '|';
-         device[sizeof(device) - 2] = '\0';
-         Str_Strcat(device, "|", sizeof(device));
-
-         if (strstr(diskDevNames, device)) {
-            knownDiskMajor[numDiskMajors++] = major;
-         }
-      }
-
-      free(buf);
-
-      if (numDiskMajors >= MAX_DISK_MAJORS) {
-         break;
-      }
-   }
-
-   fclose(f);
-}
-
-#else
-
-static void
-WiperCollectDiskMajors(void)
-{
-}
-
-#endif
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -269,7 +135,7 @@ WiperCollectDiskMajors(void)
  */
 
 #if defined(sun) /* SunOS { */
-static Bool
+static INLINE Bool
 WiperIsDiskDevice(MNTINFO *mnt,         // IN: file system being considered
                   struct stat *s)       // IN: stat(2) info of fs source
 {
@@ -310,25 +176,29 @@ WiperIsDiskDevice(MNTINFO *mnt,         // IN: file system being considered
 
 #elif defined(__linux__) /* } linux { */
 
-static Bool
+static INLINE Bool
 WiperIsDiskDevice(MNTINFO *mnt,         // IN: file system being considered
                   struct stat *s)       // IN: stat(2) info of fs source
 {
-   int majorN = major(s->st_rdev);
-   int i;
+   int major;
 
-   for (i = 0; i < numDiskMajors; i++) {
-      if (majorN == knownDiskMajor[i]) {
-         return TRUE;
-      }
+   major = (s->st_rdev >> 8) & 0xFF;
+   if (! (   (major == 3)  /* First MFM, RLL and IDE hard disk/CD-ROM
+                              interface. Inside a VM, this is simply the First
+                              IDE hard disk/CD-ROM interface because we don't
+                              support others */
+             || (major == 22) /* Second IDE hard disk/CD-ROM interface */
+             || (major == 8)  /* SCSI disk devices */
+             || (major == 43) /* Network block device */)) {
+      return FALSE;
    }
 
-   return FALSE;
+   return TRUE;
 }
 
-#elif defined(__FreeBSD__) /* } FreeBSD { */
+#elif defined(__FreeBSD__) || defined(__APPLE__) /* } FreeBSD { */
 
-static Bool
+static INLINE Bool
 WiperIsDiskDevice(MNTINFO *mnt,         // IN: file system being considered
                   struct stat *s)       // IN: stat(2) info of fs source
 {
@@ -340,7 +210,7 @@ WiperIsDiskDevice(MNTINFO *mnt,         // IN: file system being considered
     * function, as a whole, does not even apply to OS X, so this caveat is
     * only minor.
     */
-#if __FreeBSD_version < 500000
+#if BSD_VERSION < 50
    /*
     * Before FreeBSD 5, device nodes had static major/minor numbers.
     * (FreeBSD 5 included devfs which got rid of this concept.)  So
@@ -360,7 +230,7 @@ WiperIsDiskDevice(MNTINFO *mnt,         // IN: file system being considered
        (S_ISCHR(s->st_mode) && ((maj == 3) || (maj == 13)))) {
       retval = TRUE;
    }
-#else
+#else /* Also the Apple case */
    /*
     * Begin by testing whether file system source is really a character
     * device node.  (FreeBSD killed off block devices long ago.)  Next,
@@ -376,27 +246,10 @@ WiperIsDiskDevice(MNTINFO *mnt,         // IN: file system being considered
          retval = TRUE;
       }
    }
-#undef MASK_ATA_DISK
-#undef MASK_SCSI_DISK
-#endif /* __FreeBSD_version */
+#endif /* BSD_VERSION */
 
    return retval;
 }
-
-#elif defined(__APPLE__) /* } { */
-
-static Bool
-WiperIsDiskDevice(MNTINFO *mnt,     // IN
-                  struct stat *s)   // IN
-{
-   /*
-    * Differently from FreeBSD, Mac OS still lists disks as block devices,
-    * it seems. Disks devices also seem to start with "/dev/disk".
-    */
-   return S_ISBLK(s->st_mode) &&
-          StrUtil_StartsWith(MNTINFO_NAME(mnt), "/dev/disk");
-}
-
 #endif /* } */
 
 /*
@@ -420,62 +273,145 @@ WiperPartitionFilter(WiperPartition *item,         // IN/OUT
                      MNTINFO *mnt)                 // IN
 {
    struct stat s;
-   const char *comment = NULL;
-   const char *fsname = MNTINFO_FSTYPE(mnt);
-   const PartitionInfo *info;
-   size_t i;
 
-   item->type = PARTITION_UNSUPPORTED;
-
-   for (i = 0; i < ARRAYSIZE(gKnownPartitions); i++) {
-      info = &gKnownPartitions[i];
-      if (strcmp(info->name, fsname) == 0) {
-         item->type = info->type;
-         comment = info->comment;
-         break;
-      }
+   /*
+    * Let's ignore remote filesystems before we do a stat(2) on the actual
+    * mountpoint. This should prevent a deadlock in guestd for guests that
+    * still use an HGFS pserver (Solaris).
+    */
+   if (strcmp(MNTINFO_FSTYPE(mnt), "autofs") == 0) {
+      /* XXX Should we look at autofs' config files? --hpreg */
+      item->comment = "Not implemented. Contact VMware";
+      return;
    }
 
-   if (i == ARRAYSIZE(gKnownPartitions)) {
-      comment = "Unknown filesystem. Contact VMware.";
-   } else if (item->type != PARTITION_UNSUPPORTED) {
-      /*
-       * If the partition is supported by the wiper library, do some other
-       * checks before declaring it shrinkable.
-       */
+   if (strcmp(MNTINFO_FSTYPE(mnt), "vmhgfs") == 0) {
+      item->comment = "Remote partition";
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "nfs") == 0) {
+      item->comment = "Remote filesystem";
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "smbfs") == 0) {
+      item->comment = "Remote filesystem";
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "swap") == 0) {
+      item->comment = "Swap partition";
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), PROCFS) == 0) {
+      item->comment = "Proc partition";
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "devpts") == 0) {
+      item->comment = "Devpts partition";
+      return;
+   }
+
+   if (Posix_Stat(MNTINFO_NAME(mnt), &s) < 0) {
+      item->comment = "Unknown device";
+      return;
+   }
 
-      if (info->diskBacked) {
-         if (Posix_Stat(MNTINFO_NAME(mnt), &s) < 0) {
-            comment = "Unknown device.";
 #if defined(sun) || defined(__linux__)
-         } else if (!S_ISBLK(s.st_mode)) {
-            comment = "Not a block device.";
+   if (! S_ISBLK(s.st_mode)) {
+      item->comment = "Not a block device";
+      return;
+   }
 #endif
-         } else if (!WiperIsDiskDevice(mnt, &s)) {
-            comment = "Not a disk device.";
-         } else if (MNTINFO_MNT_IS_RO(mnt)) {
-            comment = "Not writable.";
-         }
-      } else if (Posix_Access(MNTINFO_MNTPT(mnt), W_OK) != 0) {
-         comment = "Mount point not writable.";
-      }
 
-      if (comment != NULL) {
-         item->type = PARTITION_UNSUPPORTED;
-      }
+   if (!WiperIsDiskDevice(mnt, &s)) {
+      item->comment = "Not a disk device";
+      return;
    }
 
-   if (item->type == PARTITION_UNSUPPORTED) {
-      ASSERT(comment);
-      item->comment = Util_SafeStrdup(comment);
+   if (MNTINFO_MNT_IS_RO(mnt)) {
+      item->comment = "Not writable";
+      return;
    }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "ext2") == 0) {
+      item->comment = "";
+      item->type = PARTITION_EXT2;
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "ext3") == 0) {
+      item->comment = "";
+      item->type = PARTITION_EXT3;
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "reiserfs") == 0) {
+      item->comment = "";
+      item->type = PARTITION_REISERFS;
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "ntfs") == 0) {
+      item->comment = "";
+      item->type = PARTITION_NTFS;
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "vfat") == 0) {
+      item->comment = "";
+      item->type = PARTITION_FAT;
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "ufs") == 0) {
+      item->comment = "";
+      item->type = PARTITION_UFS;
+      return;
+   }
+
+   if (strcmp(MNTINFO_FSTYPE(mnt), "pcfs") == 0) {
+      item->comment = "";
+      item->type = PARTITION_PCFS;
+      return;
+   }
+
+   item->comment = "Unknown filesystem. Contact VMware";
+   return;
 }
 
 
 /*
  *-----------------------------------------------------------------------------
  *
- * WiperSinglePartition_Open --
+ * SingleWiperPartition_Close --
+ *
+ *      Destroy the information returned by a previous call to
+ *      SingleWiperPartition_Open().
+ *
+ * Results:
+ *      None
+ *
+ * Side Effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+SingleWiperPartition_Close(WiperPartition *p)      // IN
+{
+   free(p);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * SingleWiperPartition_Open --
  *
  *      Return information about the input 'mountPoint' partition.
  *
@@ -490,20 +426,26 @@ WiperPartitionFilter(WiperPartition *item,         // IN/OUT
  */
 
 WiperPartition *
-WiperSinglePartition_Open(const char *mountPoint)      // IN
+SingleWiperPartition_Open(const char *mountPoint)      // IN
 {
    char *mntpt = NULL;
    MNTHANDLE fp;
    int len = 0;
    DECLARE_MNTINFO(mnt);
-   WiperPartition *p = NULL;
+   WiperPartition *p;
 
    ASSERT(initDone);
+   fp = NULL;
+   p = (WiperPartition *) malloc(sizeof *p);
+   if (p == NULL) {
+      Log("Not enough memory while opening a partition.\n");
+      goto error;
+   }
 
    fp = OPEN_MNTFILE("r");
    if (fp == NULL) {
       Log("Could not open %s\n", MNTFILE);
-      return NULL;
+      goto error;
    }
 
    mntpt = Util_SafeStrdup(mountPoint);
@@ -521,30 +463,30 @@ WiperSinglePartition_Open(const char *mountPoint)      // IN
    len = strlen(mntpt);
    while (GETNEXT_MNTINFO(fp, mnt)) {
       if (strncmp(MNTINFO_MNTPT(mnt), mntpt, len) == 0) {
-
-         p = WiperSinglePartition_Allocate();
-         if (p == NULL) {
-            Log("Not enough memory while opening a partition.\n");
-         } else if (Str_Snprintf(p->mountPoint, NATIVE_MAX_PATH,
-                                 "%s", MNTINFO_MNTPT(mnt)) == -1) {
+         if (Str_Snprintf(p->mountPoint, NATIVE_MAX_PATH,
+                          "%s", MNTINFO_MNTPT(mnt)) == -1) {
             Log("NATIVE_MAX_PATH is too small.\n");
-            WiperSinglePartition_Close(p);
-            p = NULL;
-         } else {
-            WiperCollectDiskMajors();
-            WiperPartitionFilter(p, mnt);
+            goto error;
          }
 
-         goto out;
+         WiperPartitionFilter(p, mnt);
+         (void) CLOSE_MNTFILE(fp);
+
+         fp = NULL;
+
+         return p;
       }
    }
 
    Log("Could not find a mount point for %s in %s\n", mntpt, MNTFILE);
 
- out:
-   free(mntpt);
-   (void) CLOSE_MNTFILE(fp);
-   return p;
+  error:
+   SingleWiperPartition_Close(p);
+   if (fp != NULL) {
+      (void) CLOSE_MNTFILE(fp);
+      fp = NULL;
+   }
+   return NULL;
 }
 
 /*
@@ -574,8 +516,6 @@ WiperSinglePartition_GetSpace(const WiperPartition *p,  // IN
 #else
    struct statfs statfsbuf;
 #endif
-   uint64 blockSize;
-
    ASSERT(p);
 
 #ifdef sun
@@ -586,18 +526,12 @@ WiperSinglePartition_GetSpace(const WiperPartition *p,  // IN
       return "Unable to statfs() the mount point";
    }
 
-#ifdef sun
-   blockSize = statfsbuf.f_frsize;
-#else
-   blockSize = statfsbuf.f_bsize;
-#endif
-
    if (geteuid()== 0) {
-      *free = (uint64)statfsbuf.f_bfree * blockSize;
+      *free = (uint64)statfsbuf.f_bfree * statfsbuf.f_bsize;
    } else {
-      *free = (uint64)statfsbuf.f_bavail * blockSize;
+      *free = (uint64)statfsbuf.f_bavail * statfsbuf.f_bsize;
    }
-   *total = (uint64)statfsbuf.f_blocks * blockSize;
+   *total = (uint64)statfsbuf.f_blocks * statfsbuf.f_bsize;
 
    return "";
 }
@@ -619,78 +553,102 @@ WiperSinglePartition_GetSpace(const WiperPartition *p,  // IN
  *-----------------------------------------------------------------------------
  */
 
-Bool
-WiperPartition_Open(WiperPartition_List *pl)
+WiperPartition_List *
+WiperPartition_Open(void)
 {
+   WiperPartition_List *pl;
    MNTHANDLE fp;
    DECLARE_MNTINFO(mnt);
-   Bool rc = TRUE;
 
    ASSERT(initDone);
+   fp = NULL;
 
-   DblLnkLst_Init(&pl->link);
+   pl = (WiperPartition_List *)malloc(sizeof *pl);
+   if (pl == NULL) {
+      Log("Not enough memory while opening a partition.\n");
+      goto error;
+   }
+
+   pl->partitions = NULL;
+   pl->size = 0;
 
    /* Basically call functions to parse /etc/mtab ... */
    fp = OPEN_MNTFILE("r");
    if (fp == NULL) {
       Log("Unable to open mount file.\n");
-      return FALSE;
+      goto error;
    }
 
-   WiperCollectDiskMajors();
-
    while (GETNEXT_MNTINFO(fp, mnt)) {
-      WiperPartition *part = WiperSinglePartition_Allocate();
+      WiperPartition item;
 
-      if (part == NULL) {
-         Log("Not enough memory while opening a partition.\n");
-         rc = FALSE;
-         break;
-      }
-
-      if (Str_Snprintf(part->mountPoint, NATIVE_MAX_PATH, "%s",
+      if (Str_Snprintf(item.mountPoint, NATIVE_MAX_PATH, "%s",
                        MNTINFO_MNTPT(mnt)) == -1) {
-         Log("NATIVE_MAX_PATH is too small.\n");
-         WiperSinglePartition_Close(part);
-         rc = FALSE;
-         break;
+              Log("NATIVE_MAX_PATH is too small.\n");
+              goto error;
       }
 
-      WiperPartitionFilter(part, mnt);
-      DblLnkLst_LinkLast(&pl->link, &part->link);
+      WiperPartitionFilter(&item, mnt);
+
+      /* Add the partition to the list */
+      pl->partitions = (WiperPartition *)realloc(pl->partitions,
+                                                 (pl->size + 1) *
+                                                 sizeof *pl->partitions);
+      if (pl->partitions == NULL) {
+         Log("Not enough memory while opening a partition.\n");
+              goto error;
+      }
+
+      pl->partitions[pl->size] = item;
+      pl->size++;
    }
 
-   if (!rc)
-      WiperPartition_Close(pl);
-
    (void) CLOSE_MNTFILE(fp);
-   return rc;
+   fp = NULL;
+
+   return pl;
+
+error:
+   WiperPartition_Close(pl);
+   pl = NULL;
+   if (fp != NULL) {
+      (void) CLOSE_MNTFILE(fp);
+      fp = NULL;
+   }
+   return NULL;
 }
 
 
 /*
- *---------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
- * Wiper_IsWipeSupported --
+ * WiperPartition_Close --
  *
- *      Query if wipe is supported on the specified wiper partition.
+ *      Destroy the information returned by a previous call to
+ *      WiperPartition_Open()
  *
  * Results:
- *      FALSE always.
+ *      None
  *
  * Side Effects:
  *      None
  *
- *---------------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
-Bool
-Wiper_IsWipeSupported(const WiperPartition *part)
+void
+WiperPartition_Close(WiperPartition_List *pl)      // IN/OUT
 {
-   return FALSE;
+   if (pl != NULL) {
+      free(pl->partitions);
+      pl->partitions = NULL;
+      free(pl);
+   }
 }
 
 
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -996,11 +954,8 @@ Wiper_Cancel(Wiper_State **s)      // IN/OUT
  *
  * Wiper_Init --
  *
- *      On Solaris, this function is defined only to provide a uniform
- *      interface to the library.  On Linux, the /proc/devices file is
- *      read to initialize an array with device numbers that correspond
- *      to the device-mapper devices.  This is to differentiate partitions
- *      that use the device-mapper from other non-disk devices.
+ *      On Linux and Solaris, this function is defined only to provide a
+ *      uniform interface to the library.
  *
  * Results:
  *      Always TRUE.
@@ -1012,8 +967,7 @@ Wiper_Cancel(Wiper_State **s)      // IN/OUT
  */
 
 Bool
-Wiper_Init(WiperInitData *clientData)
+Wiper_Init(void *clientData)
 {
    return initDone = TRUE;
 }
-
