@@ -37,8 +37,7 @@
 /* os.h includes necessary OS-specific headers. */
 #include "os.h"
 
-#if defined(vmblock_fuse)
-#elif defined(linux)
+#ifdef linux
 # include "vmblockInt.h"
 #elif defined(sun)
 # include "module.h"
@@ -351,7 +350,7 @@ BlockWaitOnFile(const char *filename,   // IN: file to block on
     */
    if (cookie == NULL) {
       os_read_lock(&blockedFilesLock);
-      block = GetBlock(filename, OS_UNKNOWN_BLOCKER);
+      block = GetBlock(filename, NULL);
       os_read_unlock(&blockedFilesLock);
 
       if (!block) {
@@ -420,7 +419,7 @@ BlockLookup(const char *filename,               // IN: pathname to test for
    os_read_lock(&blockedFilesLock);
 
    block = GetBlock(filename, blocker);
-
+   
    os_read_unlock(&blockedFilesLock);
 
    return block;
@@ -530,13 +529,8 @@ GetBlock(const char *filename,          // IN: file to find block for
 {
    struct DblLnkLst_Links *curr;
 
-   /*
-    * On FreeBSD we have a mechanism to assert (but not simply check)
-    * that a lock is held. Since semantic is different (panic that
-    * happens if assertion fails can not be suppressed) we are using
-    * different name.
-    */
-#ifdef os_assert_rwlock_held
+   /* XXX The following is only temporary. */
+#ifdef __FreeBSD__
    os_assert_rwlock_held(&blockedFilesLock);
 #else
    ASSERT(os_rwlock_held(&blockedFilesLock));
