@@ -188,7 +188,7 @@
 #include <sys/filedesc.h>
 #include <sys/kdb.h>
 #include "compat_freebsd.h"
-#if __FreeBSD_version >= 700055
+#if __FreeBSD_version >= 700000
 #include <sys/priv.h>
 #endif
 
@@ -252,9 +252,6 @@ typedef struct VMBlockIoctlArgs VMBlockIoctlArgs;
 struct vop_vector VMBlockVnodeOps = {
    .vop_bypass =                  VMBlockVopBypass,
    .vop_access =                  VMBlockVopAccess,
-#if __FreeBSD_version >= 900013
-   .vop_advlockpurge =            vop_stdadvlockpurge,
-#endif
    .vop_bmap =                    VOP_EOPNOTSUPP,
    .vop_getattr =                 VMBlockVopGetAttr,
    .vop_getwritemount =           VMBlockVopGetWriteMount,
@@ -272,21 +269,19 @@ struct vop_vector VMBlockVnodeOps = {
    .vop_unlock =                  VMBlockVopUnlock,
 };
 
-
-
 /*
  * VMBlockFS file descriptor operations vector --
  *   There are a few special cases where we need to control behavior beyond
  *   the file system layer.  For this we define our own fdesc op vector,
  *   install our own handlers for these special cases, and fall back to the
- *   badfileops vnode ops for everything else.
+ *   original vnode ops for everything else.
  *
  *   VMBlock instances are keyed on/indexed by the file descriptor that received
  *   the ioctl request[1].  Since the relationship between file descriptors and
  *   vnodes is N:1, we need to intercept ioctl requests at the file descriptor
  *   level, rather than at the vnode level, in order to have a record of which
  *   descriptor received the request.  Similarly, we need to remove VMBlocks
- *   issued on a file descriptor when said descriptor is closed.
+ *   issued on a file descriptor when said descriptor is closed. 
  *
  *   NOTICE --
  *     This applies -only- when a user opens the FS mount point directly.  All
@@ -299,34 +294,10 @@ struct vop_vector VMBlockVnodeOps = {
  *     dies, even though the same descriptor is open.
  */
 
-static struct fileops VMBlockFileOps;
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMBlockSetupFileOps --
- *
- *      Sets up secial file operations vector used for root vnode _only_
- *      (see the comment for VMBlockFileOps above).
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-VMBlockSetupFileOps(void)
-{
-   VMBlockFileOps = badfileops;
-   VMBlockFileOps.fo_stat = vnops.fo_stat;
-   VMBlockFileOps.fo_flags = vnops.fo_flags;
-   VMBlockFileOps.fo_ioctl = VMBlockFileIoctl;
-   VMBlockFileOps.fo_close = VMBlockFileClose;
-}
+struct fileops VMBlockFileOps = {
+   .fo_ioctl    = VMBlockFileIoctl,
+   .fo_close    = VMBlockFileClose,
+};
 
 
 /*
@@ -548,7 +519,6 @@ struct vop_generic_args {
       }
       vppp = VOPARG_OFFSETTO(struct vnode***, descp->vdesc_vpp_offset,ap);
       if (*vppp) {
-         /* FIXME: set proper name for the vnode */
          error = VMBlockNodeGet(old_vps[0]->v_mount, **vppp, *vppp, NULL);
       }
    }
@@ -600,8 +570,7 @@ struct vop_lookup_args {
    BlockHandle blockCookie;
    int flags = cnp->cn_flags;
    int error = 0;
-   char *pathname;
-   size_t pathname_len;
+   char *savename = NULL, *pathname = NULL;
 
    /*
     * Fail attempts to modify a read-only filesystem w/o bothering with a
@@ -629,26 +598,8 @@ struct vop_lookup_args {
     *
     * If we find we were forcibly unmounted, fail with EIO.
     */
-
-   pathname = uma_zalloc(VMBlockPathnameZone, M_WAITOK);
-   if (pathname == NULL) {
-      return ENOMEM;
-   }
-
-   /*
-    * FIXME: we need to ensure that vnode always has name set up.
-    * Currently VMBlockVopBypass() may produce vnodes without a name.
-    */
-   pathname_len = strlcpy(pathname,
-                          VPTOVMB(dvp)->name ? VPTOVMB(dvp)->name : ".",
-                          MAXPATHLEN);
-   /*
-    * Make sure we have room in the buffer to add our component.
-    * + 1 is for separator (slash).
-    */
-   if (pathname_len + 1 + cnp->cn_namelen >= MAXPATHLEN) {
-      error = ENAMETOOLONG;
-      goto out;
+   if ((pathname = VMBlockBuildBlockName(dvp)) == NULL) {
+      return ENAMETOOLONG;
    }
 
    if ((blockCookie = BlockLookup(pathname, OS_UNKNOWN_BLOCKER)) != NULL) {
@@ -673,11 +624,6 @@ struct vop_lookup_args {
       }
    }
 
-   /* We already verified that buffer is big enough. */
-   pathname[pathname_len] = '/';
-   bcopy(cnp->cn_nameptr, &pathname[pathname_len + 1], cnp->cn_namelen);
-   pathname[pathname_len + 1 + cnp->cn_namelen] = 0;
-
    /*
     * Although it is possible to call VMBlockVopBypass(), we'll do a direct
     * call to reduce overhead
@@ -685,6 +631,12 @@ struct vop_lookup_args {
    ldvp = VMBVPTOLOWERVP(dvp);
    vp = lvp = NULL;
 
+   /*
+    * The subsequent VOP_LOOKUP() may advance cnp->cn_nameptr, so save a copy
+    * in order to refer to the current component later.
+    */
+   savename = cnp->cn_nameptr;
+
    error = VOP_LOOKUP(ldvp, &lvp, cnp);
    if (error == EJUSTRETURN && (flags & ISLASTCN) &&
        (dvp->v_mount->mnt_flag & MNT_RDONLY) &&
@@ -702,20 +654,19 @@ struct vop_lookup_args {
          VREF(dvp);
          vrele(lvp);
       } else {
-         error = VMBlockNodeGet(dvp->v_mount, lvp, &vp, pathname);
+         error = VMBlockNodeGet(dvp->v_mount, lvp, &vp, dvp);
          if (error) {
             /* XXX Cleanup needed... */
             panic("VMBlockNodeGet failed");
          }
          *ap->a_vpp = vp;
-         /* The vnode now owns pathname so don't try to free it below. */
-         pathname = NULL;
+         VMBlockSetNodeName(vp, savename);
       }
    }
 
 out:
    if (pathname) {
-      uma_zfree(VMBlockPathnameZone, pathname);
+      VMBlockDestroyBlockName(pathname);
    }
    return error;
 }
@@ -747,10 +698,7 @@ struct vop_open_args {
    int fflag;           // IN: fdesc flags (see fcntl(2))
    struct ucred *cred;  // IN: caller's credentials (usually real uid vs euid)
    struct thread *td;   // IN: calling thread's context      
-   FreeBSD <= 6 --
-      int fdidx;        // IN: file descriptor number alloc'd to this open()
-   FreeBSD >= 7 --
-      struct file *fp   // IN: struct associated with this particular open()
+   int fdidx;           // IN: file descriptor number alloc'd to this open()
 };
 */
 {
@@ -774,12 +722,9 @@ struct vop_open_args {
        *
        * NB:  Allowing only the superuser to open this directory breaks
        *      readdir() of the filesystem root for non-privileged users.
-       *
-       * Also, on FreeBSD 8.0 and newer we check for a specific module priv
-       * because none of the existing privs seemed to match very well.
        */
-      if ((retval = compat_priv_check(ap->a_td, PRIV_DRIVER)) == 0) {
-#if __FreeBSD_version >= 700055
+      if ((retval = suser(ap->a_td)) == 0) {
+#if __FreeBSD_version >= 700000
          fp = ap->a_fp;
 #else
          fp = ap->a_td->td_proc->p_fd->fd_ofiles[ap->a_fdidx];
@@ -943,7 +888,7 @@ struct vop_ioctl_args {
       /*
        * Don't block the mount point!
        */
-      if (!strcmp(VPTOVMB(vp)->name, pathbuf)) {
+      if (!strcmp(VPTOVMB(vp)->componentName, pathbuf)) {
          ret = EINVAL;
       } else {
          ret = (ap->a_command == VMBLOCK_ADD_FILEBLOCK) ?
@@ -1062,7 +1007,7 @@ struct vop_access_args {
 */
 {
    struct vnode *vp = ap->a_vp;
-   compat_accmode_t mode = ap->compat_a_accmode;
+   mode_t mode = ap->a_mode;
 
    /*
     * Disallow write attempts on read-only layers; unless the file is a
@@ -1137,7 +1082,7 @@ struct vop_rename_args {
       vrele(fvp);
       return EXDEV;
    }
-
+   
    return VMBlockVopBypass((struct vop_generic_args *)ap);
 }
 
@@ -1165,11 +1110,11 @@ struct vop_rename_args {
 static int
 VMBlockVopLock(compat_vop_lock_args *ap)
 /*
-struct vop_lock_args {
-   struct vnode *vp;    // IN: vnode operand
-   int flags;           // IN: lockmgr(9) flags
-   struct thread *td;   // IN: calling thread's context
-};
+struct {
+   struct vnode *a_vp;    // IN: vnode operand 
+   int a_flags;           // IN: lockmgr(9) flags
+   struct thread *a_td;   // IN: calling thread's context
+} *ap;
 */
 {
    struct vnode *vp = ap->a_vp;
@@ -1440,8 +1385,8 @@ struct vop_reclaim_args {
    /*
     * Clean up VMBlockNode attachment.
     */
-   if (xp->name) {
-      uma_zfree(VMBlockPathnameZone, xp->name);
+   if (xp->componentName) {
+      free(xp->componentName, M_VMBLOCKFSNODE);
    }
    free(xp, M_VMBLOCKFSNODE);
 
