@@ -26,27 +26,26 @@
 #include <sys/types.h>
 #include <sys/malloc.h>
 
-#if defined __APPLE__
+#if defined(__APPLE__)
 #  include <libkern/libkern.h>  // for rindex
 #endif
 
 #include "fsutil.h"
 #include "cpName.h"
-#include "hgfsEscape.h"
-#include "cpNameLite.h"
+#include "staticEscape.h"
 #include "os.h"
 
-#if defined __APPLE__
+#if defined(__APPLE__)
 char *rindex(const char *ptr, int chr);
 #endif
 
 /*
- * Mac OS sets vnode attributes through the use of a VATTR_RETURN function.
+ * OS X sets vnode attributes through the use of a VATTR_RETURN function.
  * FreeBSD sets vnode attributes directly in the structure. To enable a shared
  * implementation of HgfsAttrToBSD and HgfsSetattrCopy, we define VATTR_RETURN
  * for FreeBSD.
  */
-#if defined __FreeBSD__
+#if defined(__FreeBSD__)
 #define VATTR_RETURN(vn, attr, val) \
    do { (vn)-> attr = (val); } while (0)
 #endif
@@ -117,14 +116,14 @@ HgfsSubmitRequest(HgfsSuperInfo *sip,   // IN: Superinfo containing request list
 /*
  *----------------------------------------------------------------------------
  *
- * HgfsGetStatus --
- *    
- *    Gets the status of the reply packet. If the size of the reply packet
- *    does not lie between the minimum expected size and maximum allowed packet
- *    size, then EPROTO is returned.   
+ * HgfsValidateReply --
+ *
+ *    Validates a reply to ensure that its state is set appropriately and the
+ *    reply is at least the minimum expected size and not greater than the
+ *    maximum allowed packet size.
  *
  * Results:
- *    Returns zero on success, and an error code on error.
+ *    Returns zero on success, and a non-zero on error.
  *
  * Side effects:
  *    None.
@@ -133,48 +132,26 @@ HgfsSubmitRequest(HgfsSuperInfo *sip,   // IN: Superinfo containing request list
  */
 
 int
-HgfsGetStatus(HgfsKReqHandle req,    // IN: Request that contains reply data
-              uint32_t minSize)      // IN: Minimum size expected for the reply
+HgfsValidateReply(HgfsKReqHandle req,    // IN: Request that contains reply data
+                  uint32_t minSize)      // IN: Minimum size expected for the reply
 {
-   HgfsReply *replyHeader;
-   size_t repSize = 0;
-   int ret = 0;
-
    ASSERT(req);
    ASSERT(minSize <= HGFS_PACKET_MAX);  /* we want to know if this fails */
 
    switch (HgfsKReq_GetState(req)) {
-
    case HGFS_REQ_ERROR:
       DEBUG(VM_DEBUG_FAIL, "received reply with error.\n");
-      ret = EPROTO;
-      break;
+      return -1;
 
    case HGFS_REQ_COMPLETED:
-      repSize = HgfsKReq_GetPayloadSize(req);
-      /*
-       * Server sets the packet size equal to size of HgfsReply when it
-       * encounters an error. In order to return correct error code, 
-       * we should first check the status and then check if packet size
-       * lies between minimum expected size and maximum allowed packet size.
-       */
-
-      if (repSize >= sizeof *replyHeader) {
-         replyHeader = (HgfsReply *)HgfsKReq_GetPayload(req);
-         ret = HgfsStatusToBSD(replyHeader->status);
-         if (ret) {
-            break;
-         }
-      }
-
-      if (repSize < minSize || repSize > HGFS_PACKET_MAX) {
+      if ((HgfsKReq_GetPayloadSize(req) < minSize) || (HgfsKReq_GetPayloadSize(req) > HGFS_PACKET_MAX)) {
          DEBUG(VM_DEBUG_FAIL, "successfully "
                "completed reply is too small/big: !(%d < %" FMTSZ "d < %d).\n",
-               minSize, repSize, HGFS_PACKET_MAX);
-         ret = EPROTO;   
+               minSize, HgfsKReq_GetPayloadSize(req), HGFS_PACKET_MAX);
+         return -1;
+      } else {
+         return 0;
       }
-      break;
-
    /*
     * If we get here then there is a programming error in this module:
     *  HGFS_REQ_UNUSED should be for requests in the free list
@@ -184,10 +161,104 @@ HgfsGetStatus(HgfsKReqHandle req,    // IN: Request that contains reply data
     */
    default:
       NOT_REACHED();
-      ret = EPROTO;        /* avoid compiler warning */
+      return -1;        /* avoid compiler warning */
    }
+}
 
-   return ret;
+
+/*
+ * XXX: These were taken directly from hgfs/solaris/vnode.c.  Should we
+ * move them to hgfsUtil.c or similar?  (And Solaris took them from the Linux
+ * implementation.)
+ */
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * HgfsEscapeBuffer --
+ *
+ *    Escape any characters that are not legal in a linux filename,
+ *    which is just the character "/". We also of course have to
+ *    escape the escape character, which is "%".
+ *
+ *    sizeBufOut must account for the NUL terminator.
+ *
+ *    XXX: See the comments in staticEscape.c and staticEscapeW.c to understand
+ *    why this interface sucks.
+ *
+ * Results:
+ *    On success, the size (excluding the NUL terminator) of the
+ *    escaped, NUL terminated buffer.
+ *    On failure (bufOut not big enough to hold result), negative value.
+ *
+ * Side effects:
+ *    None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+HgfsEscapeBuffer(char const *bufIn, // IN:  Buffer with unescaped input
+                 uint32 sizeIn,     // IN:  Size of input buffer (chars)
+                 uint32 sizeBufOut, // IN:  Size of output buffer (bytes)
+                 char *bufOut)      // OUT: Buffer for escaped output
+{
+   /*
+    * This is just a wrapper around the more general escape
+    * routine; we pass it the correct bitvector and the
+    * buffer to escape. [bac]
+    */
+   EscBitVector bytesToEsc;
+
+   ASSERT(bufIn);
+   ASSERT(bufOut);
+
+   /* Set up the bitvector for "/" and "%" */
+   EscBitVector_Init(&bytesToEsc);
+   EscBitVector_Set(&bytesToEsc, (unsigned char)'%');
+   EscBitVector_Set(&bytesToEsc, (unsigned char)'/');
+
+   return StaticEscape_Do('%',
+                          &bytesToEsc,
+                          bufIn,
+                          sizeIn,
+                          sizeBufOut,
+                          bufOut);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * HgfsUnescapeBuffer --
+ *
+ *    Unescape a buffer that was escaped using HgfsEscapeBuffer.
+ *
+ *    The unescaping is done in place in the input buffer, and
+ *    can not fail.
+ *
+ * Results:
+ *    The size (excluding the NUL terminator) of the unescaped, NUL
+ *    terminated buffer.
+ *
+ * Side effects:
+ *    None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+HgfsUnescapeBuffer(char *bufIn,   // IN: Buffer to be unescaped
+                   uint32 sizeIn) // IN: Size of input buffer
+{
+   /*
+    * This is just a wrapper around the more general unescape
+    * routine; we pass it the correct escape characer and the
+    * buffer to unescape. [bac]
+    */
+   ASSERT(bufIn);
+   return StaticEscape_Undo('%', bufIn, sizeIn);
 }
 
 /*
@@ -224,9 +295,9 @@ HgfsGetOpenMode(uint32 flags) // IN: Open flags
     * Preprocessor wrapper kept for when this function is factored out
     * into a common file.
     */
-#if defined _KERNEL || defined KERNEL
+#if defined(_KERNEL) || defined(KERNEL)
    /*
-    * FreeBSD / Mac OS use different values from those in the linux kernel. These are defined in
+    * FreeBSD / OS X use different values from those in the linux kernel. These are defined in
     * <sys/fcntl.h>.
     */
    #undef O_RDONLY
@@ -498,80 +569,84 @@ HgfsMakeFullName(const char *path,      // IN:  Path of directory containing fil
  */
 
 Bool
-HgfsSetattrCopy(HgfsVnodeAttr *vap,      // IN:  Attributes to change to
-                HgfsAttrV2 *hgfsAttrV2,  // OUT: Hgfs attributes to fill in
-                HgfsAttrHint *hints)     // OUT: Hgfs attribute hints
+HgfsSetattrCopy(HGFS_VNODE_ATTR *vap,    // IN:  Attributes to change to
+                HgfsAttr *hgfsAttr,      // OUT: Hgfs attributes to fill in
+                HgfsAttrChanges *update) // OUT: Hgfs attribute changes to make
 {
    Bool ret = FALSE;
 
    ASSERT(vap);
-   ASSERT(hgfsAttrV2);
-   ASSERT(hints);
+   ASSERT(hgfsAttr);
+   ASSERT(update);
 
-   memset(hgfsAttrV2, 0, sizeof *hgfsAttrV2);
-   memset(hints, 0, sizeof *hints);
+   memset(hgfsAttr, 0, sizeof *hgfsAttr);
+   memset(update, 0, sizeof *update);
 
    /*
     * Hgfs supports changing these attributes:
     * o mode bits (permissions)
-    * o uid/gid
     * o size
     * o access/write times
     */
 
-   if (HGFS_VATTR_MODE_IS_ACTIVE(vap, HGFS_VA_MODE)){
+#if defined(__APPLE__)
+   if (VATTR_IS_ACTIVE(vap, va_mode)) {
       DEBUG(VM_DEBUG_COMM, "updating permissions.\n");
-      hgfsAttrV2->mask |= HGFS_ATTR_VALID_SPECIAL_PERMS |
-                          HGFS_ATTR_VALID_OWNER_PERMS |
-                          HGFS_ATTR_VALID_GROUP_PERMS |
-                          HGFS_ATTR_VALID_OTHER_PERMS;
-      hgfsAttrV2->specialPerms = (vap->HGFS_VA_MODE & (S_ISUID | S_ISGID |
-                                  S_ISVTX)) >> HGFS_ATTR_SPECIAL_PERM_SHIFT;
-      hgfsAttrV2->ownerPerms = (vap->HGFS_VA_MODE & S_IRWXU) >>
-                                HGFS_ATTR_OWNER_PERM_SHIFT;
-      hgfsAttrV2->groupPerms = (vap->HGFS_VA_MODE & S_IRWXG) >>
-                                HGFS_ATTR_GROUP_PERM_SHIFT;
-      hgfsAttrV2->otherPerms = vap->HGFS_VA_MODE & S_IRWXO;
+      *update |= HGFS_ATTR_PERMISSIONS;
+      hgfsAttr->permissions = (vap->va_mode & S_IRWXU) >> HGFS_ATTR_MODE_SHIFT;
       ret = TRUE;
    }
 
-   if (HGFS_VATTR_IS_ACTIVE(vap, HGFS_VA_UID)) {
-      DEBUG(VM_DEBUG_COMM, "updating user id.\n");
-      hgfsAttrV2->mask |= HGFS_ATTR_VALID_USERID;
-      hgfsAttrV2->userId = vap->HGFS_VA_UID;
-      ret = TRUE;
-   }
-
-   if (HGFS_VATTR_IS_ACTIVE(vap, HGFS_VA_GID)) {
-      DEBUG(VM_DEBUG_COMM, "updating group id.\n");
-      hgfsAttrV2->mask |= HGFS_ATTR_VALID_GROUPID;
-      hgfsAttrV2->groupId = vap->HGFS_VA_GID;
-      ret = TRUE;
-   }
-
-   if (HGFS_VATTR_IS_ACTIVE(vap, HGFS_VA_ACCESS_TIME_SEC)) {
-      DEBUG(VM_DEBUG_COMM, "updating access time.\n");
-      *hints |= HGFS_ATTR_HINT_SET_ACCESS_TIME;
-      hgfsAttrV2->mask |= HGFS_ATTR_VALID_ACCESS_TIME;
-      hgfsAttrV2->accessTime = HGFS_GET_TIME(vap->HGFS_VA_ACCESS_TIME);
-      ret = TRUE;
-   }
-
-   if (HGFS_VATTR_IS_ACTIVE(vap, HGFS_VA_MODIFY_TIME_SEC)) {
-      DEBUG(VM_DEBUG_COMM, "updating write time.\n");
-      *hints |= HGFS_ATTR_HINT_SET_WRITE_TIME;
-      hgfsAttrV2->mask |= HGFS_ATTR_VALID_WRITE_TIME;
-      hgfsAttrV2->writeTime = HGFS_GET_TIME(vap->HGFS_VA_MODIFY_TIME);
-      ret = TRUE;
-   }
-
-   if (HGFS_VATTR_SIZE_IS_ACTIVE(vap, HGFS_VA_DATA_SIZE)) {
+   if (VATTR_IS_ACTIVE(vap, va_data_size)) {
       DEBUG(VM_DEBUG_COMM, "updating size.\n");
-      hgfsAttrV2->mask |= HGFS_ATTR_VALID_SIZE;
-      hgfsAttrV2->size = vap->HGFS_VA_DATA_SIZE;
+      *update |= HGFS_ATTR_SIZE;
+      hgfsAttr->size = vap->va_data_size;
       ret = TRUE;
    }
 
+   if (VATTR_IS_ACTIVE(vap, va_access_time)) {
+      DEBUG(VM_DEBUG_COMM, "updating access time.\n");
+      *update |= HGFS_ATTR_ACCESS_TIME;
+      hgfsAttr->accessTime = HGFS_GET_TIME(vap->va_access_time);
+      ret = TRUE;
+   }
+
+   if (VATTR_IS_ACTIVE(vap, va_modify_time)) {
+      DEBUG(VM_DEBUG_COMM, "updating write time.\n");
+      *update |= HGFS_ATTR_WRITE_TIME;
+      hgfsAttr->writeTime = HGFS_GET_TIME(vap->va_modify_time);
+      ret = TRUE;
+   }
+#elif defined(__FreeBSD__)
+   if (vap->va_mode != (mode_t)VNOVAL) {
+      DEBUG(VM_DEBUG_COMM, "updating permissions.\n");
+      *update |= HGFS_ATTR_PERMISSIONS;
+      hgfsAttr->permissions = (vap->va_mode & S_IRWXU) >> HGFS_ATTR_MODE_SHIFT;
+      ret = TRUE;
+   }
+
+   if (vap->va_size != (u_quad_t)VNOVAL) {
+      DEBUG(VM_DEBUG_COMM, "updating size.\n");
+      *update |= HGFS_ATTR_SIZE;
+      hgfsAttr->size = vap->va_size;
+      ret = TRUE;
+   }
+
+   if (vap->va_atime.tv_sec != VNOVAL) {
+      DEBUG(VM_DEBUG_COMM, "updating access time.\n");
+      *update |= HGFS_ATTR_ACCESS_TIME;
+      hgfsAttr->accessTime = HGFS_GET_TIME(vap->va_atime);
+      ret = TRUE;
+   }
+
+   if (vap->va_mtime.tv_sec != VNOVAL) {
+      DEBUG(VM_DEBUG_COMM, "updating write time.\n");
+      *update |= HGFS_ATTR_WRITE_TIME;
+      hgfsAttr->writeTime = HGFS_GET_TIME(vap->va_mtime);
+      ret = TRUE;
+   }
+#endif
+
    return ret;
 }
 
@@ -581,7 +656,7 @@ HgfsSetattrCopy(HgfsVnodeAttr *vap,      // IN:  Attributes to change to
  *
  * HgfsAttrToBSD --
  *
- *    Maps Hgfs attributes to Mac OS/BSD attributes, filling the provided BSD
+ *    Maps Hgfs attributes to Solaris attributes, filling the provided Solaris
  *    attribute structure appropriately.
  *
  * Results:
@@ -596,11 +671,8 @@ HgfsSetattrCopy(HgfsVnodeAttr *vap,      // IN:  Attributes to change to
 void
 HgfsAttrToBSD(struct vnode *vp,             // IN:  The vnode for this file
               const HgfsAttrV2 *hgfsAttrV2, // IN:  Hgfs attributes to copy
-              HgfsVnodeAttr *vap)           // OUT: BSD attributes to fill
+              HGFS_VNODE_ATTR *vap)         // OUT: BSD attributes to fill
 {
-   short mode = 0;
-   HgfsSuperInfo *sip = HGFS_VP_TO_SIP(vp);
-
    ASSERT(vp);
    ASSERT(hgfsAttrV2);
    ASSERT(vap);
@@ -610,126 +682,102 @@ HgfsAttrToBSD(struct vnode *vp,             // IN:  The vnode for this file
    DEBUG(VM_DEBUG_ENTRY, "%p -> %p\n", hgfsAttrV2, vap);
 
    /*
-    * Initialize all fields to zero. We don't need to do this for Mac OS
+    * Initialize all fields to zero. We don't need to do this for OS X
     * because the VATTR_RETURN macros take care of it for us.
     */
-#if defined __FreeBSD__
+#if defined(__FreeBSD__)
    VATTR_NULL(vap);
 #endif
 
-   if ((hgfsAttrV2->mask & HGFS_ATTR_VALID_TYPE)) {
-      /* Set the file type. */
-      switch (hgfsAttrV2->type) {
-      case HGFS_FILE_TYPE_REGULAR:
-         HGFS_VATTR_TYPE_RETURN(vap, VREG);
-         DEBUG(VM_DEBUG_ATTR, " Type: VREG\n");
-         break;
+   /* Set the file type. */
+   switch (hgfsAttrV2->type) {
+   case HGFS_FILE_TYPE_REGULAR:
+      VATTR_RETURN(vap, va_type, VREG);
+      DEBUG(VM_DEBUG_ATTR, " Type: VREG\n");
+      break;
 
-      case HGFS_FILE_TYPE_DIRECTORY:
-         HGFS_VATTR_TYPE_RETURN(vap, VDIR);
-         DEBUG(VM_DEBUG_ATTR, " Type: VDIR\n");
-         break;
+   case HGFS_FILE_TYPE_DIRECTORY:
+      VATTR_RETURN(vap, va_type, VDIR);
+      DEBUG(VM_DEBUG_ATTR, " Type: VDIR\n");
+      break;
 
-      case HGFS_FILE_TYPE_SYMLINK:
-         HGFS_VATTR_TYPE_RETURN(vap, VLNK);
-         DEBUG(VM_DEBUG_ATTR, " Type: VLNK\n");
-         break;
-
-      default:
-         /*
-          * There are only the above three filetypes.  If there is an error
-          * elsewhere that provides another value, we set the Solaris type to
-          * none and ASSERT in devel builds.
-          */
-         HGFS_VATTR_TYPE_RETURN(vap, VNON);
-         DEBUG(VM_DEBUG_FAIL, "invalid HgfsFileType provided.\n");
-      }
-   } else {
-      HGFS_VATTR_TYPE_RETURN(vap, VNON);
-      DEBUG(VM_DEBUG_FAIL, "invalid HgfsFileType provided\n");
+   default:
+      /*
+       * There are only the above two filetypes.  If there is an error
+       * elsewhere that provides another value, we set the Solaris type to
+       * none and ASSERT in devel builds.
+       */
+      VATTR_RETURN(vap, va_type, VNON);
+      DEBUG(VM_DEBUG_FAIL, "invalid HgfsFileType provided.\n");
+      ASSERT(0);
    }
 
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_SPECIAL_PERMS) {
-      mode |= hgfsAttrV2->specialPerms << HGFS_ATTR_SPECIAL_PERM_SHIFT;
-   }
+   /* We only have permissions for owners. */
+   VATTR_RETURN(vap, va_mode, (hgfsAttrV2->ownerPerms << HGFS_ATTR_MODE_SHIFT));
 
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_OWNER_PERMS) {
-      mode |= hgfsAttrV2->ownerPerms << HGFS_ATTR_OWNER_PERM_SHIFT;
-   }
+   VATTR_RETURN(vap, va_nlink, 1);               /* fake */
+   VATTR_RETURN(vap, va_uid, 0);                 /* XXX root? */
+   VATTR_RETURN(vap, va_gid, 0);                 /* XXX root? */
 
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_GROUP_PERMS) {
-      mode |= hgfsAttrV2->groupPerms << HGFS_ATTR_GROUP_PERM_SHIFT;
-   }
-
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_OWNER_PERMS) {
-      mode |= hgfsAttrV2->otherPerms;
-   }
-
-   HGFS_VATTR_MODE_RETURN(vap, mode);
-
-   HGFS_VATTR_NLINK_RETURN(vap, 1);               /* fake */
-
-   if (sip->uidSet || (hgfsAttrV2->mask & HGFS_ATTR_VALID_USERID) == 0) {
-      HGFS_VATTR_UID_RETURN(vap, sip->uid);
-   } else {
-      HGFS_VATTR_UID_RETURN(vap, hgfsAttrV2->userId);
-   }
-
-   if (sip->gidSet || (hgfsAttrV2->mask & HGFS_ATTR_VALID_GROUPID) == 0) {
-      HGFS_VATTR_GID_RETURN(vap, sip->gid);
-   } else {
-      HGFS_VATTR_GID_RETURN(vap, hgfsAttrV2->groupId);
-   }
-
-   HGFS_VATTR_FSID_RETURN(vap, HGFS_VP_TO_STATFS(vp)->f_fsid.val[0]);
+   VATTR_RETURN(vap, va_fsid, HGFS_VP_TO_STATFS(vp)->f_fsid.val[0]);
 
    /* Get the node id calculated for this file in HgfsVnodeGet() */
-   HGFS_VATTR_FILEID_RETURN(vap, HGFS_VP_TO_NODEID(vp));
+   VATTR_RETURN(vap, va_fileid, HGFS_VP_TO_NODEID(vp));
+   DEBUG(VM_DEBUG_ATTR, "*HgfsAttrToBSD: fileName %s\n",
+         HGFS_VP_TO_FILENAME(vp));
 
-   HGFS_VATTR_BLOCKSIZE_RETURN(vap, HGFS_BLOCKSIZE);
+    /*
+     * Some of the attribute names or meanings have changed slightly between
+     * OS X and FreeBSD. We handle these special case items here.
+     */
 
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_SIZE) {
-      HGFS_VATTR_BYTES_RETURN(vap, hgfsAttrV2->size);
-      HGFS_VATTR_SIZE_RETURN(vap, hgfsAttrV2->size);
+    DEBUG(VM_DEBUG_ATTR, " Setting size to %"FMT64"u\n", hgfsAttrV2->size);
+
+#if defined(__APPLE__)
+    DEBUG(VM_DEBUG_ATTR, " Node ID: %"FMT64"u\n", vap->va_fileid);
+
+    /* va_iosize is the Optimal I/O blocksize. */
+    VATTR_RETURN(vap, va_iosize, HGFS_BLOCKSIZE);
+    VATTR_RETURN(vap, va_data_size, hgfsAttrV2->size);
+
+    HGFS_SET_TIME(vap->va_access_time, hgfsAttrV2->accessTime);
+    HGFS_SET_TIME(vap->va_modify_time, hgfsAttrV2->writeTime);
+
+   /* Since Windows doesn't keep ctime, we may need to use mtime instead. */
+   if (HGFS_SET_TIME(vap->va_create_time, hgfsAttrV2->attrChangeTime)) {
+      vap->va_create_time = vap->va_modify_time;
    }
 
-   /* 
-    * HGFS_SET_TIME does not mark the attribute as supported (unlike
-    * VATTR_RETURN on Mac OS) so we have to do it explicitly with calls to
-    * VATTR_SET_SUPPORTED. For FreeBSD, HGFS_VATTR_*_SET_SUPPORTED is just a NULL
-    * macro.
+   /* HGFS_SET_TIME does not mark the attribute as supported (unlike
+    * VATTR_RETURN on OS X) so we have to do it explicitly with calls to
+    * VATTR_SET_SUPPORTED.
     */
+   VATTR_SET_SUPPORTED(vap, va_access_time);
+   VATTR_SET_SUPPORTED(vap, va_modify_time);
+   VATTR_SET_SUPPORTED(vap, va_create_time);
 
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_ACCESS_TIME) {
-      HGFS_SET_TIME(vap->HGFS_VA_ACCESS_TIME, hgfsAttrV2->accessTime);
-      HGFS_VATTR_ACCESS_TIME_SET_SUPPORTED(vap);
+#elif defined(__FreeBSD__)
+    DEBUG(VM_DEBUG_ATTR, " Node ID: %ld\n", vap->va_fileid);
+
+    VATTR_RETURN(vap, va_bytes, hgfsAttrV2->size);
+    VATTR_RETURN(vap, va_size, hgfsAttrV2->size);
+    VATTR_RETURN(vap, va_blocksize, HGFS_BLOCKSIZE);
+
+    HGFS_SET_TIME(vap->va_atime, hgfsAttrV2->accessTime);
+    HGFS_SET_TIME(vap->va_mtime, hgfsAttrV2->writeTime);
+   /* Since Windows doesn't keep ctime, we may need to use mtime instead. */
+   if (HGFS_SET_TIME(vap->va_ctime, hgfsAttrV2->attrChangeTime)) {
+      vap->va_ctime = vap->va_mtime;
    }
 
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_WRITE_TIME) {
-      HGFS_SET_TIME(vap->HGFS_VA_MODIFY_TIME, hgfsAttrV2->writeTime);
-      HGFS_VATTR_MODIFY_TIME_SET_SUPPORTED(vap);
-   }
+   /*
+    * This is only set for FreeBSD as there does not seem to be an analogous
+    * attribute for OS X.
+    */
+   DEBUG(VM_DEBUG_ATTR, " Setting birthtime\n");
+   HGFS_SET_TIME(vap->va_birthtime, hgfsAttrV2->creationTime);
 
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_CHANGE_TIME) {
-      HGFS_SET_TIME(vap->HGFS_VA_CHANGE_TIME, hgfsAttrV2->attrChangeTime);
-      HGFS_VATTR_CHANGE_TIME_SET_SUPPORTED(vap);
-   }
-
-   if (hgfsAttrV2->mask & HGFS_ATTR_VALID_CREATE_TIME) {
-      /* Since Windows doesn't keep ctime, we may need to use mtime instead. */
-       HGFS_SET_TIME(vap->HGFS_VA_CREATE_TIME, hgfsAttrV2->creationTime);
-       HGFS_VATTR_CREATE_TIME_SET_SUPPORTED(vap);
-   } else if (hgfsAttrV2->mask & HGFS_ATTR_VALID_WRITE_TIME) {
-       DEBUG(VM_DEBUG_ATTR, "Set create time from write time\n");
-       vap->HGFS_VA_CREATE_TIME = vap->HGFS_VA_MODIFY_TIME;
-       HGFS_VATTR_CREATE_TIME_SET_SUPPORTED(vap);
-   } else {
-       DEBUG(VM_DEBUG_ATTR, "Do not set create time\n");
-   }
-
-   DEBUG(VM_DEBUG_ATTR, "Attrib mask %lld\n", hgfsAttrV2->mask);
-   DEBUG(VM_DEBUG_ATTR, "Supported %lld, active %lld\n", vap->va_supported,
-         vap->va_active);
+#endif
 
    HgfsDebugPrintVattr(vap);
 }
@@ -813,7 +861,7 @@ HgfsStatusToBSD(HgfsStatus hgfsStatus) // IN: Hgfs status msg to be converted
  * rindex --
  *
  *      Search a character string for the last instance of chr. This is only
- *      implemented for Mac OS because it is not exported by the Mac OS kernel.
+ *      implemented for OS X because it is not exported by the OS X kernel.
  *
  * Results:
  *      Pointer to the last instance of chr in the string.
@@ -824,7 +872,7 @@ HgfsStatusToBSD(HgfsStatus hgfsStatus) // IN: Hgfs status msg to be converted
  *----------------------------------------------------------------------------
  */
 
-#if defined __APPLE__
+#if defined(__APPLE__)
 char *
 rindex(const char *ptr, // IN: String to search.
        int chr)         // IN: Char to look for.
@@ -891,8 +939,7 @@ HgfsAttemptToCreateShare(const char *path,  // IN: Path
  *      3) Cross platform string is finally unescaped.
  *
  * Results:
- *      Returns the size (excluding the NULL terminator)  on success and
- *      negative error code on failure.
+ *      Returns 0 on success and an error code on failure.
  *
  * Side effects:
  *      None.
@@ -914,13 +961,12 @@ HgfsNameToWireEncoding(const char *bufIn,  // IN: Buffer to be normalized
       /* Allocating precomposed buffer to be equal to Output buffer. */
       precomposedBuf = os_malloc(bufOutSize, M_WAITOK);
       if (!precomposedBuf) {
-         return -ENOMEM;
+         return ENOMEM;
       }
 
       ret = os_path_to_utf8_precomposed(bufIn, bufInSize, precomposedBuf, bufOutSize);
       if (ret < 0) {
          DEBUG(VM_DEBUG_FAIL, "os_path_to_utf8_precomposed failed.");
-         ret = -EINVAL;
          goto out;
       }
       utf8Buf = precomposedBuf;
@@ -932,10 +978,11 @@ HgfsNameToWireEncoding(const char *bufIn,  // IN: Buffer to be normalized
    if (ret < 0) {
       DEBUG(VM_DEBUG_FAIL,
             "CPName_ConvertTo: Conversion to cross platform name failed.\n");
-      ret = -ENAMETOOLONG;
+      ret = ENAMETOOLONG;
       goto out;
    }
 
+   ret = HgfsUnescapeBuffer(bufOut, ret); // Cannot fail
 out:
    if (precomposedBuf != NULL) {
       os_free(precomposedBuf, bufOutSize);
@@ -948,12 +995,11 @@ out:
  *----------------------------------------------------------------------------
  *
  * HgfsNameFromWireEncoding --
- *      1) Converts input from CPName form if necessary.
- *      2) Result is converted into decomposed form.
+ *      1) Input string is converted into decomposed form.
+ *      2) Decomposed string is finally escaped.
  *
  * Results:
- *      Returns the size (excluding the NULL terminator)  on success and
- *      negative error code on failure.
+ *      Returns 0 on success and an error code on failure.
  *
  * Side effects:
  *      None.
@@ -963,65 +1009,48 @@ out:
 
 int
 HgfsNameFromWireEncoding(const char *bufIn,  // IN: Buffer to be encoded
-                         uint32 inputLength, // IN: Number of characters in the input
+                         uint32 bufInSize,   // IN: Size of input buffer
                          char *bufOut,       // OUT: Encoded buffer will be stored here
                          uint32 bufOutSize)  // IN: Size of output buffer
 {
-   size_t escapedLen;
+   char *decomposedBuf = NULL; //Allocated from M_TEMP; free when done
+   const char *utf8Buf;
+   size_t lenOut;
    int ret = 0;
 
-   /* Output buffer needs one additional byte for NUL terminator. */
-   if (inputLength >= bufOutSize) {
-      return -ENOMEM;
-   }
-   escapedLen = HgfsEscape_GetSize(bufIn, inputLength);
-   if (escapedLen != 0) {
-      HgfsEscape_Do(bufIn, inputLength, bufOutSize, bufOut);
-   } else {
-      escapedLen = inputLength;
-      memcpy(bufOut, bufIn, inputLength);
-   }
-   CPNameLite_ConvertFrom(bufOut, escapedLen, '/');
-
    if (os_utf8_conversion_needed()) {
-      size_t decomposedLen;
-      char *decomposedBuf = NULL;
-      /*
-       * The decomposed form a string can be a lot bigger than the input
-       * buffer size. We allocate a buffer equal to the output buffer.
+      /* 
+       * The decomposed form a string can be a lot bigger than the input 
+       * buffer size. We allocate a buffer equal to the output buffer.  
        */
       decomposedBuf = os_malloc(bufOutSize, M_WAITOK);
       if (!decomposedBuf) {
-         DEBUG(VM_DEBUG_FAIL, "Not enough memory for decomposed buffer size %d.\n",
-			   bufOutSize);
-         return -ENOMEM;
+         return ENOMEM;
       }
       /*
        * Convert the input buffer into decomposed form. Higher layers in
-       * Mac OS expects the name to be in decomposed form.
+       * MacOS X expects the name to be in decomposed form.
        */
-      ret = os_component_to_utf8_decomposed(bufOut, escapedLen, decomposedBuf,
-                                            &decomposedLen, bufOutSize);
+      ret = os_component_to_utf8_decomposed(bufIn, bufInSize, decomposedBuf,
+                                             &lenOut, bufOutSize);
       /*
        * If the decomposed name didn't fit in the buffer or it contained
        * illegal utf8 characters, return back to the caller.
-       * os_component_to_utf8_decomposed returns 0 on success or OS_ERR on failure.
        */
-      if (ret != 0){
+      if (ret < 0){
          DEBUG(VM_DEBUG_FAIL, "os_component_to_utf8_decomposed failed.\n");
-         ret = -EINVAL;
-      } else {
-         if (decomposedLen < bufOutSize) {
-            ret = decomposedLen;
-            memcpy(bufOut, decomposedBuf, decomposedLen + 1);
-          } else {
-            DEBUG(VM_DEBUG_FAIL, "Output buffer is too small.\n");
-            ret = -ENOMEM;
-         }
+         goto out;
       }
-      os_free(decomposedBuf, bufOutSize);
+      utf8Buf = decomposedBuf;
    } else {
-      ret = escapedLen;
+      utf8Buf = bufIn;
+   }
+
+    /* Escape any illegal characters */
+    ret = HgfsEscapeBuffer(utf8Buf, strlen(utf8Buf), bufOutSize, bufOut);
+out:
+   if (decomposedBuf != NULL) {
+      os_free(decomposedBuf, bufOutSize);
    }
    return ret;
 }
