@@ -23,24 +23,20 @@
  *      and thread synchronization routines.
  */
 
-#if !defined _KERNEL
+#if !defined(_KERNEL)
 #  error "This os.c file can only be compiled for the FreeBSD kernel."
 #endif
 
-#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/malloc.h>
 #include <sys/lock.h>         // for struct mtx
 #include <sys/kernel.h>
 #include <vm/uma.h>           // for uma_zone_t
 #include <sys/kthread.h>      // for kthread_create()
-#include <vm/vm.h>
-#include <vm/vm_extern.h>     // for vnode_pager_setsize
 
 #include "vm_basic_types.h"
 #include "os.h"
 #include "debug.h"
-#include "channel.h"
 #include "compat_freebsd.h"
 
 /*
@@ -188,20 +184,12 @@ os_zone_destroy(OS_ZONE_T *zone) // IN
 
 void *
 os_zone_alloc(OS_ZONE_T *zone, // IN
-              int flags)       // IN
+	      int flags)       // IN
 {
-   void *mem;
-   HgfsTransportChannel *channel = gHgfsChannel;
-   HgfsKReqObject *req;
    ASSERT(zone);
    ASSERT(zone->umaZone);
 
-   mem = uma_zalloc(zone->umaZone, flags | M_ZERO);
-   if (mem) {
-      req = (HgfsKReqObject *)mem;
-      req->channel = channel;
-   }
-   return mem;
+   return uma_zalloc(zone->umaZone, flags);
 }
 
 
@@ -224,7 +212,7 @@ os_zone_alloc(OS_ZONE_T *zone, // IN
 
 void
 os_zone_free(OS_ZONE_T *zone, // IN
-             void *mem)       // IN
+	     void *mem)       // IN
 {
    ASSERT(zone);
    ASSERT(zone->umaZone);
@@ -238,7 +226,7 @@ os_zone_free(OS_ZONE_T *zone, // IN
  *
  * os_malloc --
  *
- *      Malloc some memory in a FreeBSD / Mac OS kernel independent manner.
+ *      Malloc some memory in a FreeBSD / OS X kernel independent manner.
  *      This just calls the internal kernel malloc function. According to the
  *      FreeBSD commments, if M_WAITOK is passed to the flags, malloc will never
  *      return NULL.
@@ -265,7 +253,7 @@ os_malloc(size_t size, // IN
  *
  * os_free --
  *
- *      Free some memory in a FreeBSD / Mac OS kernel independent manner.
+ *      Free some memory in a FreeBSD / OS X kernel independent manner.
  *      This just calls the internal kernel free function.
  *
  * Results:
@@ -291,7 +279,7 @@ os_free(void *mem,   // IN
  * os_mutex_alloc_init --
  *
  *      Allocate and initialize a FreeBSD mutex in an OS independent way.
- *      Mtx_name is not used on Mac OS.
+ *      Mtx_name is not used on OS X.
  *
  * Results:
  *      A new mtx which has been allocated and is ready for use.
@@ -316,9 +304,9 @@ os_mutex_alloc_init(const char *mtxName) // IN
 /*
  *----------------------------------------------------------------------------
  *
- * os_mutex_free --
+ * os_mutex_destroy --
  *
- *      Frees a FreeBSD mutex in an OS independent way.
+ *      Destroy a FreeBSD mutex in an OS independent way.
  *
  * Results:
  *      None.
@@ -330,7 +318,7 @@ os_mutex_alloc_init(const char *mtxName) // IN
  */
 
 void
-os_mutex_free(OS_MUTEX_T *mtx) // IN
+os_mutex_destroy(OS_MUTEX_T *mtx) // IN
 {
    mtx_destroy(mtx);
    os_free(mtx, sizeof *mtx);
@@ -413,9 +401,9 @@ os_rw_lock_alloc_init(const char *lckName) // IN
 /*
  *----------------------------------------------------------------------------
  *
- * os_rw_lock_free --
+ * os_rw_lock_destory --
  *
- *      Frees a FreeBSD rwlock in an OS independent way.
+ *      Destroy a FreeBSD rwlock in an OS independent way.
  *
  * Results:
  *      None.
@@ -427,7 +415,7 @@ os_rw_lock_alloc_init(const char *lckName) // IN
  */
 
 void
-os_rw_lock_free(OS_RWLOCK_T *lck) // IN
+os_rw_lock_destroy(OS_RWLOCK_T *lck) // IN
 {
    sx_destroy(lck);
    os_free(lck, sizeof *lck);
@@ -533,7 +521,7 @@ os_rw_lock_unlock_exclusive(OS_RWLOCK_T *lck) // IN
  *
  * os_cv_init --
  *
- *      Initialize a cv under FreeBSD. Under Mac OS, we are actually passed an
+ *      Initialize a cv under FreeBSD. Under OS X, we are actually passed an
  *      object address we will use in place of a cv in later functions. Here
  *      we simply do nothing.
  *
@@ -559,7 +547,7 @@ os_cv_init(OS_CV_T *cv,      // IN
  *
  * os_cv_destroy --
  *
- *      Destroy a cv under FreeBSD. Under Mac OS, we are actually passed an
+ *      Destroy a cv under FreeBSD. Under OS X, we are actually passed an
  *      object address we will use in place of a cv in later functions. Here
  *      we simply do nothing.
  *
@@ -583,7 +571,7 @@ os_cv_destroy(OS_CV_T *cv) // IN
  *
  * os_cv_signal --
  *
- *      Signal a thread to wakeup in a FreeBSD/Mac OS independent way.
+ *      Signal a thread to wakeup in a FreeBSD/OS X independent way.
  *
  * Results:
  *      None.
@@ -638,7 +626,7 @@ os_cv_wait(OS_CV_T *cv,      // IN
  *
  * os_thread_create --
  *
- *      Create an Mac OS or FreeBSD kernel thread in an OS independent way.
+ *      Create an OS X or FreeBSD kernel thread in an OS independent way.
  *
  * Results:
  *      None.
@@ -719,7 +707,7 @@ os_thread_release(OS_THREAD_T thread) // IN
  * Hgfsthreadexit --
  *
  *      Called when a thread is exiting. ErrorCode is returned as the thread exit code
- *      under FreeBSD and ignored under Mac OS.
+ *      under FreeBSD and ignored under OS X.
  *
  * Results:
  *      None.
@@ -873,56 +861,3 @@ os_path_to_utf8_precomposed(char const *bufIn,  // IN
    NOT_IMPLEMENTED();
    return OS_ERR;
 }
-
-
-/*
- *----------------------------------------------------------------------------
- *
- * os_SetSize  --
- *
- *      Notifies memory management system that file size has been changed.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *----------------------------------------------------------------------------
- */
-
-void
-os_SetSize(struct vnode* vp,          // IN: vnode which size has changed
-           off_t newSize)             // IN: new file size
-{
-   vnode_pager_setsize(vp, newSize);
-}
-
-
-/*
- *----------------------------------------------------------------------------
- *
- * os_FlushRange  --
- *
- *      Flushes dirty pages associated with the file.
- *
- * Results:
- *      Always retun 0 (success) for now since it is NOOP.
- *
- * Side effects:
- *      None.
- *----------------------------------------------------------------------------
- */
-
-int
-os_FlushRange(struct vnode *vp,    // IN: vnode which data needs flushing
-              off_t start,         // IN: starting offset in the file to flush
-              uint32_t length)     // IN: length of data to flush
-{
-   /*
-    * XXX: NOOP for now. This routine is needed to maintain coherence
-    *      between memory mapped data and data for read/write operations.
-    *      Will need to implement when adding support for memory mapped files to HGFS
-    *      for FreeBsd.
-    */
-   return 0;
-}
