@@ -30,7 +30,6 @@
 
 #include "hgfs_kernel.h"
 #include "requestInt.h"
-#include "channel.h"
 
 /*
  * Macros
@@ -63,10 +62,10 @@ OS_CV_T hgfsKReqWorkItemCv;
  * Local functions (prototypes)
  */
 
-static int   HgfsKReqZCtor(void *mem, int size, void *arg, int flags);
-static void  HgfsKReqZDtor(void *mem, int size, void *arg);
-static int   HgfsKReqZInit(void *mem, int size, int flags);
-static void  HgfsKReqZFini(void *mem, int size);
+   static int   HgfsKReqZCtor(void *mem, int size, void *arg, int flags);
+   static void  HgfsKReqZDtor(void *mem, int size, void *arg);
+   static int   HgfsKReqZInit(void *mem, int size, int flags);
+   static void  HgfsKReqZFini(void *mem, int size);
 
 /*
  * Global functions (definitions)
@@ -112,7 +111,7 @@ HgfsKReq_SysInit(void)
    }
 
    /*
-    * This is a nop on Mac OS because we don't actually have a condition variable
+    * This is a nop on OS X because we don't actually have a condition variable
     * to initialize.
     */
    os_cv_init(&hgfsKReqWorkItemCv, HGFS_FS_NAME "_workcv");
@@ -125,7 +124,7 @@ HgfsKReq_SysInit(void)
    if (ret != 0) {
       os_cv_destroy(&hgfsKReqWorkItemCv);
       os_zone_destroy(hgfsKReqZone);
-      os_mutex_free(hgfsKReqWorkItemLock);
+      os_mutex_destroy(hgfsKReqWorkItemLock);
       return HGFS_ERR;
    }
 
@@ -170,7 +169,7 @@ HgfsKReq_SysFini(void)
    os_thread_release(hgfsKReqWorkerThread);
    os_zone_destroy(hgfsKReqZone);
    os_cv_destroy(&hgfsKReqWorkItemCv);
-   os_mutex_free(hgfsKReqWorkItemLock);
+   os_mutex_destroy(hgfsKReqWorkItemLock);
 
    return 0;
 }
@@ -237,7 +236,7 @@ HgfsKReq_FreeContainer(HgfsKReqContainerHandle container) // IN: file system's
    ASSERT(container);
    ASSERT(DblLnkLst_IsLinked(&container->list) == FALSE);
 
-   os_mutex_free(container->listLock);
+   os_mutex_destroy(container->listLock);
    os_free(container, sizeof(*container));
 }
 
@@ -371,7 +370,7 @@ HgfsKReq_ContainerIsEmpty(HgfsKReqContainerHandle container)       // IN:
  *      interrupted by a signal.
  *
  * Results:
- *      Pointer to fresh HgfsKReqHandle or NULL on failure.
+ *      Pointer to fresh HgfsKReqObject or NULL on failure.
  *
  * Side effects:
  *      Request inserted into caller's requests container.  This routine may
@@ -380,37 +379,15 @@ HgfsKReq_ContainerIsEmpty(HgfsKReqContainerHandle container)       // IN:
  *----------------------------------------------------------------------------
  */
 
-HgfsKReqHandle
-HgfsKReq_AllocateRequest(HgfsKReqContainerHandle container,  // IN:
-                         int *errorRet)                      // OUT:
+HgfsKReqObject *
+HgfsKReq_AllocateRequest(HgfsKReqContainerHandle container)        // IN
 {
    HgfsKReqObject *req;
 
-   ASSERT(errorRet);
    ASSERT(container);
 
-   *errorRet = 0;
-
-   if (!gHgfsChannel) {
-      *errorRet = EIO;
-      return NULL;
-   }
-
-   /*
-    * In case we don't have any channel currently, set up a new channel.
-    * Note that we remember the channel from which request was allocated
-    * and sent, thereby making sure that we free it via correct channel.
-    */
-   if (gHgfsChannel->status != HGFS_CHANNEL_CONNECTED) {
-      if (!HgfsSetupNewChannel()) {
-         *errorRet = EIO;
-         return NULL;
-      }
-   }
-
    req = os_zone_alloc(hgfsKReqZone, M_WAITOK);
    if (!req) {
-      *errorRet = ENOMEM;
       return NULL;
    }
 
@@ -606,8 +583,7 @@ HgfsKReq_GetId(HgfsKReqHandle request) // IN: Request to get the ID for
  *
  *      Return a pointer to the payload area of a request.  Callers may write
  *      Hgfs packet data directly to this area.  It's guaranteed to hold at
- *      most HGFS_PACKET_MAX (6144) bytes. For Hgfs version 3, the caller should
- *      explicitly write request header (HgfsRequest) into this area.
+ *      most HGFS_PACKET_MAX (6144) bytes.
  *
  * Results:
  *      Pointer to the payload area.
@@ -619,7 +595,7 @@ HgfsKReq_GetId(HgfsKReqHandle request) // IN: Request to get the ID for
  */
 
 char *
-HgfsKReq_GetPayload(HgfsKReqHandle request)  // IN: Request to get pointer to payload 
+HgfsKReq_GetPayload(HgfsKReqHandle request)     // IN:
 {
    ASSERT(request);
 
@@ -672,7 +648,7 @@ HgfsKReq_GetPayloadSize(HgfsKReqHandle request) // IN: Request to get the size o
 
 void
 HgfsKReq_SetPayloadSize(HgfsKReqHandle request, // IN: Request object
-                        size_t payloadSize)     // IN: New payload size
+                        size_t payloadSize) // IN: New payload size
 {
    ASSERT(request);
    ASSERT(payloadSize <= HGFS_PACKET_MAX);
@@ -739,12 +715,22 @@ HgfsKReqZInit(void *mem,     // IN: Pointer to the allocated object
               int size,      // IN: Size of item being initialized [ignored]
               int flags)     // IN: malloc(9) style flags
 {
-   static unsigned int id = 0;
    HgfsKReqObject *req = (HgfsKReqObject *)mem;
    ASSERT(size == sizeof *req);
 
-   os_add_atomic(&id, 1);
-   req->id = id;
+   /*
+    * Zero out the object.  (Do NOT pass UMA_ZEROINIT to uma_zcreate, as
+    * that will override this routine and zero everything -after- us.)
+    */
+   bzero(req, sizeof *req);
+
+   /*
+    * Request IDs are a 32-bit unsigned integer.  Conveniently enough for us,
+    * our memory addresses provide (at least) 32 bits.
+    *
+    * Assumption:  Kernel memory will never straddle a 2**32 byte boundary.
+    */
+   req->id = (uint32_t)((unsigned long)req & 0xffffffff);
    req->state = HGFS_REQ_UNUSED;
    req->stateLock = os_mutex_alloc_init("hgfs_req_mtx");
    if (!req->stateLock) {
@@ -756,10 +742,11 @@ HgfsKReqZInit(void *mem,     // IN: Pointer to the allocated object
    /* Reset list pointers. */
    DblLnkLst_Init(&req->fsNode);
    DblLnkLst_Init(&req->pendingNode);
-   DblLnkLst_Init(&req->sentNode);
 
    /* Clear packet of request before allocating to clients. */
    bzero(&req->__rpc_packet, sizeof req->__rpc_packet);
+   bcopy(HGFS_SYNC_REQREP_CLIENT_CMD, req->__rpc_packet._command,
+         HGFS_SYNC_REQREP_CLIENT_CMD_LEN);
 
    return 0;
 }
@@ -790,7 +777,7 @@ HgfsKReqZFini(void *mem,     // IN: Pointer to object leaving the UMA cache
    HgfsKReqObject *req = (HgfsKReqObject *)mem;
    ASSERT(size == sizeof *req);
    ASSERT(req->state == HGFS_REQ_UNUSED);
-   os_mutex_free(req->stateLock);
+   os_mutex_destroy(req->stateLock);
    os_cv_destroy(&req->stateCv);
 }
 
@@ -829,6 +816,9 @@ HgfsKReqZCtor(void *mem,     // IN: Pointer to memory allocated to user
    /* Initialize state & reference count. */
    req->state = HGFS_REQ_ALLOCATED;
    req->refcnt = 1;
+
+   ASSERT(!strncmp(req->__rpc_packet._command, HGFS_SYNC_REQREP_CLIENT_CMD, HGFS_SYNC_REQREP_CLIENT_CMD_LEN));
+
    return 0;
 }
 
