@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,24 +27,28 @@
 #include "driver-config.h"
 
 #include <asm/atomic.h>
+#include <asm/semaphore.h>
 #include <linux/errno.h>
 #include <linux/list.h>
 #include <linux/module.h>
 #include <linux/pagemap.h>
-#include "compat_cred.h"
+#include "compat_completion.h"
 #include "compat_dcache.h"
 #include "compat_fs.h"
 #include "compat_kernel.h"
+#include "compat_kthread.h"
 #include "compat_sched.h"
-#include "compat_semaphore.h"
 #include "compat_slab.h"
 #include "compat_spinlock.h"
 #include "compat_string.h"
 #include "compat_uaccess.h"
 #include "compat_version.h"
 
+/* Must be included after sched.h. */
+#include <linux/smp_lock.h>
+
+#include "bdhandler.h"
 #include "filesystem.h"
-#include "transport.h"
 #include "hgfsDevLinux.h"
 #include "hgfsProto.h"
 #include "hgfsUtil.h"
@@ -56,58 +60,78 @@
 #include "rpcout.h"
 #include "hgfs.h"
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 25)
+#define KERNEL_25_FS 0
+#else
+#define KERNEL_25_FS 1
+#endif
+
+#define HGFS_BD_THREAD_NAME "VMware hgfs backdoor handler"
+
 /* Synchronization primitives. */
-DEFINE_SPINLOCK(hgfsBigLock);
+spinlock_t hgfsBigLock = SPIN_LOCK_UNLOCKED;
+long hgfsReqThreadFlags;
+wait_queue_head_t hgfsReqThreadWait;
+struct task_struct *hgfsReqThread;
+COMPAT_KTHREAD_DECLARE_STOP_INFO();
 
 /* Other variables. */
-compat_kmem_cache *hgfsInodeCache;
+compat_kmem_cache *hgfsReqCache = NULL;
+compat_kmem_cache *hgfsInodeCache = NULL;
+RpcOut *hgfsRpcOut = NULL;
+unsigned int hgfsIdCounter = 0;
+struct list_head hgfsReqsUnsent;
 
 /* Global protocol version switch. */
-HgfsOp hgfsVersionOpen;
-HgfsOp hgfsVersionRead;
-HgfsOp hgfsVersionWrite;
-HgfsOp hgfsVersionClose;
-HgfsOp hgfsVersionSearchOpen;
-HgfsOp hgfsVersionSearchRead;
-HgfsOp hgfsVersionSearchClose;
-HgfsOp hgfsVersionGetattr;
-HgfsOp hgfsVersionSetattr;
-HgfsOp hgfsVersionCreateDir;
-HgfsOp hgfsVersionDeleteFile;
-HgfsOp hgfsVersionDeleteDir;
-HgfsOp hgfsVersionRename;
-HgfsOp hgfsVersionQueryVolumeInfo;
-HgfsOp hgfsVersionCreateSymlink;
+atomic_t hgfsVersionOpen;
+atomic_t hgfsVersionRead;
+atomic_t hgfsVersionWrite;
+atomic_t hgfsVersionClose;
+atomic_t hgfsVersionSearchOpen;
+atomic_t hgfsVersionSearchRead;
+atomic_t hgfsVersionSearchClose;
+atomic_t hgfsVersionGetattr;
+atomic_t hgfsVersionSetattr;
+atomic_t hgfsVersionCreateDir;
+atomic_t hgfsVersionDeleteFile;
+atomic_t hgfsVersionDeleteDir;
+atomic_t hgfsVersionRename;
+atomic_t hgfsVersionQueryVolumeInfo;
+atomic_t hgfsVersionCreateSymlink;
 
 /* Private functions. */
 static inline unsigned long HgfsComputeBlockBits(unsigned long blockSize);
 static compat_kmem_cache_ctor HgfsInodeCacheCtor;
-static HgfsSuperInfo *HgfsInitSuperInfo(void *rawData,
-                                        uint32 mountInfoVersion);
+static HgfsSuperInfo *HgfsInitSuperInfo(HgfsMountInfo *mountInfo);
 static int HgfsReadSuper(struct super_block *sb,
                          void *rawData,
                          int flags);
-static void HgfsResetOps(void);
 
 
 /* HGFS filesystem high-level operations. */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
-static struct dentry *HgfsMount(struct file_system_type *fs_type,
-                                int flags,
-                                const char *dev_name,
-                                void *rawData);
-#elif defined(VMW_GETSB_2618)
+#if KERNEL_25_FS /* { */
+#   if defined(VMW_GETSB_2618)
 static int HgfsGetSb(struct file_system_type *fs_type,
                      int flags,
                      const char *dev_name,
                      void *rawData,
                      struct vfsmount *mnt);
-#else
+#   elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 70)
 static struct super_block *HgfsGetSb(struct file_system_type *fs_type,
                                      int flags,
                                      const char *dev_name,
                                      void *rawData);
-#endif
+#   else
+static struct super_block *HgfsGetSb(struct file_system_type *fs_type,
+                                     int flags,
+                                     char *dev_name,
+                                     void *rawData);
+#   endif
+#else /* } { */
+static struct super_block *HgfsReadSuper24(struct super_block *sb,
+                                           void *rawData,
+                                           int flags);
+#endif /* } */
 
 /* HGFS filesystem type structure. */
 static struct file_system_type hgfsType = {
@@ -115,14 +139,15 @@ static struct file_system_type hgfsType = {
    .name         = HGFS_NAME,
 
    .fs_flags     = FS_BINARY_MOUNTDATA,
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
-   .mount        = HgfsMount,
-#else
+#if KERNEL_25_FS
    .get_sb       = HgfsGetSb,
-#endif
    .kill_sb      = kill_anon_super,
+#else
+   .read_super   = HgfsReadSuper24,
+#endif
 };
 
+
 /*
  * Private functions implementations.
  */
@@ -175,9 +200,18 @@ HgfsComputeBlockBits(unsigned long blockSize)
  *-----------------------------------------------------------------------------
  */
 
+#ifdef VMW_KMEMCR_CTOR_HAS_3_ARGS
 static void
-HgfsInodeCacheCtor(COMPAT_KMEM_CACHE_CTOR_ARGS(slabElem)) // IN: slab item to initialize
+HgfsInodeCacheCtor(void *slabElem,           // IN: slab item to initialize
+                   compat_kmem_cache *cache, // IN: cache slab is from
+                   unsigned long flags)      // IN: flags associated with allocation
+#else
+static void
+HgfsInodeCacheCtor(compat_kmem_cache *cache, // IN: cache slab is from
+                   void *slabElem)           // IN: slab item to initialize
+#endif
 {
+#ifdef VMW_EMBED_INODE
    HgfsInodeInfo *iinfo = (HgfsInodeInfo *)slabElem;
 
    /*
@@ -186,262 +220,7 @@ HgfsInodeCacheCtor(COMPAT_KMEM_CACHE_CTOR_ARGS(slabElem)) // IN: slab item to in
     * much of the VFS inode members.
     */
    inode_init_once(&iinfo->inode);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsValidateMountInfo --
- *
- *    Validate the the user mode mounter information.
- *
- * Results:
- *    Zero on success or -EINVAL if we pass in an unknown version.
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-HgfsValidateMountInfo(void *rawData,             // IN: Fs-specific mount data
-                      uint32 *mountInfoVersion)  // OUT: Mount flags
-{
-   HgfsMountInfoV1 *infoV1;
-   HgfsMountInfo *info;
-   uint32 *magicNumber;
-   int retVal = -EINVAL;
-
-   ASSERT(mountInfoVersion);
-
-   /* Sanity check the incoming user data. */
-   if (rawData == NULL) {
-      LOG(4, (KERN_CRIT LGPFX "%s: error: no user supplied mount data\n",
-              __func__));
-      goto exit;
-   }
-
-   /* Magic number is always first 4 bytes of the header. */
-   magicNumber = rawData;
-   if (*magicNumber != HGFS_SUPER_MAGIC) {
-      LOG(4, (KERN_CRIT LGPFX "%s: error: user supplied mount data is not valid!\n",
-              __func__));
-      goto exit;
-   }
-
-   /*
-    * Looks like HGFS data, now validate the version so that we can
-    * proceed and extract the required settings from the user.
-    */
-   info = rawData;
-   infoV1 = rawData;
-   if ((info->version == HGFS_PROTOCOL_VERSION_1 ||
-        info->version == HGFS_PROTOCOL_VERSION) &&
-        info->infoSize == sizeof *info) {
-      /*
-       * The current version is validated with the size and magic number.
-       * Note the version can be either 1 or 2 as it was not bumped initially.
-       * Furthermore, return the version as HGFS_PROTOCOL_VERSION (2) only since
-       * the objects are the same and it simplifies field extractions.
-       */
-      LOG(4, (KERN_DEBUG LGPFX "%s: mount data version %d passed\n",
-              __func__, info->version));
-      *mountInfoVersion = HGFS_PROTOCOL_VERSION;
-      retVal = 0;
-   } else if (infoV1->version == HGFS_PROTOCOL_VERSION_1) {
-      /*
-       * The version 1 is validated with the version and magic number.
-       * Note the version can be only be 1 and if so does not collide with version 2 of
-       * the header (which would be the info size field).
-       */
-      LOG(4, (KERN_DEBUG LGPFX "%s: mount data version %d passed\n",
-              __func__, info->version));
-      *mountInfoVersion = infoV1->version;
-      retVal = 0;
-   } else {
-      /*
-       * The version and info size fields could not be validated
-       * for the known structure. It is probably a newer version.
-       */
-      LOG(4, (KERN_DEBUG LGPFX "%s: error: user supplied mount data version %d\n",
-              __func__, infoV1->version));
-   }
-
-exit:
-   return retVal;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsGetMountInfoV1 --
- *
- *    Gets the fields of interest from the user mode mounter version 1.
- *
- * Results:
- *    None
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-HgfsGetMountInfoV1(HgfsMountInfoV1 *mountInfo, // IN: version 1 mount data
-                   uint32 *mntFlags,           // OUT: Mount flags
-                   uint32 *ttl,                // OUT: seconds until revalidate
-                   uid_t *uid,                 // OUT: owner
-                   gid_t *gid,                 // OUT: group
-                   mode_t *fmask,              // OUT: file mask
-                   mode_t *dmask,              // OUT: directory mask
-                   const char **shareHost,     // OUT: share host name
-                   const char **shareDir)      // OUT: share directory
-{
-   ASSERT(mountInfo);
-
-   *mntFlags = 0;
-   /*
-    * If the mounter specified a uid or gid, we will prefer them over any uid
-    * or gid given to us by the server.
-    */
-   if (mountInfo->uidSet) {
-      *mntFlags |= HGFS_MNT_SET_UID;
-      *uid = mountInfo->uid;
-   }
-
-   if (mountInfo->gidSet) {
-      *mntFlags |= HGFS_MNT_SET_GID;
-      *gid = mountInfo->gid;
-   }
-
-   *fmask = mountInfo->fmask;
-   *dmask = mountInfo->dmask;
-   *ttl = mountInfo->ttl;
-   *shareHost = mountInfo->shareNameHost;
-   *shareDir = mountInfo->shareNameDir;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsGetMountInfoV2 --
- *
- *    Gets the fields of interest from the user mode mounter version 2.
- *
- * Results:
- *    None
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-HgfsGetMountInfoV2(HgfsMountInfo *mountInfo,   // IN: version 2 mount data
-                   uint32 *mntFlags,           // OUT: Mount flags
-                   uint32 *ttl,                // OUT: seconds until revalidate
-                   uid_t *uid,                 // OUT: owner
-                   gid_t *gid,                 // OUT: group
-                   mode_t *fmask,              // OUT: file mask
-                   mode_t *dmask,              // OUT: directory mask
-                   const char **shareHost,     // OUT: share host name
-                   const char **shareDir)      // OUT: share directory
-{
-   ASSERT(mountInfo);
-
-   *mntFlags = 0;
-
-   if ((mountInfo->flags & HGFS_MNTINFO_SERVER_INO) != 0) {
-      *mntFlags |= HGFS_MNT_SERVER_INUM;
-   }
-
-   /*
-    * If the mounter specified a uid or gid, we will prefer them over any uid
-    * or gid given to us by the server.
-    */
-   if (mountInfo->uidSet) {
-      *mntFlags |= HGFS_MNT_SET_UID;
-      *uid = mountInfo->uid;
-   }
-
-   if (mountInfo->gidSet) {
-      *mntFlags |= HGFS_MNT_SET_GID;
-      *gid = mountInfo->gid;
-   }
-
-   *fmask = mountInfo->fmask;
-   *dmask = mountInfo->dmask;
-   *ttl = mountInfo->ttl;
-   *shareHost = mountInfo->shareNameHost;
-   *shareDir = mountInfo->shareNameDir;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsGetMountInfo --
- *
- *    Gets the fields of interest from the user mode mounter.
- *
- * Results:
- *    Zero on success or -EINVAL if we pass in an unknown version.
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-HgfsGetMountInfo(void *rawData,            // IN: Fs-specific mount data
-                 uint32 mountInfoVersion,  // IN: mount information version
-                 uint32 *mntFlags,         // OUT: Mount flags
-                 uint32 *ttl,             // OUT: seconds until revalidate
-                 uid_t *uid,              // OUT: owner
-                 gid_t *gid,              // OUT: group
-                 mode_t *fmask,           // OUT: file mask
-                 mode_t *dmask,           // OUT: directory mask
-                 const char **shareHost,  // OUT: share host name
-                 const char **shareDir)   // OUT: share path
-{
-   int result = 0;
-
-   switch (mountInfoVersion) {
-   case HGFS_PROTOCOL_VERSION_1:
-      HgfsGetMountInfoV1(rawData,
-                         mntFlags,
-                         ttl,
-                         uid,
-                         gid,
-                         fmask,
-                         dmask,
-                         shareHost,
-                         shareDir);
-      break;
-   case HGFS_PROTOCOL_VERSION:
-      HgfsGetMountInfoV2(rawData,
-                         mntFlags,
-                         ttl,
-                         uid,
-                         gid,
-                         fmask,
-                         dmask,
-                         shareHost,
-                         shareDir);
-      break;
-   default:
-      ASSERT(FALSE);
-      result = -EINVAL;
-   }
-
-   return result;
+#endif
 }
 
 
@@ -463,79 +242,39 @@ HgfsGetMountInfo(void *rawData,            // IN: Fs-specific mount data
  */
 
 static HgfsSuperInfo *
-HgfsInitSuperInfo(void *rawData,            // IN: Passed down from the user
-                  uint32 mountInfoVersion)  // IN: version
+HgfsInitSuperInfo(HgfsMountInfo *mountInfo) // IN: Passed down from the user
 {
    HgfsSuperInfo *si = NULL;
    int result = 0;
    int len;
-   char *tmpName = NULL;
+   char *tmpName;
    Bool hostValid;
-   uint32 mntFlags = 0;
-   uint32 ttl = 0;
-   uid_t uid = 0;
-   gid_t gid = 0;
-   mode_t fmask = 0;
-   mode_t dmask = 0;
-   const char *shareHost;
-   const char *shareDir;
 
    si = kmalloc(sizeof *si, GFP_KERNEL);
    if (!si) {
       result = -ENOMEM;
-      goto out_error_si;
-   }
-   memset(si, 0, sizeof *si);
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
-   result = bdi_setup_and_register(&si->bdi, HGFS_NAME);
-   if (result) {
-      LOG(6, (KERN_DEBUG "VMware hgfs: %s: initialize backing device info"
-              "failed. (%d)\n", __func__, result));
-      goto out_error_si;
-   }
-#endif
-
-   result = HgfsGetMountInfo(rawData,
-                             mountInfoVersion,
-                             &mntFlags,
-                             &ttl,
-                             &uid,
-                             &gid,
-                             &fmask,
-                             &dmask,
-                             &shareHost,
-                             &shareDir);
-   if (result < 0) {
-      LOG(6, (KERN_DEBUG LGPFX "%s: error: get mount info %d\n", __func__, result));
-      goto out_error_last;
+      goto out2;
    }
 
    /*
-    * Initialize with the default flags.
+    * If the mounter specified a uid or gid, we will prefer them over any uid
+    * or gid given to us by the server.
     */
-   si->mntFlags = mntFlags;
-
-   si->uid = current_uid();
-   if ((si->mntFlags & HGFS_MNT_SET_UID) != 0) {
-      kuid_t mntUid = make_kuid(current_user_ns(), uid);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
-      if (uid_valid(mntUid))
-#endif
-         si->uid = mntUid;
+   si->uidSet = mountInfo->uidSet;
+   if (si->uidSet) {
+      si->uid = mountInfo->uid;
+   } else {
+      si->uid = current->uid;
    }
-
-   si->gid = current_gid();
-   if ((si->mntFlags & HGFS_MNT_SET_GID) != 0) {
-      kgid_t mntGid = make_kgid(current_user_ns(), gid);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
-      if (gid_valid(mntGid))
-#endif
-         si->gid = mntGid;
+   si->gidSet = mountInfo->gidSet;
+   if (si->gidSet) {
+      si->gid = mountInfo->gid;
+   } else {
+      si->gid = current->gid;
    }
-   si->fmask = fmask;
-   si->dmask = dmask;
-   si->ttl = ttl * HZ; // in ticks
+   si->fmask = mountInfo->fmask;
+   si->dmask = mountInfo->dmask;
+   si->ttl = mountInfo->ttl * HZ; // in ticks
 
    /*
     * We don't actually care about this field (though we may care in the
@@ -551,15 +290,15 @@ HgfsInitSuperInfo(void *rawData,            // IN: Passed down from the user
       LOG(6, (KERN_DEBUG "VMware hgfs: HgfsInitSuperInfo: could not obtain "
               "memory for filename\n"));
       result = -ENOMEM;
-      goto out_error_last;
+      goto out2;
    }
 
-   len = strncpy_from_user(tmpName, shareHost, PATH_MAX);
+   len = strncpy_from_user(tmpName, mountInfo->shareNameHost, PATH_MAX);
    if (len < 0 || len >= PATH_MAX) {
       LOG(6, (KERN_DEBUG "VMware hgfs: HgfsInitSuperInfo: strncpy_from_user "
               "on host string failed\n"));
       result = len < 0 ? len : -ENAMETOOLONG;
-      goto out_error_last;
+      goto out;
    }
 
    hostValid = strcmp(tmpName, ".host") == 0;
@@ -567,26 +306,26 @@ HgfsInitSuperInfo(void *rawData,            // IN: Passed down from the user
       LOG(6, (KERN_DEBUG "VMware hgfs: HgfsInitSuperInfo: host string is "
               "invalid\n"));
       result = -EINVAL;
-      goto out_error_last;
+      goto out;
    }
 
    /*
     * Perform a simple sanity check on the directory portion: it must begin
     * with forward slash.
     */
-   len = strncpy_from_user(tmpName, shareDir, PATH_MAX);
+   len = strncpy_from_user(tmpName, mountInfo->shareNameDir, PATH_MAX);
    if (len < 0 || len >= PATH_MAX) {
       LOG(6, (KERN_DEBUG "VMware hgfs: HgfsInitSuperInfo: strncpy_from_user "
               "on dir string failed\n"));
       result = len < 0 ? len : -ENAMETOOLONG;
-      goto out_error_last;
+      goto out;
    }
 
    if (*tmpName != '/') {
       LOG(6, (KERN_DEBUG "VMware hgfs: HgfsInitSuperInfo: dir string is "
               "invalid\n"));
       result = -EINVAL;
-      goto out_error_last;
+      goto out;
    }
 
    /*
@@ -604,25 +343,18 @@ HgfsInitSuperInfo(void *rawData,            // IN: Passed down from the user
       LOG(6, (KERN_DEBUG "VMware hgfs: HgfsInitSuperInfo: kstrdup on "
               "dir string failed\n"));
       result = -ENOMEM;
-      goto out_error_last;
+      goto out;
    }
    si->shareNameLen = strlen(si->shareName);
 
-out_error_last:
-   if (tmpName) {
-      compat___putname(tmpName);
-   }
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
-   if (result) {
-      bdi_destroy(&si->bdi);
-   }
-#endif
-out_error_si:
+  out:
+   compat___putname(tmpName);
+  out2:
    if (result) {
+      /* If we failed, si->shareName couldn't have been allocated. */
       kfree(si);
       si = ERR_PTR(result);
    }
-
    return si;
 }
 
@@ -656,23 +388,27 @@ HgfsReadSuper(struct super_block *sb, // OUT: Superblock object
               void *rawData,          // IN: Fs-specific mount data
               int flags)              // IN: Mount flags
 {
-   int result = 0;
+   int result;
    HgfsSuperInfo *si;
-   struct dentry *rootDentry = NULL;
-   uint32 mountInfoVersion;
+   HgfsMountInfo *mountInfo;
+   struct dentry *rootDentry;
 
    ASSERT(sb);
 
    LOG(6, (KERN_DEBUG "VMware hgfs: HgfsReadSuper: entered\n"));
 
    /* Sanity check the incoming user data. */
-   result = HgfsValidateMountInfo(rawData, &mountInfoVersion);
-   if (result < 0) {
-      return result;
+   mountInfo = (HgfsMountInfo *)rawData;
+   if (!mountInfo ||
+       mountInfo->magicNumber != HGFS_SUPER_MAGIC ||
+       mountInfo->version != HGFS_PROTOCOL_VERSION) {
+      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsReadSuper: bad mount data passed "
+              "in by user, failing!\n"));
+      return -EINVAL;
    }
 
    /* Setup both our superblock and the VFS superblock. */
-   si = HgfsInitSuperInfo(rawData, mountInfoVersion);
+   si = HgfsInitSuperInfo(mountInfo);
    if (IS_ERR(si)) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsReadSuper: superinfo "
               "init failed\n"));
@@ -682,14 +418,6 @@ HgfsReadSuper(struct super_block *sb, // OUT: Superblock object
    sb->s_magic = HGFS_SUPER_MAGIC;
    sb->s_op = &HgfsSuperOperations;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
-   sb->s_d_op = &HgfsDentryOperations;
-#endif
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
-   sb->s_bdi = &si->bdi;
-#endif
-
    /*
     * If s_maxbytes isn't initialized, the generic write path may fail. In
     * most kernels, s_maxbytes is initialized by the kernel's superblock
@@ -697,7 +425,9 @@ HgfsReadSuper(struct super_block *sb, // OUT: Superblock object
     * it. Note that we'll initialize it anyway, because the default value is
     * MAX_NON_LFS, which caps our filesize at 2^32 bytes.
     */
+#ifdef VMW_SB_HAS_MAXBYTES
    sb->s_maxbytes = MAX_LFS_FILESIZE;
+#endif
 
    /*
     * These two operations will make sure that our block size and the bits
@@ -708,15 +438,27 @@ HgfsReadSuper(struct super_block *sb, // OUT: Superblock object
    sb->s_blocksize = 1 << sb->s_blocksize_bits;
 
    /*
-    * Create the root dentry and its corresponding inode.
+    * We can't use d_alloc_root() here directly because it requires a valid
+    * inode, which only HgfsInstantiate will create. So instead, we'll do the
+    * work in pieces. First we'll allocate the dentry and setup its parent
+    * and superblock. Then HgfsInstantiate will do the rest, issuing a getattr,
+    * getting the inode, and instantiating the dentry with it.
     */
-   result = HgfsInstantiateRoot(sb, &rootDentry);
+   rootDentry = compat_d_alloc_name(NULL, "/");
+   if (rootDentry == NULL) {
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsReadSuper: Could not allocate "
+              "root dentry\n"));
+      result = -ENOMEM;
+      goto exit;
+   }
+   rootDentry->d_parent = rootDentry;
+   rootDentry->d_sb = sb;
+   result = HgfsInstantiate(rootDentry, HGFS_ROOT_INO, NULL);
    if (result) {
       LOG(4, (KERN_WARNING "VMware hgfs: HgfsReadSuper: Could not instantiate "
               "root dentry\n"));
       goto exit;
    }
-
    sb->s_root = rootDentry;
 
    LOG(6, (KERN_DEBUG "VMware hgfs: HgfsReadSuper: finished %s\n", si->shareName));
@@ -724,10 +466,6 @@ HgfsReadSuper(struct super_block *sb, // OUT: Superblock object
   exit:
    if (result) {
       dput(rootDentry);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
-      bdi_destroy(&si->bdi);
-      sb->s_bdi = NULL;
-#endif
       kfree(si->shareName);
       kfree(si);
    }
@@ -739,33 +477,8 @@ HgfsReadSuper(struct super_block *sb, // OUT: Superblock object
  * HGFS filesystem high-level operations.
  */
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
-/*
- *-----------------------------------------------------------------------------
- *
- * HgfsMount --
- *
- *    Invokes generic kernel code to mount a deviceless filesystem.
- *
- * Results:
- *    Mount's root dentry structure on success
- *    ERR_PTR()-encoded negative error code on failure
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-struct dentry *
-HgfsMount(struct file_system_type *fs_type, // IN: file system type of mount
-          int flags,                        // IN: mount flags
-          const char *dev_name,             // IN: device mounting on
-          void *rawData)                    // IN: mount arguments
-{
-   return mount_nodev(fs_type, flags, rawData, HgfsReadSuper);
-}
-#elif defined VMW_GETSB_2618
+#if KERNEL_25_FS
+#if defined(VMW_GETSB_2618)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -812,53 +525,51 @@ HgfsGetSb(struct file_system_type *fs_type,
  *-----------------------------------------------------------------------------
  */
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 70)
 static struct super_block *
 HgfsGetSb(struct file_system_type *fs_type,
 	  int flags,
 	  const char *dev_name,
 	  void *rawData)
+#else
+static struct super_block *
+HgfsGetSb(struct file_system_type *fs_type,
+	  int flags,
+	  char *dev_name,
+	  void *rawData)
+#endif
 {
    return get_sb_nodev(fs_type, flags, rawData, HgfsReadSuper);
 }
 #endif
+#else
 
 
 /*
  *-----------------------------------------------------------------------------
  *
- * HgfsResetOps --
+ * HgfsReadSuper24 --
  *
- *      Reset ops with more than one opcode back to the desired opcode.
+ *    Compatibility wrapper for 2.4.x kernels read_super.
+ *    Converts success to sb, and failure to NULL.
  *
  * Results:
- *      None.
+ *    The initialized superblock on success
+ *    NULL on failure
  *
  * Side effects:
- *      None.
+ *    None
  *
  *-----------------------------------------------------------------------------
  */
 
-static void
-HgfsResetOps(void)
-{
-   hgfsVersionOpen            = HGFS_OP_OPEN_V3;
-   hgfsVersionRead            = HGFS_OP_READ_V3;
-   hgfsVersionWrite           = HGFS_OP_WRITE_V3;
-   hgfsVersionClose           = HGFS_OP_CLOSE_V3;
-   hgfsVersionSearchOpen      = HGFS_OP_SEARCH_OPEN_V3;
-   hgfsVersionSearchRead      = HGFS_OP_SEARCH_READ_V3;
-   hgfsVersionSearchClose     = HGFS_OP_SEARCH_CLOSE_V3;
-   hgfsVersionGetattr         = HGFS_OP_GETATTR_V3;
-   hgfsVersionSetattr         = HGFS_OP_SETATTR_V3;
-   hgfsVersionCreateDir       = HGFS_OP_CREATE_DIR_V3;
-   hgfsVersionDeleteFile      = HGFS_OP_DELETE_FILE_V3;
-   hgfsVersionDeleteDir       = HGFS_OP_DELETE_DIR_V3;
-   hgfsVersionRename          = HGFS_OP_RENAME_V3;
-   hgfsVersionQueryVolumeInfo = HGFS_OP_QUERY_VOLUME_INFO_V3;
-   hgfsVersionCreateSymlink   = HGFS_OP_CREATE_SYMLINK_V3;
+static struct super_block *
+HgfsReadSuper24(struct super_block *sb,
+		void *rawData,
+		int flags) {
+   return HgfsReadSuper(sb, rawData, flags) ? NULL : sb;
 }
-
+#endif
 
 /*
  * Public function implementations.
@@ -883,9 +594,26 @@ HgfsResetOps(void)
 Bool
 HgfsInitFileSystem(void)
 {
+   Bool success = FALSE;
+
    /* Initialize primitives. */
+   INIT_LIST_HEAD(&hgfsReqsUnsent);
+   init_waitqueue_head(&hgfsReqThreadWait);
+   hgfsReqThread = NULL;
+   hgfsReqThreadFlags = 0;
    HgfsResetOps();
 
+   /* Setup the request slab allocator. */
+   hgfsReqCache = compat_kmem_cache_create("hgfsReqCache",
+                                           sizeof (HgfsReq),
+                                           0,
+                                           SLAB_HWCACHE_ALIGN,
+                                           NULL);
+   if (hgfsReqCache == NULL) {
+      printk(KERN_WARNING "VMware hgfs: failed to create request allocator\n");
+      goto exit;
+   }
+
    /* Setup the inode slab allocator. */
    hgfsInodeCache = compat_kmem_cache_create("hgfsInodeCache",
                                              sizeof (HgfsInodeInfo),
@@ -894,11 +622,15 @@ HgfsInitFileSystem(void)
                                              HgfsInodeCacheCtor);
    if (hgfsInodeCache == NULL) {
       printk(KERN_WARNING "VMware hgfs: failed to create inode allocator\n");
-      return FALSE;
+      goto exit;
    }
 
-   /* Initialize the transport. */
-   HgfsTransportInit();
+   /* Create backdoor handler. */
+   hgfsReqThread = compat_kthread_run(HgfsBdHandler, NULL, HGFS_NAME);
+   if (IS_ERR(hgfsReqThread)) {
+      printk(KERN_WARNING "VMware hgfs: failed to create kernel thread\n");
+      goto exit;
+   }
 
    /*
     * Register the filesystem. This should be the last thing we do
@@ -906,12 +638,29 @@ HgfsInitFileSystem(void)
     */
    if (register_filesystem(&hgfsType)) {
       printk(KERN_WARNING "VMware hgfs: failed to register filesystem\n");
-      kmem_cache_destroy(hgfsInodeCache);
-      return FALSE;
+      goto exit;
    }
    LOG(4, (KERN_DEBUG "VMware hgfs: Module Loaded\n"));
+#ifdef HGFS_ENABLE_WRITEBACK
+   LOG(4, (KERN_DEBUG "VMware hgfs: writeback cache enabled\n"));
+#endif
+   success = TRUE;
 
-   return TRUE;
+  exit:
+
+   /* Cleanup if an error occurred. */
+   if (success == FALSE) {
+      if (!IS_ERR(hgfsReqThread)) {
+         compat_kthread_stop(hgfsReqThread);
+      }
+      if (hgfsInodeCache != NULL) {
+         kmem_cache_destroy(hgfsInodeCache);
+      }
+      if (hgfsReqCache != NULL) {
+         kmem_cache_destroy(hgfsReqCache);
+      }
+   }
+   return success;
 }
 
 
@@ -936,6 +685,14 @@ HgfsCleanupFileSystem(void)
 {
    Bool success = TRUE;
 
+/* FIXME: Check actual kernel version when RR's modules went in */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 45)
+   if (MOD_IN_USE) {
+      printk(KERN_WARNING "VMware hgfs: filesystem in use, removal failed\n");
+      success = FALSE;
+   }
+#endif
+
   /*
    * Unregister the filesystem. This should be the first thing we do in
    * the module cleanup code.
@@ -945,11 +702,12 @@ HgfsCleanupFileSystem(void)
       success = FALSE;
    }
 
-   /* Transport cleanup. */
-   HgfsTransportExit();
+   /* Kill the backdoor handler thread. */
+   compat_kthread_stop(hgfsReqThread);
 
    /* Destroy the inode and request slabs. */
    kmem_cache_destroy(hgfsInodeCache);
+   kmem_cache_destroy(hgfsReqCache);
 
    LOG(4, (KERN_DEBUG "VMware hgfs: Module Unloaded\n"));
    return success;
