@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,12 +27,12 @@
 
 /* Must come before any kernel header file. */
 #include "driver-config.h"
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
-#include <linux/backing-dev.h>
-#endif
+
 #include <asm/atomic.h>
+#include <asm/semaphore.h>
+#include "compat_completion.h"
 #include "compat_fs.h"
-#include "compat_semaphore.h"
+#include "compat_kthread.h"
 #include "compat_slab.h"
 #include "compat_spinlock.h"
 #include "compat_version.h"
@@ -47,7 +47,7 @@
 /* Logging stuff. */
 #ifdef VMX86_DEVEL
 extern int LOGLEVEL_THRESHOLD;
-#define LGPFX "VMware hgfs: "
+
 #define LOG(level, args) ((void) (LOGLEVEL_THRESHOLD >= (level) ? (printk args) : 0))
 #else
 #define LOG(level, args)
@@ -76,39 +76,40 @@ extern int LOGLEVEL_THRESHOLD;
  * Macros for accessing members that are private to this code in
  * sb/inode/file structs.
  */
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 5, 0)
-typedef uid_t kuid_t;
-typedef gid_t kgid_t;
-#define from_kuid(_ns, _kuid)            (_kuid)
-#define from_kgid(_ns, _kgid)            (_kgid)
-#define make_kuid(_ns, _uid)             (_uid)
-#define make_kgid(_ns, _gid)             (_gid)
-#endif
-
-/*
- * Since the f_dentry disappeared we do this locally.
- * It is used quite extensively and only one other driver
- * is affected by this so it is done locally and not
- * as part of the common compat_fs.h includes.
- */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 19, 0)
-#ifndef f_dentry
-#define f_dentry                         f_path.dentry
-#endif
-#endif
-
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 42)
+#define HGFS_SET_SB_TO_COMMON(sb, common) do { (sb)->u.generic_sbp = (common); } while (0)
+#define HGFS_SB_TO_COMMON(sb)             ((HgfsSuperInfo *)(sb)->u.generic_sbp)
+#else
 #define HGFS_SET_SB_TO_COMMON(sb, common) do { (sb)->s_fs_info = (common); } while (0)
 #define HGFS_SB_TO_COMMON(sb)             ((HgfsSuperInfo *)(sb)->s_fs_info)
+#endif
 
+#ifdef VMW_EMBED_INODE
 #define INODE_GET_II_P(_inode) container_of(_inode, HgfsInodeInfo, inode)
+#elif defined(VMW_INODE_2618)
+#define INODE_GET_II_P(inode) ((HgfsInodeInfo *)(inode)->i_private)
+#else
+#define INODE_GET_II_P(inode) ((HgfsInodeInfo *)(inode)->u.generic_ip)
+#endif
 
-#if defined VMW_INODE_2618
+#if defined(VMW_INODE_2618)
 #define INODE_SET_II_P(inode, info) do { (inode)->i_private = (info); } while (0)
 #else
 #define INODE_SET_II_P(inode, info) do { (inode)->u.generic_ip = (info); } while (0)
 #endif
 
+/* 2.5.x kernels support nanoseconds timestamps. */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 48)
+#define HGFS_DECLARE_TIME(unixtm) time_t unixtm
+#define HGFS_EQUAL_TIME(unixtm1, unixtm2) (unixtm1 == unixtm2)
+#define HGFS_SET_TIME(unixtm,nttime) HgfsConvertFromNtTime(&unixtm, nttime)
+#define HGFS_GET_TIME(unixtm) HgfsConvertToNtTime(unixtm, 0L)
+#define HGFS_GET_CURRENT_TIME() HgfsConvertToNtTime(CURRENT_TIME, 0L)
+/*
+ * Beware! This macro returns list of two elements. Do not add braces around.
+ */
+#define HGFS_PRINT_TIME(unixtm) unixtm, 0L
+#else
 #define HGFS_DECLARE_TIME(unixtm) struct timespec unixtm
 #define HGFS_EQUAL_TIME(unixtm1, unixtm2) timespec_equal(&unixtm1, &unixtm2)
 #define HGFS_SET_TIME(unixtm,nttime) HgfsConvertFromNtTimeNsec(&unixtm, nttime)
@@ -122,6 +123,15 @@ typedef gid_t kgid_t;
  * Beware! This macro returns list of two elements. Do not add braces around.
  */
 #define HGFS_PRINT_TIME(unixtm) unixtm.tv_sec, unixtm.tv_nsec
+#endif
+
+/*
+ * The writeback support we're using (set_page_dirty()) was added in
+ * 2.5.12, so we only support writeback from then on.
+ */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 12)
+#define HGFS_ENABLE_WRITEBACK
+#endif
 
 /*
  * For files opened in our actual Host/Guest filesystem, the
@@ -132,35 +142,34 @@ typedef gid_t kgid_t;
 #define FILE_SET_FI_P(file, info) do { (file)->private_data = info; } while (0)
 #define FILE_GET_FI_P(file)         ((HgfsFileInfo *)(file)->private_data)
 
-#define HGFS_MNT_SET_UID            (1 << 0) /* Was the UID specified at mount-time? */
-#define HGFS_MNT_SET_GID            (1 << 1) /* Was the GID specified at mount-time? */
-#define HGFS_MNT_SERVER_INUM        (1 << 2) /* Use inode numbers from the server? */
+/*
+ * When waking up the request handler thread, these are the possible operations
+ * one can ask it to perform.
+ */
+#define HGFS_REQ_THREAD_SEND  (1 << 0)
+#define HGFS_REQ_THREAD_EXIT  (1 << 1)
 
 /* Data kept in each superblock in sb->u. */
 typedef struct HgfsSuperInfo {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
-   struct backing_dev_info bdi;     /* Kernel VFS uses it to check whether our backend
-                                       need to writeback dirty pages among other things. */
-#endif
-   kuid_t uid;                      /* UID of user who mounted this fs. */
-   kgid_t gid;                      /* GID of user who mounted this fs. */
+   uid_t uid;                       /* UID of user who mounted this fs. */
+   Bool uidSet;                     /* Was the UID specified at mount-time? */
+   gid_t gid;                       /* GID of user who mounted this fs. */
+   Bool gidSet;                     /* Was the GID specified at mount-time? */
    mode_t fmask;                    /* File permission mask. */
    mode_t dmask;                    /* Directory permission mask. */
    uint32 ttl;                      /* Maximum dentry age (in ticks). */
    char *shareName;                 /* Mounted share name. */
    size_t shareNameLen;             /* To avoid repeated strlen() calls. */
-   uint32 mntFlags;                 /* HGFS mount flags */
 } HgfsSuperInfo;
 
 /*
  * HGFS specific per-inode data.
  */
 typedef struct HgfsInodeInfo {
+#ifdef VMW_EMBED_INODE
    /* Embedded inode. */
    struct inode inode;
-
-   /* Inode number given by the host. */
-   uint64 hostFileId;
+#endif
 
    /* Was the inode number for this inode generated via iunique()? */
    Bool isFakeInodeNumber;
@@ -168,12 +177,8 @@ typedef struct HgfsInodeInfo {
    /* Is this a fake inode created in HgfsCreate that has yet to be opened? */
    Bool createdAndUnopened;
 
-   /*
-    * The number of write back pages to the file which is tracked so any
-    * concurrent file validations such as reads will not invalidate the cache.
-    */
-   unsigned long numWbPages;
-   struct list_head listWbPages;
+   /* Is this inode referenced by HGFS? (needed by HgfsInodeLookup()) */
+   Bool isReferencedInode;
 
    /* List of open files for this inode. */
    struct list_head files;
@@ -195,16 +200,6 @@ typedef struct HgfsFileInfo {
     * choose one with appropriate permissions.
     */
    HgfsOpenMode mode;
-
-   /*
-    * Do we need to reopen a directory ? Note that this is only used
-    * for directories.
-    */
-   Bool isStale;
-
-   /* Directory read position for tracking. */
-   loff_t direntPos;
-
 } HgfsFileInfo;
 
 
@@ -218,6 +213,19 @@ typedef struct HgfsFileInfo {
  */
 extern spinlock_t hgfsBigLock;
 
+/*
+ * The request handler thread uses hgfsReqThreadWait to wake up and handle
+ * IO. Possible operations include:
+ *   -Sending outgoing HGFS requests.
+ *   -Shutting down the request handler thread.
+ *
+ * Finally, we use hgfsReqThread to synchronize the stopping of the
+ * backdoor handler thread.
+ */
+extern long hgfsReqThreadFlags;
+extern wait_queue_head_t hgfsReqThreadWait;
+extern struct task_struct *hgfsReqThread;
+
 /* Hgfs filesystem structs. */
 extern struct super_operations HgfsSuperOperations;
 extern struct dentry_operations HgfsDentryOperations;
@@ -229,22 +237,26 @@ extern struct file_operations HgfsDirFileOperations;
 extern struct address_space_operations HgfsAddressSpaceOperations;
 
 /* Other global state. */
+extern compat_kmem_cache *hgfsReqCache;
 extern compat_kmem_cache *hgfsInodeCache;
+extern RpcOut *hgfsRpcOut;
+extern unsigned int hgfsIdCounter;
+extern struct list_head hgfsReqsUnsent;
 
-extern HgfsOp hgfsVersionOpen;
-extern HgfsOp hgfsVersionRead;
-extern HgfsOp hgfsVersionWrite;
-extern HgfsOp hgfsVersionClose;
-extern HgfsOp hgfsVersionSearchOpen;
-extern HgfsOp hgfsVersionSearchRead;
-extern HgfsOp hgfsVersionSearchClose;
-extern HgfsOp hgfsVersionGetattr;
-extern HgfsOp hgfsVersionSetattr;
-extern HgfsOp hgfsVersionCreateDir;
-extern HgfsOp hgfsVersionDeleteFile;
-extern HgfsOp hgfsVersionDeleteDir;
-extern HgfsOp hgfsVersionRename;
-extern HgfsOp hgfsVersionQueryVolumeInfo;
-extern HgfsOp hgfsVersionCreateSymlink;
+extern atomic_t hgfsVersionOpen;
+extern atomic_t hgfsVersionRead;
+extern atomic_t hgfsVersionWrite;
+extern atomic_t hgfsVersionClose;
+extern atomic_t hgfsVersionSearchOpen;
+extern atomic_t hgfsVersionSearchRead;
+extern atomic_t hgfsVersionSearchClose;
+extern atomic_t hgfsVersionGetattr;
+extern atomic_t hgfsVersionSetattr;
+extern atomic_t hgfsVersionCreateDir;
+extern atomic_t hgfsVersionDeleteFile;
+extern atomic_t hgfsVersionDeleteDir;
+extern atomic_t hgfsVersionRename;
+extern atomic_t hgfsVersionQueryVolumeInfo;
+extern atomic_t hgfsVersionCreateSymlink;
 
 #endif // _HGFS_DRIVER_MODULE_H_
