@@ -40,6 +40,7 @@
 
 #include <asm/bug.h>
 #include <asm/uaccess.h>
+#include <asm/semaphore.h>
 #include <asm/string.h>
 #include <linux/buffer_head.h>
 #include <linux/proc_fs.h>
@@ -47,7 +48,6 @@
 #include "compat_fs.h"
 #include "compat_module.h"
 #include "compat_namei.h"
-#include "compat_mutex.h"
 #include "compat_slab.h"
 #include "compat_workqueue.h"
 
@@ -65,13 +65,6 @@ MODULE_AUTHOR("VMware, Inc.");
 MODULE_DESCRIPTION("VMware Sync Driver");
 MODULE_VERSION(VMSYNC_DRIVER_VERSION_STRING);
 MODULE_LICENSE("GPL v2");
-/*
- * Starting with SLE10sp2, Novell requires that IHVs sign a support agreement
- * with them and mark their kernel modules as externally supported via a
- * change to the module header. If this isn't done, the module will not load
- * by default (i.e., neither mkinitrd nor modprobe will accept it).
- */
-MODULE_INFO(supported, "external");
 
 static int VmSyncRelease(struct inode* inode,
                          struct file *file);
@@ -112,7 +105,7 @@ typedef struct VmSyncBlockDevice {
 
 typedef struct VmSyncState {
    struct list_head     devices;
-   compat_mutex_t       lock;
+   struct semaphore     lock;
    compat_delayed_work  thawTask;
 } VmSyncState;
 
@@ -121,7 +114,7 @@ typedef struct VmSyncState {
  * Serializes freeze operations. Used to make sure that two different
  * fds aren't allowed to freeze the same device.
  */
-static compat_mutex_t gFreezeLock;
+static struct semaphore gFreezeLock;
 
 /* A global count of how many devices are currently frozen by the driver. */
 static atomic_t gFreezeCount;
@@ -158,7 +151,7 @@ VmSyncThawDevices(void  *_state)  // IN
 
    state = (VmSyncState *) _state;
 
-   compat_mutex_lock(&state->lock);
+   down(&state->lock);
    cancel_delayed_work(&state->thawTask);
    list_for_each_safe(cur, tmp, &state->devices) {
       dev = list_entry(cur, VmSyncBlockDevice, list);
@@ -169,7 +162,7 @@ VmSyncThawDevices(void  *_state)  // IN
       list_del_init(&dev->list);
       kmem_cache_free(gBlockDeviceCache, dev);
    }
-   compat_mutex_unlock(&state->lock);
+   up(&state->lock);
 }
 
 
@@ -227,7 +220,7 @@ VmSyncAddPath(const VmSyncState *state,   // IN
    struct nameidata nd;
    VmSyncBlockDevice *dev;
 
-   if ((result = compat_path_lookup(path, LOOKUP_FOLLOW, &nd)) != 0) {
+   if ((result = path_lookup(path, LOOKUP_FOLLOW, &nd)) != 0) {
       goto exit;
    }
    inode = compat_vmw_nd_to_dentry(nd)->d_inode;
@@ -322,8 +315,8 @@ VmSyncFreezeDevices(VmSyncState *state,            // IN
       return PTR_ERR(paths);
    }
 
-   compat_mutex_lock(&gFreezeLock);
-   compat_mutex_lock(&state->lock);
+   down(&gFreezeLock);
+   down(&state->lock);
 
    /*
     * First, try to add all paths to the list of paths to be frozen.
@@ -368,8 +361,8 @@ VmSyncFreezeDevices(VmSyncState *state,            // IN
       }
    }
 
-   compat_mutex_unlock(&state->lock);
-   compat_mutex_unlock(&gFreezeLock);
+   up(&state->lock);
+   up(&gFreezeLock);
 
    if (result == 0) {
       compat_schedule_delayed_work(&state->thawTask, VMSYNC_THAW_TASK_DELAY);
@@ -583,11 +576,18 @@ VmSyncRelease(struct inode *inode,  // IN
  *-----------------------------------------------------------------------------
  */
 
+#ifdef VMW_KMEMCR_CTOR_HAS_3_ARGS
 static void
-VmSyncBlockDeviceCtor(COMPAT_KMEM_CACHE_CTOR_ARGS(slabelem))  // IN
+VmSyncBlockDeviceCtor(void *slabelem,           // IN
+                      compat_kmem_cache *cache, // IN
+                      unsigned long flags)      // IN
+#else
+static void
+VmSyncBlockDeviceCtor(compat_kmem_cache *cache, // IN
+                      void *slabelem)           // IN
+#endif
 {
-   VmSyncBlockDevice *dev = slabelem;
-
+   VmSyncBlockDevice *dev = (VmSyncBlockDevice *) slabelem;
    INIT_LIST_HEAD(&dev->list);
    dev->bdev = NULL;
    dev->sb = NULL;
@@ -610,15 +610,22 @@ VmSyncBlockDeviceCtor(COMPAT_KMEM_CACHE_CTOR_ARGS(slabelem))  // IN
  *-----------------------------------------------------------------------------
  */
 
+#ifdef VMW_KMEMCR_CTOR_HAS_3_ARGS
 static void
-VmSyncStateCtor(COMPAT_KMEM_CACHE_CTOR_ARGS(slabelem))  // IN
+VmSyncStateCtor(void *slabelem,           // IN
+                compat_kmem_cache *cache, // IN
+                unsigned long flags)      // IN
+#else
+static void
+VmSyncStateCtor(compat_kmem_cache *cache, // IN
+                void *slabelem)           // IN
+#endif
 {
-   VmSyncState *state = slabelem;
-
+   VmSyncState *state = (VmSyncState *) slabelem;
    INIT_LIST_HEAD(&state->devices);
    COMPAT_INIT_DELAYED_WORK(&state->thawTask,
-                            VmSyncThawDevicesCallback, state);
-   compat_mutex_init(&state->lock);
+                            VmSyncThawDevicesCallback, &state);
+   init_MUTEX(&state->lock);
 }
 
 
@@ -645,7 +652,7 @@ init_module(void)
    struct proc_dir_entry *controlProcEntry;
 
    atomic_set(&gFreezeCount, 0);
-   compat_mutex_init(&gFreezeLock);
+   init_MUTEX(&gFreezeLock);
 
    /* Create the slab allocators for the module. */
    gBlockDeviceCache = compat_kmem_cache_create("VmSyncBlockDeviceCache",
@@ -670,9 +677,9 @@ init_module(void)
    }
 
    /* Create /proc/driver/vmware-sync */
-   controlProcEntry = create_proc_entry("driver/vmware-sync",
+   controlProcEntry = create_proc_entry("vmware-sync",
                                         S_IFREG | S_IRUSR | S_IRGRP | S_IROTH,
-                                        NULL);
+                                        proc_root_driver);
    if (!controlProcEntry) {
       printk(KERN_ERR "vmsync: could not create /proc/driver/vmware-sync\n");
       kmem_cache_destroy(gSyncStateCache);
@@ -704,7 +711,7 @@ init_module(void)
 void
 cleanup_module(void)
 {
-   remove_proc_entry("driver/vmware-sync", NULL);
+   remove_proc_entry("vmware-sync", proc_root_driver);
    kmem_cache_destroy(gSyncStateCache);
    kmem_cache_destroy(gBlockDeviceCache);
 }
