@@ -25,16 +25,12 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <gdk/gdkkeysyms.h>
 
-#include "toolboxGtkInt.h"
+#include "toolboxInt.h"
 #include "vm_assert.h"
 #include "vm_app.h"
 #include "eventManager.h"
 #include "guestApp.h"
-#if !defined(OPEN_VM_TOOLS)
-#  include "installerdb.h"
-#endif
 #include "vmcheck.h"
 #include "debug.h"
 #include "strutil.h"
@@ -46,8 +42,6 @@
 #include "smallIcon.xpm"
 #include "conf.h"
 #include "toolboxgtk_version.h"
-#include "util.h"
-#include "system.h"
 
 #include "embed_version.h"
 VM_EMBED_VERSION(TOOLBOXGTK_VERSION_STRING);
@@ -68,26 +62,13 @@ static char THIS_FILE[] = __FILE__;
 /*
  * Globals
  */
-static char *hlpDir = NULL;
+static char hlpDir[PATH_MAX];
 static Display *gXDisplay;
 static Window gXRoot;
 RpcIn *gRpcInCtlPanel;
 static GtkWidget *toolsMain;
 static Bool optionAutoHide;
 static guint gTimeoutId;
-const char **gNativeEnviron;
-
-/* Help pages. These need to be in the same order as the tabs in the UI. */
-#define HELP_INDEX      "index.html"
-#define HELP_TOPIC(t)   HELP_INDEX "?context=vm_tools&topic=" #t
-static const char *gHelpPages[] = {
-   HELP_INDEX,
-   HELP_TOPIC(IDH_OPTIONS),
-   HELP_TOPIC(IDH_DEVICES),
-   HELP_TOPIC(IDH_SCRIPTS),
-   HELP_TOPIC(IDH_SHRINK),
-   HELP_TOPIC(IDH_ABOUT)
-};
 
 /*
  * All signals that:
@@ -107,35 +88,32 @@ static int const gSignals[] = {
 /*
  * From toolboxInt.h
  */
-GList *gIconList;
+GdkPixmap* pixmap;
+GdkBitmap* bitmask;
+GdkColormap* colormap;
 GtkWidget *optionsTimeSync;
 GtkWidget *scriptsApply;
 DblLnkLst_Links *gEventQueue;
 
-static void ToolsMainCleanupRpc(void);
-static void ToolsMainSignalHandler(int sig);
-static void ToolsMain_OnDestroy(GtkWidget *widget, gpointer data);
-static void ToolsMain_YesNoBoxOnClicked(GtkButton *btn, gpointer user_data);
-static void ToolsMain_YesNoBoxOnDestroy(GtkWidget *widget, gpointer data);
-static void ToolsMain_OnHelp(gpointer btn, gpointer data);
-static void ToolsMain_OpenHelp(const char *help);
-static gint ToolsMain_CheckF1Help(GtkWidget *widget, GdkEventKey *event,
-                                  gpointer data);
+void ToolsMainCleanupRpc(void);
+void ToolsMainSignalHandler(int sig);
+void ToolsMain_OnDestroy(GtkWidget *widget, gpointer data);
+void ToolsMain_YesNoBoxOnClicked(GtkButton *btn, gpointer user_data);
+void ToolsMain_OnHelp(gpointer btn, gpointer data);
+void ToolsMain_OpenHelp(const char *help);
 
-static GtkWidget* ToolsMain_Create(void);
+GtkWidget* ToolsMain_Create(void);
 
 
-static Bool RpcInResetCB(RpcInData *data);
-static Bool RpcInSetOptionCB(char const **result, size_t *resultLen,
-                             const char *name, const char *args,
-                             size_t argsSize, void *clientData);
-static Bool RpcInCapRegCB(char const **result, size_t *resultLen,
-                          const char *name, const char *args,
-                          size_t argsSize, void *clientData);
-static void RpcInErrorCB(void *clientdata, char const *status);
-static gint EventQueuePump(gpointer data);
+Bool RpcInResetCB(RpcInData *data);
+Bool RpcInSetOptionCB(char const **result, size_t *resultLen, const char *name,
+                      const char *args, size_t argsSize, void *clientData);
+Bool RpcInCapRegCB(char const **result, size_t *resultLen, const char *name,
+                   const char *args, size_t argsSize, void *clientData);
+void RpcInErrorCB(void *clientdata, char const *status);
+gint EventQueuePump(gpointer data);
 
-static void ShowUsage(char const *prog);
+void ShowUsage(char const *prog);
 
 
 /*
@@ -154,7 +132,7 @@ static void ShowUsage(char const *prog);
  *-----------------------------------------------------------------------------
  */
 
-static void
+void
 ToolsMainCleanupRpc(void)
 {
    if (gRpcInCtlPanel) {
@@ -190,8 +168,7 @@ ToolsMainCleanupRpc(void)
  *-----------------------------------------------------------------------------
  */
 
-static void
-ToolsMainSignalHandler(int sig) // IN
+void ToolsMainSignalHandler(int sig) // IN
 {
    /* We want to kill the event manager before gtk_main_quit. */
    ToolsMainCleanupRpc();
@@ -215,31 +192,23 @@ ToolsMainSignalHandler(int sig) // IN
  *-----------------------------------------------------------------------------
  */
 
-static void
+void
 ToolsMain_OpenHelp(const char *help) // IN
 {
-   gchar *helpPage;
+   char helpPage[1000];
 
-   if (hlpDir == NULL) {
-      ToolsMain_MsgBox("Error", "Unable to determine where help pages are stored.");
+   if (help == NULL) {
+      ToolsMain_MsgBox("Error", "No help was found for the page.");
       return;
    }
 
-   helpPage = g_strdup_printf("%s/%s", hlpDir, HELP_INDEX);
-   if (help == NULL || !g_file_test(helpPage, G_FILE_TEST_IS_REGULAR)) {
-      ToolsMain_MsgBox("Error", "No help was found.");
-      g_free(helpPage);
-      return;
-   }
-
-   g_free(helpPage);
-   helpPage = g_strdup_printf("file:%s/%s", hlpDir, help);
+   Str_Snprintf(helpPage, sizeof helpPage, "file:%s%s", hlpDir, help);
    if (!GuestApp_OpenUrl(helpPage, FALSE)) {
       ToolsMain_MsgBox("Help Unavailable",
                        "Sorry, but help requires a web browser.  You may need "
                        "to modify your PATH environment variable accordingly.");
+      return;
    }
-   g_free(helpPage);
 }
 
 
@@ -260,11 +229,12 @@ ToolsMain_OpenHelp(const char *help) // IN
  *-----------------------------------------------------------------------------
  */
 
-static void
+void
 ToolsMain_OnHelp(gpointer btn,  // IN: Unused
                  gpointer data) // IN: notebook containing all tabs
 {
-   gint page;
+   char *text;
+   GtkWidget *curPage;
    GtkNotebook *nb;
 
    nb = GTK_NOTEBOOK(data);
@@ -272,53 +242,24 @@ ToolsMain_OnHelp(gpointer btn,  // IN: Unused
       return;
    }
 
-   page = gtk_notebook_get_current_page(nb) + 1;
-   ASSERT(page > 0);
-
-   if (page >= ARRAYSIZE(gHelpPages)) {
-      char *text;
-      GtkWidget *curPage;
-      curPage = gtk_notebook_get_nth_page(nb, page - 1);
-      gtk_label_get(GTK_LABEL(gtk_notebook_get_tab_label(nb,curPage)), &text);
-      Warning("No help page for tab %s, defaulting to index.\n", text);
-      page = 0;
+   curPage = gtk_notebook_get_nth_page(nb, gtk_notebook_get_current_page(nb));
+   gtk_label_get(GTK_LABEL(gtk_notebook_get_tab_label(nb,curPage)), &text);
+   if (strcmp(text, TAB_LABEL_OPTIONS) == 0) {
+      ToolsMain_OpenHelp("tools_options.htm");
+   } else if (strcmp(text, TAB_LABEL_DEVICES) == 0) {
+      ToolsMain_OpenHelp("tools_devices.htm");
+   } else if (strcmp(text, TAB_LABEL_SCRIPTS) == 0) {
+      ToolsMain_OpenHelp("tools_scripts.htm");
+   } else if (strcmp(text, TAB_LABEL_SHRINK) == 0) {
+      ToolsMain_OpenHelp("tools_shrink.htm");
+   } else if (strcmp(text, TAB_LABEL_ABOUT) == 0) {
+   ToolsMain_OpenHelp("tools_about.htm");
+   } else {
+      ToolsMain_OpenHelp("index.htm");
    }
-
-   ToolsMain_OpenHelp(gHelpPages[page]);
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * ToolsMain_CheckF1Help --
- *
- *      Snooper function to detect F1 key press and display appropriate help
- *      page to the user.
- *
- * Results:
- *      TRUE if user pressed F1.
- *      FALSE if user pressed any other key and it should be processed further.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static gint
-ToolsMain_CheckF1Help(GtkWidget *widget,    // IN: Unused
-                      GdkEventKey *event,   // IN: event that was snooped
-                      gpointer data)        // IN: notebook containing all tabs
-{
-   if (event->type != GDK_KEY_PRESS || event->keyval != GDK_F1) {
-      return FALSE; /* pass event further */
-   }
-
-   ToolsMain_OnHelp(NULL, data);
-   return TRUE;
-}
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -353,7 +294,7 @@ ToolsMain_MsgBox(gchar *title, // IN: dialog's title
    gtk_widget_show(dialog);
    gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
    gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 10);
-   gdk_window_set_icon_list(dialog->window, gIconList);
+   gdk_window_set_icon(dialog->window, NULL, pixmap, bitmask);
 
    label = gtk_label_new (msg);
    gtk_widget_show(label);
@@ -369,8 +310,6 @@ ToolsMain_MsgBox(gchar *title, // IN: dialog's title
    gtk_widget_set_usize(okbtn, 70, 25);
    gtk_signal_connect_object(GTK_OBJECT(okbtn), "clicked",
                              GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));
-   GTK_WIDGET_SET_FLAGS(okbtn, GTK_CAN_DEFAULT);
-   gtk_widget_grab_default(okbtn);
    gtk_widget_show_all(dialog);
 }
 
@@ -410,9 +349,7 @@ ToolsMain_YesNoBox(gchar *title, gchar *msg)
    gtk_widget_show(dialog);
    gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
    gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 10);
-   gdk_window_set_icon_list(dialog->window, gIconList);
-   gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
-                      GTK_SIGNAL_FUNC(ToolsMain_YesNoBoxOnDestroy), &ret);
+   gdk_window_set_icon(dialog->window, NULL, pixmap, bitmask);
 
    label = gtk_label_new (msg);
    gtk_widget_show(label);
@@ -468,53 +405,21 @@ ToolsMain_YesNoBox(gchar *title, gchar *msg)
  *-----------------------------------------------------------------------------
  */
 
-static void
+void
 ToolsMain_YesNoBoxOnClicked(GtkButton *btn,     // IN: clicked button
                            gpointer user_data) // OUT: pointer to result value
 {
    char *text;
-   int *ret = (int *)user_data;
-
+   Bool *ret = (Bool*)user_data;
    gtk_label_get(GTK_LABEL(GTK_BIN(btn)->child), &text);
-
    if (strcmp(text, "Yes") == 0) {
       *ret = 1;
    } else if (strcmp(text, "No") == 0) {
       *ret = 2;
    }
-
    gtk_widget_destroy(GTK_WIDGET(gtk_widget_get_toplevel(GTK_WIDGET(btn))));
 }
 
-/*
- *-----------------------------------------------------------------------------
- *
- * ToolsMain_YesNoBoxOnDestroy  --
- *
- *      Callback for the gtk signal "destroy" on the Yes/No dialog.
- *      If user did not press any buttons but attempted to close
- *      the window assume that changes should not be saved.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      May cause changes to scripts and other settings to be abandoned.
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-ToolsMain_YesNoBoxOnDestroy(GtkWidget *widget, // IN: Unused
-                            gpointer data)     // OUT: result value
-{
-   int *ret = (int *) data;
-
-   if (*ret == 0) {
-      *ret = 2;
-   }
-}
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -534,7 +439,7 @@ ToolsMain_YesNoBoxOnDestroy(GtkWidget *widget, // IN: Unused
  *-----------------------------------------------------------------------------
  */
 
-static void
+void
 ToolsMain_OnDestroy(GtkWidget *widget, // IN: Unused
                     gpointer data)     // IN: Unused
 {
@@ -569,7 +474,7 @@ ToolsMain_OnDestroy(GtkWidget *widget, // IN: Unused
  *-----------------------------------------------------------------------------
  */
 
-static gint
+gint
 EventQueuePump(gpointer data) // IN: Unused
 {
    int ret;
@@ -606,7 +511,7 @@ EventQueuePump(gpointer data) // IN: Unused
  *-----------------------------------------------------------------------------
  */
 
-static GtkWidget*
+GtkWidget*
 ToolsMain_Create(void)
 {
    GtkWidget *ToolsMain;
@@ -657,28 +562,23 @@ ToolsMain_Create(void)
 #else
       gtk_notebook_append_page(GTK_NOTEBOOK(notebookMain),
                                Devices_Create(ToolsMain),
-                               gtk_label_new(TAB_LABEL_DEVICES));
+                               gtk_label_newl(TAB_LABEL_DEVICES));
 #endif
    } else {
       Debug("User not allowed to edit devices");
    }
    free(result);
 
-   /**
-    * @note If adding or removing tabs to the notebook, make sure to update
-    *       @ref gHelpPages.  The indices between these tabs directly correspond
-    *       to that variable's list of pages.
-    */
 #ifdef GTK2
    gtk_notebook_append_page(GTK_NOTEBOOK(notebookMain), Scripts_Create(ToolsMain),
-                            gtk_label_new_with_mnemonic(TAB_LABEL_SCRIPTS));
+                         gtk_label_new_with_mnemonic(TAB_LABEL_SCRIPTS));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebookMain), Shrink_Create(ToolsMain),
                             gtk_label_new_with_mnemonic(TAB_LABEL_SHRINK));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebookMain), About_Create(ToolsMain),
                             gtk_label_new_with_mnemonic(TAB_LABEL_ABOUT));
 #else
    gtk_notebook_append_page(GTK_NOTEBOOK(notebookMain), Scripts_Create(ToolsMain),
-                            gtk_label_new(TAB_LABEL_SCRIPTS));
+                         gtk_label_new(TAB_LABEL_SCRIPTS));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebookMain), Shrink_Create(ToolsMain),
                             gtk_label_new(TAB_LABEL_SHRINK));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebookMain), About_Create(ToolsMain),
@@ -712,9 +612,6 @@ ToolsMain_Create(void)
 #else
    btn = gtk_button_new_with_label("Close");
 #endif
-
-   gtk_key_snooper_install(ToolsMain_CheckF1Help, notebookMain);
-
    gtk_widget_show(btn);
    gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
    gtk_signal_connect_object(GTK_OBJECT(btn), "clicked",
@@ -743,7 +640,7 @@ ToolsMain_Create(void)
  *-----------------------------------------------------------------------------
  */
 
-static Bool
+Bool
 RpcInResetCB(RpcInData *data) // IN/OUT
 {
    Debug("----------toolbox: Received 'reset' from vmware\n");
@@ -768,7 +665,7 @@ RpcInResetCB(RpcInData *data) // IN/OUT
  *-----------------------------------------------------------------------------
  */
 
-static void
+void
 RpcInErrorCB(void *clientdata, char const *status)
 {
    Warning("Error in the RPC recieve loop: %s\n", status);
@@ -796,7 +693,7 @@ RpcInErrorCB(void *clientdata, char const *status)
  *-----------------------------------------------------------------------------
  */
 
-static Bool
+Bool
 RpcInSetOptionCB(char const **result,     // OUT
                  size_t *resultLen,       // OUT
                  const char *name,        // IN
@@ -888,7 +785,7 @@ RpcInSetOptionCB(char const **result,     // OUT
  *-----------------------------------------------------------------------------
  */
 
-static Bool
+Bool
 RpcInCapRegCB(char const **result,     // OUT
               size_t *resultLen,       // OUT
               const char *name,        // IN
@@ -953,67 +850,6 @@ OnViewportSizeRequest(GtkWidget *widget,           // IN
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * InitHelpDir --
- *
- *      Queries the location db for the location of the Toolbox Help docs.
- *      If not found, will try to fall back to semi-safe defaults.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      If a suitable directory was found, hlpDir will point to a buffer
- *      containing it.  Otherwise it will remain NULL.
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-InitHelpDir(void)
-{
-   gchar *tmpDir = NULL;
-
-   ASSERT(hlpDir == NULL);
-
-   /*
-    * XXX: open-vm-tools doesn't have an installer db, nor does it have help
-    * files yet IIRC.
-    */
-#if !defined(OPEN_VM_TOOLS)
-   if (InstallerDB_Init("/etc/vmware-tools", TRUE)) {
-      const char *libdir = InstallerDB_GetLibDir();
-      if (libdir != NULL) {
-         tmpDir = g_strdup_printf("%s/hlp", libdir);
-      }
-      InstallerDB_DeInit();
-   }
-#endif
-
-   if (!tmpDir || !File_Exists(tmpDir)) {
-      unsigned int i;
-
-      static const char *candidates[] = {
-         "/usr/lib/vmware-tools/hlp",        // Linux, Solaris
-         "/usr/local/lib/vmware-tools/hlp",  // FreeBSD
-      };
-
-      for (i = 0; i < ARRAYSIZE(candidates); i++) {
-         if (File_Exists(candidates[i])) {
-            tmpDir = g_strdup(candidates[i]);
-            break;
-         }
-      }
-   }
-
-   if (tmpDir) {
-      hlpDir = tmpDir;
-   }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1030,7 +866,7 @@ InitHelpDir(void)
  *-----------------------------------------------------------------------------
  */
 
-static void
+void
 ShowUsage(char const *prog)
 {
    fprintf(stderr,
@@ -1056,7 +892,7 @@ ShowUsage(char const *prog)
  *      This is main
  *
  * Results:
- *      0 on success.
+ *      0 on success, -1 otherwise
  *
  * Side effects:
  *      The linux toolbox ui will run and do a variety of tricks for your
@@ -1065,19 +901,35 @@ ShowUsage(char const *prog)
  *-----------------------------------------------------------------------------
  */
 int
-main(int argc,                  // IN: ARRAY_SIZEOF(argv)
-     char *argv[],              // IN: argument vector
-     const char *envp[])        // IN: environment vector
+main(int argc, char *argv[])
 {
+   char *tmp;
    Bool optIconify, optHelp, optVersion;
    struct sigaction olds[ARRAYSIZE(gSignals)];
-   GKeyFile *pConfDict;
-   GdkPixbuf *iconPixbuf;
+   GuestApp_Dict *pConfDict;
+
+   /*
+    * Setup the path to the help directory. We assume that this binary lives in
+    * "LIBDIR/<some_bin_path>" and that the help directory is "LIBDIR/hlp".
+    *
+    * If for some reason this isn't true (if argv[0] lacks enough parent
+    * directories) we'll still initialize the help directory path to something,
+    * but subsequent help requests will probably fail.
+    */
+   Str_Strcpy(hlpDir, argv[0], sizeof hlpDir);
+   tmp = Str_Strrchr(hlpDir, '/');
+   if (tmp) {
+      *tmp-- = '\0';
+      tmp = Str_Strrchr(hlpDir, '/');
+      if (tmp) {
+         *tmp = '\0';
+      }
+   }
+   Str_Strcat(hlpDir, "/hlp/", sizeof hlpDir);
 
    if (!VmCheck_IsVirtualWorld()) {
 #ifndef ALLOW_TOOLS_IN_FOREIGN_VM
-      Warning("The VMware Toolbox must be run inside a virtual machine.\n");
-      return 1;
+      Panic("The VMware Toolbox must be run inside a virtual machine.\n");
 #else
       runningInForeignVM = TRUE;
 #endif
@@ -1088,14 +940,10 @@ main(int argc,                  // IN: ARRAY_SIZEOF(argv)
       Panic("vmware-toolbox can't set signal handler\n");
    }
 
-   pConfDict = Toolbox_LoadToolsConf();
-   Debug_Set(g_key_file_get_boolean(pConfDict, "logging", CONFNAME_LOG, NULL),
-             DEBUG_PREFIX);
-   Debug_EnableToFile(g_key_file_get_string(pConfDict, "logging",
-                                            CONFNAME_LOGFILE, NULL),
-                      FALSE);
-   InitHelpDir();
-   g_key_file_free(pConfDict);
+   pConfDict = Conf_Load();
+   Debug_Set(GuestApp_GetDictEntryBool(pConfDict, CONFNAME_LOG), DEBUG_PREFIX);
+   Debug_EnableToFile(GuestApp_GetDictEntry(pConfDict, CONFNAME_LOGFILE), FALSE);
+   GuestApp_FreeDict(pConfDict);
 
    optionAutoHide = FALSE;
 
@@ -1142,13 +990,6 @@ main(int argc,                  // IN: ARRAY_SIZEOF(argv)
       exit(0);
    }
 
-   /*
-    * Determine our pre-VMware wrapper native environment for use with spawned
-    * applications.
-    */
-   gNativeEnviron = System_GetNativeEnviron(envp);
-   GuestApp_SetSpawnEnviron(gNativeEnviron);
-
    /*
     * See bug 73119. Some distros (SUSE 9.2 64-bit) set LC_CTYPE to the UTF-8 version of
     * the locale. This makes the toolbox use a bad font. If we don't cal gtk_set_locale(),
@@ -1192,14 +1033,11 @@ main(int argc,                  // IN: ARRAY_SIZEOF(argv)
                      DefaultScreen (GDK_DISPLAY ()));
    }
 
-   /*
-    * Cast below is needed, in perfect world gdk_pixbuf_new_from_xpm_data
-    * should be taking 'const char * const * data', not 'const char **data'.
-    */
-   iconPixbuf = gdk_pixbuf_new_from_xpm_data((const char**)smallIcon_xpm);
-   gIconList = g_list_append(gIconList, iconPixbuf);
-
-   gdk_window_set_icon_list(toolsMain->window, gIconList);
+   /* Create the icon from a pixmap */
+   colormap = gtk_widget_get_colormap(toolsMain);
+   pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,&bitmask,NULL,
+                                                  smallIcon_xpm);
+   gdk_window_set_icon(toolsMain->window, NULL, pixmap, bitmask);
 
    gXDisplay = GDK_WINDOW_XDISPLAY(toolsMain->window);
    gXRoot = RootWindow(gXDisplay, DefaultScreen(gXDisplay));
@@ -1217,12 +1055,9 @@ main(int argc,                  // IN: ARRAY_SIZEOF(argv)
    gtk_main();
 
    Signal_ResetGroupHandler(gSignals, olds, ARRAYSIZE(gSignals));
-   System_FreeNativeEnviron(gNativeEnviron);
 
-   g_list_foreach(gIconList, (GFunc)gdk_pixbuf_unref, NULL);
-   g_list_free(gIconList);
-
-   g_free(hlpDir);
+   gdk_pixmap_unref(pixmap);
+   gdk_bitmap_unref(bitmask);
 
    return 0;
 }
