@@ -27,20 +27,16 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if defined(__FreeBSD__)
-#include <sys/param.h>
-#include <sys/syslimits.h>
+#if defined(__FreeBSD__) && BSD_VERSION >= 53
+# include <syslimits.h>
 #endif
 
-#include "toolboxGtkInt.h"
+#include "toolboxInt.h"
 #include "guestApp.h"
 #include "conf.h"
-#include "debug.h"
 #include "str.h"
-#include "posix.h"
 #include "procMgr.h"
 #include "file.h"
-#include "util.h"
 
 /*
  * From toolboxInt.h
@@ -54,7 +50,7 @@ GtkWidget *scriptsApply;
 /* X terminal application and option (to launch vi to edit scripts). */
 static char *termApp;
 static char *termAppOption;
-static GKeyFile *confDict;
+static GuestApp_Dict *confDict;
 static GtkWidget *scriptsUseScript;
 static GtkWidget *scriptsDefaultScript;
 static GtkWidget *scriptsCustomScript;
@@ -65,8 +61,6 @@ static GtkWidget *scriptsBrowse;
 static GtkWidget *scriptsFileDlg;
 static GtkWidget *scriptsCombo;
 
-static int scriptsUseFileDlg;
-
 void Scripts_UpdateEnabled(void);
 void Scripts_OnComboChanged(gpointer entry, gpointer data);
 void Scripts_OnUseScriptToggled(gpointer entry, gpointer data);
@@ -75,8 +69,6 @@ void Scripts_OnBrowse(gpointer btn, gpointer data);
 void Scripts_OnEdit(gpointer btn, gpointer data);
 void Scripts_OnRun(gpointer btn, gpointer data);
 void Scripts_BrowseOnOk(gpointer btn, gpointer data);
-void Scripts_BrowseOnCancel(gpointer btn, gpointer data);
-gint Scripts_BrowseOnClose(GtkWidget *widget, GdkEvent *event, gpointer data);
 void Scripts_BrowseOnChanged(gpointer entry, gpointer data);
 void Scripts_PathOnChanged(GtkEditable *editable, gpointer data);
 
@@ -106,7 +98,7 @@ Scripts_Create(GtkWidget* mainWnd)
    GList *items = NULL;
    GSList *radiobtn_group = NULL;
 
-   confDict = Toolbox_LoadToolsConf();
+   confDict = Conf_Load();
 
    scriptstab = gtk_vbox_new(FALSE, 10);
    gtk_widget_show(scriptstab);
@@ -123,11 +115,11 @@ Scripts_Create(GtkWidget* mainWnd)
       gtk_widget_show(hbox);
       gtk_box_pack_start(GTK_BOX(scriptstab), hbox, FALSE, FALSE, 0);
 
-#ifdef GTK2
-      label = gtk_label_new_with_mnemonic("Script Even_t");
-#else
+      #if GTK2
+      label = gtk_label_new_with_mnemonic("Script _Event");
+      #else
       label = gtk_label_new("Script Event");
-#endif
+      #endif
       gtk_widget_show(label);
       gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
@@ -144,10 +136,9 @@ Scripts_Create(GtkWidget* mainWnd)
       gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(scriptsCombo)->entry), FALSE);
       gtk_signal_connect(GTK_OBJECT(GTK_COMBO(scriptsCombo)->entry), "changed",
                          GTK_SIGNAL_FUNC(Scripts_OnComboChanged), NULL);
-#ifdef GTK2
-      gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_COMBO(scriptsCombo)->entry);
 
-      scriptsUseScript = gtk_check_button_new_with_mnemonic("Use Scr_ipt");
+#ifdef GTK2
+      scriptsUseScript = gtk_check_button_new_with_mnemonic("_Use Script");
 #else
       scriptsUseScript = gtk_check_button_new_with_label("Use Script");
 #endif
@@ -160,13 +151,13 @@ Scripts_Create(GtkWidget* mainWnd)
                          GTK_SIGNAL_FUNC(Scripts_OnUseScriptToggled), NULL);
 
       /*
-       * Try to find an available x terminal app to launch a text editor.
+       * Try to find an available x terminal app to launch vi.
        *
-       * Options with different X terminal apps to launch the editor:
-       *    xterm -e [app] foo.txt
-       *    rxvt -e [app] foo.txt
-       *    konsole -e [app] foo.txt
-       *    gnome-terminal -x [app] foo.txt
+       * Options with different X terminal apps to launch vi:
+       *    xterm -e vi foo.txt
+       *    rxvt -e vi foo.txt
+       *    konsole -e vi foo.txt
+       *    gnome-terminal -x vi foo.txt
        */
       termApp = NULL;
       termAppOption = "-e";
@@ -205,7 +196,7 @@ Scripts_Create(GtkWidget* mainWnd)
                          GTK_SIGNAL_FUNC(Scripts_OnDefaultScriptToggled), NULL);
 
 #ifdef GTK2
-      scriptsCustomScript = gtk_radio_button_new_with_mnemonic(NULL,("Cu_stom Script"));
+      scriptsCustomScript = gtk_radio_button_new_with_mnemonic(NULL,("_Custom Script"));
 #else
       scriptsCustomScript = gtk_radio_button_new_with_label(NULL,("Custom Script"));
 #endif
@@ -217,6 +208,7 @@ Scripts_Create(GtkWidget* mainWnd)
       hbox = gtk_hbox_new(FALSE, 10);
       gtk_widget_show(hbox);
       gtk_box_pack_start(GTK_BOX(scriptstab), hbox, FALSE, FALSE, 0);
+      gtk_widget_set_usize(hbox, -1, 25);
 
       scriptsPath = gtk_entry_new();
       gtk_widget_show(scriptsPath);
@@ -236,6 +228,7 @@ Scripts_Create(GtkWidget* mainWnd)
       } else {
          gtk_box_pack_end(GTK_BOX(hbox), scriptsBrowse, FALSE, FALSE, 0);
       }
+      gtk_widget_set_usize(scriptsBrowse, 70, 6);
       gtk_widget_set_sensitive(scriptsBrowse, FALSE);
       gtk_signal_connect(GTK_OBJECT(scriptsBrowse), "clicked",
                          GTK_SIGNAL_FUNC(Scripts_OnBrowse), NULL);
@@ -249,6 +242,7 @@ Scripts_Create(GtkWidget* mainWnd)
 #endif
          gtk_widget_show(scriptsEdit);
          gtk_box_pack_end(GTK_BOX(hbox), scriptsEdit, FALSE, FALSE, 0);
+         gtk_widget_set_usize(scriptsEdit, 70, 25);
          gtk_signal_connect(GTK_OBJECT(scriptsEdit), "clicked",
                             GTK_SIGNAL_FUNC(Scripts_OnEdit), NULL);
       }
@@ -264,6 +258,7 @@ Scripts_Create(GtkWidget* mainWnd)
 #endif
       gtk_widget_show(scriptsRun);
       gtk_box_pack_end(GTK_BOX(hbox), scriptsRun, FALSE, FALSE, 0);
+      gtk_widget_set_usize(scriptsRun, 70, 25);
       gtk_signal_connect(GTK_OBJECT(scriptsRun), "clicked",
                          GTK_SIGNAL_FUNC(Scripts_OnRun), NULL);
 
@@ -274,6 +269,7 @@ Scripts_Create(GtkWidget* mainWnd)
 #endif
       gtk_widget_show(scriptsApply);
       gtk_box_pack_end(GTK_BOX(hbox), scriptsApply, FALSE, FALSE, 0);
+      gtk_widget_set_usize(scriptsApply, 70, 6);
       gtk_widget_set_sensitive(scriptsApply, FALSE);
       gtk_signal_connect(GTK_OBJECT(scriptsApply), "clicked",
                          GTK_SIGNAL_FUNC(Scripts_OnApply), NULL);
@@ -344,37 +340,26 @@ void
 Scripts_OnComboChanged(gpointer entry, // IN: the entry selected
                        gpointer data)  // IN: unused
 {
-   const char *defaultPath;
+   const char *path, *defaultPath;
    const char *currentState;
-   const char *confName = NULL;
-   gchar *path;
-
    currentState = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(scriptsCombo)->entry));
    if (strcmp(currentState, SCRIPT_SUSPEND) == 0) {
-      confName = CONFNAME_SUSPENDSCRIPT;
+      path = GuestApp_GetDictEntry(confDict, CONFNAME_SUSPENDSCRIPT);
+      defaultPath = GuestApp_GetDictEntryDefault(confDict, CONFNAME_SUSPENDSCRIPT);
    } else if (strcmp(currentState, SCRIPT_RESUME) == 0) {
-      confName = CONFNAME_RESUMESCRIPT;
+      path = GuestApp_GetDictEntry(confDict, CONFNAME_RESUMESCRIPT);
+      defaultPath = GuestApp_GetDictEntryDefault(confDict, CONFNAME_RESUMESCRIPT);
    } else if (strcmp(currentState, SCRIPT_OFF) == 0) {
-      confName = CONFNAME_POWEROFFSCRIPT;
+      path = GuestApp_GetDictEntry(confDict, CONFNAME_POWEROFFSCRIPT);
+      defaultPath = GuestApp_GetDictEntryDefault(confDict, CONFNAME_POWEROFFSCRIPT);
    } else if (strcmp(currentState, SCRIPT_ON) == 0) {
-      confName = CONFNAME_POWERONSCRIPT;
-   }
-
-   if (confName != NULL) {
-      path = g_key_file_get_string(confDict, "powerops", confName, NULL);
-      defaultPath = GuestApp_GetDefaultScript(confName);
-      if (path == NULL) {
-         if (g_key_file_has_key(confDict, "powerops", confName, NULL)) {
-            path = g_strdup("");
-         } else {
-            path = g_strdup(defaultPath);
-         }
-      }
+      path = GuestApp_GetDictEntry(confDict, CONFNAME_POWERONSCRIPT);
+      defaultPath = GuestApp_GetDictEntryDefault(confDict, CONFNAME_POWERONSCRIPT);
    } else {
-      path = g_strdup("");
+      path = "";
       defaultPath = "";
    }
-   
+
    gtk_signal_handler_block_by_func(GTK_OBJECT(scriptsUseScript),
                                     GTK_SIGNAL_FUNC(Scripts_OnUseScriptToggled),
                                     NULL);
@@ -383,7 +368,7 @@ Scripts_OnComboChanged(gpointer entry, // IN: the entry selected
                                     NULL);
    if (strcmp(path, "") == 0) {
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(scriptsUseScript), FALSE);
-      path = g_strdup(defaultPath);
+      path = defaultPath;
    } else {
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(scriptsUseScript), TRUE);
    }
@@ -408,7 +393,6 @@ Scripts_OnComboChanged(gpointer entry, // IN: the entry selected
                                       GTK_SIGNAL_FUNC(Scripts_OnUseScriptToggled),
                                       NULL);
    Scripts_UpdateEnabled();
-   g_free(path);
 }
 
 
@@ -487,9 +471,8 @@ Scripts_OnApply(gpointer btn,  // IN: unused
                 gpointer data) // IN: unused
 {
    const char *path;
-   const char *defaultPath;
    const char *currentState;
-   const char *confName = NULL;
+   char *confName;
    Bool enabledUse, enabledDef;
 
    currentState = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(scriptsCombo)->entry));
@@ -502,11 +485,9 @@ Scripts_OnApply(gpointer btn,  // IN: unused
    } else if (strcmp(currentState, SCRIPT_ON) == 0) {
       confName = CONFNAME_POWERONSCRIPT;
    } else {
-      /* Not reached? */
-      return;
+      confName = "";
    }
 
-   defaultPath = GuestApp_GetDefaultScript(confName);
    enabledUse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(scriptsUseScript));
    enabledDef = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(scriptsDefaultScript));
 
@@ -514,15 +495,15 @@ Scripts_OnApply(gpointer btn,  // IN: unused
       path = "";
    } else {
       if (enabledDef) {
-         path = defaultPath;
+         path = GuestApp_GetDictEntryDefault(confDict, confName);
       } else {
          path = gtk_editable_get_chars(GTK_EDITABLE(scriptsPath), 0, -1);
       }
    }
 
-   g_key_file_set_string(confDict, "powerops", confName, path);
+   GuestApp_SetDictEntry(confDict, confName, path);
 
-   Toolbox_SaveToolsConf(confDict);
+   GuestApp_WriteDict(confDict);
    Scripts_UpdateEnabled();
    gtk_widget_set_sensitive(scriptsApply, FALSE);
 
@@ -557,10 +538,8 @@ void
 Scripts_OnEdit(gpointer btn,  // IN: unused
                gpointer data) // IN: unused
 {
-   char *editor;
-   char *scriptName;
-   const char *argv[5];
-   size_t idx = 0;
+   const char *scriptName;
+   char *cmd;
 
    if (!termApp) {
       ToolsMain_MsgBox("Error", "Unable to locate terminal application in "
@@ -568,52 +547,20 @@ Scripts_OnEdit(gpointer btn,  // IN: unused
       return;
    }
 
-   editor = Posix_Getenv("EDITOR");
-   if (editor == NULL || strlen(editor) == 0) {
-      Debug("EDITOR not set, defaulting to vi.\n");
-      editor = g_find_program_in_path("vi");
-      if (editor == NULL) {
-         ToolsMain_MsgBox("Error",
-                          "Cannot edit script because the an editor was not found.");
-         return;
-      } else {
-         /*
-          * Make sure "editor" is allocated with malloc(), so that the call to
-          * free() later does the right thing.
-          */
-         char *tmp = editor;
-         editor = Util_SafeStrdup(tmp);
-         g_free(tmp);
-      }
-   } else {
-      /*
-       * Strings retuned by Posix_Getenv should not be freed
-       * so we make a duplicate that can be freed so we can
-       * have a common code path below.
-       */
-      editor = Util_SafeStrdup(editor);
+   scriptName = gtk_entry_get_text(GTK_ENTRY(scriptsPath));
+   cmd = Str_Asprintf(NULL, "%s %s vi %s >/dev/null 2>&1",
+                      termApp, termAppOption, scriptName);
+   if (!cmd) {
+      ToolsMain_MsgBox("Error", "Failure creating command to edit script.");
+      return;
    }
 
-   scriptName = Toolbox_GetScriptPath(gtk_entry_get_text(GTK_ENTRY(scriptsPath)));
-   argv[idx++] = termApp;
-   if (termAppOption != NULL && strlen(termAppOption) > 0) {
-      argv[idx++] = termAppOption;
-   }
-   argv[idx++] = editor;
-   argv[idx++] = scriptName;
-   argv[idx++] = NULL;
-
-   /* Restore the original LD_LIBRARY_PATH before invoking external apps. */
-   if (!g_spawn_sync(NULL, (gchar **) argv, (gchar **) gNativeEnviron,
-                     G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL, NULL, NULL)) {
-      gchar *msg = g_strdup_printf("Cannot edit script: failed to execute editor %s.",
-                                   editor);
-      ToolsMain_MsgBox("Error", msg);
-      g_free(msg);
+   if (!ProcMgr_ExecSync(cmd, NULL)) {
+      ToolsMain_MsgBox("Error", "Cannot edit script because the vi editor "
+                       "was not found.");
    }
 
-   g_free(scriptName);
-   free(editor);
+   free(cmd);
 }
 
 
@@ -638,15 +585,13 @@ void
 Scripts_OnRun(gpointer btn,  // IN: unused
               gpointer data) // IN: unused
 {
-   char *scriptName;
+   const char *scriptName;
 
-   scriptName = Toolbox_GetScriptPath(gtk_entry_get_text(GTK_ENTRY(scriptsPath)));
+   scriptName = gtk_entry_get_text(GTK_ENTRY(scriptsPath));
    if (!ProcMgr_ExecSync(scriptName, NULL)) {
       ToolsMain_MsgBox("Error", "Failure executing script, please ensure the "
                        "file exists and is executable.");
    }
-
-   g_free(scriptName);
 }
 
 
@@ -678,10 +623,6 @@ Scripts_OnBrowse(gpointer btn,  // IN: unused
    struct stat statBuf;
 
    scriptsFileDlg = gtk_file_selection_new("Select a file");
-
-   scriptsUseFileDlg = TRUE;
-
-   gtk_window_set_modal(GTK_WINDOW(scriptsFileDlg), TRUE);
    gtk_widget_show(scriptsFileDlg);
 
    defaultPath = gtk_entry_get_text(GTK_ENTRY(scriptsPath));
@@ -710,26 +651,25 @@ Scripts_OnBrowse(gpointer btn,  // IN: unused
                                 GTK_SELECTION_BROWSE);
 #endif
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(scriptsFileDlg)->ok_button),
-                      "clicked", GTK_SIGNAL_FUNC(Scripts_BrowseOnOk), &path);
-   gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(scriptsFileDlg)->cancel_button),
-                      "clicked", GTK_SIGNAL_FUNC(Scripts_BrowseOnCancel), NULL);
-   gtk_signal_connect(GTK_OBJECT(scriptsFileDlg),
-                      "delete_event", GTK_SIGNAL_FUNC(Scripts_BrowseOnClose), NULL);
+                       "clicked", (GtkSignalFunc)Scripts_BrowseOnOk, &path);
+   gtk_signal_connect(GTK_OBJECT(scriptsFileDlg), "destroy",
+                      (GtkSignalFunc)gtk_widget_destroyed, &scriptsFileDlg);
+   gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(scriptsFileDlg)->
+                                        cancel_button),
+                             "clicked", (GtkSignalFunc)gtk_widget_destroy,
+                             GTK_OBJECT(scriptsFileDlg));
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(scriptsFileDlg)->selection_entry),
-                      "changed", GTK_SIGNAL_FUNC(Scripts_BrowseOnChanged), 
+                      "changed",
+                      (GtkSignalFunc)Scripts_BrowseOnChanged,
                       GTK_FILE_SELECTION(scriptsFileDlg)->ok_button);
 
 
    /* Block here and pump messages */
-   while (gtk_events_pending() || scriptsUseFileDlg) {
+   while (gtk_events_pending() || scriptsFileDlg != NULL) {
       gtk_main_iteration();
    }
 
-   if (scriptsFileDlg != NULL) {
-      gtk_widget_destroy(scriptsFileDlg);
-   }
-
-   if (*path != 0 && strcmp(path, defaultPath) != 0) {
+   if (*path != 0) {
       gtk_entry_set_text(GTK_ENTRY(scriptsPath), path);
       gtk_widget_set_sensitive(scriptsApply, TRUE);
    }
@@ -768,9 +708,9 @@ Scripts_BrowseOnChanged(gpointer entry, // IN: selection_entry
  *
  * Scripts_BrowseOnOk --
  *
- *      Callback for the gtk signal "clicked" on the Scripts File Browser OK
- *      button. We save the selected path, hide the File Browser and set the
- *      flag for File Browser use to FALSE so that to quit looping.
+ *      Callback for the gtk signal "clicked" on the Scripts file browser OK
+ *      button.  Set "okPressed" to 1 so the Scripts_OnBrowse code knows to
+ *      quit looping and that OK was the reason we closed.
  *
  * Results:
  *      None.
@@ -779,7 +719,7 @@ Scripts_BrowseOnChanged(gpointer entry, // IN: selection_entry
  *      None.
  *
  *-----------------------------------------------------------------------------
-*/
+ */
 
 void
 Scripts_BrowseOnOk(gpointer btn,  // IN: OK button
@@ -787,73 +727,12 @@ Scripts_BrowseOnOk(gpointer btn,  // IN: OK button
 {
    const char *ret;
 
-   gtk_widget_hide(scriptsFileDlg);
-
-   scriptsUseFileDlg = FALSE;
-
    ret = gtk_file_selection_get_filename(GTK_FILE_SELECTION(scriptsFileDlg));
    strncpy((char*)data, ret, PATH_MAX);
+   gtk_widget_destroy(scriptsFileDlg);
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * Scripts_BrowseOnCancel --
- *
- *      Callback for the gtk signal "clicked" on the Scripts File Browser 
- *      Cancel button. We hide the File Browser and set the flag for 
- *      File Browser use to FALSE so that to quit looping.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-Scripts_BrowseOnCancel(gpointer btn,  // IN: OK button
-                       gpointer data) // IN: data
-{
-   gtk_widget_hide(scriptsFileDlg);
-
-   scriptsUseFileDlg = FALSE;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Scripts_BrowseOnClose --
- *
- *      Callback for the gtk signal "delete_event" on the Scripts File Browser 
- *      Close button from titlebar. We hide the File Browser and set the flag 
- *      for File Browser use to FALSE so that to quit looping and return TRUE
- *      so that not to destroy the window object until we get out of events
- *      loop, after that we will destroy the window.
- *
- * Results:
- *      FALSE if you want the widget to be destroyed or TRUE otherwise.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-gint
-Scripts_BrowseOnClose(GtkWidget *widget, GdkEvent *event, gpointer data)
-{
-   gtk_widget_hide(scriptsFileDlg);
-
-   scriptsUseFileDlg = FALSE;
-
-   return TRUE;
-}
-
 /*
  *----------------------------------------------------------------------------
  *
